@@ -156,7 +156,9 @@ class CTX {
     double normals, tangents, scalingFactor;
     int autoCoherence, highlightOrphans, clip, useTransform;
     double tolerance, snap[3], transform[3][3], offset[3];
-    int occFixSmallEdges, occFixSmallFaces, occSewFaces;
+    int occFixSmallEdges, occFixSmallFaces, occSewFaces, occConnectFaces;
+    int copyMeshingMethod, exactExtrusion;
+    int matchGeomAndMesh;
   } geom;
   // mesh options 
   struct {
@@ -164,7 +166,7 @@ class CTX {
     int points, lines, triangles, quadrangles, tetrahedra, hexahedra, prisms, pyramids;
     int surfacesEdges, surfacesFaces, volumesEdges, volumesFaces, numSubEdges;
     int pointsNum, linesNum, surfacesNum, volumesNum;
-    int optimize, optimizeNetgen, refineSteps, qualityType, labelType;
+    int optimize, optimizeNetgen, refineSteps, qualityType, labelType, remove4triangles;
     double normals, tangents, explode, angleSmoothNormals, allowSwapEdgeAngle;
     double mshFileVersion, labelFrequency, pointSize, lineWidth;
     double qualityInf, qualitySup, radiusInf, radiusSup;
@@ -178,6 +180,7 @@ class CTX {
     int smoothInternalEdges, minCircPoints, minCurvPoints;
     int saveAll, saveGroupsOfNodes, binary, bdfFieldFormat, saveParametric;
     int smoothNormals, reverseAllNormals, zoneDefinition, clip;
+    int saveElementTagType;
   } mesh;
   // FIXME: putting these in the mesh struct (where they belong) causes 
   // an LNK1179 error ("duplicate COMDAT") with MSVC...
@@ -189,7 +192,6 @@ class CTX {
     int smooth, animCycle, combineTime, combineRemoveOrig;
     int fileFormat, plugins;
     double animDelay;
-    void (*pluginDrawFunction)(void*);
   }post;
   // solver options 
   struct{
