@@ -1,33 +1,13 @@
-// $Id: Octree.cpp,v 1.5 2008-03-20 11:44:02 geuzaine Exp $
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
-// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
-// 
-// Please report all bugs and problems to <gmsh@geuz.org>.
+// See the LICENSE.txt file for license information. Please report all
+// bugs and problems to <gmsh@geuz.org>.
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <list>
 #include "Octree.h"
 
-using std::list;
-
-void free_buckets(octantBucket *);
-
 Octree* Octree_Create(int maxElements, double origin[3], double size[3],   
                       void  (*BB)(void *, double*, double*),
                       void  (*Centroid)(void *, double *),
@@ -42,11 +22,34 @@ Octree* Octree_Create(int maxElements, double origin[3], double size[3],
   return myOctree;
 }
 
+void free_buckets(octantBucket * bucket)
+{
+  int i, numBuck = 8;
+  ELink ptr1, ptr2;
+
+  if(bucket->next == NULL) {
+    ptr1 = bucket->lhead;
+    while(ptr1 != NULL) {
+      ptr2 = ptr1;
+      ptr1 = ptr1->next;
+      delete ptr2;
+    }
+    bucket->listBB.clear(); 
+    return;
+  }
+
+  for(i = numBuck-1; i >= 0; i--) 
+    free_buckets((bucket->next)+i);         
+  delete []bucket->next;
+  return;
+}
+
 void Octree_Delete(Octree *myOctree)
 {
   delete myOctree->info;
   free_buckets(myOctree->root);
   delete myOctree->root;
+  delete myOctree;
 }
 
 void Octree_Insert(void * element, Octree *myOctree)
@@ -78,29 +81,7 @@ void * Octree_Search(double *pt, Octree *myOctree)
                        myOctree->function_BB, myOctree->function_inElement);
 }
 
-void free_buckets(octantBucket * bucket)
-{
-  int i, numBuck = 8;
-  ELink ptr1, ptr2;
-
-  if(bucket->next == NULL) {
-    ptr1 = bucket->lhead;
-    while(ptr1 != NULL) {
-      ptr2 = ptr1;
-      ptr1 = ptr1->next;
-      delete ptr2;
-    }
-    bucket->listBB.clear(); 
-    return;
-  }
-
-  for(i = numBuck-1; i >= 0; i--) 
-    free_buckets((bucket->next)+i);         
-  delete []bucket->next;
-  return;
-}
-
-void Octree_SearchAll(double * pt, Octree * myOctree, list<void *> * output)
+void Octree_SearchAll(double * pt, Octree * myOctree, std::list<void *> * output)
 {
   searchAllElements(myOctree->root, pt, myOctree->info, myOctree->function_BB,
                     myOctree->function_inElement, output);      
