@@ -68,6 +68,12 @@ std::string opt_solver_extension1(OPT_ARGS_STR);
 std::string opt_solver_extension2(OPT_ARGS_STR);
 std::string opt_solver_extension3(OPT_ARGS_STR);
 std::string opt_solver_extension4(OPT_ARGS_STR);
+std::string opt_solver_input_name(OPT_ARGS_STR);
+std::string opt_solver_input_name0(OPT_ARGS_STR);
+std::string opt_solver_input_name1(OPT_ARGS_STR);
+std::string opt_solver_input_name2(OPT_ARGS_STR);
+std::string opt_solver_input_name3(OPT_ARGS_STR);
+std::string opt_solver_input_name4(OPT_ARGS_STR);
 std::string opt_solver_mesh_name(OPT_ARGS_STR);
 std::string opt_solver_mesh_name0(OPT_ARGS_STR);
 std::string opt_solver_mesh_name1(OPT_ARGS_STR);
@@ -292,6 +298,7 @@ double opt_general_orthographic(OPT_ARGS_NUM);
 double opt_general_mouse_selection(OPT_ARGS_NUM);
 double opt_general_mouse_hover_meshes(OPT_ARGS_NUM);
 double opt_general_draw_bounding_box(OPT_ARGS_NUM);
+double opt_general_draw_oriented_bounding_box(OPT_ARGS_NUM);
 double opt_general_fast_redraw(OPT_ARGS_NUM);
 double opt_general_xmin(OPT_ARGS_NUM);
 double opt_general_xmax(OPT_ARGS_NUM);
@@ -422,6 +429,7 @@ double opt_geometry_light_two_side(OPT_ARGS_NUM);
 double opt_geometry_occ_fix_small_edges(OPT_ARGS_NUM);
 double opt_geometry_occ_fix_small_faces(OPT_ARGS_NUM);
 double opt_geometry_occ_sew_faces(OPT_ARGS_NUM);
+double opt_geometry_occ_connect_faces(OPT_ARGS_NUM);
 double opt_geometry_old_circle(OPT_ARGS_NUM);
 double opt_geometry_old_newreg(OPT_ARGS_NUM);
 double opt_geometry_num_sub_edges(OPT_ARGS_NUM);
@@ -432,9 +440,13 @@ double opt_geometry_snap0(OPT_ARGS_NUM);
 double opt_geometry_snap1(OPT_ARGS_NUM);
 double opt_geometry_snap2(OPT_ARGS_NUM);
 double opt_geometry_clip(OPT_ARGS_NUM);
+double opt_geometry_copy_meshing_method(OPT_ARGS_NUM);
+double opt_geometry_exact_extrusion(OPT_ARGS_NUM);
+double opt_geometry_match_geom_and_mesh(OPT_ARGS_NUM);
 double opt_mesh_label_frequency(OPT_ARGS_NUM);
 double opt_mesh_optimize(OPT_ARGS_NUM);
 double opt_mesh_optimize_netgen(OPT_ARGS_NUM);
+double opt_mesh_remove_4_triangles(OPT_ARGS_NUM);
 double opt_mesh_refine_steps(OPT_ARGS_NUM);
 double opt_mesh_normals(OPT_ARGS_NUM);
 double opt_mesh_num_sub_edges(OPT_ARGS_NUM);
@@ -502,6 +514,7 @@ double opt_mesh_dual(OPT_ARGS_NUM);
 double opt_mesh_voronoi(OPT_ARGS_NUM);
 double opt_mesh_draw_skin_only(OPT_ARGS_NUM);
 double opt_mesh_save_all(OPT_ARGS_NUM);
+double opt_mesh_save_element_tag_type(OPT_ARGS_NUM);
 double opt_mesh_save_parametric(OPT_ARGS_NUM);
 double opt_mesh_save_groups_of_nodes(OPT_ARGS_NUM);
 double opt_mesh_color_carousel(OPT_ARGS_NUM);
@@ -806,11 +819,11 @@ void PrintOptions(int num, int level, int diff, int help, const char *filename);
 void PrintOptionsDoc();
 
 bool StringOption(int action, const char *category, int num, 
-		  const char *name, std::string &val);
+                  const char *name, std::string &val);
 bool NumberOption(int action, const char *category, int num, 
-		  const char *name, double &val);
+                  const char *name, double &val);
 bool ColorOption(int action, const char *category, int num, 
-		 const char *name, unsigned int &val);
+                 const char *name, unsigned int &val);
 
 GmshColorTable *GetColorTable(int num);
 
