@@ -24,15 +24,7 @@ int     Tree_Query(Tree_T *Tree, void *data);
 void   *Tree_PQuery(Tree_T *Tree, void *data);
 int     Tree_Suppress(Tree_T *Tree, void *data);
 int     Tree_Size(Tree_T *tree) ;
-
-inline void Tree_Action(Tree_T *tree, void (*action) (void *data, void *dummy))
-{
-  if(!tree) return;
-
-  avl_foreach(tree->root, action, AVL_FORWARD);
-}
-
+void    Tree_Action(Tree_T *tree, void (*action) (void *data, void *dummy));
 List_T *Tree2List(Tree_T *pTree);
 
 #endif
-
