@@ -3,13 +3,15 @@
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
+#include <string.h>
 #include <algorithm>
+#include "GmshMessage.h"
 #include "VertexArray.h"
 #include "Context.h"
 #include "Numeric.h"
 
-template<int N> float ElementDataLessThan<N>::tolerance = 0.;
-float BarycenterLessThan::tolerance = 0.;
+template<int N> float ElementDataLessThan<N>::tolerance = 0.0F;
+float BarycenterLessThan::tolerance = 0.0F;
 
 VertexArray::VertexArray(int numVerticesPerElement, int numElements) 
   : _numVerticesPerElement(numVerticesPerElement)
@@ -40,7 +42,7 @@ void VertexArray::_addNormal(float nx, float ny, float nz)
 }
 
 void VertexArray::_addColor(unsigned char r, unsigned char g, unsigned char b,
-			    unsigned char a)
+                            unsigned char a)
 {
   _colors.push_back(r);
   _colors.push_back(g);
@@ -72,14 +74,14 @@ void VertexArray::add(double *x, double *y, double *z, SVector3 *n,
 }
 
 void VertexArray::add(double *x, double *y, double *z, SVector3 *n, unsigned char *r, 
-		      unsigned char *g, unsigned char *b, unsigned char *a, 
-		      MElement *ele, bool unique, bool boundary)
+                      unsigned char *g, unsigned char *b, unsigned char *a, 
+                      MElement *ele, bool unique, bool boundary)
 {
   int npe = getNumVerticesPerElement();
 
   if(boundary && npe == 3){
     ElementData<3> e(x, y, z, n, r, g, b, a, ele);
-    ElementDataLessThan<3>::tolerance = CTX::instance()->lc * 1.e-12;
+    ElementDataLessThan<3>::tolerance = (float)(CTX::instance()->lc * 1.e-12);
     std::set<ElementData<3>, ElementDataLessThan<3> >::iterator it = _data3.find(e);
     if(it == _data3.end())
       _data3.insert(e);
@@ -89,18 +91,18 @@ void VertexArray::add(double *x, double *y, double *z, SVector3 *n, unsigned cha
   }
 
   if(unique){
-    Barycenter pc(0., 0., 0.);
+    Barycenter pc(0.0F, 0.0F, 0.0F);
     for(int i = 0; i < npe; i++)
       pc += Barycenter(x[i], y[i], z[i]);
-    BarycenterLessThan::tolerance = CTX::instance()->lc * 1.e-12;
+    BarycenterLessThan::tolerance = (float)(CTX::instance()->lc * 1.e-12);
     if(_barycenters.find(pc) != _barycenters.end()) 
       return;
     _barycenters.insert(pc);
   }
 
   for(int i = 0; i < npe; i++){
-    _addVertex(x[i], y[i], z[i]);
-    if(n) _addNormal(n[i].x(), n[i].y(), n[i].z());
+    _addVertex((float)x[i], (float)y[i], (float)z[i]);
+    if(n) _addNormal((float)n[i].x(), (float)n[i].y(), (float)n[i].z());
     if(r && g && b && a) _addColor(r[i], g[i], b[i], a[i]);
     _addElement(ele);
   }
@@ -195,11 +197,11 @@ void VertexArray::sort(double x, double y, double z)
       for(int k = 0; k < 3; k++)
         sortedVertices.push_back(elements[i].v[3 * j + k]);
       if(elements[i].n)
-	for(int k = 0; k < 3; k++)
+        for(int k = 0; k < 3; k++)
           sortedNormals.push_back(elements[i].n[3 * j + k]);
       if(elements[i].c)
-	for(int k = 0; k < 4; k++)
-	  sortedColors.push_back(elements[i].c[4 * j + k]);
+        for(int k = 0; k < 4; k++)
+          sortedColors.push_back(elements[i].c[4 * j + k]);
     }
   }
   
@@ -214,3 +216,68 @@ int VertexArray::getMemoryUsage()
     _colors.size() * sizeof(unsigned char);
   return bytes / 1024 / 1024;
 }
+
+char *VertexArray::toChar(int num, int type, double min, double max, double time,
+                          SBoundingBox3d bbox, int &len)
+{
+  int vn = _vertices.size(), nn = _normals.size(), cn = _colors.size();
+  int vs = vn * sizeof(float), ns = nn * sizeof(char), cs = cn * sizeof(unsigned char);
+  int is = sizeof(int), ds = sizeof(double);
+  double xmin = bbox.min().x(), ymin = bbox.min().y(), zmin = bbox.min().z();
+  double xmax = bbox.max().x(), ymax = bbox.max().y(), zmax = bbox.max().z();
+
+  len = 5 * is + 9 * ds + vs + ns + cs;
+  char *bytes = new char[len];
+  int index = 0;
+  memcpy(&bytes[index], &num, is); index += is;
+  memcpy(&bytes[index], &type, is); index += is;
+  memcpy(&bytes[index], &min, ds); index += ds;
+  memcpy(&bytes[index], &max, ds); index += ds;
+  memcpy(&bytes[index], &time, ds); index += ds;
+  memcpy(&bytes[index], &xmin, ds); index += ds;
+  memcpy(&bytes[index], &ymin, ds); index += ds;
+  memcpy(&bytes[index], &zmin, ds); index += ds;
+  memcpy(&bytes[index], &xmax, ds); index += ds;
+  memcpy(&bytes[index], &ymax, ds); index += ds;
+  memcpy(&bytes[index], &zmax, ds); index += ds;
+  memcpy(&bytes[index], &vn, is); index += is;
+  memcpy(&bytes[index], &_vertices[0], vs); index += vs;
+  memcpy(&bytes[index], &nn, is); index += is;
+  memcpy(&bytes[index], &_normals[0], ns); index += ns;
+  memcpy(&bytes[index], &cn, is); index += is;
+  memcpy(&bytes[index], &_colors[0], cs); index += cs;
+  return bytes;
+}
+
+void VertexArray::fromChar(const char *bytes)
+{
+  int is = sizeof(int), ds = sizeof(double), index = 0;
+
+  int num; memcpy(&num, &bytes[index], is); index += is;
+
+  if(num > 65535)
+    Msg::Error("Should swap data in vertex array stream");
+
+  int type; memcpy(&type, &bytes[index], is); index += is;
+  double min; memcpy(&min, &bytes[index], ds); index += ds;
+  double max; memcpy(&max, &bytes[index], ds); index += ds;
+  double time; memcpy(&time, &bytes[index], ds); index += ds;
+  double xmin; memcpy(&xmin, &bytes[index], ds); index += ds;
+  double ymin; memcpy(&ymin, &bytes[index], ds); index += ds;
+  double zmin; memcpy(&zmin, &bytes[index], ds); index += ds;
+  double xmax; memcpy(&xmax, &bytes[index], ds); index += ds;
+  double ymax; memcpy(&ymax, &bytes[index], ds); index += ds;
+  double zmax; memcpy(&zmax, &bytes[index], ds); index += ds;
+
+  int vn; memcpy(&vn, &bytes[index], is); index += is;
+  _vertices.resize(vn); int vs = vn * sizeof(float);
+  memcpy(&_vertices[0], &bytes[index], vs); index += vs;
+
+  int nn; memcpy(&nn, &bytes[index], is); index += is;
+  _normals.resize(nn); int ns = nn * sizeof(char); 
+  memcpy(&_normals[0], &bytes[index], ns); index += ns;
+
+  int cn; memcpy(&cn, &bytes[index], is); index += is;
+  _colors.resize(cn); int cs = cn * sizeof(unsigned char); 
+  memcpy(&_colors[0], &bytes[index], cs); index += cs;
+}
