@@ -4,17 +4,17 @@
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <string>
-#include "GUI.h"
-#include "menuWindow.h"
 #include "Gmsh.h"
 #include "GmshMessage.h"
-#include "Draw.h"
+#include "GmshRemote.h"
+#include "FlGui.h"
+#include "menuWindow.h"
+#include "drawContext.h"
 #include "Context.h"
 #include "Options.h"
 #include "Parser.h"
 #include "OpenFile.h"
 #include "CommandLine.h"
-#include "Solvers.h"
 #include "PluginManager.h"
 #include "GModel.h"
 #include "Field.h"
@@ -47,7 +47,7 @@ int main(int argc, char *argv[])
   }
 
   // Create the GUI
-  GUI::instance(argc, argv);
+  FlGui::instance(argc, argv);
 
   // Set all previously defined options in the GUI
   InitOptionsGUI(0);
@@ -66,11 +66,11 @@ int main(int argc, char *argv[])
   Msg::Info("Packager       : %s", GetGmshPackager());
   Msg::Info("Home directory : %s", CTX::instance()->homeDir.c_str());
   Msg::Info("Launch date    : %s", Msg::GetLaunchDate().c_str());
-  Msg::Info("Command line   : %s", Msg::GetCommandLine().c_str());
+  Msg::Info("Command line   : %s", Msg::GetCommandLineArgs().c_str());
   Msg::Info("-------------------------------------------------------");
 
   // Display the GUI immediately to have a quick "a la Windows" launch time
-  GUI::instance()->check();
+  FlGui::instance()->check();
 
   // Open project file and merge all other input files
   OpenProject(GModel::current()->getFileName());
@@ -85,20 +85,20 @@ int main(int argc, char *argv[])
   
   if(CTX::instance()->post.combineTime){
     PView::combine(true, 2, CTX::instance()->post.combineRemoveOrig);
-    GUI::instance()->updateViews();
+    FlGui::instance()->updateViews();
   }
 
   // Init first context
   switch (CTX::instance()->initialContext) {
-  case 1: GUI::instance()->menu->setContext(menu_geometry, 0); break;
-  case 2: GUI::instance()->menu->setContext(menu_mesh, 0); break;
-  case 3: GUI::instance()->menu->setContext(menu_solver, 0); break;
-  case 4: GUI::instance()->menu->setContext(menu_post, 0); break;
+  case 1: FlGui::instance()->menu->setContext(menu_geometry, 0); break;
+  case 2: FlGui::instance()->menu->setContext(menu_mesh, 0); break;
+  case 3: FlGui::instance()->menu->setContext(menu_solver, 0); break;
+  case 4: FlGui::instance()->menu->setContext(menu_post, 0); break;
   default: // automatic
     if(PView::list.size())
-      GUI::instance()->menu->setContext(menu_post, 0);
+      FlGui::instance()->menu->setContext(menu_post, 0);
     else
-      GUI::instance()->menu->setContext(menu_geometry, 0);
+      FlGui::instance()->menu->setContext(menu_geometry, 0);
     break;
   }
 
@@ -112,11 +112,12 @@ int main(int argc, char *argv[])
   }
 
   // Draw the scene
-  Draw();
+  drawContext::global()->draw();
 
   // Listen to external solvers
-  if(CTX::instance()->solver.listen) Solver(-1, NULL);
+  if(CTX::instance()->solver.listen)
+    GmshRemote::get(-1)->run("");
 
   // loop
-  return GUI::instance()->run();
+  return FlGui::instance()->run();
 }
