@@ -3,16 +3,17 @@
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
-#include <Fl/Fl_Tabs.H>
-#include <Fl/Fl_Value_Input.H>
-#include "GUI.h"
+#include <FL/Fl_Tabs.H>
+#include <FL/Fl_Value_Input.H>
+#include "FlGui.h"
 #include "classificationEditor.h"
 #include "paletteWindow.h"
 #include "Numeric.h"
-#include "Draw.h"
+#include "drawContext.h"
 #include "Options.h"
 #include "Context.h"
 #include "GmshMessage.h"
+#include "MLine.h"
 #include "meshGFaceDelaunayInsertion.h"
 #include "meshGFaceOptimize.h"
 #include "discreteEdge.h"
@@ -44,6 +45,7 @@ static void ElementsSelectedMode(classificationEditor *e)
   e->_togbuttons[CLASSTOGBUTTON_HIDE]->deactivate(); 
 }
 
+// we should 
 static void class_selectgface_cb(Fl_Widget *w, void *data)
 {
   classificationEditor *e = (classificationEditor*)data;
@@ -53,16 +55,16 @@ static void class_selectgface_cb(Fl_Widget *w, void *data)
 
   while(1) {
     CTX::instance()->mesh.changed = ENT_ALL;
-    Draw();
+    drawContext::global()->draw();
 
     Msg::StatusBar(3, false, "Select Model Face\n"
-        "[Press 'e' to end selection or 'q' to abort]");
+                   "[Press 'e' to end selection or 'q' to abort]");
     
-    char ib = GUI::instance()->selectEntity(ENT_SURFACE);
+    char ib = FlGui::instance()->selectEntity(ENT_SURFACE);
     if(ib == 'l') {
-      for(unsigned int i = 0; i < GUI::instance()->selectedFaces.size(); i++){
-        GUI::instance()->selectedFaces[i]->setSelection(1);
-        temp.push_back(GUI::instance()->selectedFaces[i]);
+      for(unsigned int i = 0; i < FlGui::instance()->selectedFaces.size(); i++){
+        FlGui::instance()->selectedFaces[i]->setSelection(1);
+        temp.push_back(FlGui::instance()->selectedFaces[i]);
       }
     }
     // ok store the list of gfaces !
@@ -80,7 +82,7 @@ static void class_selectgface_cb(Fl_Widget *w, void *data)
     }
   } 
   CTX::instance()->mesh.changed = ENT_ALL;
-  Draw();  
+  drawContext::global()->draw();  
   Msg::StatusBar(3, false, "");
 }
 
@@ -93,25 +95,25 @@ static void class_deleteedge_cb(Fl_Widget *w, void *data)
   
   while(1) {
     CTX::instance()->mesh.changed = ENT_ALL;
-    Draw();
+    drawContext::global()->draw();
 
     Msg::StatusBar(3, false, "Select Elements\n"
-        "[Press 'e' to end selection or 'q' to abort]");
+                   "[Press 'e' to end selection or 'q' to abort]");
     
-    char ib = GUI::instance()->selectEntity(ENT_ALL);
+    char ib = FlGui::instance()->selectEntity(ENT_ALL);
     if(ib == 'l') {
       if(CTX::instance()->pickElements){
-        for(unsigned int i = 0; i < GUI::instance()->selectedElements.size(); i++){
-          MElement *me = GUI::instance()->selectedElements[i];
-          if(me->getNumEdges() == 1 && me->getVisibility() != 2){
+        for(unsigned int i = 0; i < FlGui::instance()->selectedElements.size(); i++){
+          MElement *me = FlGui::instance()->selectedElements[i];
+          if(me->getType() == TYPE_LIN && me->getVisibility() != 2){
             me->setVisibility(2); ele.push_back((MLine*)me);
           }
         }
       }
     }
     if(ib == 'r') {
-      for(unsigned int i = 0; i < GUI::instance()->selectedElements.size(); i++)
-        GUI::instance()->selectedElements[i]->setVisibility(1);
+      for(unsigned int i = 0; i < FlGui::instance()->selectedElements.size(); i++)
+        FlGui::instance()->selectedElements[i]->setVisibility(1);
     }
     // ok, we compute edges !
     if(ib == 'e') {
@@ -141,7 +143,7 @@ static void class_deleteedge_cb(Fl_Widget *w, void *data)
   
   CTX::instance()->mesh.changed = ENT_ALL;
   CTX::instance()->pickElements = 0;
-  Draw();  
+  drawContext::global()->draw();  
   Msg::StatusBar(3, false, "");
 }
 
@@ -158,7 +160,7 @@ static void class_save_cb(Fl_Widget *w, void *data)
   CTX::instance()->mesh.changed = ENT_ALL;
   CTX::instance()->pickElements = 0;
   NoElementsSelectedMode(e);
-  Draw();  
+  drawContext::global()->draw();  
   Msg::StatusBar(3, false, "");
 }
 
@@ -174,7 +176,7 @@ static void class_clear_cb(Fl_Widget *w, void *data)
   CTX::instance()->mesh.changed = ENT_ALL;
   CTX::instance()->pickElements = 0;
   NoElementsSelectedMode(e);
-  Draw();  
+  drawContext::global()->draw();  
   Msg::StatusBar(3, false, "");
 }
 
@@ -194,17 +196,6 @@ static void class_ok_cb(Fl_Widget *w, void *data)
   Msg::StatusBar(3, false, "");
 }
 
-static void class_okcolor_cb(Fl_Widget *w, void *data)
-{
-  classificationEditor *e = (classificationEditor*)data;
-  e->edge_detec->deactivate();
-  e->edge_detec->show();
-  e->face_color->deactivate();
-  e->face_color->hide();
-  //  class_save_cb(w,data);
-  Msg::StatusBar(3, false, "");
-}
-
 static int maxEdgeNum()
 {
   GModel::eiter it = GModel::current()->firstEdge();
@@ -272,7 +263,8 @@ static GEdge *getNewModelEdge(GFace *gf1, GFace *gf2,
   std::map<std::pair<int, int>, GEdge*>::iterator it = 
     newEdges.find(std::make_pair<int, int>(i1, i2));
   if(it == newEdges.end()){
-    discreteEdge *temporary = new discreteEdge(GModel::current(), maxEdgeNum() + 1);
+    discreteEdge *temporary = new discreteEdge(GModel::current(), maxEdgeNum() + 1, 0, 0);
+    printf("add new edge gf1=%d gf2=%d \n", t1, t2);
     GModel::current()->add(temporary);
     newEdges[std::make_pair<int, int>(i1, i2)] = temporary;
     return temporary;
@@ -288,6 +280,7 @@ static void recurClassifyEdges(MTri3 *t,
                                std::map<std::pair<int, int>, GEdge*> &newEdges)
 {
   if(!t->isDeleted()){
+
     t->setDeleted(true);
     GFace *gf1 = reverse[t->tri()];
     for(int i = 0; i < 3; i++){
@@ -308,6 +301,7 @@ static void recurClassifyEdges(MTri3 *t,
       if(tn)
         recurClassifyEdges(tn, reverse, lines, touched, newEdges);
     }
+
   }
 }
 
@@ -337,6 +331,9 @@ static void class_color_cb(Fl_Widget* w, void* data)
     }
   }
 
+  if ( !tris.size() )return;
+
+
   connectTriangles(tris);
 
   {
@@ -362,19 +359,117 @@ static void class_color_cb(Fl_Widget* w, void* data)
     
     it = tris.begin();
     
+    //classify edges that are bound by different GFaces
+    //--------------------------------------------------
     std::map<std::pair<int, int>, GEdge*> newEdges;
     std::set<MLine*> touched;
     recurClassifyEdges(*it, reverse, lines, touched, newEdges);
     GModel::current()->remove(e->saved);
 
+    //check if new edges should not be splitted 
+    //splitted if composed of several open or closed edges
+    //-----------------------------------------------------
+    for (std::map<std::pair<int, int>, GEdge*>::iterator it = newEdges.begin() ; it != newEdges.end() ; ++it){
+
+      GEdge *ge = it->second;
+      printf("NEW edge with tag  = %d \n", ge->tag());
+
+      std::list<MLine*> segments;
+      for (unsigned int i = 0; i < ge->lines.size(); i++){
+        segments.push_back(ge->lines[i]);
+      }
+
+      //for each actual GEdge
+      while (!segments.empty()) {
+
+        std::vector<MLine*> myLines;
+        std::list<MLine*>::iterator it = segments.begin();
+
+        MVertex *vB = (*it)->getVertex(0);
+        MVertex *vE = (*it)->getVertex(1);
+        myLines.push_back(*it);
+        segments.erase(it);
+        it++;
+
+        //printf("***candidate mline %d %d of size %d \n", vB->getNum(), vE->getNum(), segments.size());
+
+        for (int i=0; i<2; i++) {
+
+          for (std::list<MLine*>::iterator it = segments.begin() ; it != segments.end(); ++it){ 
+            MVertex *v1 = (*it)->getVertex(0);
+            MVertex *v2 = (*it)->getVertex(1);
+            //printf("mline %d %d \n", v1->getNum(), v2->getNum());
+            
+            std::list<MLine*>::iterator itp;
+            if ( v1 == vE  ){
+              //printf("->push back this mline \n");
+              myLines.push_back(*it);
+              itp = it;
+              it++;
+              segments.erase(itp);
+              vE = v2;
+              i = -1;
+            }
+            else if ( v2 == vE){
+              //printf("->push back this mline \n");
+              myLines.push_back(*it);
+              itp = it;
+              it++;
+              segments.erase(itp);
+              vE = v1;
+              i=-1;
+            }
+
+            if (it == segments.end()) break;
+
+          }
+
+          if (vB == vE) break;
+
+          if (segments.empty()) break;
+
+          //printf("not found VB=%d vE=%d\n", vB->getNum(), vE->getNum());
+          MVertex *temp = vB;
+          vB = vE;
+          vE = temp;
+          //printf("not found VB=%d vE=%d\n", vB->getNum(), vE->getNum());
+
+        }
+        
+//      printf("************ CANDIDATE NEW EDGE \n");
+//      for (std::vector<MLine*>::iterator it = myLines.begin() ; it != myLines.end() ; ++it){
+//        MVertex *v1 = (*it)->getVertex(0);
+//        MVertex *v2 = (*it)->getVertex(1);
+//        printf("Line %d %d \n", v1->getNum(), v2->getNum());
+//      }
+        GEdge *newGe = new discreteEdge(GModel::current(), maxEdgeNum() + 1, 0, 0);
+        newGe->lines.insert(newGe->lines.end(), myLines.begin(), myLines.end());
+        GModel::current()->add(newGe);
+        printf("create new edge with tag =%d\n", maxEdgeNum());
+        
+      }//end for each actual GEdge
+
+    }
+
+    printf("end new edge with tag \n");
+
+    for (std::map<std::pair<int, int>, GEdge*>::iterator it = newEdges.begin() ; it != newEdges.end() ; ++it){
+      GEdge *ge = it->second;
+      GModel::current()->remove(ge);
+    }
+
+
+
     while(it != tris.end()){
       delete *it;
       ++it;
     }
   }
   
+  printf("before drawing \n");
+  
   CTX::instance()->mesh.changed = ENT_ALL;
-  Draw();  
+  drawContext::global()->draw();  
   Msg::StatusBar(3, false, "");
 }
 
@@ -382,7 +477,7 @@ static void updateedges_cb(Fl_Widget* w, void* data)
 {
   classificationEditor *e = (classificationEditor*)data;
  
-  // printf("%d edges detected\n", e->edges_detected.size());
+  printf("%d inside edges detected\n", (int)e->edges_detected.size());
 
   for(unsigned int i = 0; i < e->temporary->lines.size(); i++){
     delete e->temporary->lines[i];
@@ -397,22 +492,24 @@ static void updateedges_cb(Fl_Widget* w, void* data)
     e->temporary->lines.push_back(new MLine(ea.v1, ea.v2));
   } 
 
+  printf("%d boundary edges detected\n", (int)e->edges_lonly.size());
   if(e->_togbuttons[CLASSTOGBUTTON_CLOS]->value()){
     for(unsigned int i = 0 ; i < e->edges_lonly.size(); i++){
       edge_angle ea = e->edges_lonly[i];
       e->temporary->lines.push_back(new MLine(ea.v1, ea.v2));
+      //check if closed loop
     } 
   }
   
   CTX::instance()->mesh.changed = ENT_ALL;
-  Draw();   
+  drawContext::global()->draw();   
 }
 
 static void class_hide_cb(Fl_Widget *w, void *data)
 {
   CTX::instance()->hideUnselected = !CTX::instance()->hideUnselected;
   CTX::instance()->mesh.changed = ENT_ALL;
-  Draw();
+  drawContext::global()->draw();
 }
 
 static void buildListOfEdgeAngle(e2t_cont adj, std::vector<edge_angle> &edges_detected,
@@ -441,25 +538,25 @@ static void class_select_cb(Fl_Widget *w, void *data)
 
   while(1) {
     CTX::instance()->mesh.changed = ENT_ALL;
-    Draw();
+    drawContext::global()->draw();
 
     Msg::StatusBar(3, false, "Select Elements\n"
-        "[Press 'e' to end selection or 'q' to abort]");
+                   "[Press 'e' to end selection or 'q' to abort]");
     
-    char ib = GUI::instance()->selectEntity(ENT_ALL);
+    char ib = FlGui::instance()->selectEntity(ENT_ALL);
     if(ib == 'l') {
       if(CTX::instance()->pickElements){
-        for(unsigned int i = 0; i < GUI::instance()->selectedElements.size(); i++){
-          MElement *me = GUI::instance()->selectedElements[i];
-          if(me->getNumEdges() == 3 && me->getVisibility() != 2){
+        for(unsigned int i = 0; i < FlGui::instance()->selectedElements.size(); i++){
+          MElement *me = FlGui::instance()->selectedElements[i];
+          if(me->getType() == TYPE_TRI && me->getVisibility() != 2){
             me->setVisibility(2); ele.push_back((MTriangle*)me);
           }
         }
       }
     }
     if(ib == 'r') {
-      for(unsigned int i = 0; i < GUI::instance()->selectedElements.size(); i++)
-        GUI::instance()->selectedElements[i]->setVisibility(1);
+      for(unsigned int i = 0; i < FlGui::instance()->selectedElements.size(); i++)
+        FlGui::instance()->selectedElements[i]->setVisibility(1);
     }
     // ok, we compute edges !
     if(ib == 'e') {
@@ -482,7 +579,7 @@ static void class_select_cb(Fl_Widget *w, void *data)
 
   CTX::instance()->mesh.changed = ENT_ALL;
   CTX::instance()->pickElements = 0;
-  Draw();  
+  drawContext::global()->draw();  
   Msg::StatusBar(3, false, "");
 }
 
@@ -619,9 +716,9 @@ classificationEditor::classificationEditor()
   // saved for the ones that have been saved by the user
   // and that will be used for next step
 
-  temporary = new discreteEdge(GModel::current(), maxEdgeNum() + 1);
+  temporary = new discreteEdge(GModel::current(), maxEdgeNum() + 1, 0, 0);
   GModel::current()->add(temporary);
-  saved = new discreteEdge(GModel::current(), maxEdgeNum() + 1);
+  saved = new discreteEdge(GModel::current(), maxEdgeNum() + 1, 0, 0);
   GModel::current()->add(saved);
   
   _window->end();
