@@ -21,21 +21,58 @@
 #include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "GmshDefines.h"
-#include "GUI.h"
+#include "FlGui.h"
 #include "CreateFile.h"
 #include "Options.h"
-#include "Draw.h"
 #include "GModel.h"
 #include "Context.h"
 
 // File chooser
 
 #if defined(HAVE_NATIVE_FILE_CHOOSER)
+
 #include <FL/Fl_Native_File_Chooser.H>
 static Fl_Native_File_Chooser *fc = 0;
+
 #else
-#include "fileChooser.h"
+
+#include <FL/Fl_File_Chooser.H>
+#include <FL/Fl_Window.H>
+#include <FL/Fl_File_Input.H>
+
+class fileChooser : public Fl_File_Chooser {
+  // we derive our own so we can set its position (The original file
+  // chooser doesn't expose its window to the world, so we need to use
+  // a cheap hack to get to it. Even worse is the hack used to get the
+  // focus on the file input widget.)
+ private:
+  Fl_Window *_win;
+  Fl_File_Input *_in;
+ public:
+  fileChooser(const char *d, const char *p, int t, const char *title)
+    : Fl_File_Chooser(d, p, t, title) 
+  { 
+    _win = dynamic_cast<Fl_Window*>(newButton->parent()->parent()); 
+    _in = dynamic_cast<Fl_File_Input *>(previewButton->parent()->parent()->resizable());
+  }
+  void show()
+  {
+    if(_win){
+      _win->show();
+      rescan(); // necessary since fltk 1.1.7
+      if(_in) _in->take_focus();
+      else _win->take_focus();
+    }
+    else
+      Fl_File_Chooser::show();
+  }
+  void position(int x, int y){ if(_win) _win->position(x,y); }
+  int x(){ if(_win) return _win->x(); else return 100; }
+  int y(){ if(_win) return _win->y(); else return 100; }
+};
+
 static fileChooser *fc = 0;
+
 #endif
 
 int file_chooser(int multi, int create, const char *message,
@@ -814,7 +851,7 @@ int bdf_dialog(const char *name)
 {
   struct _bdf_dialog{
     Fl_Window *window;
-    Fl_Choice *c;
+    Fl_Choice *c, *d;
     Fl_Check_Button *b;
     Fl_Button *ok, *cancel;
   };
@@ -827,17 +864,27 @@ int bdf_dialog(const char *name)
     {0}
   };
 
-  int BBB = BB + 9; // labels too long
+  static Fl_Menu_Item tagmenu[] = {
+    {"Elementary entity", 0, 0, 0},
+    {"Physical entity", 0, 0, 0},
+    {"Partition", 0, 0, 0},
+    {0}
+  };
+  
+  int BBB = BB + 16; // labels too long
 
   if(!dialog){
     dialog = new _bdf_dialog;
-    int h = 3 * WB + 3 * BH, w = 2 * BBB + 3 * WB, y = WB;
+    int h = 3 * WB + 4 * BH, w = 2 * BBB + 3 * WB, y = WB;
     dialog->window = new Fl_Double_Window(w, h, "BDF Options");
     dialog->window->box(GMSH_WINDOW_BOX);
     dialog->window->set_modal();
-    dialog->c = new Fl_Choice(WB, y, BBB + BBB / 2, BH, "Format"); y += BH;
+    dialog->c = new Fl_Choice(WB, y, BBB + BBB / 4, BH, "Format"); y += BH;
     dialog->c->menu(formatmenu);
     dialog->c->align(FL_ALIGN_RIGHT);
+    dialog->d = new Fl_Choice(WB, y, BBB + BBB / 4, BH, "Element tag"); y += BH;
+    dialog->d->menu(tagmenu);
+    dialog->d->align(FL_ALIGN_RIGHT);
     dialog->b = new Fl_Check_Button
       (WB, y, 2 * BBB + WB, BH, "Save all (ignore physical groups)"); y += BH;
     dialog->b->type(FL_TOGGLE_BUTTON);
@@ -848,6 +895,8 @@ int bdf_dialog(const char *name)
   }
   
   dialog->c->value(CTX::instance()->mesh.bdfFieldFormat);
+  dialog->d->value((CTX::instance()->mesh.saveElementTagType == 3) ? 2 :
+                   (CTX::instance()->mesh.saveElementTagType == 2) ? 1 : 0);
   dialog->b->value(CTX::instance()->mesh.saveAll ? 1 : 0);
   dialog->window->show();
 
@@ -858,6 +907,8 @@ int bdf_dialog(const char *name)
       if (!o) break;
       if (o == dialog->ok) {
         opt_mesh_bdf_field_format(0, GMSH_SET | GMSH_GUI, dialog->c->value());
+        opt_mesh_save_element_tag_type(0, GMSH_SET | GMSH_GUI, 
+                                       dialog->d->value() + 1);
         opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
         CreateOutputFile(name, FORMAT_BDF);
         dialog->window->hide();
@@ -875,11 +926,11 @@ int bdf_dialog(const char *name)
 // Generic mesh dialog
 
 int generic_mesh_dialog(const char *name, const char *title, int format,
-			bool binary_support)
+                        bool binary_support, bool element_tag_support)
 {
   struct _generic_mesh_dialog{
     Fl_Window *window;
-    Fl_Choice *c;
+    Fl_Choice *c, *d;
     Fl_Check_Button *b;
     Fl_Button *ok, *cancel;
   };
@@ -891,17 +942,27 @@ int generic_mesh_dialog(const char *name, const char *title, int format,
     {0}
   };
 
-  int BBB = BB + 9; // labels too long
+  static Fl_Menu_Item tagmenu[] = {
+    {"Elementary entity", 0, 0, 0},
+    {"Physical entity", 0, 0, 0},
+    {"Partition", 0, 0, 0},
+    {0}
+  };
+
+  int BBB = BB + 16; // labels too long
 
   if(!dialog){
     dialog = new _generic_mesh_dialog;
-    int h = 3 * WB + 3 * BH, w = 2 * BBB + 3 * WB, y = WB;
+    int h = 3 * WB + 4 * BH, w = 2 * BBB + 3 * WB, y = WB;
     dialog->window = new Fl_Double_Window(w, h);
     dialog->window->box(GMSH_WINDOW_BOX);
     dialog->window->set_modal();
-    dialog->c = new Fl_Choice(WB, y, BBB + BBB / 2, BH, "Format"); y += BH;
+    dialog->c = new Fl_Choice(WB, y, BBB + BBB / 4, BH, "Format"); y += BH;
     dialog->c->menu(formatmenu);
     dialog->c->align(FL_ALIGN_RIGHT);
+    dialog->d = new Fl_Choice(WB, y, BBB + BBB / 4, BH, "Element tag"); y += BH;
+    dialog->d->menu(tagmenu);
+    dialog->d->align(FL_ALIGN_RIGHT);
     dialog->b = new Fl_Check_Button
       (WB, y, 2 * BBB + WB, BH, "Save all (ignore physical groups)"); y += BH;
     dialog->b->type(FL_TOGGLE_BUTTON);
@@ -917,6 +978,12 @@ int generic_mesh_dialog(const char *name, const char *title, int format,
     dialog->c->activate();
   else
     dialog->c->deactivate();
+  dialog->d->value((CTX::instance()->mesh.saveElementTagType == 3) ? 2 :
+                   (CTX::instance()->mesh.saveElementTagType == 2) ? 1 : 0);
+  if(element_tag_support)
+    dialog->d->activate();
+  else
+    dialog->d->deactivate();
   dialog->b->value(CTX::instance()->mesh.saveAll ? 1 : 0);
   dialog->window->show();
 
@@ -927,6 +994,8 @@ int generic_mesh_dialog(const char *name, const char *title, int format,
       if (!o) break;
       if (o == dialog->ok) {
         opt_mesh_binary(0, GMSH_SET | GMSH_GUI, dialog->c->value());
+        opt_mesh_save_element_tag_type(0, GMSH_SET | GMSH_GUI, 
+                                       dialog->d->value() + 1);
         opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
         CreateOutputFile(name, format);
         dialog->window->hide();
