@@ -3,7 +3,6 @@
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
-#include <FL/gl.h>
 #include "openglWindow.h"
 #include "graphicWindow.h"
 #include "manipWindow.h"
@@ -13,40 +12,11 @@
 #include "GmshMessage.h"
 #include "GModel.h"
 #include "MElement.h"
-#include "Draw.h"
 #include "Numeric.h"
-#include "GUI.h"
+#include "FlGui.h"
 #include "VertexArray.h"
 #include "Context.h"
 
-void mousePosition::set(drawContext *ctx)
-{
-  for(int i = 0; i < 3; i++){
-    s[i] = ctx->s[i];
-    t[i] = ctx->t[i];
-  }
-
-  win[0] = (double)Fl::event_x();
-  win[1] = (double)Fl::event_y();
-  win[2] = 0.;
-
-  wnr[0] = 
-    (ctx->vxmin + win[0] / (double)ctx->viewport[2] * (ctx->vxmax - ctx->vxmin)) 
-    / ctx->s[0] - ctx->t[0] + ctx->t_init[0] / ctx->s[0];
-  wnr[1] = 
-    (ctx->vymax - win[1] / (double)ctx->viewport[3] * (ctx->vymax - ctx->vymin))
-    / ctx->s[1] - ctx->t[1] + ctx->t_init[1] / ctx->s[1];
-  wnr[2] = 0.;
-}
-
-void mousePosition::recenter(drawContext *ctx)
-{
-  // compute the equivalent translation to apply *after* the scaling
-  // so that the scaling is done around the point which was clicked:
-  ctx->t[0] = t[0] * (s[0] / ctx->s[0]) - wnr[0] * (1. - (s[0] / ctx->s[0]));
-  ctx->t[1] = t[1] * (s[1] / ctx->s[1]) - wnr[1] * (1. - (s[1] / ctx->s[1]));
-}
-  
 static void lassoZoom(drawContext *ctx, mousePosition &click1, mousePosition &click2)
 {
   if(click1.win[0] == click2.win[0] || click1.win[1] == click2.win[1])
@@ -63,8 +33,8 @@ static void lassoZoom(drawContext *ctx, mousePosition &click1, mousePosition &cl
   tmp.recenter(ctx);
 
   ctx->initPosition();
-  Draw();
-  GUI::instance()->manip->update();
+  drawContext::global()->draw();
+  FlGui::instance()->manip->update();
 }
 
 openglWindow::openglWindow(int x, int y, int w, int h, const char *l)
@@ -92,22 +62,23 @@ void openglWindow::drawScreenMessage()
     return;
 
   glColor4ubv((GLubyte *) & CTX::instance()->color.text);
-  gl_font(CTX::instance()->glFontEnum, CTX::instance()->glFontSize);
-  double h = gl_height();
+  drawContext::global()->setFont(CTX::instance()->glFontEnum, 
+                                 CTX::instance()->glFontSize);
+  double h = drawContext::global()->getStringHeight();
   
   if(screenMessage[0].size()){
     const char *txt = screenMessage[0].c_str();
-    double w = gl_width(txt);
+    double w = drawContext::global()->getStringWidth(txt);
     glRasterPos2d(_ctx->viewport[2] / 2. - w / 2., 
                   _ctx->viewport[3] - 1.2 * h);
-    gl_draw(txt);
+    drawContext::global()->drawString(txt);
   }
   if(screenMessage[1].size()){
     const char *txt = screenMessage[1].c_str();
-    double w = gl_width(txt);
+    double w = drawContext::global()->getStringWidth(txt);
     glRasterPos2d(_ctx->viewport[2] / 2. - w / 2.,
                   _ctx->viewport[3] - 2.4 * h);
-    gl_draw(txt);
+    drawContext::global()->drawString(txt);
   }
 }
 
@@ -124,7 +95,7 @@ void openglWindow::drawBorder()
       Fl::get_color(FL_BACKGROUND_COLOR, r, g, b);
     */
     glColor3ub(r, g, b);
-    glLineWidth(1);
+    glLineWidth(1.0F);
     glBegin(GL_LINE_LOOP);
     glVertex2d(_ctx->viewport[0], _ctx->viewport[1]);
     glVertex2d(_ctx->viewport[2], _ctx->viewport[1]);
@@ -169,7 +140,7 @@ void openglWindow::draw()
     // glBlendEquation(GL_FUNC_ADD);
     glBlendFunc(GL_ONE_MINUS_DST_COLOR, GL_ZERO);
     glEnable(GL_BLEND);
-    glLineWidth(0.2);
+    glLineWidth(0.2F);
     glBegin(GL_LINE_LOOP);
     glVertex2d(_click.win[0], _ctx->viewport[3] - _click.win[1]);
     glVertex2d(_lassoXY[0], _ctx->viewport[3] - _click.win[1]);
@@ -194,13 +165,15 @@ void openglWindow::draw()
       CTX::instance()->mesh.draw = 0;
       CTX::instance()->post.draw = 0;
     }
-    glClearColor(CTX::instance()->unpackRed(CTX::instance()->color.bg) / 255.,
-                 CTX::instance()->unpackGreen(CTX::instance()->color.bg) / 255.,
-                 CTX::instance()->unpackBlue(CTX::instance()->color.bg) / 255., 0.);
+    glClearColor
+      ((GLclampf)(CTX::instance()->unpackRed(CTX::instance()->color.bg) / 255.),
+       (GLclampf)(CTX::instance()->unpackGreen(CTX::instance()->color.bg) / 255.),
+       (GLclampf)(CTX::instance()->unpackBlue(CTX::instance()->color.bg) / 255.), 
+       0.0F);
     glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
     _ctx->draw3d();
     glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
-    glPointSize(CTX::instance()->geom.pointSize);
+    glPointSize((float)CTX::instance()->geom.pointSize);
     glBegin(GL_POINTS);
     glVertex3d(_point[0], _point[1], _point[2]);
     glEnd();
@@ -212,9 +185,11 @@ void openglWindow::draw()
   }
   else{
     // draw the whole scene
-    glClearColor(CTX::instance()->unpackRed(CTX::instance()->color.bg) / 255.,
-                 CTX::instance()->unpackGreen(CTX::instance()->color.bg) / 255.,
-                 CTX::instance()->unpackBlue(CTX::instance()->color.bg) / 255., 0.);
+    glClearColor
+      ((GLclampf)(CTX::instance()->unpackRed(CTX::instance()->color.bg) / 255.),
+       (GLclampf)(CTX::instance()->unpackGreen(CTX::instance()->color.bg) / 255.),
+       (GLclampf)(CTX::instance()->unpackBlue(CTX::instance()->color.bg) / 255.), 
+       0.0F);
     glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
     _ctx->draw3d();
     _ctx->draw2d();
@@ -230,17 +205,11 @@ openglWindow *openglWindow::_lastHandled = 0;
 void openglWindow::_setLastHandled(openglWindow* w)
 {
   _lastHandled = w;
-  GUI::instance()->visibility->updatePerWindow();
+  FlGui::instance()->visibility->updatePerWindow();
 }
 
 int openglWindow::handle(int event)
 {
-  // The event model in FLTK is pretty different from other toolkits:
-  // the events are passed to the widget handle of the widget that has
-  // the focus. If this handle returns 1, then the event is considered
-  // as treated, and is suppressed. If the handle returns 0, the event
-  // is passed to the parent.
-
   switch (event) {
 
   case FL_FOCUS: // accept the focus when I'm asked if I want it
@@ -250,14 +219,14 @@ int openglWindow::handle(int event)
   case FL_SHORTCUT:
   case FL_KEYBOARD:
     // override the default widget arrow-key-navigation
-    if(GUI::instance()->testArrowShortcuts())
+    if(FlGui::instance()->testArrowShortcuts())
       return 1;
     return Fl_Gl_Window::handle(event);
     
   case FL_PUSH:
     _setLastHandled(this);
     take_focus(); // force keyboard focus when we click in the window
-    _curr.set(_ctx);
+    _curr.set(_ctx, Fl::event_x(), Fl::event_y());
     if(Fl::event_button() == 1 && 
        !Fl::event_state(FL_SHIFT) && !Fl::event_state(FL_ALT)) {
       if(!lassoMode && Fl::event_state(FL_CTRL)) {
@@ -333,20 +302,20 @@ int openglWindow::handle(int event)
         lassoMode = false;
       }
     }
-    _click.set(_ctx);
-    _prev.set(_ctx);
-    GUI::instance()->manip->update();
+    _click.set(_ctx, Fl::event_x(), Fl::event_y());
+    _prev.set(_ctx, Fl::event_x(), Fl::event_y());
+    FlGui::instance()->manip->update();
     return 1;
 
   case FL_RELEASE:
-    _curr.set(_ctx);
+    _curr.set(_ctx, Fl::event_x(), Fl::event_y());
     CTX::instance()->drawRotationCenter = 0;
     if(!lassoMode) {
       CTX::instance()->mesh.draw = 1;
       CTX::instance()->post.draw = 1;
       redraw();
     }
-    _prev.set(_ctx);
+    _prev.set(_ctx, Fl::event_x(), Fl::event_y());
     return 1;
 
   case FL_MOUSEWHEEL:
@@ -359,11 +328,11 @@ int openglWindow::handle(int event)
       _prev.recenter(_ctx);
       redraw();
     }
-    GUI::instance()->manip->update();
+    FlGui::instance()->manip->update();
     return 1;
 
   case FL_DRAG:
-    _curr.set(_ctx);
+    _curr.set(_ctx, Fl::event_x(), Fl::event_y());
     {
       double dx = _curr.win[0] - _prev.win[0];
       double dy = _curr.win[1] - _prev.win[1];
@@ -416,12 +385,12 @@ int openglWindow::handle(int event)
         redraw();
       }
     }
-    _prev.set(_ctx);
-    GUI::instance()->manip->update();
+    _prev.set(_ctx, Fl::event_x(), Fl::event_y());
+    FlGui::instance()->manip->update();
     return 1;
 
   case FL_MOVE:
-    _curr.set(_ctx);
+    _curr.set(_ctx, Fl::event_x(), Fl::event_y());
     if(lassoMode){
       redraw();
     }
@@ -446,11 +415,11 @@ int openglWindow::handle(int event)
       }
       char str[32];
       sprintf(str, "%g", _point[0]); 
-      GUI::instance()->geoContext->input[2]->value(str);
+      FlGui::instance()->geoContext->input[2]->value(str);
       sprintf(str, "%g", _point[1]); 
-      GUI::instance()->geoContext->input[3]->value(str);
+      FlGui::instance()->geoContext->input[3]->value(str);
       sprintf(str, "%g", _point[2]); 
-      GUI::instance()->geoContext->input[4]->value(str);
+      FlGui::instance()->geoContext->input[4]->value(str);
       redraw();
     }
     else{ // hover mode
@@ -483,7 +452,7 @@ int openglWindow::handle(int event)
                        me ? me->getInfoString().c_str() : "");
       }
     }
-    _prev.set(_ctx);
+    _prev.set(_ctx, Fl::event_x(), Fl::event_y());
     return 1;
 
   default:
@@ -718,7 +687,7 @@ char openglWindow::selectEntity(int type,
     faces.clear();
     regions.clear();
     elements.clear();
-    GUI::instance()->wait();
+    FlGui::instance()->wait();
     if(quitSelection) {
       _selection = ENT_NONE;
       selectionMode = false;
