@@ -12,8 +12,8 @@
 #include <FL/Fl_Box.H>
 #include <FL/Fl_Return_Button.H>
 #include "GmshConfig.h"
-#include "GUI.h"
-#include "Draw.h"
+#include "FlGui.h"
+#include "drawContext.h"
 #include "visibilityWindow.h"
 #include "paletteWindow.h"
 #include "contextWindow.h"
@@ -195,7 +195,7 @@ class VisibilityList { // singleton
     }
     else if(type == ElementaryEntities){
       for(GModel::piter it = m->firstElementaryName(); it != m->lastElementaryName(); ++it)
-        setLabel(it->first, it->second, 1);
+        setLabel(it->first.second, it->second, 1);
       for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); it++)
         _entities.push_back(new VisElementary(*it));
       for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); it++)
@@ -207,7 +207,7 @@ class VisibilityList { // singleton
     }
     else if(type == PhysicalEntities){
       for(GModel::piter it = m->firstPhysicalName(); it != m->lastPhysicalName(); ++it)
-        setLabel(it->first, it->second, 1);
+        setLabel(it->first.second, it->second, 1);
       std::map<int, std::vector<GEntity*> > groups[4];
       m->getPhysicalGroups(groups);
       for(int i = 0; i < 4; i++){
@@ -288,10 +288,10 @@ VisibilityList *VisibilityList::_instance = 0;
 
 static void _rebuild_list_browser()
 {
-  GUI::instance()->visibility->browser->clear();
+  FlGui::instance()->visibility->browser->clear();
 
   VisibilityList::VisibilityType type;
-  switch(GUI::instance()->visibility->browser_type->value()){
+  switch(FlGui::instance()->visibility->browser_type->value()){
   case 0: type = VisibilityList::Models; break;
   case 2: type = VisibilityList::PhysicalEntities; break;
   case 3: type = VisibilityList::MeshPartitions; break;
@@ -300,17 +300,17 @@ static void _rebuild_list_browser()
 
   VisibilityList::instance()->update(type);
   for(int i = 0; i < VisibilityList::instance()->getNumEntities(); i++){
-    GUI::instance()->visibility->browser->add
+    FlGui::instance()->visibility->browser->add
       (VisibilityList::instance()->getBrowserLine(i).c_str());
     if(VisibilityList::instance()->getVisibility(i))
-      GUI::instance()->visibility->browser->select(i + 1);
+      FlGui::instance()->visibility->browser->select(i + 1);
   }
   
   // activate/deactivate delete button
   if(type == VisibilityList::PhysicalEntities)
-    GUI::instance()->visibility->push[0]->activate();
+    FlGui::instance()->visibility->push[0]->activate();
   else
-    GUI::instance()->visibility->push[0]->deactivate();
+    FlGui::instance()->visibility->push[0]->deactivate();
 }
 
 static void visibility_browser_apply_cb(Fl_Widget *w, void *data)
@@ -319,9 +319,9 @@ static void visibility_browser_apply_cb(Fl_Widget *w, void *data)
   // browser and apply them into the model
   if(VisibilityList::instance()->getNumEntities()){
     CTX::instance()->mesh.changed |= (ENT_LINE | ENT_SURFACE | ENT_VOLUME);
-    bool recursive = GUI::instance()->visibility->butt[0]->value() ? true : false;
+    bool recursive = FlGui::instance()->visibility->butt[0]->value() ? true : false;
     VisibilityList::VisibilityType type;
-    switch(GUI::instance()->visibility->browser_type->value()){
+    switch(FlGui::instance()->visibility->browser_type->value()){
     case 0: type = VisibilityList::Models; break;
     case 2: type = VisibilityList::PhysicalEntities; break;
     case 3: type = VisibilityList::MeshPartitions; break;
@@ -329,13 +329,13 @@ static void visibility_browser_apply_cb(Fl_Widget *w, void *data)
     }
     VisibilityList::instance()->setAllInvisible(type);
     for(int i = 0; i < VisibilityList::instance()->getNumEntities(); i++)
-      if(GUI::instance()->visibility->browser->selected(i + 1))
+      if(FlGui::instance()->visibility->browser->selected(i + 1))
         VisibilityList::instance()->setVisibility(i, 1, recursive);
     // then refresh the browser to account for recursive selections
     for(int i = 0; i < VisibilityList::instance()->getNumEntities(); i++)
       if(VisibilityList::instance()->getVisibility(i))
-        GUI::instance()->visibility->browser->select(i + 1);
-    Draw();
+        FlGui::instance()->visibility->browser->select(i + 1);
+    drawContext::global()->draw();
   }
 }
 
@@ -343,7 +343,7 @@ static void visibility_delete_cb(Fl_Widget *w, void *data)
 {
   bool all = true;
   for(int i = 0; i < VisibilityList::instance()->getNumEntities(); i++){
-    if(!GUI::instance()->visibility->browser->selected(i + 1)){
+    if(!FlGui::instance()->visibility->browser->selected(i + 1)){
       all = false;
       break;
     }
@@ -353,7 +353,7 @@ static void visibility_delete_cb(Fl_Widget *w, void *data)
   }
   else{
     for(int i = 0; i < VisibilityList::instance()->getNumEntities(); i++){
-      if(GUI::instance()->visibility->browser->selected(i + 1)){
+      if(FlGui::instance()->visibility->browser->selected(i + 1)){
         Vis *v = VisibilityList::instance()->getEntity(i);
         GModel::current()->deletePhysicalGroup(v->getDim(), v->getTag());
       }
@@ -381,37 +381,37 @@ static void visibility_sort_cb(Fl_Widget *w, void *data)
 
   if(val == 0) { // select or deselect everything
     int selectall = 0;
-    for(int i = 0; i < GUI::instance()->visibility->browser->size(); i++)
-      if(!GUI::instance()->visibility->browser->selected(i + 1)) {
+    for(int i = 0; i < FlGui::instance()->visibility->browser->size(); i++)
+      if(!FlGui::instance()->visibility->browser->selected(i + 1)) {
         selectall = 1;
         break;
       }
     if(selectall)
-      for(int i = 0; i < GUI::instance()->visibility->browser->size(); i++)
-        GUI::instance()->visibility->browser->select(i + 1);
+      for(int i = 0; i < FlGui::instance()->visibility->browser->size(); i++)
+        FlGui::instance()->visibility->browser->select(i + 1);
     else
-      GUI::instance()->visibility->browser->deselect();
+      FlGui::instance()->visibility->browser->deselect();
   }
   else if(val == -1){ // invert the selection
-    int *state = new int[GUI::instance()->visibility->browser->size()];
-    for(int i = 0; i < GUI::instance()->visibility->browser->size(); i++)
-      state[i] = GUI::instance()->visibility->browser->selected(i + 1);
-    GUI::instance()->visibility->browser->deselect();
-    for(int i = 0; i < GUI::instance()->visibility->browser->size(); i++)
-      if(!state[i]) GUI::instance()->visibility->browser->select(i + 1);
+    int *state = new int[FlGui::instance()->visibility->browser->size()];
+    for(int i = 0; i < FlGui::instance()->visibility->browser->size(); i++)
+      state[i] = FlGui::instance()->visibility->browser->selected(i + 1);
+    FlGui::instance()->visibility->browser->deselect();
+    for(int i = 0; i < FlGui::instance()->visibility->browser->size(); i++)
+      if(!state[i]) FlGui::instance()->visibility->browser->select(i + 1);
     delete [] state;
   }
   else if(val == -2){ // create new parameter name for selection
-    for(int i = 0; i < GUI::instance()->visibility->browser->size(); i++){
-      if(GUI::instance()->visibility->browser->selected(i + 1)){
+    for(int i = 0; i < FlGui::instance()->visibility->browser->size(); i++){
+      if(FlGui::instance()->visibility->browser->selected(i + 1)){
         static char tmpstr[256];
         sprintf(tmpstr, "%d", VisibilityList::instance()->getTag(i));
-        GUI::instance()->geoContext->input[1]->value(tmpstr);
+        FlGui::instance()->geoContext->input[1]->value(tmpstr);
         break;
       }
     }
-    GUI::instance()->geoContext->input[0]->value("NewName");
-    GUI::instance()->geoContext->show(0);
+    FlGui::instance()->geoContext->input[0]->value("NewName");
+    FlGui::instance()->geoContext->show(0);
   }
   else { // set new sorting mode
     VisibilityList::instance()->setSortMode(val);
@@ -443,87 +443,98 @@ class listBrowser : public Fl_Browser{
     : Fl_Browser(x, y, w, h, c){}
 };
 
-#if defined(HAVE_TREE_BROWSER)
+#if defined(HAVE_FL_TREE)
 
-static void _add_vertex(GVertex *gv, Flu_Tree_Browser::Node *n)
+static void _add_vertex(GVertex *gv, Fl_Tree *tree, std::string path)
 {
-  char str[128];
-  sprintf(str, "Point %d", gv->tag());
-  Flu_Tree_Browser::Node *n2 = n->add(str);
-  if(gv->getVisibility()) n2->select(true);
-  n2->user_data((void*)gv);
+  std::ostringstream vertex;
+  vertex << path << "Point " << gv->tag() << "/";
+  Fl_Tree_Item *n = tree->add(vertex.str().c_str());
+  if(gv->getVisibility()) n->select(1);
+  n->userdata((void*)gv);
+  n->close();
 }
 
-static void _add_edge(GEdge *ge, Flu_Tree_Browser::Node *n)
+static void _add_edge(GEdge *ge, Fl_Tree *tree, std::string path)
 {
-  char str[128];
-  sprintf(str, "Line %d/", ge->tag());
-  Flu_Tree_Browser::Node *n2 = n->add(str);
-  if(ge->getVisibility()) n2->select(true);
-  n2->user_data((void*)ge);
+  std::ostringstream edge;
+  edge << path << "Line " << ge->tag() << "/";
+  Fl_Tree_Item *n = tree->add(edge.str().c_str());
+  if(ge->getVisibility()) n->select(1);
+  n->userdata((void*)ge);
+  n->close();
   if(ge->getBeginVertex())
-    _add_vertex(ge->getBeginVertex(), n2);
+    _add_vertex(ge->getBeginVertex(), tree, edge.str());
   if(ge->getEndVertex())
-    _add_vertex(ge->getEndVertex(), n2);
+    _add_vertex(ge->getEndVertex(), tree, edge.str());
 }
 
-static void _add_face(GFace *gf, Flu_Tree_Browser::Node *n)
+static void _add_face(GFace *gf, Fl_Tree *tree, std::string path)
 {
-  char str[128];
-  sprintf(str, "Surface %d/", gf->tag());
-  Flu_Tree_Browser::Node *n2 = n->add(str);
-  if(gf->getVisibility()) n2->select(true);
-  n2->user_data((void*)gf);
+  std::ostringstream face;
+  face << path << "Surface " << gf->tag() << "/";
+  Fl_Tree_Item *n = tree->add(face.str().c_str());
+  if(gf->getVisibility()) n->select(1);
+  n->userdata((void*)gf);
+  n->close();
   std::list<GEdge*> edges = gf->edges();
   for(std::list<GEdge*>::iterator it = edges.begin(); it != edges.end(); it++)
-    _add_edge(*it, n2);
+    _add_edge(*it, tree, face.str());
 }
 
-static void _add_region(GRegion *gr, Flu_Tree_Browser::Node *n)
+static void _add_region(GRegion *gr, Fl_Tree *tree, std::string path)
 {
-  char str[128];
-  sprintf(str, "Volume %d/", gr->tag());
-  Flu_Tree_Browser::Node *n2 = n->add(str);
-  if(gr->getVisibility()) n2->select(true);
-  n2->user_data((void*)gr);
+  std::ostringstream region;
+  region << path << "Volume " << gr->tag() << "/";
+  Fl_Tree_Item *n = tree->add(region.str().c_str());
+  if(gr->getVisibility()) n->select(1);
+  n->userdata((void*)gr);
+  n->close();
   std::list<GFace*> faces = gr->faces();
   for(std::list<GFace*>::iterator it = faces.begin(); it != faces.end(); it++)
-    _add_face(*it, n2);
+    _add_face(*it, tree, region.str());
 }
 
 static void _add_physical_group(int dim, int num, std::vector<GEntity*> &ge,
-                                Flu_Tree_Browser::Node *n)
+				std::map<int, std::string> &oldLabels, 
+                                Fl_Tree *tree, std::string path)
 {
   if(ge.empty()) return;
-  char str[256];
-  Flu_Tree_Browser::Node *n2 = 0;
-  std::string name;
-  if(ge[0]->model()->getPhysicalName(num).size())
-    name += std::string(" <<") + ge[0]->model()->getPhysicalName(num) + ">>";
+  std::string name = ge[0]->model()->getPhysicalName(dim, num);
+  if(name.empty() && oldLabels.count(num)) name = oldLabels[num];
+  if(name.size()) name = std::string(" <<") + name + ">>";
+
+  Fl_Tree_Item *n;
+  std::ostringstream group;
+  group << path;
   switch(dim){
   case 3:
-    sprintf(str, "Physical Volume %d%s/", num, name.c_str());
-    n2 = n->add(str);
+    group << "Physical Volume " << num << name << "/";
+    n = tree->add(group.str().c_str());
+    n->close();
     for(unsigned int i = 0; i < ge.size(); i++)
-      _add_region((GRegion*)ge[i], n2);
+      _add_region((GRegion*)ge[i], tree, group.str());
     break;
   case 2:
-    sprintf(str, "Physical Surface %d%s/", num, name.c_str());
-    n2 = n->add(str);
+    group << "Physical Surface " << num << name << "/";
+    n = tree->add(group.str().c_str());
+    n->close();
     for(unsigned int i = 0; i < ge.size(); i++)
-      _add_face((GFace*)ge[i], n2);
+      _add_face((GFace*)ge[i], tree, group.str());
     break;
   case 1:
-    sprintf(str, "Physical Line %d%s/", num, name.c_str());
-    n2 = n->add(str);
+    group << "Physical Line " << num << name << "/";
+    n = tree->add(group.str().c_str());
+    n->close();
     for(unsigned int i = 0; i < ge.size(); i++)
-      _add_edge((GEdge*)ge[i], n2);
+      _add_edge((GEdge*)ge[i], tree, group.str());
     break;
   case 0:
-    sprintf(str, "Physical Point %d%s/", num, name.c_str());
-    n2 = n->add(str);
+    group << "Physical Point " << num << name << "/";
+    n = tree->add(group.str().c_str());
+    n->close();
     for(unsigned int i = 0; i < ge.size(); i++)
-      _add_vertex((GVertex*)ge[i], n2);
+      _add_vertex((GVertex*)ge[i], tree, group.str());
     break;
   default:
     break;
@@ -541,44 +552,63 @@ static void _rebuild_tree_browser(bool force)
         GModel::list[i]->getNumVertices();
     }
     if(numEnt > 10000){
-      GUI::instance()->visibility->tree->hide();
-      GUI::instance()->visibility->tree_create->show();
+      FlGui::instance()->visibility->tree->hide();
+      FlGui::instance()->visibility->tree_create->show();
       return;
     }
   }
 
-  GUI::instance()->visibility->tree_create->hide();
-  GUI::instance()->visibility->tree->show();
-  GUI::instance()->visibility->tree->clear();
+  FlGui::instance()->visibility->tree_create->hide();
+  FlGui::instance()->visibility->tree->show();
+  FlGui::instance()->visibility->tree->clear();
 
-  char str[128];
   for(unsigned int i = 0; i < GModel::list.size(); i++){
     GModel *m = GModel::list[i];
-    std::string s(" <<");
-    s += m->getName() + ">>";
-    if(m == GModel::current()) s += " (Active)";
-    sprintf(str, "Model [%d] %s/", i, s.c_str());
-    Flu_Tree_Browser::Node *n = GUI::instance()->visibility->tree->add(str);
-    if(m->getVisibility()) n->select(true);
-    Flu_Tree_Browser::Node *e = n->add("Elementary entities/");
-    e->select(true);
+    std::ostringstream model;
+    model << "Model [" << i << "] <<" << m->getName() << ">>";
+    if(m == GModel::current()) model << " (Active)";
+    model << "/";
+
+    Fl_Tree_Item *n;
+    n = FlGui::instance()->visibility->tree->add(model.str().c_str());
+    if(m->getVisibility()) n->select(1);
+    n->close();
+
+    std::string elementary = model.str() + "Elementary entities/";
+    n = FlGui::instance()->visibility->tree->add(elementary.c_str());
+    n->close();
+
     for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); it++)
-      _add_region(*it, e);
+      _add_region(*it, FlGui::instance()->visibility->tree, elementary);
     for(GModel::fiter it = m->firstFace(); it != m->lastFace(); it++)
-      _add_face(*it, e);
+      _add_face(*it, FlGui::instance()->visibility->tree, elementary);
     for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); it++)
-      _add_edge(*it, e);
+      _add_edge(*it, FlGui::instance()->visibility->tree, elementary);
     for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); it++)
-      _add_vertex(*it, e);
-    Flu_Tree_Browser::Node *g = n->add("Physical groups/");
-    g->select(true);
+      _add_vertex(*it, FlGui::instance()->visibility->tree, elementary);
+
+    std::string physical = model.str() + "Physical groups/";
+    n = FlGui::instance()->visibility->tree->add(physical.c_str());
+    n->close();
+
     std::map<int, std::vector<GEntity*> > groups[4];
     m->getPhysicalGroups(groups);
+    std::map<int, std::string> oldLabels;
+#if !defined(HAVE_NO_PARSER)
+    for(std::map<std::string, std::vector<double> >::iterator it = gmsh_yysymbols.begin();
+        it != gmsh_yysymbols.end(); ++it)
+      for(unsigned int i = 0; i < it->second.size(); i++)
+        oldLabels[(int)it->second[i]] = it->first;
+#endif
     for(int i = 3; i >= 0; i--)
       for(std::map<int, std::vector<GEntity*> >::iterator it = groups[i].begin();
           it != groups[i].end(); it++)
-        _add_physical_group(i, it->first, it->second, g);
+        _add_physical_group(i, it->first, it->second, oldLabels,
+                            FlGui::instance()->visibility->tree, physical);
   }
+
+  FlGui::instance()->visibility->tree->root_label("Gmsh");
+  FlGui::instance()->visibility->tree->redraw();
 }
 
 static void build_tree_cb(Fl_Widget *w, void *data)
@@ -586,18 +616,18 @@ static void build_tree_cb(Fl_Widget *w, void *data)
   _rebuild_tree_browser(true);
 }
 
-static void _recur_select(Flu_Tree_Browser::Node *n)
+static void _recur_select(Fl_Tree_Item *n)
 {
-  n->select(true);
+  n->select(1);
   for(int i = 0; i < n->children(); i++)
     _recur_select(n->child(i));
 }
 
-static void _recur_set_visible(Flu_Tree_Browser::Node *n)
+static void _recur_set_visible(Fl_Tree_Item *n)
 {
-  if(n->user_data() && n->selected()){
-    GEntity *ge = (GEntity*)n->user_data();
-    bool recursive = GUI::instance()->visibility->butt[0]->value() ? true : false;
+  if(n->userdata() && n->is_selected()){
+    GEntity *ge = (GEntity*)n->userdata();
+    bool recursive = FlGui::instance()->visibility->butt[0]->value() ? true : false;
     ge->setVisibility(1, recursive);
     // force this: if we ask to see an entity, let's assume that we
     // don't want the whole model to be invisible
@@ -607,11 +637,11 @@ static void _recur_set_visible(Flu_Tree_Browser::Node *n)
     _recur_set_visible(n->child(i));
 }
 
-static void _recur_update_selected(Flu_Tree_Browser::Node *n)
+static void _recur_update_selected(Fl_Tree_Item *n)
 {
-  if(n->user_data()){
-    GEntity *ge = (GEntity*)n->user_data();
-    n->select(ge->getVisibility() ? true : false);
+  if(n->userdata()){
+    GEntity *ge = (GEntity*)n->userdata();
+    n->select(ge->getVisibility() ? 1 : 0);
   }
   for(int i = 0; i < n->children(); i++)
     _recur_update_selected(n->child(i));
@@ -619,23 +649,25 @@ static void _recur_update_selected(Flu_Tree_Browser::Node *n)
 
 static void visibility_tree_apply_cb(Fl_Widget *w, void *data)
 {
-  bool recursive = GUI::instance()->visibility->butt[0]->value() ? true : false;
-  Flu_Tree_Browser::Node *root = GUI::instance()->visibility->tree->first();
+  CTX::instance()->mesh.changed |= (ENT_LINE | ENT_SURFACE | ENT_VOLUME);
+  bool recursive = FlGui::instance()->visibility->butt[0]->value() ? true : false;
+
+  Fl_Tree_Item *root = FlGui::instance()->visibility->tree->root();
   for(int i = 0; i < root->children(); i++){
     GModel *m = GModel::list[i];
-    Flu_Tree_Browser::Node *n = root->child(i);
+    Fl_Tree_Item *n = root->child(i);
     // treat special levels separately
     if(recursive){
-      if(n->selected()){ // if model is selected
+      if(root->is_selected() || n->is_selected()){ // if root or model is selected
         _recur_select(n);
       }
       else{
         for(int j = 0; j < n->children(); j++){
-          if(n->child(j)->selected()) // if elementary/physical is selected
+          if(n->child(j)->is_selected()) // if elementary/physical is selected
             _recur_select(n->child(j));
           else if(j == 1){
             for(int k = 0; k < n->child(j)->children(); k++){
-              if(n->child(j)->child(k)->selected()) // if physical ent is selected
+              if(n->child(j)->child(k)->is_selected()) // if physical ent is selected
                 _recur_select(n->child(j)->child(k));
             }
           }
@@ -652,26 +684,26 @@ static void visibility_tree_apply_cb(Fl_Widget *w, void *data)
     // update tree selection
     _recur_update_selected(n);
   }
-  Draw();
+  FlGui::instance()->visibility->tree->redraw();
+  drawContext::global()->draw();
 }
 
-class treeBrowser : public Flu_Tree_Browser{
+class treeBrowser : public Fl_Tree{
   int handle(int event)
   {
     switch(event){
     case FL_SHORTCUT:
     case FL_KEYBOARD:
-      if(Fl::test_shortcut(FL_Enter) || 
-         Fl::test_shortcut(FL_KP_Enter)){
+      if(Fl::test_shortcut(FL_Enter) || Fl::test_shortcut(FL_KP_Enter)){
         visibility_tree_apply_cb(NULL, NULL);
         return 1;
       }
     }
-    return Flu_Tree_Browser::handle(event);
+    return Fl_Tree::handle(event);
   }
  public:
   treeBrowser(int x, int y, int w , int h, const char* c = 0)
-    : Flu_Tree_Browser(x, y, w, h, c){}
+    : Fl_Tree(x, y, w, h, c){}
 };
 
 #endif
@@ -682,15 +714,15 @@ void visibility_cb(Fl_Widget *w, void *data)
   // accordingly
   const char *str = (const char*)data;
   if(str && !strcmp(str, "redraw_only"))
-    GUI::instance()->visibility->show(true);
+    FlGui::instance()->visibility->show(true);
   else
-    GUI::instance()->visibility->show(false);
+    FlGui::instance()->visibility->show(false);
 
   _rebuild_list_browser();
-#if defined(HAVE_TREE_BROWSER)
+#if defined(HAVE_FL_TREE)
   _rebuild_tree_browser(false);
 #endif
-  GUI::instance()->visibility->updatePerWindow(true);
+  FlGui::instance()->visibility->updatePerWindow(true);
 }
 
 static void visibility_save_cb(Fl_Widget *w, void *data)
@@ -752,53 +784,24 @@ static void _set_visibility_by_number(int what, int num, char val, bool recursiv
   bool all = (num < 0) ? true : false;
 
   GModel *m = GModel::current();
+  std::vector<GEntity*> entities;
+  m->getEntities(entities);
 
   switch(what){
   case 0: // nodes
-    for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); it++)
-      for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-        if(all || (*it)->mesh_vertices[i]->getNum() == num) 
-          (*it)->mesh_vertices[i]->setVisibility(val);
-    for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); it++)
-      for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-        if(all || (*it)->mesh_vertices[i]->getNum() == num) 
-          (*it)->mesh_vertices[i]->setVisibility(val);
-    for(GModel::fiter it = m->firstFace(); it != m->lastFace(); it++)
-      for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-        if(all || (*it)->mesh_vertices[i]->getNum() == num) 
-          (*it)->mesh_vertices[i]->setVisibility(val);
-    for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); it++)
-      for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-        if(all || (*it)->mesh_vertices[i]->getNum() == num) 
-          (*it)->mesh_vertices[i]->setVisibility(val);
+    for(unsigned int i = 0; i < entities.size(); i++){
+      for(unsigned int j = 0; j < entities[i]->mesh_vertices.size(); j++){
+        MVertex *v = entities[i]->mesh_vertices[j];
+        if(all || v->getNum() == num) v->setVisibility(val);
+      }
+    }
     break;
   case 1: // elements
-    for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); it++){
-      for(unsigned int i = 0; i < (*it)->lines.size(); i++)
-        if(all || (*it)->lines[i]->getNum() == num) 
-          (*it)->lines[i]->setVisibility(val);
-    }
-    for(GModel::fiter it = m->firstFace(); it != m->lastFace(); it++){
-      for(unsigned int i = 0; i < (*it)->triangles.size(); i++)
-        if(all || (*it)->triangles[i]->getNum() == num) 
-          (*it)->triangles[i]->setVisibility(val);
-      for(unsigned int i = 0; i < (*it)->quadrangles.size(); i++)
-        if(all || (*it)->quadrangles[i]->getNum() == num) 
-          (*it)->quadrangles[i]->setVisibility(val);
-    }
-    for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); it++){
-      for(unsigned int i = 0; i < (*it)->tetrahedra.size(); i++)
-        if(all || (*it)->tetrahedra[i]->getNum() == num) 
-          (*it)->tetrahedra[i]->setVisibility(val);
-      for(unsigned int i = 0; i < (*it)->hexahedra.size(); i++)
-        if(all || (*it)->hexahedra[i]->getNum() == num) 
-          (*it)->hexahedra[i]->setVisibility(val);
-      for(unsigned int i = 0; i < (*it)->prisms.size(); i++)
-        if(all || (*it)->prisms[i]->getNum() == num) 
-          (*it)->prisms[i]->setVisibility(val);
-      for(unsigned int i = 0; i < (*it)->pyramids.size(); i++)
-        if(all || (*it)->pyramids[i]->getNum() == num) 
-          (*it)->pyramids[i]->setVisibility(val);
+    for(unsigned int i = 0; i < entities.size(); i++){
+      for(unsigned int j = 0; j < entities[i]->getNumMeshElements(); j++){
+        MElement *e = entities[i]->getMeshElement(j);
+        if(all || e->getNum() == num) e->setVisibility(val);
+      }
     }
     break;
   case 2: // point
@@ -820,22 +823,26 @@ static void _set_visibility_by_number(int what, int num, char val, bool recursiv
   case 6: // physical point
     for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); it++)
       for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-        if (all || std::abs((*it)->physicals[i]) == num) (*it)->setVisibility(val, recursive);
+        if (all || std::abs((*it)->physicals[i]) == num)
+          (*it)->setVisibility(val, recursive);
     break;
   case 7: // physical line
     for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); it++)
       for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-        if (all || std::abs((*it)->physicals[i]) == num) (*it)->setVisibility(val, recursive);
+        if (all || std::abs((*it)->physicals[i]) == num) 
+          (*it)->setVisibility(val, recursive);
     break;
   case 8: // physical surface
     for(GModel::fiter it = m->firstFace(); it != m->lastFace(); it++)
       for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-        if (all || std::abs((*it)->physicals[i]) == num) (*it)->setVisibility(val, recursive);
+        if (all || std::abs((*it)->physicals[i]) == num) 
+          (*it)->setVisibility(val, recursive);
     break;
   case 9: // physical volume
     for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); it++)
       for(unsigned int i = 0; i < (*it)->physicals.size(); i++)
-        if (all || std::abs((*it)->physicals[i]) == num) (*it)->setVisibility(val, recursive);
+        if (all || std::abs((*it)->physicals[i]) == num) 
+          (*it)->setVisibility(val, recursive);
     break;
   }
 }
@@ -847,7 +854,7 @@ static void _apply_visibility(char mode, bool physical,
                               std::vector<GRegion*> &regions,
                               std::vector<MElement*> &elements)
 {
-  bool recursive = GUI::instance()->visibility->butt[0]->value() ? true : false;
+  bool recursive = FlGui::instance()->visibility->butt[0]->value() ? true : false;
 
   if(mode == 1){ // when showing a single entity, first hide everything
     if(CTX::instance()->pickElements)
@@ -893,9 +900,9 @@ static void _apply_visibility(char mode, bool physical,
           _set_visibility_by_number(9, regions[i]->physicals[j], mode, recursive);
     }
   }
-  int pos = GUI::instance()->visibility->browser->position();
+  int pos = FlGui::instance()->visibility->browser->position();
   visibility_cb(NULL, (void*)"redraw_only");
-  GUI::instance()->visibility->browser->position(pos);
+  FlGui::instance()->visibility->browser->position(pos);
 }
 
 static void visibility_number_cb(Fl_Widget *w, void *data)
@@ -915,17 +922,17 @@ static void visibility_number_cb(Fl_Widget *w, void *data)
   else{ // hide
     val = 0;
   }
-  const char *str = GUI::instance()->visibility->input[what]->value();
+  const char *str = FlGui::instance()->visibility->input[what]->value();
 
   int num = (!strcmp(str, "all") || !strcmp(str, "*")) ? -1 : atoi(str);
-  bool recursive = GUI::instance()->visibility->butt[0]->value() ? true : false;
+  bool recursive = FlGui::instance()->visibility->butt[0]->value() ? true : false;
   
   _set_visibility_by_number(what, num, val, recursive);
 
-  int pos = GUI::instance()->visibility->browser->position();
+  int pos = FlGui::instance()->visibility->browser->position();
   visibility_cb(NULL, (void*)"redraw_only");
-  GUI::instance()->visibility->browser->position(pos);
-  Draw();
+  FlGui::instance()->visibility->browser->position(pos);
+  drawContext::global()->draw();
 }
 
 static void visibility_interactive_cb(Fl_Widget *w, void *data)
@@ -1001,7 +1008,7 @@ static void visibility_interactive_cb(Fl_Widget *w, void *data)
     for(int i = 1; i <= 5; i++) // elements, points, lines, surfaces, volumes
       _set_visibility_by_number(i, -1, 1, false);
     CTX::instance()->mesh.changed = ENT_ALL;
-    Draw();  
+    drawContext::global()->draw();  
     return;
   }
   else
@@ -1016,18 +1023,18 @@ static void visibility_interactive_cb(Fl_Widget *w, void *data)
   while(1) {
     if(what == ENT_ALL) 
       CTX::instance()->mesh.changed = ENT_ALL;
-    Draw();
+    drawContext::global()->draw();
     Msg::StatusBar(3, false, "Select %s\n[Press %s'q' to abort]", 
                    str.c_str(), mode ? "" : "'u' to undo or ");
 
-    char ib = GUI::instance()->selectEntity(what);
+    char ib = FlGui::instance()->selectEntity(what);
     if(ib == 'l') {
       // store for possible undo later
-      vertices = GUI::instance()->selectedVertices;
-      edges = GUI::instance()->selectedEdges;
-      faces = GUI::instance()->selectedFaces;
-      regions = GUI::instance()->selectedRegions;
-      elements = GUI::instance()->selectedElements;
+      vertices = FlGui::instance()->selectedVertices;
+      edges = FlGui::instance()->selectedEdges;
+      faces = FlGui::instance()->selectedFaces;
+      regions = FlGui::instance()->selectedRegions;
+      elements = FlGui::instance()->selectedElements;
       _apply_visibility(mode, physical, vertices, edges, faces, regions, elements);
     }
     if(ib == 'u' && !mode){ // undo only in hide mode
@@ -1040,7 +1047,7 @@ static void visibility_interactive_cb(Fl_Widget *w, void *data)
 
   CTX::instance()->mesh.changed = ENT_ALL;
   CTX::instance()->pickElements = 0;
-  Draw();  
+  drawContext::global()->draw();  
   Msg::StatusBar(3, false, "");
 }
 
@@ -1048,31 +1055,32 @@ static void visibility_per_window_cb(Fl_Widget *w, void *data)
 {
   std::string what = (const char*)data;
   if(what == "item"){
-    drawContext *ctx = GUI::instance()->getCurrentOpenglWindow()->getDrawContext();
-    for(int i = 0; i < GUI::instance()->visibility->per_window->size(); i++){
+    drawContext *ctx = FlGui::instance()->getCurrentOpenglWindow()->getDrawContext();
+    for(unsigned int i = 0; 
+        i < (unsigned int)FlGui::instance()->visibility->per_window->size(); i++){
       if(i < GModel::list.size()){
         GModel *m = GModel::list[i];
-        if(GUI::instance()->visibility->per_window->selected(i + 1)) ctx->show(m);
+        if(FlGui::instance()->visibility->per_window->selected(i + 1)) ctx->show(m);
         else ctx->hide(m);
       }
       else if(i < GModel::list.size() + PView::list.size()){
         PView *v = PView::list[i - GModel::list.size()];
-        if(GUI::instance()->visibility->per_window->selected(i + 1)) ctx->show(v);
+        if(FlGui::instance()->visibility->per_window->selected(i + 1)) ctx->show(v);
         else ctx->hide(v);
       }
     }
   }
   else if(what == "reset_all"){
-    for(unsigned int i = 0; i < GUI::instance()->graph.size(); i++){
-      for(unsigned int j = 0; j < GUI::instance()->graph[i]->gl.size(); j++){
-        drawContext *ctx = GUI::instance()->graph[i]->gl[j]->getDrawContext();
+    for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++){
+      for(unsigned int j = 0; j < FlGui::instance()->graph[i]->gl.size(); j++){
+        drawContext *ctx = FlGui::instance()->graph[i]->gl[j]->getDrawContext();
         ctx->showAll();
       }
     }
-    for(int i = 0; i < GUI::instance()->visibility->per_window->size(); i++)
-      GUI::instance()->visibility->per_window->select(i + 1);
+    for(int i = 0; i < FlGui::instance()->visibility->per_window->size(); i++)
+      FlGui::instance()->visibility->per_window->select(i + 1);
   }
-  Draw();
+  drawContext::global()->draw();
 }
 
 visibilityWindow::visibilityWindow(int deltaFontSize)
@@ -1135,6 +1143,7 @@ visibilityWindow::visibilityWindow(int deltaFontSize)
       browser = new listBrowser
         (2 * WB, 2 * WB + 2 * BH, brw, height - 6 * WB - 4 * BH);
       browser->type(FL_MULTI_BROWSER);
+      browser->textsize(FL_NORMAL_SIZE - 1);
       browser->column_widths(cols);
       
       gg->end();
@@ -1164,19 +1173,15 @@ visibilityWindow::visibilityWindow(int deltaFontSize)
     g->end();
     Fl_Group::current()->resizable(g);
   }
-#if defined(HAVE_TREE_BROWSER)
+#if defined(HAVE_FL_TREE)
   {
     Fl_Group *g = new Fl_Group
       (WB, WB + BH, width - 2 * WB, height - 3 * WB - 2 * BH, "Tree browser");
 
-    tree = new treeBrowser
-      (2 * WB, 2 * WB + BH, brw, height - 6 * WB - 3 * BH);
-    tree->show_root(false);
-    tree->box(FL_DOWN_BOX);
-    tree->insertion_mode(FLU_INSERT_BACK);
-    tree->branch_icons(0, 0);
-    tree->branch_text(FL_BLACK, FL_HELVETICA_BOLD, FL_NORMAL_SIZE - 1);
-    tree->leaf_text(FL_BLACK, FL_HELVETICA, FL_NORMAL_SIZE - 1);
+    tree = new treeBrowser(2 * WB, 2 * WB + BH, brw, height - 6 * WB - 3 * BH);
+    tree->labelsize(FL_NORMAL_SIZE - 1);
+    tree->selectmode(FL_TREE_SELECT_MULTI);
+    tree->connectorstyle(FL_TREE_CONNECTOR_SOLID);
     tree->hide();
 
     tree_create = new Fl_Button
@@ -1388,9 +1393,9 @@ void visibilityWindow::show(bool redrawOnly)
 void visibilityWindow::updatePerWindow(bool force)
 {
   static openglWindow *gl = 0;
-  if(!force && gl == GUI::instance()->getCurrentOpenglWindow()) return;
+  if(!force && gl == FlGui::instance()->getCurrentOpenglWindow()) return;
 
-  gl = GUI::instance()->getCurrentOpenglWindow();
+  gl = FlGui::instance()->getCurrentOpenglWindow();
   drawContext *ctx = gl->getDrawContext();
  
   per_window->clear();
