@@ -5,22 +5,18 @@
 
 #include <sstream>
 #include <algorithm>
-
 #include "GmshConfig.h"
 #include "GmshDefines.h"
 #include "GmshMessage.h"
 #include "GModel.h"
 #include "GEdge.h"
 #include "GFace.h"
-#include "MElement.h"
+#include "MLine.h"
+#include "GaussLegendre1D.h"
 #include "Context.h"
 
-#if !defined(HAVE_GMSH_EMBEDDED)
-#include "GaussLegendre1D.h"
-#endif
-
 GEdge::GEdge(GModel *model, int tag, GVertex *_v0, GVertex *_v1)
-  : GEntity(model, tag), _tooSmall(false), v0(_v0), v1(_v1)
+  : GEntity(model, tag), _tooSmall(false), v0(_v0), v1(_v1), compound(0)
 {
   if(v0) v0->addEdge(this);
   if(v1 && v1 != v0) v1->addEdge(this);
@@ -106,6 +102,42 @@ SBoundingBox3d GEdge::bounds() const
   return bbox;
 }
 
+SOrientedBoundingBox GEdge::getOBB()
+{
+  if (!_obb) {
+    std::vector<SPoint3> vertices;
+    if(getNumMeshVertices() > 0) {
+      int N = getNumMeshVertices();
+      for (int i = 0; i < N; i++) {
+        MVertex* mv = getMeshVertex(i);
+        vertices.push_back(mv->point());
+      }
+      // Don't forget to add the first and last vertices...
+      SPoint3 pt1(getBeginVertex()->x(), getBeginVertex()->y(), getBeginVertex()->z());
+      SPoint3 pt2(getEndVertex()->x(), getEndVertex()->y(), getEndVertex()->z());
+      vertices.push_back(pt1);
+      vertices.push_back(pt2);
+    } 
+    else if(geomType() != DiscreteCurve && geomType() != BoundaryLayerCurve){
+      Range<double> tr = this->parBounds(0);
+      // N can be choosen arbitrarily, but 10 points seems reasonable
+      int N = 10;
+      for (int i = 0; i < N; i++) {
+        double t = tr.low() + (double)i / (double)(N - 1) * (tr.high() - tr.low());
+        GPoint p = point(t);
+        SPoint3 pt(p.x(), p.y(), p.z());
+        vertices.push_back(pt);
+      }
+    } 
+    else {
+      SPoint3 dummy(0, 0, 0);
+      vertices.push_back(dummy);
+    }
+    _obb = SOrientedBoundingBox::buildOBB(vertices);
+  }
+  return SOrientedBoundingBox(_obb);
+}
+
 void GEdge::setVisibility(char val, bool recursive)
 {
   GEntity::setVisibility(val);
@@ -174,36 +206,18 @@ SPoint2 GEdge::reparamOnFace(const GFace *face, double epar,int dir) const
 
 double GEdge::curvature(double par) const
 {
-  double eps1 = 1.e-5;
-  double eps2 = 1.e-5;
-
-  Range<double> r = parBounds(0);
-  if(r.low() == par) eps2 = 0;
-  if(r.high() == par) eps1 = 0;
-
-  SVector3 n1 = firstDer(par - eps1);
-  SVector3 n2 = firstDer(par + eps2);
-
-  GPoint P1 = point(par - eps1);
-  GPoint P2 = point(par + eps2);
-
-  double D = sqrt ((P1.x() - P2.x()) * (P1.x() - P2.x()) +
-                   (P1.y() - P2.y()) * (P1.y() - P2.y()) +
-                   (P1.z() - P2.z()) * (P1.z() - P2.z()));
-
-  n1.normalize();
-  n2.normalize();
-  const double one_over_D = 1. / D;
-  SVector3 d = one_over_D * (n2 - n1);
-  return norm(d);
+  SVector3 d1 = firstDer(par);
+  SVector3 d2 = secondDer(par);
   
+  double one_over_norm = 1. / norm(d1);
+
+  SVector3 cross_prod = crossprod(d1,d2);
+  
+  return ( norm(cross_prod) * one_over_norm * one_over_norm * one_over_norm );
 }
 
 double GEdge::length(const double &u0, const double &u1, const int nbQuadPoints)
 {
-#if defined(HAVE_GMSH_EMBEDDED)
-  return -1.;
-#else
   double *t = 0, *w = 0;
   gmshGaussLegendre1D(nbQuadPoints, &t, &w);
   double L = 0.0;
@@ -215,13 +229,11 @@ double GEdge::length(const double &u0, const double &u1, const int nbQuadPoints)
     L += d * w[i] * rapJ;
   }
   return L;
-#endif
 }
 
 GPoint GEdge::closestPoint(const SPoint3 &q, double &t) const
 {
   double tolerance = 1.e-12;
-  double dist = 1.;
 
   Range<double> interval = parBounds(0);
   
@@ -254,16 +266,16 @@ GPoint GEdge::closestPoint(const SPoint3 &q, double &t) const
   return point(t);
 }
 
-double GEdge::parFromPoint(const SVector3 &Q) const
+double GEdge::parFromPoint(const SPoint3 &P) const
 {
   double t;
-  bool success = XYZToU(Q, t);
+  XYZToU(P.x(), P.y(), P.z(), t);
   return t;
 }
 
-bool GEdge::XYZToU(const SVector3 &Q, double &u, const double relax) const
+bool GEdge::XYZToU(const double X, const double Y, const double Z, 
+                   double &u, const double relax) const
 {
-  const double Precision = 1.e-8;
   const int MaxIter = 25;
   const int NumInitGuess = 11;
 
@@ -274,7 +286,9 @@ bool GEdge::XYZToU(const SVector3 &Q, double &u, const double relax) const
   double uMin = uu.low();
   double uMax = uu.high();
 
-  SVector3 P;
+  printf("dans GEdge uMin=%g, uMax=%g \n", uMin, uMax);
+
+  SVector3 Q(X, Y, Z), P;
   
   double init[NumInitGuess];
   
@@ -310,7 +324,7 @@ bool GEdge::XYZToU(const SVector3 &Q, double &u, const double relax) const
   if(relax > 1.e-2) {
     Msg::Info("point %g %g %g on edge %d : Relaxation factor = %g", 
               Q.x(), Q.y(), Q.z(), 0.75 * relax);
-    return XYZToU(Q, u, 0.75 * relax);
+    return XYZToU(Q.x(), Q.y(), Q.z(), u, 0.75 * relax);
   }
   
   Msg::Error("Could not converge reparametrisation of point (%e,%e,%e) on edge %d",
