@@ -9,34 +9,32 @@
 #include "GFace.h"
 #include "GEdge.h"
 
-/*
-A GEdgeCompound is a model edge that is the compound of model edges.
-*/
+// A GEdgeCompound is a model edge that is the compound of model edges.
 
 class GEdgeCompound : public GEdge {
  protected:
   std::vector<GEdge*> _compound;
-  std::vector<bool> _orientation;
+  std::vector<int> _orientation;
   std::vector<double> _pars;
-  void parametrize() ;
-  void orderEdges()  ;
-  void getLocalParameter ( const double &t,
-			   int &iEdge,
-			   double & tLoc) const;
-  
-public:
+  void parametrize();
+  void orderEdges();
+ public:
+  void getLocalParameter(const double &t, int &iEdge, double & tLoc) const;
+  void getCompoundParameter(GEdge *ge, const double &tLoc, double &t) const;
   GEdgeCompound(GModel *m, int tag, std::vector<GEdge*> &compound);
+  GEdgeCompound(GModel *m, int tag, std::vector<GEdge*> &compound, 
+                std::vector<int> &orientation); // confidence in the input
   virtual ~GEdgeCompound();
   Range<double> parBounds(int i) const;
   virtual GPoint point(double par) const; 
   virtual SVector3 firstDer(double par) const; 
-  virtual GEntity::GeomType geomType() const { return CompoundCurve; }
+  virtual GEntity::GeomType geomType() const { return CompoundCurve;  }
   ModelType getNativeType() const { return GmshModel; }
   void * getNativePtr() const { return 0; }
   virtual double curvature(double t) const;
   virtual int minimumMeshSegments() const;
   virtual int minimumDrawSegments() const;
-
+  std::vector<GEdge*>  getEdgesOfCompound() const { return _compound; }
 };
 
 #endif
