@@ -8,16 +8,11 @@
 #include "GEntity.h"
 #include "MElement.h"
 #include "VertexArray.h"
-
-#if defined(HAVE_GMSH_EMBEDDED)
-#include "GmshEmbedded.h"
-#else
 #include "Context.h"
-#endif
 
 GEntity::GEntity(GModel *m, int t)
   : _model(m), _tag(t), _visible(1), _selection(0),
-    _allElementsVisible(1), va_lines(0), va_triangles(0)
+    _allElementsVisible(1), _obb(0), va_lines(0), va_triangles(0)
 {
   _color = CTX::instance()->packColor(0, 0, 255, 0);
 }
