@@ -9,17 +9,15 @@
 #include "GModel.h"
 #include "GFace.h"
 #include "GEdge.h"
-#include "MElement.h"
+#include "MTriangle.h"
+#include "MQuadrangle.h"
+#include "MElementCut.h"
 #include "VertexArray.h"
-#include "GmshMatrix.h"
+#include "fullMatrix.h"
 #include "Numeric.h"
-
-#if defined(HAVE_GMSH_EMBEDDED)
-#include "GmshEmbedded.h"
-#else
+#include "EigSolve.h"
 #include "GaussLegendre1D.h"
 #include "Context.h"
-#endif
 
 #define SQU(a)      ((a)*(a))
 
@@ -83,17 +81,20 @@ void GFace::deleteMesh()
   triangles.clear();
   for(unsigned int i = 0; i < quadrangles.size(); i++) delete quadrangles[i];
   quadrangles.clear();
+  for(unsigned int i = 0; i < polygons.size(); i++) delete polygons[i];
+  polygons.clear();
 }
 
 unsigned int GFace::getNumMeshElements()
 { 
-  return triangles.size() + quadrangles.size(); 
+  return triangles.size() + quadrangles.size() + polygons.size(); 
 }
 
 void GFace::getNumMeshElements(unsigned *const c) const
 {
   c[0] += triangles.size();
   c[1] += quadrangles.size();
+  c[2] += polygons.size();
 }
 
 MElement *const *GFace::getStartElementType(int type) const
@@ -105,6 +106,9 @@ MElement *const *GFace::getStartElementType(int type) const
   case 1:
     if(quadrangles.empty()) return 0; // msvc would throw an exception
     return reinterpret_cast<MElement *const *>(&quadrangles[0]);
+  case 2:
+    if(polygons.empty()) return 0;
+    return reinterpret_cast<MElement *const *>(&polygons[0]);
   }
   return 0;
 }
@@ -115,6 +119,8 @@ MElement *GFace::getMeshElement(unsigned int index)
     return triangles[index];
   else if(index < triangles.size() + quadrangles.size())
     return quadrangles[index - triangles.size()];
+  else if(index < triangles.size() + quadrangles.size() + polygons.size())
+    return polygons[index - triangles.size() - quadrangles.size()];
   return 0;
 }
 
@@ -143,6 +149,64 @@ SBoundingBox3d GFace::bounds() const
   return res;
 }
 
+SOrientedBoundingBox GFace::getOBB()
+{
+  if (!_obb) {
+    std::vector<SPoint3> vertices;
+    if(getNumMeshVertices() > 0) {
+      int N = getNumMeshVertices();
+      for (int i = 0; i < N; i++) {
+        MVertex* mv = getMeshVertex(i);
+        vertices.push_back(mv->point());
+      }
+      std::list<GEdge*> eds = edges();
+      for(std::list<GEdge*>::iterator ed = eds.begin(); ed != eds.end(); ed++) {
+        int N2 = (*ed)->getNumMeshVertices();
+        for (int i = 0; i < N2; i++) {
+          MVertex* mv = (*ed)->getMeshVertex(i);
+          vertices.push_back(mv->point());
+        }
+        // Don't forget to add the first and last vertices...
+        SPoint3 pt1((*ed)->getBeginVertex()->x(),
+                    (*ed)->getBeginVertex()->y(),
+                    (*ed)->getBeginVertex()->z());
+        SPoint3 pt2((*ed)->getEndVertex()->x(),
+                    (*ed)->getEndVertex()->y(),
+                    (*ed)->getEndVertex()->z());
+        vertices.push_back(pt1);
+        vertices.push_back(pt2);
+      }
+    } 
+    else if(buildSTLTriangulation()) {
+      int N = va_geom_triangles->getNumVertices();
+      for (int i = 0; i < N; i++) {
+        SPoint3 p((va_geom_triangles->getVertexArray(3 * i))[0],
+                  (va_geom_triangles->getVertexArray(3 * i))[1],
+                  (va_geom_triangles->getVertexArray(3 * i))[2]);
+        vertices.push_back(p);
+      }
+    }
+    else {
+      // Fallback, if we can't make a STL triangulation of the
+      // surface, use its edges..
+      std::list<GEdge*> b_edges = edges();
+      int N = 10;
+      for (std::list<GEdge*>::iterator b_edge = b_edges.begin();
+           b_edge != b_edges.end(); b_edge++) {
+        Range<double> tr = (*b_edge)->parBounds(0);
+        for (int j = 0; j < N; j++) {
+          double t = tr.low() + (double)j / (double)(N - 1) * (tr.high() - tr.low());
+          GPoint p = (*b_edge)->point(t);
+          SPoint3 pt(p.x(), p.y(), p.z());
+          vertices.push_back(pt);
+        }
+      }
+    }
+    _obb = SOrientedBoundingBox::buildOBB(vertices);
+  }
+  return SOrientedBoundingBox(_obb);
+}
+
 surface_params GFace::getSurfaceParams() const
 {
   surface_params p;
@@ -174,6 +238,7 @@ void GFace::setVisibility(char val, bool recursive)
     std::list<GEdge*>::iterator it = l_edges.begin();
     while (it != l_edges.end()){
       (*it)->setVisibility(val, recursive);
+
       ++it;
     }
   }
@@ -286,8 +351,8 @@ void GFace::computeMeanPlane(const std::vector<SPoint3> &points)
   ym /= (double)ndata;
   zm /= (double)ndata;
 
-  gmshMatrix<double> U(ndata, na), V(na, na);
-  gmshVector<double> sigma(na);
+  fullMatrix<double> U(ndata, na), V(na, na);
+  fullVector<double> sigma(na);
   for(int i = 0; i < ndata; i++) {
     U(i, 0) = points[i].x() - xm;
     U(i, 1) = points[i].y() - ym;
@@ -390,13 +455,13 @@ end:
   Msg::Debug("Surface: %d", tag());
   Msg::Debug("SVD    : %g,%g,%g (min=%d)", svd[0], svd[1], svd[2], min);
   Msg::Debug("Plane  : (%g x + %g y + %g z = %g)",
-	     meanPlane.a, meanPlane.b, meanPlane.c, meanPlane.d);
+             meanPlane.a, meanPlane.b, meanPlane.c, meanPlane.d);
   Msg::Debug("Normal : (%g , %g , %g )",
-	     meanPlane.a, meanPlane.b, meanPlane.c);
+             meanPlane.a, meanPlane.b, meanPlane.c);
   Msg::Debug("t1     : (%g , %g , %g )", t1[0], t1[1], t1[2]);
   Msg::Debug("t2     : (%g , %g , %g )", t2[0], t2[1], t2[2]);
   Msg::Debug("pt     : (%g , %g , %g )",
-	     meanPlane.x, meanPlane.y, meanPlane.z);
+             meanPlane.x, meanPlane.y, meanPlane.z);
 
   //check coherence for plane surfaces
   if(geomType() == Plane) {
@@ -440,7 +505,7 @@ void GFace::getMeanPlaneData(double plan[3][3]) const
       plan[i][j] = meanPlane.plan[i][j];
 }
 
-double GFace::curvature(const SPoint2 &param) const
+double GFace::curvatureDiv(const SPoint2 &param) const
 {
   if (geomType() == Plane) return 0;
 
@@ -448,9 +513,9 @@ double GFace::curvature(const SPoint2 &param) const
   // curv = div n = dnx/dx + dny/dy + dnz/dz
   // dnx/dx = dnx/du du/dx + dnx/dv dv/dx
 
-  const double eps = 1.e-3;
+  const double eps = 1.e-5;
 
-  Pair<SVector3,SVector3> der = firstDer(param);
+  Pair<SVector3, SVector3> der = firstDer(param);
 
   SVector3 du = der.first();
   SVector3 dv = der.second();
@@ -461,20 +526,79 @@ double GFace::curvature(const SPoint2 &param) const
   du.normalize();
   dv.normalize();
 
-  SVector3 n1 = normal(SPoint2(param.x() - eps, param.y()));
-  SVector3 n2 = normal(SPoint2(param.x() + eps, param.y()));
-  SVector3 n3 = normal(SPoint2(param.x(), param.y() - eps));
-  SVector3 n4 = normal(SPoint2(param.x(), param.y() + eps));
+  SVector3 n1, n2, n3, n4;
+  if(param.x() - eps < 0.0) {
+    n1 = normal(SPoint2(param.x(),       param.y()));
+    n2 = normal(SPoint2(param.x() + eps, param.y()));
+  }
+  else {
+    n1 = normal(SPoint2(param.x() - eps, param.y()));
+    n2 = normal(SPoint2(param.x(),       param.y()));
+  }
+  if(param.y() - eps < 0.0) {
+    n3 = normal(SPoint2(param.x(), param.y()      ));
+    n4 = normal(SPoint2(param.x(), param.y() + eps));
+  }
+  else {
+    n3 = normal(SPoint2(param.x(), param.y() - eps));
+    n4 = normal(SPoint2(param.x(), param.y()      ));
+  }
 
-  SVector3 dndu = 500 * (n2 - n1);
-  SVector3 dndv = 500 * (n4 - n3);
+  SVector3 dndu = 100000 * (n2 - n1);
+  SVector3 dndv = 100000 * (n4 - n3);
 
-  double ddu = dot(dndu,du);
-  double ddv = dot(dndv,dv);
+  SVector3 dudu = SVector3();
+  SVector3 dvdv = SVector3();
+  SVector3 dudv = SVector3();
+  secondDer(param, &dudu, &dvdv, &dudv);
+
+  double ddu = dot(dndu, du);
+  double ddv = dot(dndv, dv);
   
-  double c = std::max(fabs(ddu),fabs(ddv))/detJ;
-  // Msg::Info("c = %g detJ %g", c, detJ);
-  return c;
+  return (fabs(ddu) + fabs(ddv)) / detJ;
+}
+
+double GFace::curvatureMax(const SPoint2 &param) const
+{
+  if (geomType() == Plane) return 0.;
+
+  double eigVal[2], eigVec[8];
+  getMetricEigenVectors(param, eigVal, eigVec);
+
+  return fabs(eigVal[1]);
+}
+
+double GFace::curvatures(const SPoint2 &param, SVector3 *dirMax, SVector3 *dirMin,
+                         double *curvMax, double *curvMin) const
+{
+  Pair<SVector3, SVector3> D1 = firstDer(param);
+
+  if(geomType() == Plane){
+    *dirMax = D1.first();
+    *dirMin = D1.second();
+    *curvMax = 0.;
+    *curvMin = 0.;
+    return 0.;
+  }
+  
+  if(geomType() == Sphere){
+    *dirMax = D1.first();
+    *dirMin = D1.second();
+    *curvMax = curvatureDiv(param);
+    *curvMin = *curvMax;
+    return *curvMax;
+  }
+
+  double eigVal[2], eigVec[8];
+  getMetricEigenVectors(param, eigVal, eigVec);
+
+  // curvatures and main directions
+  *curvMax = fabs(eigVal[1]);
+  *curvMin = fabs(eigVal[0]);
+  *dirMax = eigVec[1] * D1.first() + eigVec[3] * D1.second();
+  *dirMin = eigVec[0] * D1.first() + eigVec[2] * D1.second();
+  
+  return *curvMax;
 }
 
 double GFace::getMetricEigenvalue(const SPoint2 &)
@@ -483,6 +607,72 @@ double GFace::getMetricEigenvalue(const SPoint2 &)
   return 0.;
 }
 
+// eigen values are absolute values and sorted from min to max of absolute values
+// eigen vectors are the corresponding COLUMNS of eigVec
+void GFace::getMetricEigenVectors(const SPoint2 &param,
+                                  double eigVal[2], double eigVec[4]) const
+{
+  // first derivatives
+  Pair<SVector3,SVector3> D1 = firstDer(param);
+  SVector3 du = D1.first();
+  SVector3 dv = D1.second();
+  SVector3 nor = crossprod(du, dv); nor.normalize();
+
+  // second derivatives
+  SVector3 dudu = SVector3();
+  SVector3 dvdv = SVector3();
+  SVector3 dudv = SVector3();
+  secondDer(param, &dudu, &dvdv, &dudv);
+
+  // first form
+  double form1[2][2];
+  form1[0][0] = normSq(du);
+  form1[1][1] = normSq(dv);
+  form1[0][1] = form1[1][0] = dot(du,dv);
+
+  // second form
+  double form2[2][2];
+  form2[0][0] = dot(nor,dudu);
+  form2[1][1] = dot(nor,dvdv);
+  form2[0][1] = form2[1][0] = dot(nor,dudv);
+
+  // inverse of first form
+  double inv_form1[2][2];
+  double inv_det_form1 = 1. / (form1[0][0] * form1[1][1] - form1[1][0] * form1[0][1]);
+  inv_form1[0][0] = inv_det_form1 * form1[1][1];
+  inv_form1[1][1] = inv_det_form1 * form1[0][0];
+  inv_form1[1][0] = inv_form1[0][1] = -1 * inv_det_form1 * form1[0][1];
+
+  // N = (inverse of form1) X (form2)
+  double N[4]; // { N00 N01 N10 N11 }
+  N[0] = inv_form1[0][0] * form2[0][0] + inv_form1[0][1] * form2[1][0];
+  N[1] = inv_form1[0][0] * form2[0][1] + inv_form1[0][1] * form2[1][1];
+  N[2] = inv_form1[1][0] * form2[0][0] + inv_form1[1][1] * form2[1][0];
+  N[3] = inv_form1[1][0] * form2[0][1] + inv_form1[1][1] * form2[1][1];
+  
+  // eigen values and vectors of N
+  int work1[2];
+  double work2[2];
+  double eigValI[2];
+  if (EigSolve(2, 2, N, eigVal, eigValI, eigVec, work1, work2) != 1) {
+    Msg::Error("Problem in eigen vectors computation");
+    Msg::Error(" N: %f %f %f %f", N[0], N[1], N[2], N[3]);
+    Msg::Error(" * Eigen values:");
+    Msg::Error("   %f + i * %f,  %f + i * %f",
+               eigVal[0], eigValI[0], eigVal[1], eigValI[1]);
+    Msg::Error(" * Eigen vectors (trust it only if eigen values are real):");
+    Msg::Error("   ( %f, %f ), ( %f, %f )",
+               eigVec[0], eigVec[2], eigVec[1], eigVec[3]);
+  }
+  if (fabs(eigValI[0]) > 1.e-12 || fabs(eigValI[1]) > 1.e-12) {
+    Msg::Error("Found imaginary eigenvalues");
+  }
+
+  eigVal[0] = fabs(eigVal[0]);
+  eigVal[1] = fabs(eigVal[1]);
+  EigSort(2, eigVal, eigValI, eigVec);
+}
+
 void GFace::XYZtoUV(const double X, const double Y, const double Z,
                     double &U, double &V, const double relax,
                     const bool onSurface) const
@@ -679,9 +869,6 @@ struct graphics_point{
 
 bool GFace::buildSTLTriangulation()
 {
-#if defined(HAVE_GMSH_EMBEDDED)
-  return false;
-#else
   // Build a simple triangulation for surfaces which we know are not
   // trimmed
   if(geomType() != ParametricSurface && geomType() != ProjectionFace)
@@ -731,7 +918,6 @@ bool GFace::buildSTLTriangulation()
   }
   va_geom_triangles->finalize();
   return true;
-#endif
 }
 
 // by default we assume that straight lines are geodesics
@@ -744,9 +930,9 @@ SPoint2 GFace::geodesic(const SPoint2 &pt1 , const SPoint2 &pt2 , double t)
     GPoint gp2 = point(pt2.x(), pt2.y());
     SPoint2 guess = pt1 + (pt2 - pt1) * t;
     GPoint gp = closestPoint(SPoint3(gp1.x() + t * (gp2.x() - gp1.x()),
-				     gp1.y() + t * (gp2.y() - gp1.y()),
-				     gp1.z() + t * (gp2.z() - gp1.z())),
-			     (double*)guess);
+                                     gp1.y() + t * (gp2.y() - gp1.y()),
+                                     gp1.z() + t * (gp2.z() - gp1.z())),
+                             (double*)guess);
     if (gp.g())
       return SPoint2(gp.u(), gp.v());
     else
@@ -765,9 +951,6 @@ SPoint2 GFace::geodesic(const SPoint2 &pt1 , const SPoint2 &pt2 , double t)
 // dC/dt = dC/du du/dt + dC/dv dv/dt
 double GFace::length(const SPoint2 &pt1, const SPoint2 &pt2, int nbQuadPoints)
 {
-#if defined(HAVE_GMSH_EMBEDDED)
-  return -1.;
-#else
   double *t = 0, *w = 0;
   double L = 0.0;
   gmshGaussLegendre1D(nbQuadPoints, &t, &w);
@@ -780,5 +963,32 @@ double GFace::length(const SPoint2 &pt1, const SPoint2 &pt2, int nbQuadPoints)
     L += d * w[i] ;
   }
   return L;
-#endif
+}
+
+int GFace::poincareMesh()  
+{
+  std::set<MEdge, Less_Edge> es;
+  std::set<MVertex*> vs;
+  for(unsigned int i = 0; i < getNumMeshElements(); i++){ 
+    MElement *e = getMeshElement(i);
+    for(int j = 0; j < e->getNumVertices(); j++) vs.insert(e->getVertex(j));
+    for(int j = 0; j < e->getNumEdges(); j++) es.insert(e->getEdge(j));
+  }
+  return vs.size() - es.size() + getNumMeshElements();  
+}
+
+int GFace::genusGeom()  
+{
+  int nSeams = 0;
+  std::set<GEdge*> single_seams;
+  for (std::list<GEdge*>::iterator it = l_edges.begin();
+       it!=l_edges.end();++it){
+    if ((*it)->isSeam(this)){
+      nSeams++;
+      std::set<GEdge*>::iterator it2 = single_seams.find(*it);
+      if (it2 != single_seams.end())single_seams.erase(it2);
+      else single_seams.insert(*it);
+    }
+  }
+  return nSeams - single_seams.size();
 }
