@@ -26,6 +26,7 @@
 
 #include <cstring>
 #include <iostream>  // DBG
+#include <cstdio>
 #include <map>
 #include <string>
 #include <vector>
@@ -279,13 +280,14 @@ int GModel::writeCGNS(const std::string &name, int zoneDefinition,
         partitions.resize(numZone);
         for(int i = 0; i != numZone; ++i)
           partitions[i].elements.reserve(getMaxPartitionSize());
-        unsigned numElem[4];
+        unsigned numElem[5];
         meshDim = getNumMeshElements(numElem);
         // Load the dummy entities with the elements in each partition
         switch(meshDim) {
         case 3:
           for(riter it = firstRegion(); it != lastRegion(); ++it) {
             numElem[0] = 0; numElem[1] = 0; numElem[2] = 0; numElem[3] = 0;
+            numElem[4] = 0;
             (*it)->getNumMeshElements(numElem);
             const int nType = (*it)->getNumElementTypes();
             for(int iType = 0; iType != nType; ++iType) {
@@ -301,6 +303,7 @@ int GModel::writeCGNS(const std::string &name, int zoneDefinition,
         case 2:
           for(fiter it = firstFace(); it != lastFace(); ++it) {
             numElem[0] = 0; numElem[1] = 0; numElem[2] = 0; numElem[3] = 0;
+            numElem[4] = 0;
             (*it)->getNumMeshElements(numElem);
             const int nType = (*it)->getNumElementTypes();
             for(int iType = 0; iType != nType; ++iType) {
@@ -349,8 +352,8 @@ int GModel::writeCGNS(const std::string &name, int zoneDefinition,
 
   if(zoneDefinition == 0) {
     numZone = 1;
-    unsigned numElem[4];
-    numElem[0] = 0; numElem[1] = 0; numElem[2] = 0; numElem[3] = 0;
+    unsigned numElem[5];
+    numElem[0] = 0; numElem[1] = 0; numElem[2] = 0; numElem[3] = 0; numElem[4] = 0;
     meshDim = getNumMeshElements(numElem);
     switch(meshDim) {
     case 3:
@@ -615,6 +618,7 @@ void expand_name(std::string &s, const int index, const char *const name)
  *   zoneDefinition     - (I) how to define the zone (see enum in code)
  *   numZone            - (I) Number of zones in the domain
  *   options            - (I) options for CGNS I/O
+ *   meshDim            - (I) dimension of the mesh elements
  *   group              - (I) the group of physicals used to define the mesh
  *   globalZoneIndex    - (I/O)
  *   globalPhysicalIt   - (I/O) a global scope iterator to the physicals
@@ -628,6 +632,7 @@ void expand_name(std::string &s, const int index, const char *const name)
 
 int get_zone_definition(GModel &model, const int zoneDefinition,
                         const int numZone, const CGNSOptions &options,
+                        const int meshDim,
                         const PhysGroupMap &group, int &globalZoneIndex,
                         PhysGroupMap::const_iterator &globalPhysicalIt,
                         int &zoneIndex, int &partition,
@@ -658,7 +663,8 @@ int get_zone_definition(GModel &model, const int zoneDefinition,
         break;
       case 2:  // Zone defined by physical
         partition = -1;
-        _zoneName = model.getPhysicalName(globalPhysicalIt->first).c_str();
+        _zoneName = model.getPhysicalName(meshDim, globalPhysicalIt->first)
+           .c_str();
         physicalItBegin = globalPhysicalIt++;
         physicalItEnd = globalPhysicalIt;
         break;
@@ -1011,9 +1017,9 @@ int write_CGNS_zones(GModel &model, const int zoneDefinition, const int numZone,
           int zoneIndex;
           int partition;
           if(get_zone_definition
-             (model, zoneDefinition, numZone, options, group, globalZoneIndex,
-              globalPhysicalIt, zoneTask.zoneIndex, partition, physicalItBegin,
-              physicalItEnd, zoneTask.zoneName)) {
+             (model, zoneDefinition, numZone, options, DIM, group,
+              globalZoneIndex, globalPhysicalIt, zoneTask.zoneIndex, partition,
+              physicalItBegin, physicalItEnd, zoneTask.zoneName)) {
             omp_set_lock(&threadWLock);
             --threadsWorking;
             omp_unset_lock(&threadWLock);
