@@ -11,6 +11,7 @@
 #include "OCCIncludes.h"
 
 #if defined(HAVE_OCC)
+#include <vector>
 
 class OCC_Options {
  private:
@@ -24,7 +25,7 @@ class OCC_Internals {
   TopoDS_Shape shape;
   TopTools_IndexedMapOfShape fmap, emap, vmap, somap, shmap, wmap;
  public:
-  enum BooleanOperator { Add, Cut }; 
+  enum BooleanOperator { Intersection, Cut, Section, Fuse }; 
   OCC_Internals()
   {
     somap.Clear();
@@ -36,17 +37,28 @@ class OCC_Internals {
   }
   void healGeometry(double tolerance, bool fixsmalledges, 
                     bool fixspotstripfaces, bool sewfaces, 
-                    bool makesolids=false);
+                    bool makesolids=false, bool connect=false);
   void loadBREP(const char *);  
+  void writeBREP(const char *);  
   void loadSTEP(const char *);
+  void writeSTEP(const char *);  
   void loadIGES(const char *);
   void loadShape(const TopoDS_Shape *);
   void buildGModel(GModel *gm);
   void buildLists();
   void removeAllDuplicates(const double &tolerance);
 
+  void Box(const SPoint3 &p1, const SPoint3 &p2, const BooleanOperator &op);
   void Sphere(const SPoint3 &center, const double &radius, const BooleanOperator &op);
-  void Cylinder(const SPoint3 &bottom_center, const SVector3 &dir, const BooleanOperator &op);
+  void Cylinder(const SPoint3 &bottom_center, const SVector3 &dir, double R, double H,
+                const BooleanOperator &op);
+  void Cone(const SPoint3 &bottom_center, const SVector3 &dir, double R1, double R2, 
+             double H,  const BooleanOperator &op);
+  void Torus(const SPoint3 &bottom_center, const SVector3 &dir, double R1, double R2, 
+             const BooleanOperator &op);
+  void Torus(const SPoint3 &bottom_center, const SVector3 &dir, double R1, double R2, 
+             double angle,  const BooleanOperator &op);
+  void Fillet(std::vector<TopoDS_Edge> &shapes, double radius);
   void applyBooleanOperator(TopoDS_Shape tool, const BooleanOperator &op);
 };
 
