@@ -10,12 +10,13 @@
 
 class GEntity;
 
-class GPoint 
+class GPoint
 {
  private:
   double X, Y, Z;
-  const GEntity *e;  
+  const GEntity *e;
   double par[2];
+  bool success;
  public:
   inline double x() const { return X; }
   inline double y() const { return Y; }
@@ -27,16 +28,19 @@ class GPoint
   inline double v() const { return par[1]; }
   inline const GEntity* g() const { return e; }
   GPoint (double _x=0, double _y=0, double _z=0, const GEntity *onwhat=0)
-    : X(_x), Y(_y), Z(_z), e(onwhat) 
+    : X(_x), Y(_y), Z(_z), e(onwhat), success(true)
   {
+    par[0] = -1.;
+    par[1] = -1.;
   }
   GPoint (double _x, double _y, double _z, const GEntity *onwhat, double p)
-    : X(_x), Y(_y), Z(_z), e(onwhat)
+    : X(_x), Y(_y), Z(_z), e(onwhat), success(true)
   {
     par[0] = p;
+    par[1] = -1.;
   }
   GPoint (double _x, double _y, double _z, const GEntity *onwhat, double p[2])
-    : X(_x), Y(_y), Z(_z), e(onwhat)
+    : X(_x), Y(_y), Z(_z), e(onwhat), success(true)
   {
     par[0] = p[0];
     par[1] = p[1];
@@ -48,6 +52,8 @@ class GPoint
     double dz = Z - p.z();
     return sqrt(dx * dx + dy * dy + dz * dz);
   }
+  bool succeeded() const{ return success; }
+  bool setNoSuccess(){ success = false; return success; }
 };
 
 #endif
