@@ -6,6 +6,9 @@
 #ifndef _GREGION_H_
 #define _GREGION_H_
 
+#include <list>
+#include <string>
+#include <vector>
 #include <stdio.h>
 #include "GEntity.h"
 
@@ -14,6 +17,7 @@ class MTetrahedron;
 class MHexahedron;
 class MPrism;
 class MPyramid;
+class MPolyhedron;
 class ExtrudeParams;
 
 // A model region.
@@ -46,6 +50,9 @@ class GRegion : public GEntity {
   // get the bounding box
   virtual SBoundingBox3d bounds() const;
 
+  // get the oriented bounding box
+  virtual SOrientedBoundingBox getOBB();
+
   // check if the region is connected to another region by an edge
   bool edgeConnected(GRegion *r) const;
 
@@ -56,7 +63,7 @@ class GRegion : public GEntity {
   virtual void writeGEO(FILE *fp);
 
   // number of types of elements
-  int getNumElementTypes() const { return 4; }
+  int getNumElementTypes() const { return 5; }
 
   // get total/by-type number of elements in the mesh
   unsigned int getNumMeshElements();
@@ -88,6 +95,7 @@ class GRegion : public GEntity {
   std::vector<MHexahedron*> hexahedra;
   std::vector<MPrism*> prisms;
   std::vector<MPyramid*> pyramids;
+  std::vector<MPolyhedron*> polyhedra;
 };
 
 #endif
