@@ -3,10 +3,10 @@
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
+#include <stdlib.h>
 #include <string.h>
 #include "GmshMessage.h"
 #include "Numeric.h"
-#include "MallocUtils.h"
 #include "Geo.h"
 #include "GModel.h"
 #include "GeoInterpolation.h"
@@ -78,6 +78,13 @@ static int compareVolume(const void *a, const void *b)
   return q->Num - w->Num;
 }
 
+static int compareLevelSet(const void *a, const void *b)
+{
+  LevelSet *q = *(LevelSet **)a;
+  LevelSet *w = *(LevelSet **)b;
+  return q->Num - w->Num;
+}
+
 static int comparePhysicalGroup(const void *a, const void *b)
 {
   PhysicalGroup *q = *(PhysicalGroup **)a;
@@ -139,7 +146,7 @@ static void Free_Vertex(void *a, void *b)
 
 PhysicalGroup *Create_PhysicalGroup(int Num, int typ, List_T *intlist, List_T *bndlist[4])
 {
-  PhysicalGroup *p = (PhysicalGroup *)Malloc(sizeof(PhysicalGroup));
+  PhysicalGroup *p = new PhysicalGroup;
   p->Entities = List_Create(List_Nbr(intlist), 1, sizeof(int));
   p->Num = Num;
   GModel::current()->getGEOInternals()->MaxPhysicalNum = 
@@ -156,9 +163,9 @@ PhysicalGroup *Create_PhysicalGroup(int Num, int typ, List_T *intlist, List_T *b
     for(int i = 0; i < 4; i++) {
       p->Boundaries[i] = List_Create(List_Nbr(bndlist[i]), 1, sizeof(int));
       for(int j = 0; j < List_Nbr(bndlist[i]); j++) {
-	int k;
-	List_Read(bndlist[i], j, &k);
-	List_Add(p->Boundaries[i], &k);
+        int k;
+        List_Read(bndlist[i], j, &k);
+        List_Add(p->Boundaries[i], &k);
       }
     }
   }
@@ -170,14 +177,14 @@ static void Free_PhysicalGroup(void *a, void *b)
   PhysicalGroup *p = *(PhysicalGroup **)a;
   if(p) {
     List_Delete(p->Entities);
-    Free(p);
+    delete p;
     p = NULL;
   }
 }
 
 EdgeLoop *Create_EdgeLoop(int Num, List_T *intlist)
 {
-  EdgeLoop *l = (EdgeLoop *)Malloc(sizeof(EdgeLoop));
+  EdgeLoop *l = new EdgeLoop;
   l->Curves = List_Create(List_Nbr(intlist), 1, sizeof(int));
   l->Num = Num;
   GModel::current()->getGEOInternals()->MaxLineLoopNum = 
@@ -195,14 +202,14 @@ static void Free_EdgeLoop(void *a, void *b)
   EdgeLoop *l = *(EdgeLoop **)a;
   if(l) {
     List_Delete(l->Curves);
-    Free(l);
+    delete l;
     l = NULL;
   }
 }
 
 SurfaceLoop *Create_SurfaceLoop(int Num, List_T *intlist)
 {
-  SurfaceLoop *l = (SurfaceLoop *)Malloc(sizeof(SurfaceLoop));
+  SurfaceLoop *l = new SurfaceLoop;
   l->Surfaces = List_Create(List_Nbr(intlist), 1, sizeof(int));
   l->Num = Num;
   GModel::current()->getGEOInternals()->MaxSurfaceLoopNum = 
@@ -220,7 +227,7 @@ static void Free_SurfaceLoop(void *a, void *b)
   SurfaceLoop *l = *(SurfaceLoop **)a;
   if(l) {
     List_Delete(l->Surfaces);
-    Free(l);
+    delete l;
     l = NULL;
   }
 }
@@ -249,7 +256,9 @@ void End_Curve(Curve *c)
       } 
     }
   }
-
+  
+  c->degenerated = false;
+  
   if(c->Typ == MSH_SEGM_CIRC || c->Typ == MSH_SEGM_CIRC_INV ||
      c->Typ == MSH_SEGM_ELLI || c->Typ == MSH_SEGM_ELLI_INV) {
 
@@ -358,7 +367,7 @@ void End_Curve(Curve *c)
     else if(!v[3] && fabs((R - R2) / (R + R2)) > 0.1){
       // check cocircular pts (allow 10% error)
       Msg::Error("Control points of Circle %d are not cocircular %g %g",
-		 c->Num, R, R2);
+                 c->Num, R, R2);
     }
 
     // A1 = angle first pt
@@ -468,7 +477,7 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T *Liste,
                           {-3, 3.0, 0, 0.0},
                           {1, 0, 0, 0.0} };
 
-  Curve *pC = (Curve *)Malloc(sizeof(Curve));
+  Curve *pC = new Curve;
   pC->Color.type = 0;
   pC->Visible = 1;
   pC->Extrude = NULL;
@@ -506,7 +515,7 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T *Liste,
   pC->uend = u2;
 
   if(Knots) {
-    pC->k = (float *)Malloc(List_Nbr(Knots) * sizeof(float));
+    pC->k = new float[List_Nbr(Knots)];
     double kmin = .0, kmax = 1.;
     List_Read(Knots, 0, &kmin);
     List_Read(Knots, List_Nbr(Knots) - 1, &kmax);
@@ -572,16 +581,16 @@ static void Free_Curve(void *a, void *b)
 {
   Curve *pC = *(Curve **)a;
   if(pC) {
-    Free(pC->k);
+    delete [] pC->k;
     List_Delete(pC->Control_Points);
-    Free(pC);
+    delete pC;
     pC = NULL;
   }
 }
 
 Surface *Create_Surface(int Num, int Typ)
 {
-  Surface *pS = (Surface *)Malloc(sizeof(Surface));
+  Surface *pS = new Surface;
   pS->Color.type = 0;
   pS->Visible = 1;
   pS->Num = Num;
@@ -613,14 +622,14 @@ static void Free_Surface(void *a, void *b)
     List_Delete(pS->Generatrices);
     List_Delete(pS->EmbeddedCurves);
     List_Delete(pS->EmbeddedPoints);
-    Free(pS);
+    delete pS;
     pS = NULL;
   }
 }
 
 Volume *Create_Volume(int Num, int Typ)
 {
-  Volume *pV = (Volume *)Malloc(sizeof(Volume));
+  Volume *pV = new Volume;
   pV->Color.type = 0;
   pV->Visible = 1;
   pV->Num = Num;
@@ -644,11 +653,28 @@ static void Free_Volume(void *a, void *b)
     List_Delete(pV->Surfaces);
     List_Delete(pV->SurfacesOrientations);
     List_Delete(pV->SurfacesByTag);
-    Free(pV);
+    delete pV;
     pV = NULL;
   }
 }
 
+LevelSet *Create_LevelSet(int Num, gLevelset *l)
+{
+  LevelSet *pL = new LevelSet;
+  pL->Num = Num;
+  pL->ls = l;
+  return pL;
+}
+
+static void Free_LevelSet(void *a, void *b)
+{
+  LevelSet *pL = *(LevelSet **)a;
+  if(pL) {
+    delete pL;
+    pL = NULL;
+  }
+}
+
 int NEWPOINT(void)
 {
   return (GModel::current()->getGEOInternals()->MaxPointNum + 1);
@@ -719,7 +745,7 @@ int NEWREG(void)
 }
 
 static int compare2Lists(List_T *List1, List_T *List2,
-			 int (*fcmp) (const void *a, const void *b))
+                         int (*fcmp) (const void *a, const void *b))
 {
   int i, found;
 
@@ -794,6 +820,17 @@ Volume *FindVolume(int inum)
   return NULL;
 }
 
+LevelSet *FindLevelSet(int inum)
+{
+  LevelSet L, *pl;
+  pl = &L;
+  pl->Num = inum; 
+  if(Tree_Query(GModel::current()->getGEOInternals()->LevelSets, &pl)) {
+    return pl;
+  }
+  return NULL;
+}
+
 EdgeLoop *FindEdgeLoop(int inum)
 {
   EdgeLoop S, *ps;
@@ -855,17 +892,16 @@ static int compareAbsCurve(const void *a, const void *b)
   return abs(q->Num) - abs(w->Num);
 }
 
-static void CopyCurve(Curve *c, Curve *cc)
+static void CopyCurve(Curve *c, Curve *cc, bool copyMeshingMethod)
 {
   int i, j;
   cc->Typ = c->Typ;
-  // We should not copy the meshing method : if the meshes are to be
-  // copied, the meshing algorithm will take care of it
-  // (e.g. ExtrudeMesh()).
-  //cc->Method = c->Method; 
-  cc->nbPointsTransfinite = c->nbPointsTransfinite;
-  cc->typeTransfinite = c->typeTransfinite;
-  cc->coeffTransfinite = c->coeffTransfinite;
+  if(copyMeshingMethod){
+    cc->Method = c->Method;
+    cc->nbPointsTransfinite = c->nbPointsTransfinite;
+    cc->typeTransfinite = c->typeTransfinite;
+    cc->coeffTransfinite = c->coeffTransfinite;
+  }
   cc->l = c->l;
   for(i = 0; i < 4; i++)
     for(j = 0; j < 4; j++)
@@ -876,21 +912,16 @@ static void CopyCurve(Curve *c, Curve *cc)
   cc->uend = c->uend;
   cc->Control_Points = List_Create(List_Nbr(c->Control_Points), 1, sizeof(Vertex *));
   List_Copy(c->Control_Points, cc->Control_Points);
-  if(c->Typ == MSH_SEGM_PARAMETRIC){
-    strcpy(cc->functu, c->functu);
-    strcpy(cc->functv, c->functv);
-    strcpy(cc->functw, c->functw);
-  }
   End_Curve(cc);
   Tree_Insert(GModel::current()->getGEOInternals()->Curves, &cc);
 }
 
-static Curve *DuplicateCurve(Curve *c)
+static Curve *DuplicateCurve(Curve *c, bool copyMeshingMethod)
 {
   Curve *pc;
   Vertex *v, *newv;
   pc = Create_Curve(NEWLINE(), 0, 1, NULL, NULL, -1, -1, 0., 1.);
-  CopyCurve(c, pc);
+  CopyCurve(c, pc, copyMeshingMethod);
   for(int i = 0; i < List_Nbr(c->Control_Points); i++) {
     List_Read(pc->Control_Points, i, &v);
     newv = DuplicateVertex(v);
@@ -903,59 +934,61 @@ static Curve *DuplicateCurve(Curve *c)
   return pc;
 }
 
-static void CopySurface(Surface *s, Surface *ss)
+static void CopySurface(Surface *s, Surface *ss, bool copyMeshingMethod)
 {
-  int i, j;
   ss->Typ = s->Typ;
-  // We should not copy the meshing method (or the recombination
-  // status): if the meshes are to be copied, the meshing algorithm
-  // will take care of it (e.g. ExtrudeMesh()).
-  //ss->Method = s->Method;
-  //ss->Recombine = s->Recombine;
-  //ss->RecombineAngle = s->RecombineAngle;
+  if(copyMeshingMethod){
+    ss->Method = s->Method;
+    ss->Recombine = s->Recombine;
+    ss->RecombineAngle = s->RecombineAngle;
+    if(List_Nbr(s->TrsfPoints))
+      Msg::Warning("Only automatic transfinite surface specifications can be copied");
+  }
   ss->Generatrices = List_Create(List_Nbr(s->Generatrices), 1, sizeof(Curve *));
   List_Copy(s->Generatrices, ss->Generatrices);
   End_Surface(ss);
   Tree_Insert(GModel::current()->getGEOInternals()->Surfaces, &ss);
 }
 
-static Surface *DuplicateSurface(Surface *s)
+static Surface *DuplicateSurface(Surface *s, bool copyMeshingMethod)
 {
   Surface *ps;
   Curve *c, *newc;
 
   ps = Create_Surface(NEWSURFACE(), 0);
-  CopySurface(s, ps);
+  CopySurface(s, ps, copyMeshingMethod);
   for(int i = 0; i < List_Nbr(ps->Generatrices); i++) {
     List_Read(ps->Generatrices, i, &c);
-    newc = DuplicateCurve(c);
+    newc = DuplicateCurve(c, copyMeshingMethod);
     List_Write(ps->Generatrices, i, &newc);
   }
   return ps;
 }
 
-static void CopyVolume(Volume *v, Volume *vv)
+static void CopyVolume(Volume *v, Volume *vv, bool copyMeshingMethod)
 {
   vv->Typ = v->Typ;
-  // We should not copy the meshing method (or the recombination
-  // status): if the meshes are to be copied, the meshing algorithm
-  // will take care of it (e.g. ExtrudeMesh()).
+  if(copyMeshingMethod){
+    vv->Method = v->Method;
+    if(List_Nbr(v->TrsfPoints))
+      Msg::Warning("Only automatic transfinite volume specifications can be copied");
+  }
   List_Copy(v->Surfaces, vv->Surfaces);
   List_Copy(v->SurfacesOrientations, vv->SurfacesOrientations);
   List_Copy(v->SurfacesByTag, vv->SurfacesByTag);
   Tree_Insert(GModel::current()->getGEOInternals()->Volumes, &vv);
 }
 
-static Volume *DuplicateVolume(Volume *v)
+static Volume *DuplicateVolume(Volume *v, bool copyMeshingMethod)
 {
   Volume *pv;
   Surface *s, *news;
 
   pv = Create_Volume(NEWVOLUME(), 0);
-  CopyVolume(v, pv);
+  CopyVolume(v, pv, copyMeshingMethod);
   for(int i = 0; i < List_Nbr(pv->Surfaces); i++) {
     List_Read(pv->Surfaces, i, &s);
-    news = DuplicateSurface(s);
+    news = DuplicateSurface(s, copyMeshingMethod);
     List_Write(pv->Surfaces, i, &news);
   }
   return pv;
@@ -986,12 +1019,11 @@ void CopyShape(int Type, int Num, int *New)
   case MSH_SEGM_ELLI:
   case MSH_SEGM_ELLI_INV:
   case MSH_SEGM_NURBS:
-  case MSH_SEGM_PARAMETRIC:
     if(!(c = FindCurve(Num))) {
       Msg::Error("Unknown curve %d", Num);
       return;
     }
-    newc = DuplicateCurve(c);
+    newc = DuplicateCurve(c, CTX::instance()->geom.copyMeshingMethod);
     *New = newc->Num;
     break;
   case MSH_SURF_TRIC:
@@ -1001,7 +1033,7 @@ void CopyShape(int Type, int Num, int *New)
       Msg::Error("Unknown surface %d", Num);
       return;
     }
-    news = DuplicateSurface(s);
+    news = DuplicateSurface(s, CTX::instance()->geom.copyMeshingMethod);
     *New = news->Num;
     break;
   case MSH_VOLUME:
@@ -1009,7 +1041,7 @@ void CopyShape(int Type, int Num, int *New)
       Msg::Error("Unknown volume %d", Num);
       return;
     }
-    newvol = DuplicateVolume(vol);
+    newvol = DuplicateVolume(vol, CTX::instance()->geom.copyMeshingMethod);
     *New = newvol->Num;
     break;
   default:
@@ -1113,7 +1145,6 @@ void DeleteShape(int Type, int Num)
   case MSH_SEGM_ELLI:
   case MSH_SEGM_ELLI_INV:
   case MSH_SEGM_NURBS:
-  case MSH_SEGM_PARAMETRIC:
     DeleteCurve(Num);
     DeleteCurve(-Num);
     break;
@@ -1178,7 +1209,6 @@ void ColorShape(int Type, int Num, unsigned int Color)
   case MSH_SEGM_ELLI:
   case MSH_SEGM_ELLI_INV:
   case MSH_SEGM_NURBS:
-  case MSH_SEGM_PARAMETRIC:
   case MSH_SEGM_DISCRETE:
     ColorCurve(Num, Color);
     break;
@@ -1206,10 +1236,16 @@ void VisibilityShape(int Type, int Num, int Mode)
 
   switch (Type) {
   case MSH_POINT:
+  case MSH_POINT_FROM_GMODEL:
     if((v = FindPoint(Num)))
       v->Visible = Mode;
-    else
-      Msg::Warning("Unknown point %d (use '*' to hide/show all points)", Num);
+    else{
+      GVertex *gv = GModel::current()->getVertexByTag(Num);
+      if(gv)
+        gv->setVisibility(Mode);
+      else
+        Msg::Warning("Unknown point %d (use '*' to hide/show all points)", Num);
+    }
     break;
   case MSH_SEGM_LINE:
   case MSH_SEGM_SPLN:
@@ -1220,28 +1256,45 @@ void VisibilityShape(int Type, int Num, int Mode)
   case MSH_SEGM_ELLI:
   case MSH_SEGM_ELLI_INV:
   case MSH_SEGM_NURBS:
-  case MSH_SEGM_PARAMETRIC:
   case MSH_SEGM_DISCRETE:
+  case MSH_SEGM_FROM_GMODEL:
     if((c = FindCurve(Num)))
       c->Visible = Mode;
-    else
-      Msg::Warning("Unknown line %d (use '*' to hide/show all lines)", Num);
+    else{
+      GEdge *ge = GModel::current()->getEdgeByTag(Num);
+      if(ge)
+        ge->setVisibility(Mode);
+      else
+        Msg::Warning("Unknown line %d (use '*' to hide/show all lines)", Num);
+    }
     break;
   case MSH_SURF_TRIC:
   case MSH_SURF_REGL:
   case MSH_SURF_PLAN:
   case MSH_SURF_DISCRETE:
+  case MSH_SURF_FROM_GMODEL:
     if((s = FindSurface(Num)))
       s->Visible = Mode;
-    else
-      Msg::Warning("Unknown surface %d (use '*' to hide/show all surfaces)", Num);
+    else{
+      GFace *gf = GModel::current()->getFaceByTag(Num);
+      if(gf)
+        gf->setVisibility(Mode);
+      else
+        Msg::Warning("Unknown surface %d (use '*' to hide/show all surfaces)", Num);
+    }
     break;
   case MSH_VOLUME:
   case MSH_VOLUME_DISCRETE:
+  case MSH_VOLUME_FROM_GMODEL:
     if((V = FindVolume(Num)))
       V->Visible = Mode;
-    else
-      Msg::Warning("Unknown volume %d (use '*' to hide/show all volumes)", Num);
+    else{
+      GRegion *gr = GModel::current()->getRegionByTag(Num);
+      if(gr)
+        gr->setVisibility(Mode);
+      else
+        Msg::Warning("Unknown volume %d (use '*' to hide/show all volumes)", Num);
+    }
     break;
   default:
     break;
@@ -1260,10 +1313,30 @@ void VisibilityShape(char *str, int Type, int Mode)
 
   if(!strcmp(str, "all") || !strcmp(str, "*")) {
     switch (Type) {
-    case 0: Tree_Action(GModel::current()->getGEOInternals()->Points, vis_nod); break;
-    case 1: Tree_Action(GModel::current()->getGEOInternals()->Curves, vis_cur); break;
-    case 2: Tree_Action(GModel::current()->getGEOInternals()->Surfaces, vis_sur); break;
-    case 3: Tree_Action(GModel::current()->getGEOInternals()->Volumes, vis_vol); break;
+    case 0:
+      Tree_Action(GModel::current()->getGEOInternals()->Points, vis_nod); 
+      for(GModel::viter it = GModel::current()->firstVertex(); 
+          it != GModel::current()->lastVertex(); it++)
+        (*it)->setVisibility(Mode);
+      break;
+    case 1: 
+      Tree_Action(GModel::current()->getGEOInternals()->Curves, vis_cur);
+      for(GModel::eiter it = GModel::current()->firstEdge(); 
+          it != GModel::current()->lastEdge(); it++)
+        (*it)->setVisibility(Mode);
+      break;
+    case 2: 
+      Tree_Action(GModel::current()->getGEOInternals()->Surfaces, vis_sur); 
+      for(GModel::fiter it = GModel::current()->firstFace(); 
+          it != GModel::current()->lastFace(); it++)
+        (*it)->setVisibility(Mode);
+      break;
+    case 3:
+      Tree_Action(GModel::current()->getGEOInternals()->Volumes, vis_vol); 
+      for(GModel::riter it = GModel::current()->firstRegion(); 
+          it != GModel::current()->lastRegion(); it++)
+        (*it)->setVisibility(Mode);
+      break;
     }
   }
   else {
@@ -1296,8 +1369,7 @@ Curve *CreateReversedCurve(Curve *c)
   }
 
   if(c->Typ == MSH_SEGM_NURBS && c->k) {
-    newc->k = (float *)malloc((c->degre + List_Nbr(c->Control_Points) + 1) *
-                              sizeof(float));
+    newc->k = new float[c->degre + List_Nbr(c->Control_Points) + 1];
     for(i = 0; i < c->degre + List_Nbr(c->Control_Points) + 1; i++)
       newc->k[c->degre + List_Nbr(c->Control_Points) - i] = c->k[i];
   }
@@ -1314,6 +1386,9 @@ Curve *CreateReversedCurve(Curve *c)
   newc->beg = c->end;
   newc->end = c->beg;
   newc->Method = c->Method;
+  newc->nbPointsTransfinite = c->nbPointsTransfinite;
+  newc->typeTransfinite = -c->typeTransfinite;
+  newc->coeffTransfinite = c->coeffTransfinite;
   newc->degre = c->degre;
   newc->ubeg = 1. - c->uend;
   newc->uend = 1. - c->ubeg;
@@ -1406,7 +1481,7 @@ static void SetTranslationMatrix(double matrix[4][4], double T[3])
 }
 
 static void SetSymmetryMatrix(double matrix[4][4], double A, double B, double C,
-			      double D)
+                              double D)
 {
   double F = -2.0 / (A * A + B * B + C * C);
   matrix[0][0] = 1. + A * A * F;
@@ -1540,6 +1615,7 @@ static void vecmat4x4(double mat[4][4], double vec[4], double res[4])
   }
 }
 
+#if 0
 static void printCurve(Curve *c)
 {
   Vertex *v;
@@ -1548,7 +1624,7 @@ static void printCurve(Curve *c)
   for(int i = 0; i < N; i++) {
     List_Read(c->Control_Points, i, &v);
     Msg::Debug("Vertex %d (%g,%g,%g,%g)", v->Num, v->Pos.X, v->Pos.Y,
-	       v->Pos.Z, v->lc);
+               v->Pos.Z, v->lc);
   }
 }
 
@@ -1563,9 +1639,10 @@ static void printSurface(Surface *s)
     printCurve(c);
   }
 }
+#endif
 
 static void ApplyTransformationToPoint(double matrix[4][4], Vertex *v,
-				       bool end_curve_surface=false)
+                                       bool end_curve_surface=false)
 {
   double pos[4], vec[4];
 
@@ -1678,7 +1755,6 @@ static void ApplicationOnShapes(double matrix[4][4], List_T *shapes)
     case MSH_SEGM_ELLI:
     case MSH_SEGM_ELLI_INV:
     case MSH_SEGM_NURBS:
-    case MSH_SEGM_PARAMETRIC:
       c = FindCurve(O.Num);
       if(c)
         ApplyTransformationToCurve(matrix, c);
@@ -1703,7 +1779,7 @@ static void ApplicationOnShapes(double matrix[4][4], List_T *shapes)
       break;
     default:
       Msg::Error("Impossible to transform entity %d (of type %d)", O.Num,
-		 O.Type);
+                 O.Type);
       break;
     }
   }
@@ -1797,7 +1873,6 @@ void BoundaryShapes(List_T *shapes, List_T *shapesBoundary)
     case MSH_SEGM_BSPLN:
     case MSH_SEGM_NURBS:
     case MSH_SEGM_BEZIER:
-    case MSH_SEGM_PARAMETRIC:
     case MSH_SEGM_BND_LAYER:
       {
         Curve *c = FindCurve(O.Num);
@@ -1858,7 +1933,7 @@ void BoundaryShapes(List_T *shapes, List_T *shapesBoundary)
       break;
     default:
       Msg::Error("Impossible to take boundary of entity %d (of type %d)", O.Num,
-		 O.Type);
+                 O.Type);
       break;
     }
   }
@@ -1898,11 +1973,11 @@ void ProtudeXYZ(double &x, double &y, double &z, ExtrudeParams *e)
 }
 
 static int Extrude_ProtudePoint(int type, int ip,
-				double T0, double T1, double T2,
-				double A0, double A1, double A2,
-				double X0, double X1, double X2, double alpha,
-				Curve **pc, Curve **prc, int final, 
-				ExtrudeParams *e)
+                                double T0, double T1, double T2,
+                                double A0, double A1, double A2,
+                                double X0, double X1, double X2, double alpha,
+                                Curve **pc, Curve **prc, int final, 
+                                ExtrudeParams *e)
 {
   double matrix[4][4], T[3], Ax[3], d;
   Vertex V, *pv, *newp, *chapeau;
@@ -2062,11 +2137,11 @@ static int Extrude_ProtudePoint(int type, int ip,
 }
 
 static int Extrude_ProtudeCurve(int type, int ic,
-				double T0, double T1, double T2,
-				double A0, double A1, double A2,
-				double X0, double X1, double X2, double alpha,
-				Surface **ps, int final, 
-				ExtrudeParams *e)
+                                double T0, double T1, double T2,
+                                double A0, double A1, double A2,
+                                double X0, double X1, double X2, double alpha,
+                                Surface **ps, int final, 
+                                ExtrudeParams *e)
 {
   double matrix[4][4], T[3], Ax[3];
   Curve *CurveBeg, *CurveEnd;
@@ -2089,7 +2164,7 @@ static int Extrude_ProtudeCurve(int type, int ic,
 
   Msg::Debug("Extrude Curve %d", ic);
 
-  chapeau = DuplicateCurve(pc);
+  chapeau = DuplicateCurve(pc, false);
 
   chapeau->Extrude = new ExtrudeParams(COPIED_ENTITY);
   chapeau->Extrude->fill(type, T0, T1, T2, A0, A1, A2, X0, X1, X2, alpha);
@@ -2222,10 +2297,10 @@ static int Extrude_ProtudeCurve(int type, int ic,
 }
 
 static int Extrude_ProtudeSurface(int type, int is,
-				  double T0, double T1, double T2,
-				  double A0, double A1, double A2,
-				  double X0, double X1, double X2, double alpha,
-				  Volume **pv, ExtrudeParams *e)
+                                  double T0, double T1, double T2,
+                                  double A0, double A1, double A2,
+                                  double X0, double X1, double X2, double alpha,
+                                  Volume **pv, ExtrudeParams *e)
 {
   double matrix[4][4], T[3], Ax[3];
   Curve *c, *c2;
@@ -2242,7 +2317,7 @@ static int Extrude_ProtudeSurface(int type, int is,
 
   Msg::Debug("Extrude Surface %d", is);
 
-  chapeau = DuplicateSurface(ps);
+  chapeau = DuplicateSurface(ps, false);
 
   chapeau->Extrude = new ExtrudeParams(COPIED_ENTITY);
   chapeau->Extrude->fill(type, T0, T1, T2, A0, A1, A2, X0, X1, X2, alpha);
@@ -2455,7 +2530,6 @@ void ExtrudeShapes(int type, List_T *list_in,
     case MSH_SEGM_ELLI:
     case MSH_SEGM_ELLI_INV:
     case MSH_SEGM_NURBS:
-    case MSH_SEGM_PARAMETRIC:
       {
         Surface *ps = 0;
         Shape top;
@@ -2520,7 +2594,7 @@ void ExtrudeShapes(int type, List_T *list_in,
       break;
     default:
       Msg::Error("Impossible to extrude entity %d (of type %d)",
-		 shape.Num, shape.Type);
+                 shape.Num, shape.Type);
       break;
     }
   }
@@ -2875,7 +2949,7 @@ struct PointSurface{
   Surface *s;
 };
 
-static void projectPS(gmshVector<double> &x, gmshVector<double> &res, void *data)
+static void projectPS(fullVector<double> &x, fullVector<double> &res, void *data)
 {
   PointSurface *ps = (PointSurface*)data;
   Vertex c = InterpolateSurface(ps->s, x(0), x(1), 0, 0);
@@ -2893,16 +2967,16 @@ static void projectPS(gmshVector<double> &x, gmshVector<double> &res, void *data
 
 bool ProjectPointOnSurface(Surface *s, Vertex &p, double uv[2])
 {
-  gmshVector<double> x(2);
+  fullVector<double> x(2);
   x(0) = uv[0];
   x(1) = uv[1];
   PointSurface ps = {&p, s};
 
   Vertex pp = InterpolateSurface(s, uv[0], uv[1], 0, 0);
   double d2 = 
-    (pp.Pos.X - p.Pos.X)*(pp.Pos.X - p.Pos.X) + 
-    (pp.Pos.Y - p.Pos.Y)*(pp.Pos.Y - p.Pos.Y) + 
-    (pp.Pos.Z - p.Pos.Z)*(pp.Pos.Z - p.Pos.Z) ;
+    (pp.Pos.X - p.Pos.X) * (pp.Pos.X - p.Pos.X) + 
+    (pp.Pos.Y - p.Pos.Y) * (pp.Pos.Y - p.Pos.Y) + 
+    (pp.Pos.Z - p.Pos.Z) * (pp.Pos.Z - p.Pos.Z) ;
   if(d2 < 1.e-12) return true;
 
   double UMIN = 0.;
@@ -2923,26 +2997,26 @@ bool ProjectPointOnSurface(Surface *s, Vertex &p, double uv[2])
     }
     x(0) = UMIN + (UMAX - UMIN) * ((rand() % 10000) / 10000.);
     x(1) = VMIN + (VMAX - VMIN) * ((rand() % 10000) / 10000.);
-    if (ITER++ > 100)break;
+    if(ITER++ > 100) break;
   }
   {
     int NSAMPLES = 500;
-    double uok, vok;
+    double uok = 0.5, vok = 0.5;
     double dmin = 1.e22;
     for (int i = 0; i < NSAMPLES; i++){
       const double U = i / (double)(NSAMPLES - 1);
       for (int j = 0; j < NSAMPLES; j++){
-	const double V = j / (double)(NSAMPLES - 1);
-	Vertex pp = InterpolateSurface(s, U, V, 0, 0);
-	double d2 =
+        const double V = j / (double)(NSAMPLES - 1);
+        Vertex pp = InterpolateSurface(s, U, V, 0, 0);
+        double d2 =
           (pp.Pos.X - p.Pos.X) * (pp.Pos.X - p.Pos.X) + 
-	  (pp.Pos.Y - p.Pos.Y) * (pp.Pos.Y - p.Pos.Y) + 
-	  (pp.Pos.Z - p.Pos.Z) * (pp.Pos.Z - p.Pos.Z);
-	if (d2 < dmin) {
-	  dmin = d2;
-	  uok = U;
-	  vok = V;
-	}
+          (pp.Pos.Y - p.Pos.Y) * (pp.Pos.Y - p.Pos.Y) + 
+          (pp.Pos.Z - p.Pos.Z) * (pp.Pos.Z - p.Pos.Z);
+        if (d2 < dmin) {
+          dmin = d2;
+          uok = U;
+          vok = V;
+        }
       }
     }
     p = InterpolateSurface(s, uok, vok, 0, 0);
@@ -3054,11 +3128,12 @@ bool SplitCurve(int line_id, List_T *vertices_id, List_T *shapes)
       if(surface_curve->Num == c->Num){
         List_Remove(s->Generatrices, j);
         List_Insert_In_List(shapes, j, s->Generatrices);
-        j+=List_Nbr(shapes)-1;
-      }else if(surface_curve->Num == -c->Num){
+        j += List_Nbr(shapes) - 1;
+      }
+      else if(surface_curve->Num == -c->Num){
         List_Remove(s->Generatrices, j);
         List_Insert_In_List(rshapes, j, s->Generatrices);
-        j+=List_Nbr(shapes)-1;
+        j += List_Nbr(shapes) - 1;
       }
     }
   }
@@ -3076,7 +3151,7 @@ struct CurveSurface{
   Surface *s;
 };
 
-static void intersectCS(gmshVector<double> &uvt, gmshVector<double> &res, void *data)
+static void intersectCS(fullVector<double> &uvt, fullVector<double> &res, void *data)
 {
   CurveSurface *cs = (CurveSurface*)data;
   Vertex vs = InterpolateSurface(cs->s, uvt(0), uvt(1), 0, 0);
@@ -3086,6 +3161,7 @@ static void intersectCS(gmshVector<double> &uvt, gmshVector<double> &res, void *
   res(2) = vs.Pos.Z - vc.Pos.Z;
 }
 
+
 bool IntersectCurvesWithSurface(List_T *curve_ids, int surface_id, List_T *shapes)
 {
   Surface *s = FindSurface(surface_id);
@@ -3099,7 +3175,7 @@ bool IntersectCurvesWithSurface(List_T *curve_ids, int surface_id, List_T *shape
     Curve *c = FindCurve((int)curve_id);
     if(c){
       CurveSurface cs = {c, s};
-      gmshVector<double> uvt(3);
+      fullVector<double> uvt(3);
       uvt(0) = 0.5;
       uvt(1) = 0.5;
       uvt(2) = 0.5;
@@ -3159,7 +3235,7 @@ void sortEdgesInLoop(int num, List_T *edges)
         if(c2->end == c0->beg) {
           if(List_Nbr(temp)) {
             Msg::Info("Starting subloop %d in Line Loop %d (are you sure about this?)",
-		      ++k, num);
+                      ++k, num);
             c0 = c1 = *(Curve **)List_Pointer(temp, 0);
             List_Add(edges, &c1->Num);
             List_PSuppress(temp, 0);
@@ -3314,6 +3390,7 @@ void GEO_Internals::alloc_all()
   Surfaces = Tree_Create(sizeof(Surface *), compareSurface);
   SurfaceLoops = Tree_Create(sizeof(SurfaceLoop *), compareSurfaceLoop);
   Volumes = Tree_Create(sizeof(Volume *), compareVolume);
+  LevelSets = Tree_Create(sizeof(LevelSet *), compareLevelSet);
   PhysicalGroups = List_Create(5, 5, sizeof(PhysicalGroup *));
 }
 
@@ -3327,6 +3404,7 @@ void GEO_Internals::free_all()
   Tree_Action(Surfaces, Free_Surface); Tree_Delete(Surfaces);
   Tree_Action(SurfaceLoops, Free_SurfaceLoop); Tree_Delete(SurfaceLoops);
   Tree_Action(Volumes, Free_Volume); Tree_Delete(Volumes);
+  Tree_Action(LevelSets, Free_LevelSet); Tree_Delete(LevelSets);
   List_Action(PhysicalGroups, Free_PhysicalGroup); List_Delete(PhysicalGroups);
 }
 
