@@ -19,36 +19,6 @@
 #include "Parser.h"
 #endif
 
-double evaluate_scalarfunction(std::string var, double val, std::string funct)
-{
-#if defined(HAVE_NO_PARSER)
-  Msg::Error("Scalar function evaluation not available without Gmsh parser");
-  return 0.;
-#else
-  std::string fileName = CTX::instance()->homeDir + CTX::instance()->tmpFileName;
-  FILE *gmsh_yyin_old = gmsh_yyin;
-  if(!(gmsh_yyin = fopen(fileName.c_str(), "w"))) {
-    Msg::Error("Unable to open temporary file '%s'", fileName.c_str());
-    gmsh_yyin = gmsh_yyin_old;
-    return 0.;
-  }
-  // pose "variable = function" and evaluate function
-  fprintf(gmsh_yyin, "%s = %.16g ;\n", var.c_str(), val);
-  fprintf(gmsh_yyin, "ValeurTemporaire__ = %s ;\n", funct.c_str());
-  fclose(gmsh_yyin);
-  gmsh_yyin = fopen(fileName.c_str(), "r");
-  while(!feof(gmsh_yyin)) {
-    gmsh_yyparse();
-  }
-  fclose(gmsh_yyin);
-  gmsh_yyin = gmsh_yyin_old;
-  // retreive value
-  if(gmsh_yysymbols.count("ValeurTemporaire__")) 
-    return gmsh_yysymbols["ValeurTemporaire__"][0];
-  return 0.;
-#endif
-}
-
 void add_infile(std::string text, std::string fileName, bool deleted_something)
 {
 #if defined(HAVE_NO_PARSER)
@@ -199,22 +169,22 @@ void add_param(std::string par, std::string value, std::string fileName)
 }
 
 void add_point(std::string fileName, std::string x, std::string y,
-	       std::string z, std::string lc)
+               std::string z, std::string lc)
 {
   std::ostringstream sstream;
   sstream << "Point(" << NEWPOINT() << ") = {" << x << ", " << y << ", " 
-	  << z ;
+          << z ;
   if(lc.size()) sstream << ", " << lc;
   sstream << "};";
   add_infile(sstream.str(), fileName);
 }
 
 void add_field_option(int field_id, std::string option_name, 
-		      std::string option_value, std::string fileName)
+                      std::string option_value, std::string fileName)
 {
   std::ostringstream sstream;
   sstream << "Field[" << field_id << "]." << option_name << " = " 
-	  << option_value << ";";
+          << option_value << ";";
   add_infile(sstream.str(), fileName);
 }
 
@@ -255,7 +225,7 @@ void add_circ(int p1, int p2, int p3, std::string fileName)
 {
   std::ostringstream sstream;
   sstream << "Circle(" << NEWLINE() << ") = {" << p1 << ", " << p2 << ", "
-	  << p3 << "};";
+          << p3 << "};";
   add_infile(sstream.str(), fileName);
 }
 
@@ -263,7 +233,7 @@ void add_ell(int p1, int p2, int p3, int p4, std::string fileName)
 {
   std::ostringstream sstream;
   sstream << "Ellipse(" << NEWLINE() << ") = {" << p1 << ", " << p2 << ", "
-	  << p3 << ", " << p4 << "};";
+          << p3 << ", " << p4 << "};";
   add_infile(sstream.str(), fileName);
 }
 
@@ -303,7 +273,7 @@ void add_physical(std::string type, List_T *list, std::string fileName)
 {
   std::ostringstream sstream;
   sstream << "Physical " << type << "(" << NEWPHYSICAL() << ") = {" 
-	  << list2string(list) << "};";
+          << list2string(list) << "};";
   add_infile(sstream.str(), fileName);
 }
 
@@ -325,7 +295,7 @@ void rotate(int add, List_T *list, std::string fileName, std::string what,
 {
   std::ostringstream sstream;
   sstream << "Rotate {{" << ax << ", " << ay << ", " << az << "}, {" 
-	  << px << ", " << py << ", " << pz << "}, " << angle << "} {\n  ";
+          << px << ", " << py << ", " << pz << "}, " << angle << "} {\n  ";
   if(add) sstream << "Duplicata { ";
   sstream << what << "{" << list2string(list) << "};";
   if(add) sstream << " }";
@@ -362,7 +332,7 @@ void extrude(List_T *list, std::string fileName, std::string what,
 {
   std::ostringstream sstream;
   sstream << "Extrude {" << tx << ", " << ty << ", " << tz << "} {\n  " << what 
-	  << "{" << list2string(list) << "};\n}";
+          << "{" << list2string(list) << "};\n}";
   add_infile(sstream.str(), fileName);
 }
 
@@ -372,8 +342,8 @@ void protude(List_T *list, std::string fileName, std::string what,
 {
   std::ostringstream sstream;
   sstream << "Extrude {{" << ax << ", " << ay << ", " << az << "}, {" 
-	  << px << ", " << py << ", " << pz << "}, " << angle << "} {\n  "
-	  << what << "{" << list2string(list) << "};\n}";
+          << px << ", " << py << ", " << pz << "}, " << angle << "} {\n  "
+          << what << "{" << list2string(list) << "};\n}";
   add_infile(sstream.str(), fileName);
 }
 
