@@ -26,12 +26,35 @@ bool MVertexLessThanLexicographic::operator()(const MVertex *v1, const MVertex *
   return false;
 }
 
+MVertex::MVertex(double x, double y, double z, GEntity *ge, int num)
+  : _visible(1), _order(1), _x(x), _y(y), _z(z), _ge(ge)
+{
+#pragma omp critical
+  {
+    if(num){
+      _num = num;
+      _globalNum = std::max(_globalNum, _num);
+    }
+    else{
+      _num = ++_globalNum;
+    }
+    _index = num;
+  }
+}
+
+void MVertex::forceNum(int num)
+{ 
+#pragma omp critical
+  {
+    _num = num; 
+    _globalNum = std::max(_globalNum, _num);
+  }
+}
+
 void MVertex::writeMSH(FILE *fp, bool binary, bool saveParametric, double scalingFactor)
 {
   if(_index < 0) return; // negative index vertices are never saved
 
-  //  printf("tag = %d index %d pos = %g %g %g\n",onWhat()->tag(),_index,x(),y(),z());
-
   int myDim = 0, myTag = 0;
   if(saveParametric){
     if(onWhat()){
@@ -65,7 +88,7 @@ void MVertex::writeMSH(FILE *fp, bool binary, bool saveParametric, double scalin
       double _u;
       getParameter(0, _u);
       if(!binary)
-        fprintf(fp, " %.16g\n", _u); 	    
+        fprintf(fp, " %.16g\n", _u);        
       else
         fwrite(&_u, sizeof(double), 1, fp);
     }
@@ -82,7 +105,7 @@ void MVertex::writeMSH(FILE *fp, bool binary, bool saveParametric, double scalin
     }
     else
       if(!binary)
-        fprintf(fp, "\n"); 	    
+        fprintf(fp, "\n");          
   }
 }
 
@@ -107,7 +130,7 @@ void MVertex::writeUNV(FILE *fp, double scalingFactor)
   sprintf(tmp, "%25.16E%25.16E%25.16E\n", x() * scalingFactor, 
           y() * scalingFactor, z() * scalingFactor);
   for(unsigned int i = 0; i < strlen(tmp); i++) if(tmp[i] == 'E') tmp[i] = 'D';
-  fprintf(fp, tmp);
+  fprintf(fp, "%s", tmp);
 }
 
 void MVertex::writeVTK(FILE *fp, bool binary, double scalingFactor, bool bigEndian)
@@ -122,7 +145,7 @@ void MVertex::writeVTK(FILE *fp, bool binary, double scalingFactor, bool bigEndi
   }
   else{
     fprintf(fp, "%.16g %.16g %.16g\n",
-	    x() * scalingFactor, y() * scalingFactor, z() * scalingFactor);
+            x() * scalingFactor, y() * scalingFactor, z() * scalingFactor);
   }
 }
 
@@ -222,19 +245,19 @@ static void getAllParameters(MVertex *v, GFace *gf, std::vector<SPoint2> &params
     bool seam = false;
     for(std::list<GEdge*>::iterator it = ed.begin(); it != ed.end(); it++){
       if((*it)->isSeam(gf)) {
-	Range<double> range = (*it)->parBounds(0);
-	if (gv == (*it)->getBeginVertex()){
-	  params.push_back((*it)->reparamOnFace(gf, range.low(),-1));
-	  params.push_back((*it)->reparamOnFace(gf, range.low(), 1));
-	}
-	else if (gv == (*it)->getEndVertex()){
-	  params.push_back((*it)->reparamOnFace(gf, range.high(),-1));
-	  params.push_back((*it)->reparamOnFace(gf, range.high(), 1));
-	}
-	else{
+        Range<double> range = (*it)->parBounds(0);
+        if (gv == (*it)->getBeginVertex()){
+          params.push_back((*it)->reparamOnFace(gf, range.low(),-1));
+          params.push_back((*it)->reparamOnFace(gf, range.low(), 1));
+        }
+        else if (gv == (*it)->getEndVertex()){
+          params.push_back((*it)->reparamOnFace(gf, range.high(),-1));
+          params.push_back((*it)->reparamOnFace(gf, range.high(), 1));
+        }
+        else{
           Msg::Warning("Strange!");
-	}
-	seam = true;
+        }
+        seam = true;
       }
     }
     if (!seam)
@@ -258,7 +281,6 @@ static void getAllParameters(MVertex *v, GFace *gf, std::vector<SPoint2> &params
 bool reparamMeshEdgeOnFace(MVertex *v1, MVertex *v2, GFace *gf, 
                            SPoint2 &param1, SPoint2 &param2)
 {
-
   std::vector<SPoint2> p1, p2;
   getAllParameters(v1, gf, p1);
   getAllParameters(v2, gf, p2);
@@ -312,8 +334,8 @@ bool reparamMeshVertexOnFace(const MVertex *v, const GFace *gf, SPoint2 &param)
     return true;
   }
 
-  if(v->onWhat()->geomType() == GEntity::DiscreteCurve || 	 
-     v->onWhat()->geomType() == GEntity::BoundaryLayerCurve){ 	 
+  if(v->onWhat()->geomType() == GEntity::DiscreteCurve ||        
+     v->onWhat()->geomType() == GEntity::BoundaryLayerCurve){    
     param = gf->parFromPoint(SPoint3(v->x(), v->y(), v->z()));
     return true;
   }
