@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -27,13 +27,13 @@ class MVertex{
  protected:
   // the maximum vertex id number in the mesh
   static int _globalNum;
-  // the id number of the vertex (this number is unique and is
-  // guaranteed never to change once a mesh has been generated)
+  // the immutable id number of the vertex (this number is unique and
+  // is guaranteed never to change once a vertex has been created)
   int _num;
-  // a vertex index, used for example when saving a mesh (this index
-  // is not necessarily unique, can change after mesh renumbering,
-  // ...). By convention, vertices with negative indices are not
-  // saved
+  // a vertex index, used for example during mesh generation or when
+  // saving a mesh (this index is not necessarily unique, can change
+  // after mesh renumbering, etc.). By convention, vertices with
+  // negative indices are not saved
   int _index;
   // a visibility and polynomial order flags
   char _visible, _order;
@@ -42,19 +42,8 @@ class MVertex{
   // the geometrical entity the vertex is associated with
   GEntity *_ge;
 
- public :
-  MVertex(double x, double y, double z, GEntity *ge=0, int num=0) 
-    : _visible(1), _order(1), _x(x), _y(y), _z(z), _ge(ge)
-  {
-    if(num){
-      _num = num;
-      _globalNum = std::max(_globalNum, _num);
-    }
-    else{
-      _num = ++_globalNum;
-    }
-    _index = num;
-  }
+ public:
+  MVertex(double x, double y, double z, GEntity *ge=0, int num=0);
   virtual ~MVertex(){}
 
   // get/reset the global node number
@@ -82,16 +71,18 @@ class MVertex{
   inline GEntity* onWhat() const { return _ge; }
   inline void setEntity(GEntity *ge) { _ge = ge; }
 
-  // get/set the number
+  // get the immutable vertex number
   inline int getNum() const { return _num; }
-  inline void setNum(int num) { _num = num; }
+
+  // force the immutable number (this should normally never be used)
+  void forceNum(int num);
 
   // get/set the index
   inline int getIndex() const { return _index; }
   inline void setIndex(int index) { _index = index; }
 
   // get/set ith parameter
-  virtual bool getParameter(int i, double &par) const{ return false; }
+  virtual bool getParameter(int i, double &par) const { par = 0.; return false; }
   virtual bool setParameter(int i, double par){ return false; }
 
   // measure distance to another vertex
@@ -113,7 +104,7 @@ class MVertex{
   void writeVRML(FILE *fp, double scalingFactor=1.0);
   void writeUNV(FILE *fp, double scalingFactor=1.0);
   void writeVTK(FILE *fp, bool binary=false, double scalingFactor=1.0,
-		bool bigEndian=false);
+                bool bigEndian=false);
   void writeMESH(FILE *fp, double scalingFactor=1.0);
   void writeBDF(FILE *fp, int format=0, double scalingFactor=1.0);
   void writeDIFF(FILE *fp, bool binary, double scalingFactor=1.0);
@@ -122,22 +113,16 @@ class MVertex{
 class MEdgeVertex : public MVertex{
  protected:
   double _u, _lc;
- public :
-  MEdgeVertex(double x, double y, double z, GEntity *ge, double u, double lc = -1.0, int num = 0) 
+ public:
+  MEdgeVertex(double x, double y, double z, GEntity *ge, double u, double lc = -1.0,
+              int num = 0) 
     : MVertex(x, y, z, ge,num), _u(u), _lc(lc)
   {
   }
   virtual ~MEdgeVertex(){}
-  virtual bool getParameter(int i, double &par) const 
-  { 
-    par = _u; 
-    return true; 
-  }
-  virtual bool setParameter(int i, double par)
-  { 
-    _u = par; 
-    return true; 
-  }
+  virtual bool getParameter(int i, double &par) const { 
+    par = _u; return true; }
+  virtual bool setParameter(int i, double par){ _u = par; return true; }
   double getLc() const { return _lc; }
 };
 
@@ -145,16 +130,12 @@ class MFaceVertex : public MVertex{
  protected:
   double _u, _v;
  public :
-  MFaceVertex(double x, double y, double z, GEntity *ge, double u, double v, int num =0) 
+  MFaceVertex(double x, double y, double z, GEntity *ge, double u, double v, int num = 0) 
     : MVertex(x, y, z, ge, num), _u(u), _v(v)
   {
   }
   virtual ~MFaceVertex(){}
-  virtual bool getParameter(int i, double &par) const 
-  { 
-    par = (i ? _v : _u);
-    return true; 
-  }
+  virtual bool getParameter(int i, double &par) const { par = (i ? _v : _u); return true; }
   virtual bool setParameter(int i, double par)
   {
     if(!i) 
@@ -167,7 +148,7 @@ class MFaceVertex : public MVertex{
 
 bool reparamMeshEdgeOnFace(MVertex *v1, MVertex *v2, GFace *gf, 
                            SPoint2 &param1, SPoint2 &param2);
-bool reparamMeshVertexOnFace(MVertex *v, GFace *gf, SPoint2 &param);
-bool reparamMeshVertexOnEdge(MVertex *v, GEdge *ge, double &param);
+bool reparamMeshVertexOnFace(const MVertex *v, const GFace *gf, SPoint2 &param);
+bool reparamMeshVertexOnEdge(const MVertex *v, const GEdge *ge, double &param);
 
 #endif
