@@ -50,7 +50,7 @@ OCCFace::OCCFace(GModel *m, TopoDS_Face _s, int num, TopTools_IndexedMapOfShape
           occe->setTrimmed(this);
         }
       }
-    }      
+    }
     
     GEdgeLoop el(l_wire);
     for(GEdgeLoop::citer it = el.begin(); it != el.end(); ++it){
@@ -122,6 +122,18 @@ Pair<SVector3,SVector3> OCCFace::firstDer(const SPoint2 &param) const
                                  SVector3(dv.X(), dv.Y(), dv.Z()));
 }
 
+void OCCFace::secondDer(const SPoint2 &param,
+                        SVector3 *dudu, SVector3 *dvdv, SVector3 *dudv) const
+{
+  gp_Pnt pnt;
+  gp_Vec du, dv, duu, dvv, duv;
+  occface->D2(param.x(), param.y(), pnt, du, dv, duu, dvv, duv);
+
+  *dudu = SVector3(duu.X(), duu.Y(), duu.Z());
+  *dvdv = SVector3(dvv.X(), dvv.Y(), dvv.Z());
+  *dudv = SVector3(duv.X(), duv.Y(), duv.Z());
+}
+
 GPoint OCCFace::point(double par1, double par2) const
 {
   double pp[2] = {par1, par2};
@@ -129,7 +141,7 @@ GPoint OCCFace::point(double par1, double par2) const
   return GPoint(val.X(), val.Y(), val.Z(), this, pp);
 }
 
-GPoint OCCFace::closestPoint(const SPoint3 & qp, const double initialGuess[2]) const
+GPoint OCCFace::closestPoint(const SPoint3 &qp, const double initialGuess[2]) const
 {
   gp_Pnt pnt(qp.x(), qp.y(), qp.z());
   GeomAPI_ProjectPointOnSurf proj(pnt, occface, umin, umax, vmin, vmax);
@@ -187,7 +199,7 @@ GEntity::GeomType OCCFace::geomType() const
   return Unknown;
 }
 
-double OCCFace::curvature(const SPoint2 &param) const
+double OCCFace::curvatureMax(const SPoint2 &param) const
 {
   const double eps = 1.e-12;
   BRepAdaptor_Surface sf(s, Standard_True);
@@ -200,6 +212,37 @@ double OCCFace::curvature(const SPoint2 &param) const
   return std::max(fabs(prop.MinCurvature()), fabs(prop.MaxCurvature()));
 }
 
+double OCCFace::curvatures(const SPoint2 &param,
+                           SVector3 *dirMax,
+                           SVector3 *dirMin,
+                           double *curvMax,
+                           double *curvMin) const
+{
+  const double eps = 1.e-12;
+  BRepAdaptor_Surface sf(s, Standard_True);
+  BRepLProp_SLProps prop(sf, 2, eps);
+  prop.SetParameters (param.x(),param.y());
+
+  if (!prop.IsCurvatureDefined()){
+    return -1.;
+  }
+
+  *curvMax = std::max(fabs(prop.MinCurvature()), fabs(prop.MaxCurvature()));
+  *curvMin = std::min(fabs(prop.MinCurvature()), fabs(prop.MaxCurvature()));
+
+  gp_Dir dMax = gp_Dir();
+  gp_Dir dMin = gp_Dir();
+  prop.CurvatureDirections(dMax,dMin);
+  (*dirMax)[0] = dMax.X();
+  (*dirMax)[1] = dMax.Y();
+  (*dirMax)[2] = dMax.Z();
+  (*dirMin)[0] = dMin.X();
+  (*dirMin)[1] = dMin.Y();
+  (*dirMin)[2] = dMin.Z();
+
+  return *curvMax;
+}
+
 bool OCCFace::containsPoint(const SPoint3 &pt) const
 { 
   if(geomType() == Plane){
@@ -251,6 +294,9 @@ surface_params OCCFace::getSurfaceParams() const
   case GEntity::Cylinder:
     p.radius = Handle(Geom_CylindricalSurface)::DownCast(occface)->Radius();
     break;
+  case GEntity::Sphere:
+    p.radius = Handle(Geom_SphericalSurface)::DownCast(occface)->Radius();
+    break;
   default:
     break;
   }
