@@ -22,6 +22,9 @@ class SBoundingBox3d {
     : MinPt(DBL_MAX,DBL_MAX,DBL_MAX), MaxPt(-DBL_MAX,-DBL_MAX,-DBL_MAX) {}
   SBoundingBox3d(const SPoint3 &pt)
     : MinPt(pt), MaxPt(pt) {}
+  SBoundingBox3d(double xmin, double ymin, double zmin,
+                 double xmax, double ymax, double zmax)
+    : MinPt(xmin, ymin, zmin), MaxPt(xmax, ymax, zmax) {}
   bool empty()
   {
     if(MinPt.x() == DBL_MAX || MinPt.y() == DBL_MAX || MinPt.z() == DBL_MAX ||
