@@ -5,15 +5,21 @@
 
 #include "GmshConfig.h"
 #include "discreteRegion.h"
-
-#if !defined(HAVE_GMSH_EMBEDDED)
+#include "MVertex.h"
 #include "Geo.h"
-#endif
 
 discreteRegion::discreteRegion(GModel *model, int num) : GRegion(model, num)
 {
-#if !defined(HAVE_GMSH_EMBEDDED)
   ::Volume *v = Create_Volume(num, MSH_VOLUME_DISCRETE);
   Tree_Add(model->getGEOInternals()->Volumes, &v);
-#endif
+}
+
+void discreteRegion::setBoundFaces()
+{
+  //in case discrete region already exist
+  //to modify to take into account appropriate faces
+  for(GModel::fiter face = model()->firstFace(); face != model()->lastFace(); face++){
+    l_faces.push_back(*face);
+    (*face)->addRegion(this);
+  }
 }
