@@ -3,6 +3,7 @@
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
+#include <sstream>
 #include "GModel.h"
 #include "GFace.h"
 #include "GFaceCompound.h"
@@ -29,7 +30,7 @@ void gmshEdge::resetMeshAttributes()
 
 Range<double> gmshEdge::parBounds(int i) const
 { 
-  return Range<double>(c->ubeg, c->uend);
+ return Range<double>(c->ubeg, c->uend);
 }
 
 GPoint gmshEdge::point(double par) const
@@ -48,7 +49,6 @@ GEntity::GeomType gmshEdge::geomType() const
 {
   switch (c->Typ){
   case MSH_SEGM_LINE : return Line;
-  case MSH_SEGM_PARAMETRIC : return ParametricCurve;
   case MSH_SEGM_CIRC :  
   case MSH_SEGM_CIRC_INV : return Circle;
   case MSH_SEGM_ELLI:
@@ -63,6 +63,24 @@ GEntity::GeomType gmshEdge::geomType() const
   }
 }
 
+std::string gmshEdge::getAdditionalInfoString()
+{
+  if(List_Nbr(c->Control_Points) > 0){
+    std::ostringstream sstream;
+    sstream << "{";
+    for(int i = 0; i < List_Nbr(c->Control_Points); i++){
+      if(i) sstream << ",";
+      Vertex *v;
+      List_Read(c->Control_Points, i, &v);
+      sstream << v->Num;
+    }
+    sstream << "}";
+    return sstream.str();    
+  }
+  else
+    return GEdge::getAdditionalInfoString();
+}
+
 int gmshEdge::minimumMeshSegments () const
 {
   int np;
