@@ -4,9 +4,7 @@
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <string>
-#include <FL/gl.h>
-#include <FL/Fl_JPEG_Image.H>
-#include <FL/Fl_PNG_Image.H>
+#include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "drawContext.h"
 #include "Trackball.h"
@@ -17,6 +15,13 @@
 #include "PViewOptions.h"
 #include "gl2ps.h"
 
+#if defined(HAVE_FLTK)
+#include <FL/Fl_JPEG_Image.H>
+#include <FL/Fl_PNG_Image.H>
+#endif
+
+drawContextGlobal *drawContext::_global = 0;
+
 drawContext::drawContext(drawTransform *transform) 
   : _transform(transform)
 {
@@ -49,6 +54,12 @@ drawContext::~drawContext()
   if(_displayLists) glDeleteLists(_displayLists, 2);
 }
 
+drawContextGlobal *drawContext::global()
+{ 
+  if(!_global) _global = new drawContextGlobal(); // create dummy default
+  return _global; 
+}
+
 void drawContext::createQuadricsAndDisplayLists()
 {
   if(!_quadric) _quadric = gluNewQuadric();
@@ -224,8 +235,8 @@ void drawContext::draw3d()
   // measurement of the change in depth relative to the screen area of
   // the polygon, and r is the smallest value that is guaranteed to
   // produce a resolvable offset for a given implementation.
-  glPolygonOffset(CTX::instance()->polygonOffsetFactor, 
-                  CTX::instance()->polygonOffsetUnits);
+  glPolygonOffset((float)CTX::instance()->polygonOffsetFactor, 
+                  (float)CTX::instance()->polygonOffsetUnits);
   if(CTX::instance()->polygonOffsetFactor || CTX::instance()->polygonOffsetUnits)
     CTX::instance()->polygonOffset = CTX::instance()->polygonOffsetAlways ? 1 : 
       needPolygonOffset();
@@ -391,10 +402,11 @@ void drawContext::initProjection(int xpick, int ypick, int wpick, int hpick)
 
     // background image
     if(CTX::instance()->bgImageFileName.size()){
+#if defined(HAVE_FLTK)
       if(_bgImage.empty()){
         int idot = CTX::instance()->bgImageFileName.find_last_of('.');
         std::string ext;
-        if(idot > 0 && idot < CTX::instance()->bgImageFileName.size())
+        if(idot > 0 && idot < (int)CTX::instance()->bgImageFileName.size())
           ext = CTX::instance()->bgImageFileName.substr(idot + 1);
         Fl_RGB_Image *img = 0;
         if(ext == "jpg" || ext == "JPG" || ext == "jpeg" || ext == "JPEG")
@@ -406,9 +418,9 @@ void drawContext::initProjection(int xpick, int ypick, int wpick, int hpick)
           for(int j = img->h() - 1; j >= 0; j--) {
             for(int i = 0; i < img->w(); i++) {
               int idx = j * img->w() * img->d() + i * img->d();
-              _bgImage.push_back((GLfloat)data[idx] / 255.);
-              _bgImage.push_back((GLfloat)data[idx + 1] / 255.);
-              _bgImage.push_back((GLfloat)data[idx + 2] / 255.);
+              _bgImage.push_back((GLfloat)data[idx] / 255.F);
+              _bgImage.push_back((GLfloat)data[idx + 1] / 255.F);
+              _bgImage.push_back((GLfloat)data[idx + 2] / 255.F);
             }
           }
           _bgImageSize[0] = img->w();
@@ -418,7 +430,7 @@ void drawContext::initProjection(int xpick, int ypick, int wpick, int hpick)
           Msg::Error("Could not load valid background image");
           // make sure we don't try to load it again
           for(int i = 0; i < 3; i++) _bgImage.push_back(0);
-          _bgImageSize[0] = _bgImageSize[1] = 1.;
+          _bgImageSize[0] = _bgImageSize[1] = 1;
         }
         if(img) delete img;
       }
@@ -436,6 +448,7 @@ void drawContext::initProjection(int xpick, int ypick, int wpick, int hpick)
                    (void*)&_bgImage[0]);
       gl2psDrawPixels(_bgImageSize[0], _bgImageSize[1], 0, 0, GL_RGB, GL_FLOAT,
                       (void*)&_bgImage[0]);
+#endif
     }
 
     glPopMatrix();
@@ -481,22 +494,31 @@ void drawContext::initRenderModel()
                              (GLfloat)CTX::instance()->lightPosition[i][3]};
       glLightfv((GLenum)(GL_LIGHT0 + i), GL_POSITION, position);
 
-      GLfloat r = CTX::instance()->unpackRed(CTX::instance()->color.ambientLight[i])/255.;
-      GLfloat g = CTX::instance()->unpackGreen(CTX::instance()->color.ambientLight[i])/255.;
-      GLfloat b = CTX::instance()->unpackBlue(CTX::instance()->color.ambientLight[i])/255.;
-      GLfloat ambient[4] = {r, g, b, 1.0};
+      GLfloat r = (GLfloat)(CTX::instance()->unpackRed
+                            (CTX::instance()->color.ambientLight[i]) / 255.);
+      GLfloat g = (GLfloat)(CTX::instance()->unpackGreen
+                            (CTX::instance()->color.ambientLight[i]) / 255.);
+      GLfloat b = (GLfloat)(CTX::instance()->unpackBlue
+                            (CTX::instance()->color.ambientLight[i]) / 255.);
+      GLfloat ambient[4] = {r, g, b, 1.0F};
       glLightfv((GLenum)(GL_LIGHT0 + i), GL_AMBIENT, ambient);
 
-      r = CTX::instance()->unpackRed(CTX::instance()->color.diffuseLight[i])/255.;
-      g = CTX::instance()->unpackGreen(CTX::instance()->color.diffuseLight[i])/255.;
-      b = CTX::instance()->unpackBlue(CTX::instance()->color.diffuseLight[i])/255.;
-      GLfloat diffuse[4] = {r, g, b, 1.0};
+      r = (GLfloat)(CTX::instance()->unpackRed
+                    (CTX::instance()->color.diffuseLight[i]) / 255.);
+      g = (GLfloat)(CTX::instance()->unpackGreen
+                    (CTX::instance()->color.diffuseLight[i]) / 255.);
+      b = (GLfloat)(CTX::instance()->unpackBlue
+                    (CTX::instance()->color.diffuseLight[i]) / 255.);
+      GLfloat diffuse[4] = {r, g, b, 1.0F};
       glLightfv((GLenum)(GL_LIGHT0 + i), GL_DIFFUSE, diffuse);
 
-      r = CTX::instance()->unpackRed(CTX::instance()->color.specularLight[i])/255.;
-      g = CTX::instance()->unpackGreen(CTX::instance()->color.specularLight[i])/255.;
-      b = CTX::instance()->unpackBlue(CTX::instance()->color.specularLight[i])/255.;
-      GLfloat specular[4] = {r, g, b, 1.0};
+      r = (GLfloat)(CTX::instance()->unpackRed
+                    (CTX::instance()->color.specularLight[i]) / 255.);
+      g = (GLfloat)(CTX::instance()->unpackGreen
+                    (CTX::instance()->color.specularLight[i]) / 255.);
+      b = (GLfloat)(CTX::instance()->unpackBlue
+                    (CTX::instance()->color.specularLight[i]) / 255.);
+      GLfloat specular[4] = {r, g, b, 1.0F};
       glLightfv((GLenum)(GL_LIGHT0 + i), GL_SPECULAR, specular);
 
       glEnable((GLenum)(GL_LIGHT0 + i));
@@ -512,12 +534,14 @@ void drawContext::initRenderModel()
   glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
   glEnable(GL_COLOR_MATERIAL);
   // "white"-only specular material reflection color
-  GLfloat spec[4] = {CTX::instance()->shine, CTX::instance()->shine, 
-                     CTX::instance()->shine, 1.0};
+  GLfloat spec[4] = {(GLfloat)CTX::instance()->shine, 
+                     (GLfloat)CTX::instance()->shine, 
+                     (GLfloat)CTX::instance()->shine, 1.0F};
   glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, spec);
   // specular exponent in [0,128] (larger means more "focused"
   // reflection)
-  glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, CTX::instance()->shineExponent);
+  glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 
+              (GLfloat)CTX::instance()->shineExponent);
 
   glShadeModel(GL_SMOOTH);
 
