@@ -3,13 +3,13 @@
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
-#include <FL/gl.h>
 #include "drawContext.h"
 #include "Context.h"
 #include "gl2ps.h"
 #include "VertexArray.h"
 #include "GModel.h"
 #include "SBoundingBox3d.h"
+#include "GmshMessage.h"
 
 class drawGVertex {
  private :
@@ -29,15 +29,15 @@ class drawGVertex {
     }
     
     if(v->getSelection()) {
-      glPointSize(CTX::instance()->geom.selectedPointSize);
-      gl2psPointSize(CTX::instance()->geom.selectedPointSize * 
-                     CTX::instance()->print.epsPointSizeFactor);
+      glPointSize((float)CTX::instance()->geom.selectedPointSize);
+      gl2psPointSize((float)(CTX::instance()->geom.selectedPointSize * 
+                             CTX::instance()->print.epsPointSizeFactor));
       glColor4ubv((GLubyte *) & CTX::instance()->color.geom.selection);
     }
     else {
-      glPointSize(CTX::instance()->geom.pointSize);
-      gl2psPointSize(CTX::instance()->geom.pointSize *
-                     CTX::instance()->print.epsPointSizeFactor);
+      glPointSize((float)CTX::instance()->geom.pointSize);
+      gl2psPointSize((float)(CTX::instance()->geom.pointSize *
+                             CTX::instance()->print.epsPointSizeFactor));
       glColor4ubv((GLubyte *) & CTX::instance()->color.geom.point);
     }
     
@@ -95,6 +95,7 @@ class drawGEdge {
   {
     if(!e->getVisibility()) return;
     if(e->geomType() == GEntity::DiscreteCurve) return;
+    if(e->geomType() == GEntity::PartitionCurve) return;
     if(e->geomType() == GEntity::BoundaryLayerCurve) return;
     
     bool select = (_ctx->render_mode == drawContext::GMSH_SELECT && 
@@ -105,15 +106,15 @@ class drawGEdge {
     }
     
     if(e->getSelection()) {
-      glLineWidth(CTX::instance()->geom.selectedLineWidth);
-      gl2psLineWidth(CTX::instance()->geom.selectedLineWidth * 
-                     CTX::instance()->print.epsLineWidthFactor);
+      glLineWidth((float)CTX::instance()->geom.selectedLineWidth);
+      gl2psLineWidth((float)(CTX::instance()->geom.selectedLineWidth * 
+                             CTX::instance()->print.epsLineWidthFactor));
       glColor4ubv((GLubyte *) & CTX::instance()->color.geom.selection);
     }
     else {
-      glLineWidth(CTX::instance()->geom.lineWidth);
-      gl2psLineWidth(CTX::instance()->geom.lineWidth * 
-                     CTX::instance()->print.epsLineWidthFactor);
+      glLineWidth((float)CTX::instance()->geom.lineWidth);
+      gl2psLineWidth((float)(CTX::instance()->geom.lineWidth * 
+                             CTX::instance()->print.epsLineWidthFactor));
       glColor4ubv((GLubyte *) & CTX::instance()->color.geom.line);
     }
     
@@ -379,7 +380,8 @@ class drawGFace {
   void operator () (GFace *f)
   {
     if(!f->getVisibility()) return;
-    if(f->geomType() == GEntity::DiscreteSurface) return;
+     if(f->geomType() == GEntity::DiscreteSurface) return;
+     if(f->geomType() == GEntity::PartitionSurface) return;
     if(f->geomType() == GEntity::BoundaryLayerSurface) return;
 
     bool select = (_ctx->render_mode == drawContext::GMSH_SELECT && 
@@ -390,15 +392,15 @@ class drawGFace {
     }
     
     if(f->getSelection()) {
-      glLineWidth(CTX::instance()->geom.selectedLineWidth / 2.);
-      gl2psLineWidth(CTX::instance()->geom.selectedLineWidth / 2. *
-                     CTX::instance()->print.epsLineWidthFactor);
+      glLineWidth((float)(CTX::instance()->geom.selectedLineWidth / 2.));
+      gl2psLineWidth((float)(CTX::instance()->geom.selectedLineWidth / 2. *
+                             CTX::instance()->print.epsLineWidthFactor));
       glColor4ubv((GLubyte *) & CTX::instance()->color.geom.selection);
     }
     else {
-      glLineWidth(CTX::instance()->geom.lineWidth / 2.);
-      gl2psLineWidth(CTX::instance()->geom.lineWidth / 2. * 
-                     CTX::instance()->print.epsLineWidthFactor);
+      glLineWidth((float)(CTX::instance()->geom.lineWidth / 2.));
+      gl2psLineWidth((float)(CTX::instance()->geom.lineWidth / 2. * 
+                             CTX::instance()->print.epsLineWidthFactor));
       glColor4ubv((GLubyte *) & CTX::instance()->color.geom.surface);
     }
 
@@ -406,7 +408,7 @@ class drawGFace {
       _drawPlaneGFace(f);
     else
       _drawParametricGFace(f);
-    
+
     if(select) {
       glPopName();
       glPopName();
@@ -455,7 +457,7 @@ class drawGRegion {
                     z + offset / _ctx->s[2]);
       _ctx->drawString(Num);
     }
-
+    
     if(select) {
       glPopName();
       glPopName();
