@@ -4,12 +4,18 @@
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <math.h>
-#include <FL/gl.h>
 #include "drawContext.h"
 #include "GmshMessage.h"
 #include "GmshDefines.h"
 #include "GModel.h"
-#include "MElement.h"
+#include "MLine.h"
+#include "MTriangle.h"
+#include "MQuadrangle.h"
+#include "MTetrahedron.h"
+#include "MHexahedron.h"
+#include "MPrism.h"
+#include "MPyramid.h"
+#include "MElementCut.h"
 #include "Context.h"
 #include "OS.h"
 #include "gl2ps.h"
@@ -47,14 +53,14 @@ static unsigned int getColorByElement(MElement *ele)
     return CTX::instance()->color.geom.selection;
   }
   else if(CTX::instance()->mesh.colorCarousel == 0){ // by element type
-    switch(ele->getNumEdges()){
-    case 1: return CTX::instance()->color.mesh.line;
-    case 3: return CTX::instance()->color.mesh.triangle;
-    case 4: return CTX::instance()->color.mesh.quadrangle;
-    case 6: return CTX::instance()->color.mesh.tetrahedron;
-    case 12: return CTX::instance()->color.mesh.hexahedron;
-    case 9: return CTX::instance()->color.mesh.prism;
-    case 8: return CTX::instance()->color.mesh.pyramid;
+    switch(ele->getType()){
+    case TYPE_LIN: return CTX::instance()->color.mesh.line;
+    case TYPE_TRI: return CTX::instance()->color.mesh.triangle;
+    case TYPE_QUA: return CTX::instance()->color.mesh.quadrangle;
+    case TYPE_TET: return CTX::instance()->color.mesh.tetrahedron;
+    case TYPE_HEX: return CTX::instance()->color.mesh.hexahedron;
+    case TYPE_PRI: return CTX::instance()->color.mesh.prism;
+    case TYPE_PYR: return CTX::instance()->color.mesh.pyramid;
     default: return CTX::instance()->color.mesh.vertex;
     }
   }
@@ -120,20 +126,20 @@ static bool isElementVisible(MElement *ele)
     bool hidden = false;
     for(int clip = 0; clip < 6; clip++){
       if(CTX::instance()->mesh.clip & (1 << clip)){
-	if(ele->getDim() < 3 && CTX::instance()->clipOnlyVolume){
-	}
-	else{
-	  double d = intersectClipPlane(clip, ele);
-	  if(ele->getDim() == 3 && 
+        if(ele->getDim() < 3 && CTX::instance()->clipOnlyVolume){
+        }
+        else{
+          double d = intersectClipPlane(clip, ele);
+          if(ele->getDim() == 3 && 
              CTX::instance()->clipOnlyDrawIntersectingVolume && d){
-	    hidden = true;
-	    break;
-	  }
-	  else if(d < 0){
-	    hidden = true;
-	    break;
-	  }
-	}
+            hidden = true;
+            break;
+          }
+          else if(d < 0){
+            hidden = true;
+            break;
+          }
+        }
       }
     }
     if(hidden) return false;
@@ -387,17 +393,17 @@ static void drawVoronoiDual(std::vector<T*> &elements)
     if(ele->getDim() == 2){
       for(int j = 0; j < ele->getNumEdges(); j++){
         MEdge e = ele->getEdge(j);
-	SVector3 p2p1(e.getVertex(1)->x() - e.getVertex(0)->x(),
-		      e.getVertex(1)->y() - e.getVertex(0)->y(),
-		      e.getVertex(1)->z() - e.getVertex(0)->z());
-	SVector3 pcp1(pc.x() - e.getVertex(0)->x(),
-		      pc.y() - e.getVertex(0)->y(),
-		      pc.z() - e.getVertex(0)->z());
-	double alpha = dot(pcp1,p2p1) / dot(p2p1,p2p1);
+        SVector3 p2p1(e.getVertex(1)->x() - e.getVertex(0)->x(),
+                      e.getVertex(1)->y() - e.getVertex(0)->y(),
+                      e.getVertex(1)->z() - e.getVertex(0)->z());
+        SVector3 pcp1(pc.x() - e.getVertex(0)->x(),
+                      pc.y() - e.getVertex(0)->y(),
+                      pc.z() - e.getVertex(0)->z());
+        double alpha = dot(pcp1,p2p1) / dot(p2p1,p2p1);
         SPoint3 p((1 - alpha)*e.getVertex(0)->x() + alpha * e.getVertex(1)->x(), 
-		  (1 - alpha)*e.getVertex(0)->y() + alpha * e.getVertex(1)->y(),
-		  (1 - alpha)*e.getVertex(0)->z() + alpha * e.getVertex(1)->z());
-	glVertex3d(pc.x(), pc.y(), pc.z());
+                  (1 - alpha)*e.getVertex(0)->y() + alpha * e.getVertex(1)->y(),
+                  (1 - alpha)*e.getVertex(0)->z() + alpha * e.getVertex(1)->z());
+        glVertex3d(pc.x(), pc.y(), pc.z());
         glVertex3d(p.x(), p.y(), p.z());
       }
     }
@@ -677,6 +683,7 @@ class initSmoothNormalsGFace {
   {
     addSmoothNormals(f, f->triangles);
     addSmoothNormals(f, f->quadrangles);
+    addSmoothNormals(f, f->polygons);
   }
 };
 
@@ -687,7 +694,8 @@ class initMeshGFace {
   {
     int num = 0;
     if(CTX::instance()->mesh.surfacesEdges){
-      num += (3 * f->triangles.size() + 4 * f->quadrangles.size()) / 2;
+      num += (3 * f->triangles.size() + 4 * f->quadrangles.size() + 
+              4 * f->polygons.size()) / 2;
       if(CTX::instance()->mesh.explode != 1.) num *= 2;
       if(_curved) num *= 2;
     }
@@ -697,7 +705,8 @@ class initMeshGFace {
   {
     int num = 0;
     if(CTX::instance()->mesh.surfacesFaces){
-      num += (f->triangles.size() + 2 * f->quadrangles.size());
+      num += (f->triangles.size() + 2 * f->quadrangles.size() +
+              2 * f->polygons.size());
       if(_curved) num *= 4;
     }
     return num + 100;
@@ -721,6 +730,7 @@ class initMeshGFace {
       f->va_triangles = new VertexArray(3, _estimateNumTriangles(f));
       if(CTX::instance()->mesh.triangles) addElementsInArrays(f, f->triangles, edg, fac);
       if(CTX::instance()->mesh.quadrangles) addElementsInArrays(f, f->quadrangles, edg, fac);
+      addElementsInArrays(f, f->polygons, edg, fac);
       f->va_lines->finalize();
       f->va_triangles->finalize();
     }
@@ -755,6 +765,8 @@ class drawMeshGFace {
       if(CTX::instance()->mesh.quadrangles)
         drawElementLabels(_ctx, f, f->quadrangles, CTX::instance()->mesh.surfacesFaces, 
                           CTX::instance()->color.mesh.line);
+      drawElementLabels(_ctx, f, f->polygons, CTX::instance()->mesh.surfacesFaces, 
+                        CTX::instance()->color.mesh.line);
     }
 
     if(CTX::instance()->mesh.points || CTX::instance()->mesh.pointsNum){
@@ -765,17 +777,20 @@ class drawMeshGFace {
           drawVerticesPerElement(_ctx, f, f->triangles);
         if(CTX::instance()->mesh.quadrangles) 
           drawVerticesPerElement(_ctx, f, f->quadrangles);
+        drawVerticesPerElement(_ctx, f, f->polygons);
       }
     }
 
     if(CTX::instance()->mesh.normals) {
       if(CTX::instance()->mesh.triangles) drawNormals(_ctx, f->triangles);
       if(CTX::instance()->mesh.quadrangles) drawNormals(_ctx, f->quadrangles);
+      drawNormals(_ctx, f->polygons);
     }
 
     if(CTX::instance()->mesh.dual) {
       if(CTX::instance()->mesh.triangles) drawBarycentricDual(f->triangles);
       if(CTX::instance()->mesh.quadrangles) drawBarycentricDual(f->quadrangles);
+      drawBarycentricDual(f->polygons);
     }
     else if(CTX::instance()->mesh.voronoi) {
       if(CTX::instance()->mesh.triangles) drawVoronoiDual(f->triangles);
@@ -798,8 +813,8 @@ class initMeshGRegion {
     if(CTX::instance()->clipWholeElements && 
        CTX::instance()->clipOnlyDrawIntersectingVolume){
       for(int clip = 0; clip < 6; clip++){
-	if(CTX::instance()->mesh.clip & (1 << clip))
-	  return (int)sqrt((double)num);
+        if(CTX::instance()->mesh.clip & (1 << clip))
+          return (int)sqrt((double)num);
       }
     }
     return num;
@@ -809,8 +824,11 @@ class initMeshGRegion {
     int num = 0;
     if(CTX::instance()->mesh.volumesEdges){
       // suppose edge shared by 4 elements on averge (pessmistic)
+      int numLP = 0;
+      for(unsigned int i = 0; i < r->polyhedra.size(); i++)
+        numLP += 2 * r->polyhedra[i]->getNumEdges();
       num += (12 * r->tetrahedra.size() + 24 * r->hexahedra.size() +
-              18 * r->prisms.size() + 16 * r->pyramids.size()) / 4;
+              18 * r->prisms.size() + 16 * r->pyramids.size() + numLP) / 4;
       num = _estimateIfClipped(num);
       if(CTX::instance()->mesh.explode != 1.) num *= 4;
       if(_curved) num *= 2;
@@ -821,8 +839,11 @@ class initMeshGRegion {
   {
     int num = 0;
     if(CTX::instance()->mesh.volumesFaces){
+      int numFP = 0;
+      for(unsigned int i = 0; i < r->polyhedra.size(); i++)
+        numFP += r->polyhedra[i]->getNumFaces();
       num += (4 * r->tetrahedra.size() + 12 * r->hexahedra.size() +
-              8 * r->prisms.size() + 6 * r->pyramids.size()) / 2;
+              8 * r->prisms.size() + 6 * r->pyramids.size() + numFP) / 2;
       num = _estimateIfClipped(num);
       if(CTX::instance()->mesh.explode != 1.) num *= 2;
       if(_curved) num *= 4;
@@ -854,6 +875,7 @@ class initMeshGRegion {
       if(CTX::instance()->mesh.hexahedra) addElementsInArrays(r, r->hexahedra, edg, fac);
       if(CTX::instance()->mesh.prisms) addElementsInArrays(r, r->prisms, edg, fac);
       if(CTX::instance()->mesh.pyramids) addElementsInArrays(r, r->pyramids, edg, fac);
+      addElementsInArrays(r, r->polyhedra, edg, fac);
       r->va_lines->finalize();
       r->va_triangles->finalize();
     }
@@ -898,6 +920,9 @@ class drawMeshGRegion {
         drawElementLabels(_ctx, r, r->pyramids, CTX::instance()->mesh.volumesFaces ||
                           CTX::instance()->mesh.surfacesFaces,
                           CTX::instance()->color.mesh.line);
+      drawElementLabels(_ctx, r, r->polyhedra, CTX::instance()->mesh.volumesFaces ||
+                        CTX::instance()->mesh.surfacesFaces,
+                        CTX::instance()->color.mesh.line);
     }
 
     if(CTX::instance()->mesh.points || CTX::instance()->mesh.pointsNum){
@@ -908,6 +933,7 @@ class drawMeshGRegion {
         if(CTX::instance()->mesh.hexahedra) drawVerticesPerElement(_ctx, r, r->hexahedra);
         if(CTX::instance()->mesh.prisms) drawVerticesPerElement(_ctx, r, r->prisms);
         if(CTX::instance()->mesh.pyramids) drawVerticesPerElement(_ctx, r, r->pyramids);
+        drawVerticesPerElement(_ctx, r, r->polyhedra);
       }
     }
 
@@ -916,6 +942,7 @@ class drawMeshGRegion {
       if(CTX::instance()->mesh.hexahedra) drawBarycentricDual(r->hexahedra);
       if(CTX::instance()->mesh.prisms) drawBarycentricDual(r->prisms);
       if(CTX::instance()->mesh.pyramids) drawBarycentricDual(r->pyramids);
+      drawBarycentricDual(r->polyhedra);
     }
 
     if(select) {
@@ -925,6 +952,24 @@ class drawMeshGRegion {
   }
 };
 
+static void beginFakeTransparency()
+{
+  return;
+  CTX::instance()->color.mesh.triangle = CTX::instance()->packColor(255, 0, 0, 128);
+  // simple additive blending "a la xpost":
+  glBlendFunc(GL_SRC_ALPHA, GL_ONE); // glBlendEquation(GL_FUNC_ADD);
+  // maximum intensity projection "a la volsuite":
+  // glBlendFunc(GL_ONE, GL_ONE); // glBlendEquation(GL_MAX);
+  glEnable(GL_BLEND);
+  glDisable(GL_DEPTH_TEST);
+}
+
+static void endFakeTransparency()
+{
+  return;
+  glDisable(GL_BLEND);
+  glEnable(GL_DEPTH_TEST);
+}
 
 // Main drawing routine
 
@@ -942,13 +987,13 @@ void drawContext::drawMesh()
         PView::list[j]->setChanged(true);
   }
 
-  glPointSize(CTX::instance()->mesh.pointSize);
-  gl2psPointSize(CTX::instance()->mesh.pointSize * 
-                 CTX::instance()->print.epsPointSizeFactor);
+  glPointSize((float)CTX::instance()->mesh.pointSize);
+  gl2psPointSize((float)(CTX::instance()->mesh.pointSize * 
+                         CTX::instance()->print.epsPointSizeFactor));
 
-  glLineWidth(CTX::instance()->mesh.lineWidth);
-  gl2psLineWidth(CTX::instance()->mesh.lineWidth *
-                 CTX::instance()->print.epsLineWidthFactor);
+  glLineWidth((float)CTX::instance()->mesh.lineWidth);
+  gl2psLineWidth((float)(CTX::instance()->mesh.lineWidth *
+                         CTX::instance()->print.epsLineWidthFactor));
   
   if(CTX::instance()->mesh.lightTwoSide)
     glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
@@ -958,9 +1003,9 @@ void drawContext::drawMesh()
   if(!CTX::instance()->clipWholeElements){
     for(int i = 0; i < 6; i++)
       if(CTX::instance()->mesh.clip & (1 << i)) 
-	glEnable((GLenum)(GL_CLIP_PLANE0 + i));
+        glEnable((GLenum)(GL_CLIP_PLANE0 + i));
       else
-	glDisable((GLenum)(GL_CLIP_PLANE0 + i));
+        glDisable((GLenum)(GL_CLIP_PLANE0 + i));
   }
 
   static bool busy = false;
@@ -991,8 +1036,11 @@ void drawContext::drawMesh()
             std::for_each(m->firstVertex(), m->lastVertex(), drawMeshGVertex(this));
           if(status >= 1)
             std::for_each(m->firstEdge(), m->lastEdge(), drawMeshGEdge(this));
-          if(status >= 2)
+          if(status >= 2){
+            beginFakeTransparency();
             std::for_each(m->firstFace(), m->lastFace(), drawMeshGFace(this));
+            endFakeTransparency();
+          }
           if(status >= 3)
             std::for_each(m->firstRegion(), m->lastRegion(), drawMeshGRegion(this));
         }
