@@ -6,7 +6,7 @@
 #include <math.h>
 #include <stdio.h>
 #include "GmshMessage.h"
-#include "GmshPredicates.h"
+#include "robustPredicates.h"
 #include "Numeric.h"
 #include "BDS.h"
 #include "GFace.h"
@@ -25,51 +25,51 @@ void outputScalarField(std::list<BDS_Face*> t, const char *iii, int param, GFace
     if (!(*tit)->deleted){
       (*tit)->getNodes(pts);
       if(param)
-	fprintf(f, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g){%d,%d,%d};\n",
-		pts[0]->u, pts[0]->v, 0.0,
-		pts[1]->u, pts[1]->v, 0.0,
-		pts[2]->u, pts[2]->v, 0.0,
-		pts[0]->iD, pts[1]->iD, pts[2]->iD);
+        fprintf(f, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g){%d,%d,%d};\n",
+                pts[0]->u, pts[0]->v, 0.0,
+                pts[1]->u, pts[1]->v, 0.0,
+                pts[2]->u, pts[2]->v, 0.0,
+                pts[0]->iD, pts[1]->iD, pts[2]->iD);
       else{
-	if(!gf){
-	  if (pts[3]){
-	    fprintf(f, "SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){%d,%d,%d,%d};\n",
-		    pts[0]->X, pts[0]->Y, pts[0]->Z,
-		    pts[1]->X, pts[1]->Y, pts[1]->Z,
-		    pts[2]->X, pts[2]->Y, pts[2]->Z, 
-		    pts[3]->X, pts[3]->Y, pts[3]->Z, 
-		    pts[0]->iD, pts[1]->iD, pts[2]->iD, pts[3]->iD);
-	  }
-	  else{
-	    fprintf(f, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g){%d,%d,%d};\n",
-		    pts[0]->X, pts[0]->Y, pts[0]->Z,
-		    pts[1]->X, pts[1]->Y, pts[1]->Z,
-		    pts[2]->X, pts[2]->Y, pts[2]->Z, 
-		    pts[0]->iD, pts[1]->iD, pts[2]->iD);
-	  }
-	}
-	else{
-	  if (pts[3]){
-	    fprintf(f, "SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){%g,%g,%g,%g};\n",
-		    pts[0]->X, pts[0]->Y, pts[0]->Z,
-		    pts[1]->X, pts[1]->Y, pts[1]->Z,
-		    pts[2]->X, pts[2]->Y, pts[2]->Z,
-		    pts[3]->X, pts[3]->Y, pts[3]->Z,
-		    gf->curvature(SPoint2(pts[0]->u, pts[0]->v)),
-		    gf->curvature(SPoint2(pts[1]->u, pts[1]->v)),	
-		    gf->curvature(SPoint2(pts[2]->u, pts[2]->v)),
-		    gf->curvature(SPoint2(pts[3]->u, pts[3]->v)));
-	  }
-	  else{
-	    fprintf(f, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g){%g,%g,%g};\n",
-		    pts[0]->X, pts[0]->Y, pts[0]->Z,
-		    pts[1]->X, pts[1]->Y, pts[1]->Z,
-		    pts[2]->X, pts[2]->Y, pts[2]->Z,
-		    gf->curvature(SPoint2(pts[0]->u, pts[0]->v)),
-		    gf->curvature(SPoint2(pts[1]->u, pts[1]->v)),
-		    gf->curvature(SPoint2(pts[2]->u, pts[2]->v)));
-	  }
-	}
+        if(!gf){
+          if (pts[3]){
+            fprintf(f, "SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){%d,%d,%d,%d};\n",
+                    pts[0]->X, pts[0]->Y, pts[0]->Z,
+                    pts[1]->X, pts[1]->Y, pts[1]->Z,
+                    pts[2]->X, pts[2]->Y, pts[2]->Z, 
+                    pts[3]->X, pts[3]->Y, pts[3]->Z, 
+                    pts[0]->iD, pts[1]->iD, pts[2]->iD, pts[3]->iD);
+          }
+          else{
+            fprintf(f, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g){%d,%d,%d};\n",
+                    pts[0]->X, pts[0]->Y, pts[0]->Z,
+                    pts[1]->X, pts[1]->Y, pts[1]->Z,
+                    pts[2]->X, pts[2]->Y, pts[2]->Z, 
+                    pts[0]->iD, pts[1]->iD, pts[2]->iD);
+          }
+        }
+        else{
+          if (pts[3]){
+            fprintf(f, "SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){%g,%g,%g,%g};\n",
+                    pts[0]->X, pts[0]->Y, pts[0]->Z,
+                    pts[1]->X, pts[1]->Y, pts[1]->Z,
+                    pts[2]->X, pts[2]->Y, pts[2]->Z,
+                    pts[3]->X, pts[3]->Y, pts[3]->Z,
+                    gf->curvatureDiv(SPoint2(pts[0]->u, pts[0]->v)),
+                    gf->curvatureDiv(SPoint2(pts[1]->u, pts[1]->v)),    
+                    gf->curvatureDiv(SPoint2(pts[2]->u, pts[2]->v)),
+                    gf->curvatureDiv(SPoint2(pts[3]->u, pts[3]->v)));
+          }
+          else{
+            fprintf(f, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g){%g,%g,%g};\n",
+                    pts[0]->X, pts[0]->Y, pts[0]->Z,
+                    pts[1]->X, pts[1]->Y, pts[1]->Z,
+                    pts[2]->X, pts[2]->Y, pts[2]->Z,
+                    gf->curvatureDiv(SPoint2(pts[0]->u, pts[0]->v)),
+                    gf->curvatureDiv(SPoint2(pts[1]->u, pts[1]->v)),
+                    gf->curvatureDiv(SPoint2(pts[2]->u, pts[2]->v)));
+          }
+        }
       }
     }
     ++tit;
@@ -79,7 +79,7 @@ void outputScalarField(std::list<BDS_Face*> t, const char *iii, int param, GFace
 }
 
 BDS_Vector::BDS_Vector(const BDS_Point &p2, const BDS_Point &p1)
-  :x(p2.X - p1.X), y(p2.Y - p1.Y), z(p2.Z - p1.Z)
+  : x(p2.X - p1.X), y(p2.Y - p1.Y), z(p2.Z - p1.Z)
 {
 }
 
@@ -170,7 +170,7 @@ BDS_Point *BDS_Mesh::add_point(int num, double u, double v, GFace *gf)
 BDS_Point *BDS_Mesh::find_point(int p)
 {
   BDS_Point P(p);
-  std::set < BDS_Point *, PointLessThan >::iterator it = points.find(&P);
+  std::set<BDS_Point *, PointLessThan>::iterator it = points.find(&P);
   if(it != points.end())
     return (BDS_Point*)(*it);
   else
@@ -210,10 +210,10 @@ int Intersect_Edges_2d(double x1, double y1, double x2, double y2,
 //   double q1[2] = {x3,y3};
 //   double q2[2] = {x4,y4};
   
-//   double rp1 = gmsh::orient2d(p1, p2, q1);
-//   double rp2 = gmsh::orient2d(p1, p2, q2);
-//   double rq1 = gmsh::orient2d(q1, q2, p1);
-//   double rq2 = gmsh::orient2d(q1, q2, p2);
+//   double rp1 = robustPredicates::orient2d(p1, p2, q1);
+//   double rp2 = robustPredicates::orient2d(p1, p2, q2);
+//   double rq1 = robustPredicates::orient2d(q1, q2, p1);
+//   double rq2 = robustPredicates::orient2d(q1, q2, p2);
 
 //   if(rp1*rp2<=0 && rq1*rq2<=0){
 // //      printf("p1 %22.15E %22.15E %22.15E %22.15E \n",x1,y1,x2,y2);
@@ -250,13 +250,13 @@ BDS_Edge *BDS_Mesh::recover_edge_fast(BDS_Point *p1, BDS_Point *p2){
       BDS_Edge *e= t->oppositeEdge (p1);
       BDS_Face *f= e->otherFace (t);
       if (!f->e4){
-	BDS_Point *p2b = f->oppositeVertex(e);
-	if (p2 == p2b){
-	  if (swap_edge(e, BDS_SwapEdgeTestQuality(false,false))){
-	    printf("coucou\n");
-	    return find_edge (p1,p2->iD);
-	  }
-	}
+        BDS_Point *p2b = f->oppositeVertex(e);
+        if (p2 == p2b){
+          if (swap_edge(e, BDS_SwapEdgeTestQuality(false,false))){
+            printf("coucou\n");
+            return find_edge (p1,p2->iD);
+          }
+        }
       }
     }
     ++it;
@@ -282,7 +282,6 @@ BDS_Edge *BDS_Mesh::recover_edge(int num1, int num2, std::set<EdgeToRecover> *e2
   Msg::Debug("edge %d %d has to be recovered", num1, num2);
   
   int ix = 0;
-  int ixMax = 300;
   double x[2];
   while(1){
     std::vector<BDS_Edge*> intersected;
@@ -293,31 +292,31 @@ BDS_Edge *BDS_Mesh::recover_edge(int num1, int num2, std::set<EdgeToRecover> *e2
     while(it != edges.end()){
       e = (*it);
       if(!e->deleted && e->p1 != p1 && e->p1 != p2 && e->p2 != p1 && e->p2 != p2)
-	if(Intersect_Edges_2d(e->p1->u, e->p1->v,
-			      e->p2->u, e->p2->v,
-			      p1->u, p1->v,
-			      p2->u, p2->v,x)){
-	  // intersect
-	  if(e2r && e2r->find(EdgeToRecover(e->p1->iD, e->p2->iD, 0)) != e2r->end()){
-	    std::set<EdgeToRecover>::iterator itr1 = 
-	      e2r->find(EdgeToRecover(e->p1->iD, e->p2->iD, 0));		    
-	    std::set<EdgeToRecover>::iterator itr2 = 
-	      e2r->find(EdgeToRecover(num1, num2, 0));
-	    Msg::Debug("edge %d %d on model edge %d cannot be recovered because"
-		" it intersects %d %d on model edge %d", num1, num2, itr2->ge->tag(),
-		e->p1->iD, e->p2->iD, itr1->ge->tag());
-	    // now throw a class that contains the diagnostic
-	    not_recovered->insert(EdgeToRecover(num1, num2, itr2->ge));
-	    not_recovered->insert(EdgeToRecover(e->p1->iD, e->p2->iD, itr1->ge));
-	    selfIntersection = true;
-	  }
-	  if (e->numfaces() != e->numTriangles())return 0;
-	  intersected.push_back(e);	  
-	}
+        if(Intersect_Edges_2d(e->p1->u, e->p1->v,
+                              e->p2->u, e->p2->v,
+                              p1->u, p1->v,
+                              p2->u, p2->v,x)){
+          // intersect
+          if(e2r && e2r->find(EdgeToRecover(e->p1->iD, e->p2->iD, 0)) != e2r->end()){
+            std::set<EdgeToRecover>::iterator itr1 = 
+              e2r->find(EdgeToRecover(e->p1->iD, e->p2->iD, 0));                    
+            std::set<EdgeToRecover>::iterator itr2 = 
+              e2r->find(EdgeToRecover(num1, num2, 0));
+            Msg::Debug("edge %d %d on model edge %d cannot be recovered because"
+                       " it intersects %d %d on model edge %d", num1, num2, itr2->ge->tag(),
+                       e->p1->iD, e->p2->iD, itr1->ge->tag());
+            // now throw a class that contains the diagnostic
+            not_recovered->insert(EdgeToRecover(num1, num2, itr2->ge));
+            not_recovered->insert(EdgeToRecover(e->p1->iD, e->p2->iD, itr1->ge));
+            selfIntersection = true;
+          }
+          if (e->numfaces() != e->numTriangles()) return 0;
+          intersected.push_back(e);       
+        }
       ++it;
     }
 
-    if (selfIntersection)return 0;
+    if (selfIntersection) return 0;
 
 //   if(ix > 300){
 //     Msg::Warning("edge %d %d cannot be recovered after %d iterations, trying again",
@@ -329,17 +328,18 @@ BDS_Edge *BDS_Mesh::recover_edge(int num1, int num2, std::set<EdgeToRecover> *e2
     if(!intersected.size() || ix > 1000){
       BDS_Edge *eee = find_edge(num1, num2);
       if(!eee){
-	outputScalarField(triangles, "debugp.pos", 1);
-	outputScalarField(triangles, "debugr.pos", 0);
-	Msg::Debug("edge %d %d cannot be recovered at all, look at debugp.pos "
-	    "and debugr.pos", num1, num2);
-	return 0;
+        outputScalarField(triangles, "debugp.pos", 1);
+        outputScalarField(triangles, "debugr.pos", 0);
+        Msg::Debug("edge %d %d cannot be recovered at all, look at debugp.pos "
+                   "and debugr.pos", num1, num2);
+        return 0;
       }
       return eee;
     }
     
     int ichoice = ix++ % intersected.size();
-    bool success = swap_edge(intersected[ichoice], BDS_SwapEdgeTestQuality(false, false));
+    //bool success = 
+      swap_edge(intersected[ichoice], BDS_SwapEdgeTestQuality(false, false));
     // printf("trying to swop %d %d = %d (%d %d)\n", intersected[ichoice]->p1->iD,
     //        intersected[ichoice]->p2->iD, success, intersected[ichoice]->deleted,
     //        intersected[ichoice]->numfaces());
@@ -503,7 +503,7 @@ void BDS_Edge::oppositeof(BDS_Point *oface[2]) const
 BDS_GeomEntity *BDS_Mesh::get_geom(int p1, int p2)
 {
   BDS_GeomEntity ge(p1, p2);
-  std::set < BDS_GeomEntity *, GeomLessThan >::iterator it = geom.find(&ge);
+  std::set<BDS_GeomEntity *, GeomLessThan >::iterator it = geom.find(&ge);
   if(it == geom.end()) return 0;
   return (BDS_GeomEntity*)(*it);
 }
@@ -528,7 +528,7 @@ void recur_tag(BDS_Face *t, BDS_GeomEntity *g)
 double PointLessThanLexicographic::t = 0;
 double BDS_Vector::t = 0;
 
-template < class IT > void DESTROOOY(IT beg, IT end)
+template <class IT> void DESTROOOY(IT beg, IT end)
 {
   while(beg != end) {
     delete *beg;
@@ -564,11 +564,11 @@ void BDS_Mesh::cleanup()
 
 BDS_Mesh::~BDS_Mesh()
 {
-   DESTROOOY(geom.begin(), geom.end());
-   DESTROOOY(points.begin(), points.end());
-   cleanup();
-   DESTROOOY(edges.begin(), edges.end());
-   DESTROOOY(triangles.begin(), triangles.end());
+  DESTROOOY(geom.begin(), geom.end());
+  DESTROOOY(points.begin(), points.end());
+  cleanup();
+  DESTROOOY(edges.begin(), edges.end());
+  DESTROOOY(triangles.begin(), triangles.end());
 }
 
 bool BDS_Mesh::split_face(BDS_Face *f, BDS_Point *mid)
@@ -731,8 +731,8 @@ bool BDS_SwapEdgeTestQuality::operator () (BDS_Point *_p1, BDS_Point *_p2,
     double op1[2] = {_q1->u,_q1->v};
     double op2[2] = {_q2->u,_q2->v};
     
-    double ori_t1 = gmsh::orient2d(op1, p1, op2);
-    double ori_t2 = gmsh::orient2d(op1, op2, p2);
+    double ori_t1 = robustPredicates::orient2d(op1, p1, op2);
+    double ori_t2 = robustPredicates::orient2d(op1, op2, p2);
     
     // printf("%d %d %d %d %g %g\n",_p1->iD,_p2->iD,_q1->iD,_q2->iD,ori_t1,ori_t2);
     return (ori_t1 * ori_t2 > 0); // the quadrangle was strictly convex !
@@ -978,8 +978,8 @@ static bool test_move_point_parametric_quad(BDS_Point *p, double u, double v, BD
   double pc[2] = {pts[2]->u, pts[2]->v};
   double pd[2] = {pts[3]->u, pts[3]->v};
 
-  double ori_init1 = gmsh::orient2d(pa, pb, pc);
-  double ori_init2 = gmsh::orient2d(pc, pd, pa);
+  double ori_init1 = robustPredicates::orient2d(pa, pb, pc);
+  double ori_init2 = robustPredicates::orient2d(pc, pd, pa);
 
   if(p == pts[0]){ 
     pa[0] = u; 
@@ -1002,8 +1002,8 @@ static bool test_move_point_parametric_quad(BDS_Point *p, double u, double v, BD
     return false;
   }
   
-  double ori_final1 = gmsh::orient2d(pa, pb, pc);
-  double ori_final2 = gmsh::orient2d(pc, pd, pa);
+  double ori_final1 = robustPredicates::orient2d(pa, pb, pc);
+  double ori_final2 = robustPredicates::orient2d(pc, pd, pa);
   // allow to move a point when a triangle was flat
   return (ori_init1 * ori_final1 > 0) && (ori_init2 * ori_final2 > 0);
 }
@@ -1026,7 +1026,7 @@ static bool test_move_point_parametric_triangle(BDS_Point *p, double u, double v
 
   if(area_init == 0.0) return true;
 
-  double ori_init = gmsh::orient2d(pa, pb, pc);
+  double ori_init = robustPredicates::orient2d(pa, pb, pc);
 
   if(p == pts[0]){ 
     pa[0] = u; 
@@ -1048,7 +1048,7 @@ static bool test_move_point_parametric_triangle(BDS_Point *p, double u, double v
   
   double area_final = fabs(a[0] * b[1] - a[1] * b[0]);
   if(area_final < 0.1 * area_init) return false;
-  double ori_final = gmsh::orient2d(pa, pb, pc);
+  double ori_final = robustPredicates::orient2d(pa, pb, pc);
   // allow to move a point when a triangle was flat
   return ori_init * ori_final > 0;
 }
@@ -1076,10 +1076,10 @@ bool BDS_Mesh::collapse_edge_parametric(BDS_Edge *e, BDS_Point *p)
   // printf("collapsing an edge :");
   // print_edge(e);
 
-  static BDS_Point *pt[3][1024];
-  static BDS_GeomEntity *gs[1024];
-  static int ept[2][1024];
-  static BDS_GeomEntity *egs[1024]; 
+  BDS_Point *pt[3][1024];
+  BDS_GeomEntity *gs[1024];
+  int ept[2][1024];
+  BDS_GeomEntity *egs[1024]; 
   int nt = 0;
   {
     p->getTriangles(t);
@@ -1177,8 +1177,8 @@ bool BDS_Mesh::smooth_point_centroid(BDS_Point *p, GFace *gf, bool test_quality)
 
   std::list<BDS_Face*> ts;
   p->getTriangles(ts);
-  std::list < BDS_Edge * >::iterator ited  = p->edges.begin();
-  std::list < BDS_Edge * >::iterator itede = p->edges.end();
+  std::list<BDS_Edge *>::iterator ited = p->edges.begin();
+  std::list<BDS_Edge *>::iterator itede = p->edges.end();
 
   double sTot = 0;
   while(ited != itede) {
@@ -1196,7 +1196,10 @@ bool BDS_Mesh::smooth_point_centroid(BDS_Point *p, GFace *gf, bool test_quality)
   LC /= (sTot);
 
   GPoint gp = gf->point(U * scalingU, V * scalingV);
-
+  
+  if (!gp.succeeded()){
+    return false;
+  }
   const double oldX = p->X;
   const double oldY = p->Y;
   const double oldZ = p->Z;
@@ -1307,6 +1310,8 @@ bool BDS_Mesh::smooth_point_parametric(BDS_Point *p, GFace *gf)
   }
 
   GPoint gp = gf->point(U * scalingU, V * scalingV);
+  if (!gp.succeeded())return false;
+
   p->u = U;
   p->v = V;
   p->lc() = LC;
