@@ -7,6 +7,7 @@
 //   Jonathan Lambrechts
 //
 
+#include <stdlib.h>
 #include <list>
 #include <math.h>
 #include <fstream>
@@ -14,20 +15,13 @@
 #include <string.h>
 #include <sstream>
 #include "GmshConfig.h"
-
-#if defined(HAVE_MATH_EVAL)
-#include "matheval.h"
-#endif
-#if defined(HAVE_ANN)
-#include "ANN/ANN.h"
-#endif
-
 #include "Context.h"
 #include "Field.h"
 #include "GeoInterpolation.h"
 #include "GModel.h"
 #include "GmshMessage.h"
 #include "Numeric.h"
+#include "mathEvaluator.h"
 
 #if !defined(HAVE_NO_POST)
 #include "OctreePost.h"
@@ -35,6 +29,10 @@
 #include "MVertex.h"
 #endif
 
+#if defined(HAVE_ANN)
+#include "ANN/ANN.h"
+#endif
+
 class FieldOptionDouble : public FieldOption
 {
  public:
@@ -462,30 +460,97 @@ class BoxField : public Field
     options["ZMin"] = new FieldOptionDouble
       (z_min, "Minimum Z coordinate of the box");
     options["ZMax"] = new FieldOptionDouble
-      (z_max, "Maximum Z coordinate of the box");
+      (z_max, "Maximum Z coordinate of the box");    
   }
   const char *getName()
   {
     return "Box";
   }
   double operator() (double x, double y, double z, GEntity *ge=0)
-  {
+  {    
     return (x <= x_max && x >= x_min && y <= y_max && y >= y_min && z <= z_max
             && z >= z_min) ? v_in : v_out;
   }
 };
 
+class CylinderField : public Field
+{
+  double v_in, v_out;
+  double xc,yc,zc;
+  double xa,ya,za;
+  double R;
+  
+ public:
+  std::string getDescription()
+  {
+    return "The value of this field is VIn inside a frustrated cylinder, VOut outside.\n"
+      "The cylinder is given by\n\n"
+      "  ||dX||^2 < R^2 &&\n"
+      "  (X-X0).A < ||A||^2\n"
+      "  dX = (X - X0) - ((X - X0).A)/(||A||^2) . A";
+  }
+  CylinderField()
+  {
+    v_in = v_out = xc = yc = zc = xa = ya = R = 0;
+    za = 1.;
+    
+    options["VIn"] = new FieldOptionDouble
+      (v_in, "Value inside the cylinder");
+    options["VOut"] = new FieldOptionDouble
+      (v_out, "Value outside the cylinder");
+
+    options["XCenter"] = new FieldOptionDouble
+      (xc, "X coordinate of the cylinder center");
+    options["YCenter"] = new FieldOptionDouble
+      (yc, "Y coordinate of the cylinder center");
+    options["ZCenter"] = new FieldOptionDouble
+      (zc, "Z coordinate of the cylinder center");
+
+    
+    options["XAxis"] = new FieldOptionDouble
+      (xa, "X component of the cylinder axis");
+    options["YAxis"] = new FieldOptionDouble
+      (ya, "Y component of the cylinder axis");
+    options["ZAxis"] = new FieldOptionDouble
+      (za, "Z component of the cylinder axis");
+
+    options["Radius"] = new FieldOptionDouble
+      (R,"Radius");    
+    
+  }
+  const char *getName()
+  {
+    return "Cylinder";
+  }
+  double operator() (double x, double y, double z, GEntity *ge=0)
+  {    
+    double dx = x-xc;
+    double dy = y-yc;
+    double dz = z-zc;
+    
+    double adx = (xa * dx + ya * dy + za * dz)/(xa*xa + ya*ya + za*za);
+    
+    dx -= adx * xa;
+    dy -= adx * ya;
+    dz -= adx * za;
+    
+    return ((dx*dx + dy*dy + dz*dz < R*R) && fabs(adx) < 1) ? v_in : v_out;
+    
+  }
+};
+
 class ThresholdField : public Field
 {
+ protected :
   int iField;
   double dmin, dmax, lcmin, lcmax;
   bool sigmoid, stopAtDistMax;
  public:
-  const char *getName()
+  virtual const char *getName()
   {
     return "Threshold";
   }
-  std::string getDescription()
+  virtual std::string getDescription()
   {
     return "F = LCMin if Field[IField] <= DistMin,\n"
       "F = LCMax if Field[IField] >= DistMax,\n"
@@ -538,6 +603,80 @@ class ThresholdField : public Field
   }
 };
 
+class BoundaryLayerField : public ThresholdField {
+public:
+  BoundaryLayerField() 
+  {  }
+  virtual bool isotropic () const {return false;}
+  virtual const char *getName()
+  {
+    return "BoundaryLayer";
+  }
+  virtual std::string getDescription()
+  {
+    return "F = LCMin if Field[IField] <= DistMin,\n"
+      "F = LCMax if Field[IField] >= DistMax,\n"
+      "F = interpolation between LcMin and LcMax if DistMin < Field[IField] < DistMax";
+  }
+  virtual void operator() (double x, double y, double z, SMetric3 &metr, GEntity *ge=0)
+  {
+    Field *field = GModel::current()->getFields()->get(iField);
+    if(!field) {
+      metr(0,0) = 1/(MAX_LC*MAX_LC);
+      metr(1,1) = 1/(MAX_LC*MAX_LC);
+      metr(2,2) = 1/(MAX_LC*MAX_LC);
+      metr(0,1) = metr(0,2) = metr(1,2) = 0;
+      return;
+    }
+    double dist = (*field) (x, y, z);
+
+    double r = (dist - dmin) / (dmax - dmin);
+    r = std::max(std::min(r, 1.), 0.);
+    double lc;
+    if(stopAtDistMax && r >= 1.){
+      lc = MAX_LC;
+    }
+    else if(sigmoid){
+      double s = exp(12. * r - 6.) / (1. + exp(12. * r - 6.));
+      lc = lcmin * (1. - s) + lcmax * s;
+    }
+    else{ // linear
+      lc = lcmin * (1 - r) + lcmax * r;
+    }
+    
+    double delta = std::min(CTX::instance()->lc / 1e4, dist);
+    double gx =
+      ((*field) (x + delta / 2, y, z) -
+       (*field) (x - delta / 2, y, z)) / delta;
+    double gy =
+      ((*field) (x, y + delta / 2, z) -
+       (*field) (x, y - delta / 2, z)) / delta;
+    double gz =
+      ((*field) (x, y, z + delta / 2) -
+       (*field) (x, y, z - delta / 2)) / delta;
+
+    SVector3 g(gx,gy,gz);
+    g.normalize();
+    SVector3 t1,t2;
+
+    if (fabs(gx) < fabs(gy) && fabs(gx) < fabs(gz))
+      t1 = SVector3(1,0,0);
+    else if (fabs(gy) < fabs(gx) && fabs(gy) < fabs(gz))
+      t1 = SVector3(0,1,0);
+    else
+      t1 = SVector3(0,0,1);
+    
+    t2 = crossprod(g,t1);
+    t2.normalize();
+    t1 = crossprod(t2,g);
+    
+    metr  = SMetric3(1./(lc*lc), 
+                     1/(lcmax*lcmax),
+                     1/(lcmax*lcmax),
+                     g,t1,t2);
+  }
+};
+
 class GradientField : public Field
 {
   int iField, kind;
@@ -594,12 +733,13 @@ class GradientField : public Field
       return sqrt(gx * gx + gy * gy + gz * gz);
     default:
       Msg::Error("Field %i : Unknown kind (%i) of gradient.", this->id,
-		 kind);
+                 kind);
       return MAX_LC;
     }
   }
 };
 
+
 class CurvatureField : public Field
 {
   int iField;
@@ -645,7 +785,7 @@ class CurvatureField : public Field
     grad_norm(*field, x, y, z + delta / 2, grad[4]);
     grad_norm(*field, x, y, z - delta / 2, grad[5]);
     return (grad[0][0] - grad[1][0] + grad[2][1] - 
-	    grad[3][1] + grad[4][2] - grad[5][2]) / delta;
+            grad[3][1] + grad[4][2] - grad[5][2]) / delta;
   }
 };
 
@@ -730,9 +870,9 @@ class LaplacianField : public Field
     Field *field = GModel::current()->getFields()->get(iField);
     if(!field) return MAX_LC;
     return ((*field) (x + delta , y, z)+ (*field) (x - delta , y, z)
-	    +(*field) (x, y + delta , z)+ (*field) (x, y - delta , z)
-	    +(*field) (x, y, z + delta )+ (*field) (x, y, z - delta )
-	    -6* (*field) (x , y, z)) / (delta*delta);
+            +(*field) (x, y + delta , z)+ (*field) (x, y - delta , z)
+            +(*field) (x, y, z + delta )+ (*field) (x, y, z - delta )
+            -6* (*field) (x , y, z)) / (delta*delta);
   }
 };
 
@@ -766,101 +906,73 @@ class MeanField : public Field
     Field *field = GModel::current()->getFields()->get(iField);
     if(!field) return MAX_LC;
     return ((*field) (x + delta , y, z) + (*field) (x - delta, y, z)
-	    + (*field) (x, y + delta, z) + (*field) (x, y - delta, z)
-	    + (*field) (x, y, z + delta) + (*field) (x, y, z - delta)
-	    + (*field) (x, y, z)) / 7;
+            + (*field) (x, y + delta, z) + (*field) (x, y - delta, z)
+            + (*field) (x, y, z + delta) + (*field) (x, y, z - delta)
+            + (*field) (x, y, z)) / 7;
   }
 };
 
-#if defined(HAVE_MATH_EVAL)
 class MathEvalExpression
 {
-  bool error_status;
-  std::list<Field*> *list;
-  int nvalues;
-  char **names;
-  double *values;
-  void *eval;
-  int *evaluators_id;
-  std::string function;
-  char *c_str_function;
+ private:
+  mathEvaluator *_f;
+  std::set<int> _fields;
  public:
-  double evaluate(double x, double y, double z)
+  MathEvalExpression() : _f(0) {}
+  ~MathEvalExpression(){ if(_f) delete _f; }
+  bool set_function(const std::string &f)
   {
-    if(error_status)
-      return MAX_LC;
-    for(int i = 0; i < nvalues; i++){
-      switch (evaluators_id[i]) {
-      case -1:
-        values[i] = x;
-        break;
-      case -2: 
-	values[i] = y;
-        break;
-      case -3: 
-	values[i] = z;
-        break;
-      default:
-        {
-          Field *f = GModel::current()->getFields()->get(evaluators_id[i]);
-          values[i] = f ? (*f) (x, y, z) : MAX_LC;
+    // get id numbers of fields appearing in the function
+    _fields.clear();
+    unsigned int i = 0;
+    while(i < f.size()){
+      unsigned int j = 0;
+      if(f[i] == 'F'){
+        std::string id("");
+        while(i + 1 + j < f.size() && f[i + 1 + j] >= '0' && f[i + 1 + j] <= '9'){
+          id += f[i + 1 + j];
+          j++;
         }
+        _fields.insert(atoi(id.c_str()));
       }
+      i += j + 1;
     }
-    return evaluator_evaluate(eval, nvalues, names, values);
-  }
-  MathEvalExpression()
-  {
-    eval = 0;
-    values = 0;
-    c_str_function = 0;
-    evaluators_id = 0;
-  }
-  bool set_function(const std::string & f)
-  {
-    free_members();
-    error_status = false;
-    c_str_function = strdup(f.c_str());
-    eval = evaluator_create(c_str_function);
-    if(!eval) {
-      error_status = true;
+    std::vector<std::string> expressions(1), variables(3 + _fields.size());
+    expressions[0] = f;
+    variables[0] = "x"; 
+    variables[1] = "y"; 
+    variables[2] = "z";
+    i = 3;
+    for(std::set<int>::iterator it = _fields.begin(); it != _fields.end(); it++){
+      std::ostringstream sstream;
+      sstream << "F" << *it;
+      variables[i++] = sstream.str();
+    }
+    if(_f) delete _f;
+    _f = new mathEvaluator(expressions, variables);
+    if(expressions.empty()) {
+      delete _f;
+      _f = 0;
       return false;
     }
-    evaluator_get_variables(eval, &names, &nvalues);
-    values = new double[nvalues];
-    evaluators_id = new int[nvalues];
-    for(int i = 0; i < nvalues; i++) {
-      int id;
-      if(!strcmp("x", names[i]))
-        evaluators_id[i] = -1;
-      else if(!strcmp("y", names[i]))
-        evaluators_id[i] = -2;
-      else if(!strcmp("z", names[i]))
-        evaluators_id[i] = -3;
-      else if(sscanf(names[i], "F%i", &id) == 1)
-        evaluators_id[i] = id;
-      else {
-        Msg::Error("Unknown matheval argument \"%s\"\n", names[i]);
-        error_status = true;
-        return false;
-      }
-    }
     return true;
   }
-  void free_members()
+  double evaluate(double x, double y, double z)
   {
-    if(c_str_function)
-      free(c_str_function);
-    if(eval)
-      evaluator_destroy(eval);
-    if(values)
-      delete[]values;
-    if(evaluators_id)
-      delete evaluators_id;
-  }
-  ~MathEvalExpression()
-  {
-    free_members();
+    if(!_f) return MAX_LC;
+    std::vector<double> values(3 + _fields.size()), res(1);
+    values[0] = x;
+    values[1] = y;
+    values[2] = z;
+    int i = 3;
+    for(std::set<int>::iterator it = _fields.begin(); it != _fields.end(); it++){
+      Field *field = GModel::current()->getFields()->get(*it);
+      values[i++] = field ? (*field)(x, y, z) : MAX_LC;
+    }
+    if(_f->eval(values, res)) 
+      return res[0];
+    else
+      return MAX_LC;
   }
 };
 
@@ -880,7 +992,7 @@ class MathEvalField : public Field
     if(update_needed) {
       if(!expr.set_function(f))
         Msg::Error("Field %i: Invalid matheval expression \"%s\"",
-		   this->id, f.c_str());
+                   this->id, f.c_str());
       update_needed = false;
     }
     return expr.evaluate(x, y, z);
@@ -928,22 +1040,21 @@ class ParametricField : public Field
       for(int i = 0; i < 3; i++) {
         if(!expr[i].set_function(f[i]))
           Msg::Error("Field %i : Invalid matheval expression \"%s\"",
-		     this->id, f[i].c_str());
+                     this->id, f[i].c_str());
       }
       update_needed = false;
     }
     Field *field = GModel::current()->getFields()->get(ifield);
     if(!field) return MAX_LC;
     return (*field)(expr[0].evaluate(x, y, z),
-		    expr[1].evaluate(x, y, z),
-		    expr[2].evaluate(x, y, z));
+                    expr[1].evaluate(x, y, z),
+                    expr[2].evaluate(x, y, z));
   }
   const char *getName()
   {
     return "Param";
   }
 };
-#endif
 
 #if !defined(HAVE_NO_POST)
 class PostViewField : public Field
@@ -1002,7 +1113,7 @@ class MinField : public Field
   MinField()
   {
     options["FieldsList"] = new FieldOptionList
-      (idlist, "Field indices",	&update_needed);
+      (idlist, "Field indices", &update_needed);
   }
   std::string getDescription()
   {
@@ -1142,7 +1253,7 @@ class AttractorField : public Field
         delete kdtree;
       }
       int totpoints = nodes_id.size() + n_nodes_by_edge * edges_id.size() + 
-	n_nodes_by_edge * n_nodes_by_edge * faces_id.size();
+        n_nodes_by_edge * n_nodes_by_edge * faces_id.size();
       if(totpoints)
         zeronodes = annAllocPts(totpoints, 4);
       int k = 0;
@@ -1198,33 +1309,33 @@ class AttractorField : public Field
         Surface *s = FindSurface(*it);
         if(s) {
           for(int i = 0; i < n_nodes_by_edge; i++) {
-	    for(int j = 0; j < n_nodes_by_edge; j++) {
-	      double u = (double)i / (n_nodes_by_edge - 1);
-	      double v = (double)j / (n_nodes_by_edge - 1);
-	      Vertex V = InterpolateSurface(s, u, v, 0, 0);
-	      zeronodes[k][0] = V.Pos.X;
-	      zeronodes[k][1] = V.Pos.Y;
-	      zeronodes[k++][2] = V.Pos.Z;
-	    }
-	  }
+            for(int j = 0; j < n_nodes_by_edge; j++) {
+              double u = (double)i / (n_nodes_by_edge - 1);
+              double v = (double)j / (n_nodes_by_edge - 1);
+              Vertex V = InterpolateSurface(s, u, v, 0, 0);
+              zeronodes[k][0] = V.Pos.X;
+              zeronodes[k][1] = V.Pos.Y;
+              zeronodes[k++][2] = V.Pos.Z;
+            }
+          }
         }
         else {
           GFace *f = GModel::current()->getFaceByTag(*it);
           if(f) {
             for(int i = 0; i < n_nodes_by_edge; i++) {
-	      for(int j = 0; j < n_nodes_by_edge; j++) {
-		double u = (double)i / (n_nodes_by_edge - 1);
-		double v = (double)j / (n_nodes_by_edge - 1);
-		Range<double> b1 = ge->parBounds(0);
-		Range<double> b2 = ge->parBounds(1);
-		double t1 = b1.low() + u * (b1.high() - b1.low());
-		double t2 = b2.low() + v * (b2.high() - b2.low());
-		GPoint gp = f->point(t1, t2);
-		zeronodes[k][0] = gp.x();
-		zeronodes[k][1] = gp.y();
-		zeronodes[k++][2] = gp.z();
-	      }
-	    }
+              for(int j = 0; j < n_nodes_by_edge; j++) {
+                double u = (double)i / (n_nodes_by_edge - 1);
+                double v = (double)j / (n_nodes_by_edge - 1);
+                Range<double> b1 = ge->parBounds(0);
+                Range<double> b2 = ge->parBounds(1);
+                double t1 = b1.low() + u * (b1.high() - b1.low());
+                double t2 = b2.low() + v * (b2.high() - b2.low());
+                GPoint gp = f->point(t1, t2);
+                zeronodes[k][0] = gp.x();
+                zeronodes[k][1] = gp.y();
+                zeronodes[k++][2] = gp.z();
+              }
+            }
           }
         }
       }
@@ -1236,6 +1347,7 @@ class AttractorField : public Field
     return sqrt(dist[0]);
   }
 };
+
 #endif
 
 template<class F> class FieldFactoryT : public FieldFactory {
@@ -1252,7 +1364,9 @@ FieldManager::FieldManager()
 {
   map_type_name["Structured"] = new FieldFactoryT<StructuredField>();
   map_type_name["Threshold"] = new FieldFactoryT<ThresholdField>();
+  map_type_name["BoundaryLayer"] = new FieldFactoryT<BoundaryLayerField>();
   map_type_name["Box"] = new FieldFactoryT<BoxField>();
+  map_type_name["Cylinder"] = new FieldFactoryT<CylinderField>();
   map_type_name["LonLat"] = new FieldFactoryT<LonLatField>();
 #if !defined(HAVE_NO_POST)
   map_type_name["PostView"] = new FieldFactoryT<PostViewField>();
@@ -1265,10 +1379,8 @@ FieldManager::FieldManager()
   map_type_name["Laplacian"] = new FieldFactoryT<LaplacianField>();
   map_type_name["Mean"] = new FieldFactoryT<MeanField>();
   map_type_name["Curvature"] = new FieldFactoryT<CurvatureField>();
-#if defined(HAVE_MATH_EVAL)
   map_type_name["Param"] = new FieldFactoryT<ParametricField>();
   map_type_name["MathEval"] = new FieldFactoryT<MathEvalField>();
-#endif
 #if defined(HAVE_ANN)
   map_type_name["Attractor"] = new FieldFactoryT<AttractorField>();
 #endif
@@ -1276,10 +1388,6 @@ FieldManager::FieldManager()
   background_field = -1;
 }
 
-Field::Field()
-{
-}
-
 #if !defined(HAVE_NO_POST)
 void Field::putOnNewView()
 {
@@ -1309,11 +1417,11 @@ void Field::putOnView(PView *view, int comp)
     for(int ele = 0; ele < data->getNumElements(0, ent); ele++){
       if(data->skipElement(0, ent, ele)) continue;
       for(int nod = 0; nod < data->getNumNodes(0, ent, ele); nod++){
-	double x, y, z;
-	data->getNode(0, ent, ele, nod, x, y, z);
+        double x, y, z;
+        data->getNode(0, ent, ele, nod, x, y, z);
         double val = (*this)(x, y, z);
-	for(int comp = 0; comp < data->getNumComponents(0, ent, ele); comp++)
-	  data->setValue(0, ent, ele, nod, comp, val);
+        for(int comp = 0; comp < data->getNumComponents(0, ent, ele); comp++)
+          data->setValue(0, ent, ele, nod, comp, val);
       }
     }
   }
