@@ -10,7 +10,13 @@
 #include "Context.h"
 #include "OS.h"
 #include "GModel.h"
-#include "MElement.h"
+#include "MLine.h"
+#include "MTriangle.h"
+#include "MQuadrangle.h"
+#include "MTetrahedron.h"
+#include "MHexahedron.h"
+#include "MPrism.h"
+#include "MPyramid.h"
 #include "meshGEdge.h"
 #include "meshGFace.h"
 #include "meshGFaceBDS.h"
@@ -39,8 +45,8 @@ static MVertex* isEquivalentTo(std::multimap<MVertex*, MVertex*> &m, MVertex *v)
 }
 
 static void buildASetOfEquivalentMeshVertices(GFace *gf, 
-					      std::multimap<MVertex*, MVertex *> &equivalent,
-					      std::map<GVertex*, MVertex*> &bm)
+                                              std::multimap<MVertex*, MVertex *> &equivalent,
+                                              std::map<GVertex*, MVertex*> &bm)
 {
   // an edge is degenerated when is length is considered to be
   // zero. In some cases, a model edge can be considered as too
@@ -59,11 +65,11 @@ static void buildASetOfEquivalentMeshVertices(GFace *gf,
       MVertex *va = *((*it)->getBeginVertex()->mesh_vertices.begin());
       MVertex *vb = *((*it)->getEndVertex()->mesh_vertices.begin());
       if (va != vb){
-	equivalent.insert(std::make_pair(va, vb));
-	equivalent.insert(std::make_pair(vb, va));
-	bm[(*it)->getBeginVertex()] = va;
-	bm[(*it)->getEndVertex()] = vb;
-	printf("%d equivalent to %d\n", va->getNum(), vb->getNum());
+        equivalent.insert(std::make_pair(va, vb));
+        equivalent.insert(std::make_pair(vb, va));
+        bm[(*it)->getBeginVertex()] = va;
+        bm[(*it)->getEndVertex()] = vb;
+        printf("%d equivalent to %d\n", va->getNum(), vb->getNum());
       }
     }
     ++it;
@@ -75,10 +81,10 @@ static void buildASetOfEquivalentMeshVertices(GFace *gf,
       MVertex *va = *((*it)->getBeginVertex()->mesh_vertices.begin());
       MVertex *vb = *((*it)->getEndVertex()->mesh_vertices.begin());
       if (va != vb){
-	equivalent.insert(std::make_pair(va, vb));
-	equivalent.insert(std::make_pair(vb, va));
-	bm[(*it)->getBeginVertex()] = va;
-	bm[(*it)->getEndVertex()] = vb;
+        equivalent.insert(std::make_pair(va, vb));
+        equivalent.insert(std::make_pair(vb, va));
+        bm[(*it)->getBeginVertex()] = va;
+        bm[(*it)->getEndVertex()] = vb;
       }
     }
     ++it;
@@ -111,6 +117,30 @@ geomTresholdVertexEquivalence::geomTresholdVertexEquivalence(GModel *g)
       printf("Finally : %d equivalent to %d\n", v->getNum(), other->getNum());
       g->mesh_vertices.clear();
       g->mesh_vertices.push_back(other);
+      std::list<GEdge*> ed = g->edges();
+      for (std::list<GEdge*>::iterator ite = ed.begin() ; ite != ed.end() ; ++ite){
+        std::vector<MLine*> newl;
+        for (unsigned int i = 0; i < (*ite)->lines.size(); ++i){
+          MLine *l = (*ite)->lines[i];
+          MVertex *v1 = l->getVertex(0);
+          MVertex *v2 = l->getVertex(1);
+          if (v1 == v && v2 != other){  
+            delete l;       
+            l = new MLine(other,v2);
+            newl.push_back(l);
+          }       
+          else if (v1 != other && v2 == v){     
+            delete l;       
+            l = new MLine(v1,other);
+            newl.push_back(l);
+          }       
+          else if (v1 != v && v2 != v)  
+            newl.push_back(l);    
+          else 
+            delete l;
+        }
+        (*ite)->lines = newl;
+      }       
     }
   }
 }
@@ -210,28 +240,28 @@ void GetStatistics(double stat[50], double quality[4][100])
     double disto=0., distoMin=1., distoMax=0.;
     if (m->firstRegion() == m->lastRegion()){
       for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it){
-	GetQualityMeasure((*it)->quadrangles, gamma, gammaMin, gammaMax,
-			  eta, etaMin, etaMax, rho, rhoMin, rhoMax, 
-			  disto, distoMin, distoMax, quality);
-	GetQualityMeasure((*it)->triangles, gamma, gammaMin, gammaMax,
-			  eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-			  disto, distoMin, distoMax, quality);
+        GetQualityMeasure((*it)->quadrangles, gamma, gammaMin, gammaMax,
+                          eta, etaMin, etaMax, rho, rhoMin, rhoMax, 
+                          disto, distoMin, distoMax, quality);
+        GetQualityMeasure((*it)->triangles, gamma, gammaMin, gammaMax,
+                          eta, etaMin, etaMax, rho, rhoMin, rhoMax,
+                          disto, distoMin, distoMax, quality);
       }
     }
     else{
       for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it){
-	GetQualityMeasure((*it)->tetrahedra, gamma, gammaMin, gammaMax,
-			  eta, etaMin, etaMax, rho, rhoMin, rhoMax, 
-			  disto, distoMin, distoMax, quality);
-	GetQualityMeasure((*it)->hexahedra, gamma, gammaMin, gammaMax,
-			  eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-			  disto, distoMin, distoMax, quality);
-	GetQualityMeasure((*it)->prisms, gamma, gammaMin, gammaMax,
-			  eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-			  disto, distoMin, distoMax,quality);
-	GetQualityMeasure((*it)->pyramids, gamma, gammaMin, gammaMax,
-			  eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-			  disto, distoMin, distoMax, quality);
+        GetQualityMeasure((*it)->tetrahedra, gamma, gammaMin, gammaMax,
+                          eta, etaMin, etaMax, rho, rhoMin, rhoMax, 
+                          disto, distoMin, distoMax, quality);
+        GetQualityMeasure((*it)->hexahedra, gamma, gammaMin, gammaMax,
+                          eta, etaMin, etaMax, rho, rhoMin, rhoMax,
+                          disto, distoMin, distoMax, quality);
+        GetQualityMeasure((*it)->prisms, gamma, gammaMin, gammaMax,
+                          eta, etaMin, etaMax, rho, rhoMin, rhoMax,
+                          disto, distoMin, distoMax,quality);
+        GetQualityMeasure((*it)->pyramids, gamma, gammaMin, gammaMax,
+                          eta, etaMin, etaMax, rho, rhoMin, rhoMax,
+                          disto, distoMin, distoMax, quality);
       }
     }
     double N = stat[9] + stat[10] + stat[11] + stat[12];
@@ -297,8 +327,8 @@ static bool CancelDelaunayHybrid(GModel *m)
     return !Msg::GetBinaryAnswer
       ("You are trying to generate a mixed structured/unstructured grid using\n"
        "the 3D Delaunay algorithm. This algorithm cannot garantee that the\n"
-       "final mesh will be conforming. You should probably use the Frontal\n"
-       "Netgen algorithm instead. Do you really want to continue?\n\n"
+       "final mesh will be conforming. You should probably use the 3D Frontal\n"
+       "algorithm instead. Do you really want to continue?\n\n"
        "(To disable this warning in the future, select `Enable expert mode'\n"
        "in the option dialog.)",
        "Continue", "Cancel");
@@ -333,7 +363,13 @@ static void PrintMesh2dStatistics(GModel *m)
   int nTotT = 0, nTotE = 0, nTotGoodLength = 0, nTotGoodQuality = 0;
   int nUnmeshed = 0, numFaces = 0;
 
-  Msg::Info("2D Mesh Statistics :");
+  if(CTX::instance()->createAppendMeshStatReport == 1){
+    fprintf(statreport, "2D stats\tname\t\t#faces\t\t#fail\t\t"
+            "#t\t\tQavg\t\tQbest\t\tQworst\t\t#Q>90\t\t#Q>90/#t\t"
+            "#e\t\ttau\t\t#Egood\t\t#Egood/#e\tCPU\n");
+    if(m->empty()) return;
+  }
+
   for(GModel::fiter it = m->firstFace() ; it != m->lastFace(); ++it){
     worst = std::min((*it)->meshStatistics.worst_element_shape, worst);
     best = std::max((*it)->meshStatistics.best_element_shape, best);
@@ -350,12 +386,6 @@ static void PrintMesh2dStatistics(GModel *m)
     numFaces++;
   }
 
-  if(CTX::instance()->createAppendMeshStatReport == 1){
-    fprintf(statreport, "2D stats\tname\t\t#faces\t\t#fail\t\t"
-            "#t\t\tQavg\t\tQbest\t\tQworst\t\t#Q>90\t\t#Q>90/#t\t"
-            "#e\t\ttau\t\t#Egood\t\t#Egood/#e\tCPU\n");
-  }
-
   fprintf(statreport,"\t%16s\t%d\t\t%d\t\t", m->getName().c_str(), numFaces, nUnmeshed);
   fprintf(statreport,"%d\t\t%8.7f\t%8.7f\t%8.7f\t%d\t\t%8.7f\t",
           nTotT, avg / (double)nTotT, best, worst, nTotGoodQuality,
@@ -373,18 +403,18 @@ static void Mesh2D(GModel *m)
   double t1 = Cpu();
 
   // skip short mesh edges
-  geomTresholdVertexEquivalence inst (m);
+  geomTresholdVertexEquivalence inst(m);
 
   // boundary layers are special: their generation (including vertices
   // and curve meshes) is global as it depends on a smooth normal
   // field generated from the surface mesh of the source surfaces
   if(!Mesh2DWithBoundaryLayers(m)){
-    std::for_each(m->firstFace(), m->lastFace(), meshGFace());        
+    std::for_each(m->firstFace(), m->lastFace(), meshGFace());
     int nIter = 0;
     while(1){
       meshGFace mesher;
       int nbPending = 0;
-      for(GModel::fiter it = m->firstFace() ; it!=m->lastFace(); ++it){
+      for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it){
         if ((*it)->meshStatistics.status == GFace::PENDING){
           mesher(*it);
           nbPending++;
@@ -395,7 +425,7 @@ static void Mesh2D(GModel *m)
     }
   }
   
-  // gmshCollapseSmallEdges (*m);
+  // collapseSmallEdges(*m);
 
   double t2 = Cpu();
   CTX::instance()->meshTimer[1] = t2 - t1;
@@ -406,7 +436,7 @@ static void Mesh2D(GModel *m)
 }
 
 static void FindConnectedRegions(std::vector<GRegion*> &delaunay, 
-				 std::vector<std::vector<GRegion*> > &connected)
+                                 std::vector<std::vector<GRegion*> > &connected)
 {
   // FIXME: need to split region vector into connected components here!
   connected.push_back(delaunay);
@@ -437,8 +467,9 @@ static void Mesh3D(GModel *m)
   // quality reasons)
   std::vector<std::vector<GRegion*> > connected;
   FindConnectedRegions(delaunay, connected);
-  for(unsigned int i = 0; i < connected.size(); i++)
+  for(unsigned int i = 0; i < connected.size(); i++){
     MeshDelaunayVolume(connected[i]);
+  }
 
   double t2 = Cpu();
   CTX::instance()->meshTimer[2] = t2 - t1;
@@ -559,7 +590,7 @@ void GenerateMesh(GModel *m, int ask)
               CTX::instance()->mesh.secondOrderIncomplete);
 
   Msg::Info("%d vertices %d elements",
-	    m->getNumMeshVertices(), m->getNumMeshElements());
+            m->getNumMeshVertices(), m->getNumMeshElements());
 
   Msg::PrintErrorCounter("Mesh generation error summary");
 
