@@ -8,14 +8,20 @@
 //
 
 #include "HighOrder.h"
-#include "gmshSmoothHighOrder.h"
-#include "MElement.h"
+#include "highOrderSmoother.h"
+#include "MLine.h"
+#include "MTriangle.h"
+#include "MQuadrangle.h"
+#include "MTetrahedron.h"
+#include "MHexahedron.h"
+#include "MPrism.h"
+#include "MPyramid.h"
 #include "GmshMessage.h"
 #include "OS.h"
 #include "Numeric.h"
 #include "Context.h"
-#include "GmshMatrix.h"
-#include "FunctionSpace.h"
+#include "fullMatrix.h"
+#include "functionSpace.h"
 
 #define SQU(a)      ((a)*(a))
 
@@ -40,7 +46,7 @@ static bool mappingIsInvertible(MTetrahedron *e)
     if (det0 * detN <= 0.) return false;
   }
 
-  const gmshMatrix<double> &points = e->getFunctionSpace()->points;
+  const fullMatrix<double> &points = e->getFunctionSpace()->points;
 
   for (int i = 0; i < e->getNumPrimaryVertices(); i++) {
     const double u = points(i,0);
@@ -62,7 +68,7 @@ static double mylength(GEdge *ge, int i, double *u)
   return ge->length(u[i], u[i+1], 10);
 }
 
-static void myresid(int N, GEdge *ge, double *u, gmshVector<double> &r)
+static void myresid(int N, GEdge *ge, double *u, fullVector<double> &r)
 {
   double L[100];
   for (int i = 0; i < N - 1; i++) L[i] = mylength(ge, i, u);
@@ -89,10 +95,10 @@ static bool computeEquidistantParameters(GEdge *ge, double u0, double uN, int N,
 
   // create the tangent matrix
   const int M = N - 2;
-  gmshMatrix<double> J(M, M);
-  gmshVector<double> DU(M);
-  gmshVector<double> R(M);
-  gmshVector<double> Rp(M);
+  fullMatrix<double> J(M, M);
+  fullVector<double> DU(M);
+  fullVector<double> R(M);
+  fullVector<double> Rp(M);
   
   int iter = 1;
 
@@ -134,7 +140,7 @@ static double mylength(GFace *gf, int i, double *u, double *v)
   return gf->length(SPoint2(u[i], v[i]), SPoint2(u[i + 1], v[i + 1]), 10);
 }
 
-static void myresid(int N, GFace *gf, double *u, double *v, gmshVector<double> &r)
+static void myresid(int N, GFace *gf, double *u, double *v, fullVector<double> &r)
 {
   double L[100];
   for (int i = 0; i < N - 1; i++) L[i] = mylength(gf, i, u, v);  
@@ -168,10 +174,10 @@ static bool computeEquidistantParameters(GFace *gf, double u0, double uN,
 
   // create the tangent matrix
   const int M = N - 2;
-  gmshMatrix<double> J(M, M);
-  gmshVector<double> DU(M);
-  gmshVector<double> R(M);
-  gmshVector<double> Rp(M);
+  fullMatrix<double> J(M, M);
+  fullVector<double> DU(M);
+  fullVector<double> R(M);
+  fullVector<double> Rp(M);
   
   int iter = 1;
 
@@ -221,8 +227,8 @@ static bool computeEquidistantParameters(GFace *gf, double u0, double uN,
 
 static void getEdgeVertices(GEdge *ge, MElement *ele, std::vector<MVertex*> &ve,
                             edgeContainer &edgeVertices, bool linear,
-                            int nPts = 1, gmshHighOrderSmoother *displ2D = 0,
-                            gmshHighOrderSmoother *displ3D = 0)
+                            int nPts = 1, highOrderSmoother *displ2D = 0,
+                            highOrderSmoother *displ3D = 0)
 {
   if(ge->geomType() == GEntity::DiscreteCurve ||
      ge->geomType() == GEntity::BoundaryLayerCurve)
@@ -282,10 +288,10 @@ static void getEdgeVertices(GEdge *ge, MElement *ele, std::vector<MVertex*> &ve,
           GPoint pc = ge->point(US[j + 1]);
           v = new MEdgeVertex(pc.x(), pc.y(), pc.z(), ge, US[j + 1]);
             
-          if (displ2D){
+          if (displ2D || displ3D){
             SPoint3 pc2 = edge.interpolate(t);          
-            displ2D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
-            displ3D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
+            if (displ2D)displ2D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
+            if (displ3D)displ3D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
           }
         }
         temp.push_back(v);
@@ -304,8 +310,9 @@ static void getEdgeVertices(GEdge *ge, MElement *ele, std::vector<MVertex*> &ve,
 
 static void getEdgeVertices(GFace *gf, MElement *ele, std::vector<MVertex*> &ve,
                             edgeContainer &edgeVertices, bool linear,
-                            int nPts = 1, gmshHighOrderSmoother *displ2D = 0,
-                            gmshHighOrderSmoother *displ3D = 0)
+                            int nPts = 1, 
+                            highOrderSmoother *displ2D = 0,
+                            highOrderSmoother *displ3D = 0)
 {
   if(gf->geomType() == GEntity::DiscreteSurface ||
      gf->geomType() == GEntity::BoundaryLayerSurface)
@@ -342,11 +349,12 @@ static void getEdgeVertices(GFace *gf, MElement *ele, std::vector<MVertex*> &ve,
         }
         else{
           GPoint pc = gf->point(US[j + 1], VS[j + 1]);
-	  v = new MFaceVertex(pc.x(), pc.y(), pc.z(), gf, US[j + 1], VS[j + 1]);
-	  if (displ3D){
-	    SPoint3 pc2 = edge.interpolate(t);          
-	    displ3D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
-	  }
+          v = new MFaceVertex(pc.x(), pc.y(), pc.z(), gf, US[j + 1], VS[j + 1]);
+          if (displ2D || displ3D){
+            SPoint3 pc2 = edge.interpolate(t);          
+            if (displ3D) displ3D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
+            if (displ2D) displ2D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
+          }
         }
         temp.push_back(v);
         gf->mesh_vertices.push_back(v);
@@ -362,8 +370,8 @@ static void getEdgeVertices(GFace *gf, MElement *ele, std::vector<MVertex*> &ve,
 
 static void getEdgeVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &ve,
                             edgeContainer &edgeVertices, bool linear,
-                            int nPts = 1, gmshHighOrderSmoother *displ2D = 0,
-                            gmshHighOrderSmoother *displ3D = 0)
+                            int nPts = 1, highOrderSmoother *displ2D = 0,
+                            highOrderSmoother *displ3D = 0)
 {
   for(int i = 0; i < ele->getNumEdges(); i++){
     MEdge edge = ele->getEdge(i);
@@ -394,27 +402,27 @@ static void getEdgeVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &v
 
 static void getFaceVertices(GFace *gf, MElement *incomplete, MElement *ele, 
                             std::vector<MVertex*> &vf, faceContainer &faceVertices, 
-                            bool linear, int nPts = 1, gmshHighOrderSmoother *displ2D = 0,
-                            gmshHighOrderSmoother *displ3D = 0)
+                            bool linear, int nPts = 1, highOrderSmoother *displ2D = 0,
+                            highOrderSmoother *displ3D = 0)
 {
   if(gf->geomType() == GEntity::DiscreteSurface ||
      gf->geomType() == GEntity::BoundaryLayerSurface)
     linear = true;
 
-  gmshMatrix<double> points;
+  fullMatrix<double> points;
   int start = 0;
 
   switch (nPts){
   case 2 :
-    points = gmshFunctionSpaces::find(MSH_TRI_10).points;
+    points = functionSpaces::find(MSH_TRI_10).points;
     start = 9;
     break;
   case 3 :
-    points = gmshFunctionSpaces::find(MSH_TRI_15).points;
+    points = functionSpaces::find(MSH_TRI_15).points;
     start = 12;
     break;
   case 4 :
-    points = gmshFunctionSpaces::find(MSH_TRI_21).points;
+    points = functionSpaces::find(MSH_TRI_21).points;
     start = 15;
     break;
   default :  
@@ -433,8 +441,8 @@ static void getFaceVertices(GFace *gf, MElement *incomplete, MElement *ele,
       SPoint2 pts[20];
       bool reparamOK = true;
       if(!linear){
-	for(int k = 0; k < incomplete->getNumVertices(); k++)
-	  reparamOK &= reparamMeshVertexOnFace(incomplete->getVertex(k), gf, pts[k]);
+        for(int k = 0; k < incomplete->getNumVertices(); k++)
+          reparamOK &= reparamMeshVertexOnFace(incomplete->getVertex(k), gf, pts[k]);
       }
       if(face.getNumVertices() == 3 && nPts > 1){ // tri face
         for(int k = start; k < points.size1(); k++){
@@ -446,35 +454,36 @@ static void getFaceVertices(GFace *gf, MElement *incomplete, MElement *ele,
             v = new MVertex(pc.x(), pc.y(), pc.z(), gf);
           }
           else{
-	    double X(0), Y(0), Z(0), GUESS[2] = {0, 0};
+            double X(0), Y(0), Z(0), GUESS[2] = {0, 0};
             double sf[256]; 
             incomplete->getShapeFunctions(t1, t2, 0, sf);
-	    for (int j = 0; j < incomplete->getNumVertices(); j++){
-	      MVertex *vt = incomplete->getVertex(j);
+            for (int j = 0; j < incomplete->getNumVertices(); j++){
+              MVertex *vt = incomplete->getVertex(j);
               X += sf[j] * vt->x();
-	      Y += sf[j] * vt->y();
-	      Z += sf[j] * vt->z();
-	      if (reparamOK){
-		GUESS[0] += sf[j] * pts[j][0];
-		GUESS[1] += sf[j] * pts[j][1];
-	      }
-	    }
-	    if(reparamOK){
-	      GPoint gp = gf->closestPoint(SPoint3(X, Y, Z), GUESS);
-	      if (gp.g()){
-		v = new MFaceVertex(gp.x(), gp.y(), gp.z(), gf, gp.u(), gp.v());
-	      }
-	      else{
-		v = new MVertex(X, Y, Z, gf);
-	      }
-	    }
-	    else{
-	      v = new MVertex(X, Y, Z, gf);
-	    }
-	    if(displ3D){
-	      SPoint3 pc2 = face.interpolate(t1, t2);
-	      displ3D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
-	    }	    
+              Y += sf[j] * vt->y();
+              Z += sf[j] * vt->z();
+              if (reparamOK){
+                GUESS[0] += sf[j] * pts[j][0];
+                GUESS[1] += sf[j] * pts[j][1];
+              }
+            }
+            if(reparamOK){
+              GPoint gp = gf->closestPoint(SPoint3(X, Y, Z), GUESS);
+              if (gp.g()){
+                v = new MFaceVertex(gp.x(), gp.y(), gp.z(), gf, gp.u(), gp.v());
+              }
+              else{
+                v = new MVertex(X, Y, Z, gf);
+              }
+            }
+            else{
+              v = new MVertex(X, Y, Z, gf);
+            }
+            if(displ3D || displ2D){
+              SPoint3 pc2 = face.interpolate(t1, t2);
+              if(displ3D)displ3D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
+              if(displ2D)displ2D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
+            }       
           }
           // should be expensive -> induces a new search each time
           vtcs.push_back(v);
@@ -550,20 +559,20 @@ static void getFaceVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &v
                             faceContainer &faceVertices, edgeContainer &edgeVertices,
                             bool linear, int nPts = 1)
 {
-  gmshMatrix<double> points;
+  fullMatrix<double> points;
   int start = 0;
   
   switch (nPts){
   case 2 :
-    points = gmshFunctionSpaces::find(MSH_TRI_10).points;
+    points = functionSpaces::find(MSH_TRI_10).points;
     start = 9;
     break;
   case 3 :
-    points = gmshFunctionSpaces::find(MSH_TRI_15).points;
+    points = functionSpaces::find(MSH_TRI_15).points;
     start = 12;
     break;
   case 4 :
-    points = gmshFunctionSpaces::find(MSH_TRI_21).points;
+    points = functionSpaces::find(MSH_TRI_21).points;
     start = 15;
     break;
   default :  
@@ -643,16 +652,16 @@ static void getFaceVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &v
 static void getRegionVertices(GRegion *gr, MElement *incomplete, MElement *ele, 
                               std::vector<MVertex*> &vr, bool linear, int nPts = 1)
 {
-  gmshMatrix<double> points;
+  fullMatrix<double> points;
   int start = 0;
 
   switch (nPts){
   case 3 :
-    points = gmshFunctionSpaces::find(MSH_TET_35).points;
+    points = functionSpaces::find(MSH_TET_35).points;
     start = 34;
     break;
   case 4 :
-    points = gmshFunctionSpaces::find(MSH_TET_56).points;
+    points = functionSpaces::find(MSH_TET_56).points;
     start = 52;
     break;
   default :  
@@ -674,8 +683,8 @@ static void getRegionVertices(GRegion *gr, MElement *incomplete, MElement *ele,
 }
 
 static void setHighOrder(GEdge *ge, edgeContainer &edgeVertices, bool linear, 
-                         int nbPts = 1, gmshHighOrderSmoother *displ2D = 0,
-                         gmshHighOrderSmoother *displ3D = 0)
+                         int nbPts = 1, highOrderSmoother *displ2D = 0,
+                         highOrderSmoother *displ3D = 0)
 {
   std::vector<MLine*> lines2;
   for(unsigned int i = 0; i < ge->lines.size(); i++){
@@ -692,37 +701,54 @@ static void setHighOrder(GEdge *ge, edgeContainer &edgeVertices, bool linear,
   ge->deleteVertexArrays();
 }
 
-static void setHighOrder(GFace *gf, edgeContainer &edgeVertices, 
-                         faceContainer &faceVertices, bool linear, bool incomplete,
-                         int nPts = 1, gmshHighOrderSmoother *displ2D = 0,
-                         gmshHighOrderSmoother *displ3D = 0)
+MTriangle* setHighOrder(MTriangle *t, GFace *gf, 
+                        edgeContainer &edgeVertices, 
+                        faceContainer &faceVertices, 
+                        bool linear, bool incomplete, int nPts, 
+                        highOrderSmoother *displ2D,
+                        highOrderSmoother *displ3D)
 {
-  std::vector<MTriangle*> triangles2;
-  for(unsigned int i = 0; i < gf->triangles.size(); i++){
-    MTriangle *t = gf->triangles[i];
-    std::vector<MVertex*> ve, vf;
-    getEdgeVertices(gf, t, ve, edgeVertices, linear, nPts, displ2D, displ3D);
-    if(nPts == 1){
-      triangles2.push_back
-        (new MTriangle6(t->getVertex(0), t->getVertex(1), t->getVertex(2),
-                        ve[0], ve[1], ve[2]));
+  std::vector<MVertex*> ve, vf;
+  getEdgeVertices(gf, t, ve, edgeVertices, linear, nPts, displ2D, displ3D);
+  if(nPts == 1){
+    return new MTriangle6(t->getVertex(0), t->getVertex(1), t->getVertex(2),
+                          ve[0], ve[1], ve[2]);
+  }
+  else{
+    if(incomplete){
+      return new MTriangleN(t->getVertex(0), t->getVertex(1), t->getVertex(2),
+                            ve, nPts + 1);
     }
     else{
-      if(incomplete){
-        triangles2.push_back
-          (new MTriangleN(t->getVertex(0), t->getVertex(1), t->getVertex(2),
-                          ve, nPts + 1));
+      if (displ2D && gf->geomType() == GEntity::Plane){
+        MTriangle incpl(t->getVertex(0), t->getVertex(1), t->getVertex(2));
+        getFaceVertices(gf, &incpl, t, vf, faceVertices, linear, nPts, displ2D, 
+                        displ3D);
       }
       else{
         MTriangleN incpl(t->getVertex(0), t->getVertex(1), t->getVertex(2),
                          ve, nPts + 1);
-        getFaceVertices(gf, &incpl, t, vf, faceVertices, linear, nPts, displ2D, displ3D);
-        ve.insert(ve.end(), vf.begin(), vf.end());
-        triangles2.push_back
-          (new MTriangleN(t->getVertex(0), t->getVertex(1), t->getVertex(2),
-                          ve, nPts + 1));
+        getFaceVertices(gf, &incpl, t, vf, faceVertices, linear, nPts, displ2D, 
+                        displ3D);
       }
+      ve.insert(ve.end(), vf.begin(), vf.end());
+      return new MTriangleN(t->getVertex(0), t->getVertex(1), t->getVertex(2),
+                            ve, nPts + 1);
     }
+  }  
+}
+
+static void setHighOrder(GFace *gf, edgeContainer &edgeVertices, 
+                         faceContainer &faceVertices, bool linear, bool incomplete,
+                         int nPts = 1, highOrderSmoother *displ2D = 0,
+                         highOrderSmoother *displ3D = 0)
+{
+  std::vector<MTriangle*> triangles2;
+  for(unsigned int i = 0; i < gf->triangles.size(); i++){
+    MTriangle *t = gf->triangles[i];
+    MTriangle *tNew = setHighOrder(t,gf,edgeVertices,faceVertices, linear, incomplete,
+                                   nPts,displ2D,displ3D);
+    triangles2.push_back(tNew);
     delete t;
   }
   gf->triangles = triangles2;
@@ -751,8 +777,8 @@ static void setHighOrder(GFace *gf, edgeContainer &edgeVertices,
 
 static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices, 
                          faceContainer &faceVertices, bool linear, bool incomplete,
-                         int nPts = 1, gmshHighOrderSmoother *displ2D = 0,
-                         gmshHighOrderSmoother *displ3D = 0)
+                         int nPts = 1, highOrderSmoother *displ2D = 0,
+                         highOrderSmoother *displ3D = 0)
 {
   int nbCorr = 0;
   
@@ -763,8 +789,8 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
     getEdgeVertices(gr, t, ve, edgeVertices, linear, nPts, displ2D, displ3D);
     if(nPts == 1){
       tetrahedra2.push_back
-	(new MTetrahedron10(t->getVertex(0), t->getVertex(1), t->getVertex(2), 
-			    t->getVertex(3), ve[0], ve[1], ve[2], ve[3], ve[4], ve[5]));
+        (new MTetrahedron10(t->getVertex(0), t->getVertex(1), t->getVertex(2), 
+                            t->getVertex(3), ve[0], ve[1], ve[2], ve[3], ve[4], ve[5]));
     }
     else{
       getFaceVertices(gr, t, vf, faceVertices, edgeVertices, linear, nPts);
@@ -926,29 +952,71 @@ void SetOrder1(GModel *m)
     removeHighOrderVertices(*it);
 }
 
-static void checkHighOrderTriangles(GModel *m, std::vector<MElement*> &bad, double &minJGlob)
+static void checkHighOrderTriangles(const char* cc, GModel *m, 
+                                    std::vector<MElement*> &bad, double &minJGlob)
 {
   bad.clear();
   minJGlob = 1.0;
+  double minGGlob = 1.0;
+  double avg = 0.0;
+  int count = 0, nbfair=0;
   for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it){
     for(unsigned int i = 0; i < (*it)->triangles.size(); i++){
       MTriangle *t = (*it)->triangles[i];
-      double disto = t->distoShapeMeasure();
+      double disto_ = t->distoShapeMeasure();
+      double gamma_ = t->gammaShapeMeasure();
+      double disto = disto_;
       minJGlob = std::min(minJGlob, disto);
+      minGGlob = std::min(minGGlob, gamma_);
+      avg += disto; count++;
       if (disto < 0) bad.push_back(t);
+      else if (disto < 0.2) nbfair++;
     }
   }
-  if (minJGlob > 0) Msg::Info("Worst Element Smoothness %g", minJGlob);
-  else Msg::Warning("Worst Element Smoothness %g", minJGlob);
+  if (minJGlob > 0) 
+    Msg::Info("%s : Worst Triangle Smoothness %g Gamma %g NbFair = %d", 
+              cc, minJGlob, minGGlob,nbfair );
+  else
+    Msg::Warning("%s : Worst Triangle Smoothness %g (%d negative jacobians) "
+                 "Worst Gamma %g Avg Smoothness %g", cc, minJGlob, bad.size(),
+                 minGGlob, avg / (count ? count : 1));
+}
+
+static void checkHighOrderTetrahedron(const char* cc, GModel *m, 
+                                      std::vector<MElement*> &bad, double &minJGlob)
+{
+  bad.clear();
+  minJGlob = 1.0;
+  double minGGlob = 1.0;
+  double avg = 0.0;
+  int count = 0, nbfair=0;
+  for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it){
+    for(unsigned int i = 0; i < (*it)->tetrahedra.size(); i++){
+      MTetrahedron *t = (*it)->tetrahedra[i];
+      double disto_ = t->distoShapeMeasure();
+      double gamma_ = t->gammaShapeMeasure();
+      double disto = disto_;
+      minJGlob = std::min(minJGlob, disto);
+      minGGlob = std::min(minGGlob, gamma_);
+      avg += disto; count++;
+      if (disto < 0) bad.push_back(t);
+      else if (disto < 0.2) nbfair++;
+    }
+  }
+  if (minJGlob > 0)
+    Msg::Info("%s : Worst Tetrahedron Smoothness %g Gamma %g NbFair = %d", 
+              cc, minJGlob, minGGlob, nbfair);
+  else 
+    Msg::Warning("%s : Worst Tetrahedron Smoothness %g (%d negative jacobians) "
+                 "Worst Gamma %g Avg Smoothness %g", cc, minJGlob, bad.size(),
+                 minGGlob, avg / (count ? count : 1));
 }
 
 extern double mesh_functional_distorsion(MTriangle *t, double u, double v);
 
 static void printJacobians(GModel *m, const char *nm)
 {
-  return;
-
-  const int n = 25;
+  const int n = 100;
   double D[n][n], X[n][n], Y[n][n], Z[n][n];
 
   FILE *f = fopen(nm,"w");
@@ -962,7 +1030,10 @@ static void printJacobians(GModel *m, const char *nm)
           double u = (double)i / (n - 1);
           double v = (double)k / (n - 1);         
           t->pnt(u, v, 0, pt);
-	  D[i][k] = mesh_functional_distorsion(t, u, v);
+          D[i][k] = mesh_functional_distorsion(t, u, v);
+          //X[i][k] = u;
+          //Y[i][k] = v;
+          //Z[i][k] = 0.0;
           X[i][k] = pt.x();
           Y[i][k] = pt.y();
           Z[i][k] = pt.z();
@@ -1018,11 +1089,11 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete)
   // first, make sure to remove any existsing second order vertices/elements
   SetOrder1(m);    
 
-  gmshHighOrderSmoother *displ2D = 0; 
-  gmshHighOrderSmoother *displ3D = 0; 
+  highOrderSmoother *displ2D = 0; 
+  highOrderSmoother *displ3D = 0; 
   if(CTX::instance()->mesh.smoothInternalEdges){
-    displ2D = new gmshHighOrderSmoother(2);
-    displ3D = new gmshHighOrderSmoother(3);
+    displ2D = new highOrderSmoother(2);
+    displ3D = new highOrderSmoother(3);
   }
 
   // then create new second order vertices/elements
@@ -1045,13 +1116,10 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete)
   std::vector<MElement*> bad;
   double worst;
   if (displ2D){
-    checkHighOrderTriangles(m, bad, worst);
+    checkHighOrderTriangles("Before optimization", m, bad, worst);
     for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it)
-      if ((*it)->geomType() == GEntity::Plane) {
-	gmshSwapHighOrderTriangles(*it);
-	if (displ2D) displ2D->smooth(*it);
-      }
-    // will have to smooth in the planar coordinates, using the metric
+      displ2D->optimize(*it,edgeVertices,faceVertices);
+    checkHighOrderTriangles("After optimization", m, bad, worst);
   }
 
   for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it)
@@ -1060,18 +1128,17 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete)
 
   // smooth the 3D regions
   if (displ3D){
+    checkHighOrderTetrahedron("Before optimization", m, bad, worst);
     for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it)
       displ3D->smooth(*it);
+    checkHighOrderTetrahedron("After optimization", m, bad, worst);
   }
 
-  if(displ2D){    
-    delete displ2D;
-    delete displ3D;
-  }
-
-  printJacobians(m, "detjIni.pos");
-  checkHighOrderTriangles(m, bad, worst);
+  if(displ2D) delete displ2D;
+  if(displ3D) delete displ3D;
 
+  //  printJacobians(m, "smoothness.pos");
+  
   double t2 = Cpu();
   Msg::Info("Meshing order %d complete (%g s)", order, t2 - t1);
   Msg::StatusBar(1, false, "Mesh");
