@@ -6,7 +6,7 @@
 #include "GModel.h"
 #include "meshGEdge.h"
 #include "GEdge.h"
-#include "MElement.h"
+#include "MLine.h"
 #include "BackgroundMesh.h"
 #include "Numeric.h"
 #include "GmshMessage.h"
@@ -33,30 +33,6 @@ struct xi2lc {
 
 static std::vector<xi2lc> interpLc;
 
-static void smoothInterpLc(int nbSmooth)
-{
-  for(int j = 0; j < nbSmooth; j++){
-    for(int i = 0 ; i < (int)interpLc.size(); i++){               
-      xi2lc &left = (i == 0) ? interpLc[0] : interpLc[i - 1];
-      xi2lc &mid = interpLc[i];
-      xi2lc &right = (i == (int)interpLc.size() - 1) ?
-        interpLc[interpLc.size() - 1] : interpLc[i+1];
-      if(1. / mid.lc > 1.1 * 1. / left.lc) mid.lc = left.lc / 1.1;
-      if(1. / mid.lc > 1.1 * 1. / right.lc) mid.lc = right.lc / 1.1;
-    }
-  } 
-}
-
-static void printInterpLc(const char *name)
-{
-  FILE *f = fopen(name,"w");
-  for(unsigned int i = 0; i < interpLc.size(); i++){              
-    xi2lc &interp = interpLc[i];
-    fprintf(f,"%12.5E %12.5E\n", interp.xi, 1 / interp.lc);
-  }
-  fclose(f);
-}
-
 static void buildInterpLc(const std::vector<IntPoint> &lcPoints)
 {
   IntPoint p;
@@ -74,13 +50,11 @@ static double F_Lc_usingInterpLc(GEdge *ge, double t)
   double t1 = it->xi;
   double l1 = it->lc;
   it++;
-  SVector3 der = ge->firstDer(t);
-  const double d = norm(der);
-  if(it == interpLc.end()) return d * l1;
+  if(it == interpLc.end()) return l1;
   double t2 = it->xi;
   double l2 = it->lc;
   double l = l1 + ((t - t1) / (t2 - t1)) * (l2 - l1);
-  return d * l;
+  return l;
 }
 
 static double F_Lc_usingInterpLcBis(GEdge *ge, double t)
@@ -92,6 +66,9 @@ static double F_Lc_usingInterpLcBis(GEdge *ge, double t)
   double t_begin = bounds.low();
   double t_end = bounds.high();
 
+  SVector3 der = ge->firstDer(t);
+  const double d = norm(der);
+
   if(t == t_begin)
     lc_here = BGM_MeshSize(ge->getBeginVertex(), t, 0, p.x(), p.y(), p.z());
   else if(t == t_end)
@@ -99,7 +76,7 @@ static double F_Lc_usingInterpLcBis(GEdge *ge, double t)
   else
     lc_here = BGM_MeshSize(ge, t, 0, p.x(), p.y(), p.z());
 
-  return 1 / lc_here;
+  return d / lc_here;
 }
 
 static double F_Lc(GEdge *ge, double t)
@@ -120,7 +97,11 @@ static double F_Lc(GEdge *ge, double t)
 
   SVector3 der = ge->firstDer(t);
   const double d = norm(der);
+
+  //printf("lc_here=%g d=%g nb =%g\n", lc_here,d,  d/lc_here);
+
   return d / lc_here;
+
 }
 
 static double F_Transfinite(GEdge *ge, double t)
@@ -193,9 +174,9 @@ static double trapezoidal(IntPoint * P1, IntPoint * P2)
 }
 
 static void RecursiveIntegration(GEdge *ge, IntPoint *from, IntPoint *to,
-				 double (*f) (GEdge *e, double X), 
-				 std::vector<IntPoint> &Points,
-				 double Prec, int *depth)
+                                 double (*f) (GEdge *e, double X), 
+                                 std::vector<IntPoint> &Points,
+                                 double Prec, int *depth)
 {
   IntPoint P, p1;
 
@@ -227,8 +208,8 @@ static void RecursiveIntegration(GEdge *ge, IntPoint *from, IntPoint *to,
 }
 
 static double Integration(GEdge *ge, double t1, double t2, 
-			  double (*f) (GEdge *e, double X),
-			  std::vector<IntPoint> &Points, double Prec)
+                          double (*f) (GEdge *e, double X),
+                          std::vector<IntPoint> &Points, double Prec)
 {
   IntPoint from, to;
 
@@ -305,18 +286,15 @@ void meshGEdge::operator() (GEdge *ge)
       Integration(ge, t_begin, t_end, F_Lc_usingInterpLcBis, lcPoints, 
                   CTX::instance()->mesh.lcIntegrationPrecision);
       buildInterpLc(lcPoints);
-      // printInterpLc("toto1.dat");
-      // smoothInterpLc(20);
-      // printInterpLc("toto2.dat");
       a = Integration(ge, t_begin, t_end, F_Lc_usingInterpLc, Points, 1.e-8);
     }
     else{
       a = Integration(ge, t_begin, t_end, F_Lc, Points,
-		      CTX::instance()->mesh.lcIntegrationPrecision);
+                      CTX::instance()->mesh.lcIntegrationPrecision);
     }
     N = std::max(ge->minimumMeshSegments() + 1, (int)(a + 1.));
   }
-  
+
   // if the curve is periodic and if the begin vertex is identical to
   // the end vertex and if this vertex has only one model curve
   // adjacent to it, then the vertex is not connecting any other
@@ -326,6 +304,7 @@ void meshGEdge::operator() (GEdge *ge)
   if(ge->getBeginVertex() == ge->getEndVertex() && 
      ge->getBeginVertex()->edges().size() == 1){
     end_p = beg_p = ge->point(t_begin);
+    Msg::Debug("Meshing periodic closed curve");
   }
   else{
     MVertex *v0 = ge->getBeginVertex()->mesh_vertices[0];
@@ -355,7 +334,6 @@ void meshGEdge::operator() (GEdge *ge)
         double lc  = d/(P1.lc + dlc / dp * (d - P1.p));
         GPoint V = ge->point(t);
         ge->mesh_vertices[NUMP - 1] = new MEdgeVertex(V.x(), V.y(), V.z(), ge, t, lc);
-        // printf("lc = %12.5E %12.5E \n",lc,P1.lc,P2.lc);
         NUMP++;
       }
       else {
@@ -366,9 +344,9 @@ void meshGEdge::operator() (GEdge *ge)
   }
 
   for(unsigned int i = 0; i < ge->mesh_vertices.size() + 1; i++){
-    MVertex *v0 = (i == 0) ? 
+    MVertex *v0 = (i == 0) ?
       ge->getBeginVertex()->mesh_vertices[0] : ge->mesh_vertices[i - 1];
-    MVertex *v1 = (i == ge->mesh_vertices.size()) ? 
+    MVertex *v1 = (i == ge->mesh_vertices.size()) ?
       ge->getEndVertex()->mesh_vertices[0] : ge->mesh_vertices[i];
     ge->lines.push_back(new MLine(v0, v1));
   }
