@@ -38,15 +38,9 @@ class orientMeshGFace {
 };
 
 void fourthPoint(double *p1, double *p2, double *p3, double *p4);
-
-// Compute edge loops of the face, all_mvertices are the vertices of
-// the
-void computeEdgeLoops(const GFace *gf,
-                      std::vector<MVertex*> &all_mvertices,
-                      std::vector<int> &indices);
 void findTransfiniteCorners(GFace *gf, std::vector<MVertex*> &corners);
 int MeshTransfiniteSurface(GFace *gf);
-int MeshExtrudedSurface(GFace *gf, 
-                        std::set<std::pair<MVertex*, MVertex*> > *constrainedEdges=0);
+int MeshExtrudedSurface(GFace *gf, std::set<std::pair<MVertex*, MVertex*> > 
+                        *constrainedEdges=0);
 
 #endif
