@@ -6,7 +6,8 @@
 #ifndef _MESH_GFACE_DELAUNAY_INSERTIONFACE_H_
 #define _MESH_GFACE_DELAUNAY_INSERTIONFACE_H_
 
-#include "MElement.h"
+#include "MTriangle.h"
+#include "STensor3.h"
 #include <list>
 #include <set>
 #include <map>
@@ -17,21 +18,18 @@ class BDS_Mesh;
 class BDS_Point;
 
 void buildMetric(GFace *gf, double *uv, double *metric);
-int inCircumCircleAniso(GFace *gf, double *p1, double *p2, double *p3, double *p4,
-                        double *metric);
-int inCircumCircleAniso(GFace *gf, MTriangle *base, const double *uv, const double *metric,
-                        const std::vector<double> &Us, const std::vector<double> &Vs);
-void circumCenterXYZ(double *p1, double *p2, double *p3, double *res, double *uv=0);
-void circumCenterMetric(double *pa, double *pb, double *pc,
-                        const double *metric,
+int inCircumCircleAniso(GFace *gf, double *p1, double *p2, double *p3, 
+                        double *p4, double *metric);
+int inCircumCircleAniso(GFace *gf, MTriangle *base, const double *uv, 
+                        const double *metric, const std::vector<double> &Us,
+                        const std::vector<double> &Vs);
+void circumCenterMetric(double *pa, double *pb, double *pc, const double *metric,
                         double *x, double &Radius2);
-void circumCenterMetric(MTriangle *base,
-                        const double *metric,
-                        const std::vector<double> &Us,
+void circumCenterMetric(MTriangle *base, const double *metric,
+                        const std::vector<double> &Us, 
                         const std::vector<double> &Vs,
                         double *x, double &Radius2);
-bool circumCenterMetricInTriangle(MTriangle *base,
-                                  const double *metric,
+bool circumCenterMetricInTriangle(MTriangle *base, const double *metric,
                                   const std::vector<double> &Us,
                                   const std::vector<double> &Vs);
 bool invMapUV(MTriangle *t, double *p,
@@ -46,12 +44,11 @@ class MTri3
   MTri3 *neigh[3];
 
  public :
-  //  char done;
-  bool isDeleted () const { return deleted; }
-  void forceRadius (double r){ circum_radius = r; }
-  double getRadius () const { return circum_radius; }
+  bool isDeleted() const { return deleted; }
+  void forceRadius(double r) { circum_radius = r; }
+  double getRadius() const { return circum_radius; }
 
-  MTri3(MTriangle *t, double lc);
+  MTri3(MTriangle *t, double lc, SMetric3 *m = 0);
   inline MTriangle *tri() const { return base; }
   inline void  setNeigh(int iN , MTri3 *n) { neigh[iN] = n; }
   inline MTri3 *getNeigh(int iN ) const { return neigh[iN]; }
@@ -65,29 +62,29 @@ class MTri3
   {
     return inCircumCircle(v->x(), v->y());
   }
-  inline void setDeleted (bool d){ deleted = d; }
+  inline void setDeleted(bool d){ deleted = d; }
   inline bool assertNeigh() const
   {
-    if (deleted) return true;
-    for (int i = 0; i < 3; i++)
-      if (neigh[i] && (neigh[i]->isNeigh(this) == false)) return false;
+    if(deleted) return true;
+    for(int i = 0; i < 3; i++)
+      if(neigh[i] && (neigh[i]->isNeigh(this) == false)) return false;
     return true;
   }
   inline bool isNeigh(const MTri3 *t) const
   {
-    for (int i = 0; i < 3; i++)
-      if (neigh[i] == t) return true;
+    for(int i = 0; i < 3; i++)
+      if(neigh[i] == t) return true;
     return false;
   }
 };
 
 class compareTri3Ptr
 {
-public:
+ public:
   inline bool operator () (const MTri3 *a, const MTri3 *b)  const
   {
-    if (a->getRadius() > b->getRadius()) return true;
-    if (a->getRadius() < b->getRadius()) return false;
+    if(a->getRadius() > b->getRadius()) return true;
+    if(a->getRadius() < b->getRadius()) return false;
     return a<b;
   }
 };
@@ -95,8 +92,8 @@ public:
 void connectTriangles(std::list<MTri3*> &);
 void connectTriangles(std::vector<MTri3*> &);
 void connectTriangles(std::set<MTri3*,compareTri3Ptr> &AllTris);
-void gmshBowyerWatson(GFace *gf);
-void gmshBowyerWatsonFrontal(GFace *gf);
+void bowyerWatson(GFace *gf);
+void bowyerWatsonFrontal(GFace *gf);
 
 struct edgeXface
 {
@@ -111,9 +108,9 @@ struct edgeXface
   }
   inline bool operator < ( const edgeXface &other) const
   {
-    if (v[0] < other.v[0]) return true;
-    if (v[0] > other.v[0]) return false;
-    if (v[1] < other.v[1]) return true;
+    if(v[0] < other.v[0]) return true;
+    if(v[0] > other.v[0]) return false;
+    if(v[1] < other.v[1]) return true;
     return false;
   }
 };
