@@ -11,6 +11,7 @@
 #include "MElement.h"
 #include "MEdge.h"
 #include "meshGFaceDelaunayInsertion.h"
+#include "STensor3.h"
 
 class GFace;
 class MVertex;
@@ -23,23 +24,24 @@ void buildEdgeToTriangle(std::vector<MTriangle*> &, e2t_cont &adj);
 void laplaceSmoothing(GFace *gf);
 void edgeSwappingLawson(GFace *gf);
 
-enum gmshSwapCriterion {SWCR_DEL, SWCR_QUAL, SWCR_NORM, SWCR_CLOSE};
-enum gmshSplitCriterion {SPCR_CLOSE, SPCR_QUAL, SPCR_ALLWAYS};
+enum swapCriterion {SWCR_DEL, SWCR_QUAL, SWCR_NORM, SWCR_CLOSE};
+enum splitCriterion {SPCR_CLOSE, SPCR_QUAL, SPCR_ALLWAYS};
 
 int edgeSwapPass(GFace *gf, 
                  std::set<MTri3*, compareTri3Ptr> &allTris, 
-                 const gmshSwapCriterion &cr,
+                 const swapCriterion &cr,
                  const std::vector<double> &Us, 
                  const std::vector<double> &Vs,
                  const std::vector<double> &vSizes, 
                  const std::vector<double> &vSizesBGM);
 int edgeSplitPass(double maxLC, GFace *gf, 
                   std::set<MTri3*, compareTri3Ptr> &allTris,
-                  const gmshSplitCriterion &cr,   
+                  const splitCriterion &cr,   
                   std::vector<double> &Us,
                   std::vector<double> &Vs,
                   std::vector<double> &vSizes ,
                   std::vector<double> &vSizesBGM);
+void removeFourTrianglesNodes(GFace *gf, bool replace_by_quads);
 int edgeCollapsePass(double minLC, GFace *gf, 
                      std::set<MTri3*, compareTri3Ptr> &allTris,
                      std::vector<double> &Us,
@@ -50,11 +52,12 @@ void buildMeshGenerationDataStructures(GFace *gf,
                                        std::set<MTri3*, compareTri3Ptr> &AllTris,
                                        std::vector<double> &vSizes,
                                        std::vector<double> &vSizesBGM,
+                                       std::vector<SMetric3> &vMetricsBGM,
                                        std::vector<double> &Us,
                                        std::vector<double> &Vs);
 void transferDataStructure(GFace *gf, std::set<MTri3*, compareTri3Ptr> &AllTris,
                            std::vector<double> &Us, std::vector<double> &Vs);
-void gmshRecombineIntoQuads(GFace *gf);
+void recombineIntoQuads(GFace *gf);
 
 struct swapquad{
   int v[4];
