@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -8,7 +8,9 @@
 #include "GFace.h"
 #include "GRegion.h"
 #include "MVertex.h"
-#include "MElement.h"
+#include "MTetrahedron.h"
+#include "MHexahedron.h"
+#include "MPrism.h"
 #include "Context.h"
 #include "GmshMessage.h"
 
@@ -97,13 +99,13 @@
                                       tab[i + 1][j + 1][k    ])
 
 static double transfiniteHex(double f1, double f2, double f3, double f4, 
-			     double f5, double f6,
-			     double c1, double c2, double c3, double c4, 
-			     double c5, double c6, double c7, double c8, 
-			     double c9, double c10, double c11, double c12,
-			     double s1, double s2, double s3, double s4, 
-			     double s5, double s6, double s7, double s8,
-			     double u, double v, double w)
+                             double f5, double f6,
+                             double c1, double c2, double c3, double c4, 
+                             double c5, double c6, double c7, double c8, 
+                             double c9, double c10, double c11, double c12,
+                             double s1, double s2, double s3, double s4, 
+                             double s5, double s6, double s7, double s8,
+                             double u, double v, double w)
 {
   return (1-u)*f4 + u*f2 + (1-v)*f1 + v*f3 + (1-w)*f5 + w*f6 -
     ((1-u)*(1-v)*c9 + (1-u)*v*c12 + u*(1-v)*c10 + u*v*c11) -
@@ -114,14 +116,14 @@ static double transfiniteHex(double f1, double f2, double f3, double f4,
 }
 
 static MVertex *transfiniteHex(GRegion *gr, 
-			       MVertex *f1, MVertex *f2, MVertex *f3, MVertex *f4, 
-			       MVertex *f5, MVertex *f6,
-			       MVertex *c1, MVertex *c2, MVertex *c3, MVertex *c4, 
-			       MVertex *c5, MVertex *c6, MVertex *c7, MVertex *c8, 
-			       MVertex *c9, MVertex *c10, MVertex *c11, MVertex *c12,
-			       MVertex *s1, MVertex *s2, MVertex *s3, MVertex *s4, 
-			       MVertex *s5, MVertex *s6, MVertex *s7, MVertex *s8,
-			       double u, double v, double w)
+                               MVertex *f1, MVertex *f2, MVertex *f3, MVertex *f4, 
+                               MVertex *f5, MVertex *f6,
+                               MVertex *c1, MVertex *c2, MVertex *c3, MVertex *c4, 
+                               MVertex *c5, MVertex *c6, MVertex *c7, MVertex *c8, 
+                               MVertex *c9, MVertex *c10, MVertex *c11, MVertex *c12,
+                               MVertex *s1, MVertex *s2, MVertex *s3, MVertex *s4, 
+                               MVertex *s5, MVertex *s6, MVertex *s7, MVertex *s8,
+                               double u, double v, double w)
 {
   double x = transfiniteHex(f1->x(), f2->x(), f3->x(), f4->x(), f5->x(), f6->x(),
                             c1->x(), c2->x(), c3->x(), c4->x(), c5->x(), c6->x(),
@@ -156,7 +158,7 @@ public:
     : _gf(0), _LL(0), _HH(0), _permutation(-1), _index(-1)
   {
   }
-  GOrientedTransfiniteFace(GFace *gf, std::vector<GVertex*> &corners) 
+  GOrientedTransfiniteFace(GFace *gf, std::vector<MVertex*> &corners)
     : _gf(gf), _LL(0), _HH(0), _permutation(-1), _index(-1)
   { 
     _LL = gf->transfinite_vertices.size() - 1;
@@ -169,24 +171,19 @@ public:
     std::vector<MVertex*> s(8);
     if(corners.size() == 8){
       for(int i = 0; i < 8; i++)
-        s[i] = corners[i]->mesh_vertices[0];
+        s[i] = corners[i];
     }
     else if(corners.size() == 6){
-      s[0] = corners[0]->mesh_vertices[0];
-      s[1] = corners[1]->mesh_vertices[0];
-      s[2] = corners[2]->mesh_vertices[0];
-      s[3] = corners[0]->mesh_vertices[0];
-      s[4] = corners[3]->mesh_vertices[0];
-      s[5] = corners[4]->mesh_vertices[0];
-      s[6] = corners[5]->mesh_vertices[0];
-      s[7] = corners[3]->mesh_vertices[0];
+      s[0] = corners[0]; s[1] = corners[1]; s[2] = corners[2]; s[3] = corners[0];
+      s[4] = corners[3]; s[5] = corners[4]; s[6] = corners[5]; s[7] = corners[3];
     }
     else
       return;
     
     // get the corners of the transfinite surface mesh
     std::vector<MVertex*> c(4);
-    if(_gf->meshAttributes.corners.size() == 4){
+    if(_gf->meshAttributes.corners.empty() || 
+       _gf->meshAttributes.corners.size() == 4){
       c[0] = _gf->transfinite_vertices[0][0];
       c[1] = _gf->transfinite_vertices[_LL][0];
       c[2] = _gf->transfinite_vertices[_LL][_HH];
@@ -261,6 +258,53 @@ public:
   }
 };
 
+void findTransfiniteCorners(GRegion *gr, std::vector<MVertex*> &corners)
+{
+  if(gr->meshAttributes.corners.size()){
+    // corners have been specified explicitly
+    for(unsigned int i = 0; i < gr->meshAttributes.corners.size(); i++)
+      corners.push_back(gr->meshAttributes.corners[i]->mesh_vertices[0]);
+  }
+  else{
+    // try to find the corners automatically
+    std::list<GFace*> faces = gr->faces();
+    GFace *gf = 0;
+    if(faces.size() == 6){
+      // any face will do as a starting face
+      gf = faces.front();
+    }
+    else if(faces.size() == 5){
+      // we need to start with a triangular face
+      for(std::list<GFace*>::iterator it = faces.begin(); it != faces.end(); it++){
+        if((*it)->edges().size() == 3 || (*it)->meshAttributes.corners.size() == 3){
+          gf = *it;
+          break;
+        }
+      }
+    }
+    if(gf){
+      std::list<GEdge*> fedges = gf->edges();
+      std::list<GEdge*> redges = gr->edges();
+      for(std::list<GEdge*>::iterator it = fedges.begin(); it != fedges.end(); it++)
+        redges.erase(std::find(redges.begin(), redges.end(), *it));
+      findTransfiniteCorners(gf, corners);
+      unsigned int N = corners.size();
+      for(unsigned int i = 0; i < N; i++){
+        for(std::list<GEdge*>::iterator it = redges.begin(); it != redges.end(); it++){
+          if((*it)->getBeginVertex()->mesh_vertices[0] == corners[i]){
+            corners.push_back((*it)->getEndVertex()->mesh_vertices[0]);
+            break;
+          }
+          else if((*it)->getEndVertex()->mesh_vertices[0] == corners[i]){
+            corners.push_back((*it)->getBeginVertex()->mesh_vertices[0]);
+            break;
+          }
+        }
+      }
+    }
+  }
+}
+
 int MeshTransfiniteVolume(GRegion *gr)
 {
   if(gr->meshAttributes.Method != MESH_TRANSFINITE) return 0;
@@ -273,12 +317,20 @@ int MeshTransfiniteVolume(GRegion *gr)
     return 0;
   }
 
+  std::vector<MVertex*> corners;
+  findTransfiniteCorners(gr, corners);
+  if(corners.size() != 6 && corners.size() != 8){
+    Msg::Error("Volume %d is transfinite but has %d corners",
+               gr->tag(), corners.size());
+    return 0;
+  }
+  
   std::vector<GOrientedTransfiniteFace> orientedFaces(6);
   for(std::list<GFace*>::iterator it = faces.begin(); it != faces.end(); ++it){
-    GOrientedTransfiniteFace f(*it, gr->meshAttributes.corners);
+    GOrientedTransfiniteFace f(*it, corners);
     if(f.index() < 0){
       Msg::Error("Incompatible surface %d in transfinite volume %d", 
-		 (*it)->tag(), gr->tag());
+                 (*it)->tag(), gr->tag());
       return 0;
     }
     orientedFaces[f.index()] = f;
@@ -364,7 +416,7 @@ int MeshTransfiniteVolume(GRegion *gr)
         MVertex *f1 = orientedFaces[1].getVertex(j, k);
         MVertex *f2 = orientedFaces[2].getVertex(i, k);
         MVertex *f3;
-        if(gr->meshAttributes.corners.size() == 8)
+        if(corners.size() == 8)
           f3 = orientedFaces[3].getVertex(j, k);
         else
           f3 = c8;
