@@ -13,6 +13,17 @@
 #include "meshPartition.h"
 #include "meshPartitionObjects.h"
 #include "meshPartitionOptions.h"
+#include "MTriangle.h"
+#include "MQuadrangle.h"
+#include "MTetrahedron.h"
+#include "MHexahedron.h"
+#include "MPrism.h"
+#include "MPyramid.h"
+#include "MElementCut.h"
+#include "MPoint.h"
+#include "partitionVertex.h"
+#include "partitionEdge.h"
+#include "partitionFace.h"
 
 //--Prototype for Chaco interface
 
@@ -124,6 +135,10 @@ int PartitionMesh(GModel *const model, meshPartitionOptions &options)
   }
 
   model->recomputeMeshPartitions();
+
+  //  if (options.createPartitionBoundaries)
+  CreatePartitionBoundaries (model);
+
   Msg::Info("Partitioning complete");
   Msg::StatusBar(1, false, "Mesh");
   return 0;
@@ -154,9 +169,9 @@ int PartitionGraph(Graph &graph, meshPartitionOptions &options)
       // Some setup (similar to that of Chaco/input/input.c)
       if(options.global_method != 2) options.rqi_flag = 0;
       if(options.global_method == 1 || options.rqi_flag) {
-	if (options.vmax < 2*(1 << options.ndims)) {
+        if (options.vmax < 2*(1 << options.ndims)) {
           options.vmax = 2*(1 << options.ndims);
-	}
+        }
       }
       // Ensure num_partitions reflects values used by Chaco
       switch(options.architecture) {
@@ -284,11 +299,13 @@ int MakeGraph(GModel *const model, Graph &graph, BoElemGrVec *const boElemGrVec)
     ElemTypeTetra = 0,
     ElemTypeHexa = 1,
     ElemTypePrism = 2,
-    ElemTypePyramid = 3
+    ElemTypePyramid = 3,
+    ElemTypePolyh = 4
   };
   enum {
     ElemTypeTri = 0,
     ElemTypeQuad = 1,
+    ElemTypePolyg = 2
   };
 
   int ier = 0;  
@@ -303,7 +320,7 @@ int MakeGraph(GModel *const model, Graph &graph, BoElemGrVec *const boElemGrVec)
 
 //--Get the dimension of the mesh and count the numbers of elements
 
-      unsigned numElem[4];
+      unsigned numElem[5];
       const int meshDim = model->getNumMeshElements(numElem);
       if(meshDim < 2) {
         Msg::Error("No mesh elements were found");
@@ -317,9 +334,10 @@ int MakeGraph(GModel *const model, Graph &graph, BoElemGrVec *const boElemGrVec)
         {
           try {
             // Allocate memory for the graph
-            const int numGrVert = numElem[ElemTypeTri] + numElem[ElemTypeQuad];
-            const int maxGrEdge =
-              (numElem[ElemTypeTri]*3 + numElem[ElemTypeQuad]*4)/2;
+            const int numGrVert = numElem[ElemTypeTri] + numElem[ElemTypeQuad]
+                                  + numElem[ElemTypePolyg];
+            const int maxGrEdge = (numElem[ElemTypeTri]*3 + numElem[ElemTypeQuad]*4
+                                   + numElem[ElemTypePolyg]*4)/2;
             graph.allocate(numGrVert, maxGrEdge);
             // Make the graph
             MakeGraphDIM<2>(model->firstFace(), model->lastFace(),
@@ -338,10 +356,12 @@ int MakeGraph(GModel *const model, Graph &graph, BoElemGrVec *const boElemGrVec)
             // Allocate memory for the graph
             const int numGrVert =
               numElem[ElemTypeTetra] + numElem[ElemTypeHexa] +
-              numElem[ElemTypePrism] + numElem[ElemTypePyramid];
+              numElem[ElemTypePrism] + numElem[ElemTypePyramid] +
+              numElem[ElemTypePolyh];
             const int maxGrEdge =
               (numElem[ElemTypeTetra]*4 + numElem[ElemTypeHexa]*6 +
-               (numElem[ElemTypePrism] + numElem[ElemTypePyramid])*5)/2;
+               numElem[ElemTypePrism]*5 + numElem[ElemTypePyramid]*5 +
+               numElem[ElemTypePolyh]*5)/2;
             graph.allocate(numGrVert, maxGrEdge);
             // Make the graph
             MakeGraphDIM<3>(model->firstRegion(), model->lastRegion(),
@@ -538,8 +558,8 @@ struct MakeGraphFromEntity
   static void eval(const GEntity *const entity, FaceMap &faceMap,
                    GrVertexMap &grVertMap, Graph &graph)
   {
-    unsigned numElem[4];
-    numElem[0] = 0; numElem[1] = 0; numElem[2] = 0; numElem[3] = 0;
+    unsigned numElem[5];
+    numElem[0] = 0; numElem[1] = 0; numElem[2] = 0; numElem[3] = 0; numElem[4] = 0;
     entity->getNumMeshElements(numElem);
     // Loop over all types of elements
     int nType = entity->getNumElementTypes();
@@ -613,8 +633,8 @@ struct MatchBoElemToGrVertex
                    const GrVertexMap &grVertMap, const Graph &graph,
                    std::vector<BoElemGr> &boElemGrVec)
   {
-    unsigned numElem[4];
-    numElem[0] = 0; numElem[1] = 0; numElem[2] = 0; numElem[3] = 0;
+    unsigned numElem[5];
+    numElem[0] = 0; numElem[1] = 0; numElem[2] = 0; numElem[3] = 0; numElem[4] = 0;
     entity->getNumMeshElements(numElem);
     // Loop over all types of elements
     int nType = entity->getNumElementTypes();
@@ -639,6 +659,270 @@ struct MatchBoElemToGrVertex
 };
 
 
+template <class ITERATOR>
+void fillit_(std::multimap<MFace,MElement*,Less_Face> &faceToElement,
+	     ITERATOR it_beg, ITERATOR it_end)
+{
+  for (ITERATOR IT = it_beg; IT != it_end ; ++IT){
+    MElement *el = *IT;
+    for(int j=0;j < el->getNumFaces();j++) {
+      MFace e = el->getFace(j);
+      faceToElement.insert(std::make_pair(e,el));
+    }// for every edge of the elem
+  }// for every elem of the face
+}
+
+template <class ITERATOR>
+void fillit_(std::multimap<MEdge,MElement*,Less_Edge> &edgeToElement,
+	     ITERATOR it_beg, ITERATOR it_end)
+{
+  for (ITERATOR IT = it_beg; IT != it_end ; ++IT){
+    MElement *el = *IT;
+    for(int j=0;j < el->getNumEdges();j++) {
+      MEdge e = el->getEdge(j);
+      edgeToElement.insert(std::make_pair(e,el));
+    }// for every edge of the elem
+  }// for every elem of the face
+}
+
+template <class ITERATOR>
+void fillit_(std::multimap<MVertex*,MElement*> &vertexToElement,
+	     ITERATOR it_beg, ITERATOR it_end)
+{
+  for (ITERATOR IT = it_beg; IT != it_end ; ++IT){
+    MElement *el = *IT;
+    for(int j=0;j < el->getNumVertices();j++) {
+      MVertex* e = el->getVertex(j);
+      vertexToElement.insert(std::make_pair(e,el));
+    }// for every edge of the elem
+  }// for every elem of the face
+}
+
+void assignPartitionBoundary(GModel *model,
+			     MFace &me,
+			     std::set<partitionFace*, Less_partitionFace> &pfaces,
+			     std::vector<MElement*> &v)
+{
+  std::vector<int> v2;
+  v2.push_back(v[0]->getPartition());
+  for (unsigned int i = 1; i < v.size(); i++){
+    bool found = false;
+    for (unsigned int j = 0; j < v2.size(); j++){
+      if (v[i]->getPartition() == v2[j]){
+	found = true;
+	break;
+      }
+    }
+    if (!found)v2.push_back(v[i]->getPartition());
+  }
+  if (v2.size() < 2)return;
+  
+  partitionFace pe(model, 1, v2);
+  std::set<partitionFace*, Less_partitionFace>::iterator it = pfaces.find(&pe);
+  partitionFace *ppe;
+  if (it == pfaces.end()){
+    ppe = new  partitionFace(model, -(int)pfaces.size()-1, v2);
+    pfaces.insert (ppe);
+    model->add(ppe);
+    printf("created partitionFace %d (", ppe->tag());
+    for (unsigned int i = 0; i < v2.size(); ++i) printf("%d ", v2[i]);
+    printf(")\n");
+  }
+  else ppe = *it;
+  // to finish !!
+  ppe->triangles.push_back(new MTriangle (me.getVertex(0),me.getVertex(1),
+					  me.getVertex(2)));
+}
+
+void assignPartitionBoundary(GModel *model,
+			     MEdge &me,
+			     std::set<partitionEdge*, Less_partitionEdge> &pedges,
+			     std::vector<MElement*> &v,
+			     std::set<partitionFace*, Less_partitionFace> &pfaces)
+{
+  std::vector<int> v2;
+  v2.push_back(v[0]->getPartition());
+  for (unsigned int i = 1; i < v.size(); i++){
+    bool found = false;
+    for (unsigned int j = 0; j < v2.size(); j++){
+      if (v[i]->getPartition() == v2[j]){
+	found = true;
+	break;
+      }
+    }
+    if (!found)v2.push_back(v[i]->getPartition());
+  }
+  if (v2.size() < 2)return;
+
+  partitionFace pf(model, 1, v2);
+  std::set<partitionFace*, Less_partitionFace>::iterator itf = pfaces.find(&pf);
+  if (itf != pfaces.end())return;
+  
+  partitionEdge pe  (model, 1, 0, 0, v2);
+  std::set<partitionEdge*, Less_partitionEdge>::iterator it = pedges.find(&pe);
+  partitionEdge *ppe;
+  if (it == pedges.end()){
+    ppe = new  partitionEdge(model, -(int)pedges.size()-1, 0, 0, v2);
+    pedges.insert (ppe);
+    model->add(ppe);
+    printf("created partitionEdge %d (", ppe->tag());
+    for (unsigned int i = 0; i < v2.size(); ++i) printf("%d ", v2[i]);
+    printf(")\n");
+  }
+  else ppe = *it;
+  ppe->lines.push_back(new MLine (me.getVertex(0),me.getVertex(1)));
+}
+
+void assignPartitionBoundary(GModel *model,
+			     MVertex *ve,
+			     std::set<partitionVertex*, Less_partitionVertex> &pvertices,
+			     std::vector<MElement*> &v,
+			     std::set<partitionEdge*, Less_partitionEdge> &pedges,
+			     std::set<partitionFace*, Less_partitionFace> &pfaces)
+{
+  std::vector<int> v2;
+  v2.push_back(v[0]->getPartition());
+  for (unsigned int i = 1; i < v.size(); i++){
+    bool found = false;
+    for (unsigned int j = 0; j < v2.size(); j++){
+      if (v[i]->getPartition() == v2[j]){
+	found = true;
+	break;
+      }
+    }
+    if (!found)v2.push_back(v[i]->getPartition());
+  }
+  if (v2.size() < 2)return;
+  
+  partitionFace pf(model, 1, v2);
+  std::set<partitionFace*, Less_partitionFace>::iterator itf = pfaces.find(&pf);
+  if (itf != pfaces.end()) return;
+
+  partitionEdge pe(model, 1, 0, 0, v2);
+  std::set<partitionEdge*, Less_partitionEdge>::iterator ite = pedges.find(&pe);
+  if (ite != pedges.end()) return;
+
+  partitionVertex pv(model, 1, v2);
+  std::set<partitionVertex*, Less_partitionVertex>::iterator it = pvertices.find(&pv);
+  partitionVertex *ppv;
+  if (it == pvertices.end()){
+    ppv = new  partitionVertex(model, -(int)pvertices.size()-1,v2);
+    pvertices.insert (ppv);
+    model->add(ppv);
+    printf("created partitionVertex %d (", ppv->tag());
+    for (unsigned int i = 0; i < v2.size(); ++i) printf("%d ", v2[i]);
+    printf(")\n");
+  }
+  else ppv = *it;
+  ppv->points.push_back(new MPoint (ve));
+}
+
+int CreatePartitionBoundaries(GModel *model)
+{
+  unsigned numElem[5];
+  const int meshDim = model->getNumMeshElements(numElem);
+  std::set<partitionEdge*, Less_partitionEdge> pedges;
+  std::set<partitionVertex*, Less_partitionVertex> pvertices;
+  std::set<partitionFace*, Less_partitionFace> pfaces;
+
+  // assign partition faces
+  if (meshDim == 3){
+    std::multimap<MFace,MElement*,Less_Face> faceToElement;    
+    for(GModel::riter it = model->firstRegion(); it != model->lastRegion(); ++it){
+      fillit_ ( faceToElement,(*it)->tetrahedra.begin(),(*it)->tetrahedra.end());
+      fillit_ ( faceToElement,(*it)->hexahedra.begin(),(*it)->hexahedra.end());
+      fillit_ ( faceToElement,(*it)->prisms.begin(),(*it)->prisms.end());
+      fillit_ ( faceToElement,(*it)->pyramids.begin(),(*it)->pyramids.end());
+      fillit_ ( faceToElement,(*it)->polyhedra.begin(),(*it)->polyhedra.end());
+    }    
+    {
+      std::multimap<MFace,MElement*,Less_Face>::iterator it = faceToElement.begin();
+      Equal_Face oper;
+      while ( it != faceToElement.end() ){
+	MFace e = it->first;
+	std::vector<MElement*> voe;
+	do {
+	  voe.push_back(it->second);
+	  ++it;
+	  if (it ==  faceToElement.end() ) break;
+	}while (oper (e,it->first));
+	assignPartitionBoundary (model,e,pfaces,voe);
+      }
+    }
+  }
+  
+  // assign partition edges
+  if (meshDim > 1){
+    std::multimap<MEdge,MElement*,Less_Edge> edgeToElement;
+    if (meshDim == 2){
+      for(GModel::fiter it = model->firstFace(); it != model->lastFace(); ++it){
+	fillit_ ( edgeToElement,(*it)->triangles.begin(),(*it)->triangles.end());
+	fillit_ ( edgeToElement,(*it)->quadrangles.begin(),(*it)->quadrangles.end());
+	fillit_ ( edgeToElement,(*it)->polygons.begin(),(*it)->polygons.end());
+      }
+    }
+    else if (meshDim == 3){
+      for(GModel::riter it = model->firstRegion(); it != model->lastRegion(); ++it){
+	fillit_ ( edgeToElement,(*it)->tetrahedra.begin(),(*it)->tetrahedra.end());
+	fillit_ ( edgeToElement,(*it)->hexahedra.begin(),(*it)->hexahedra.end());
+	fillit_ ( edgeToElement,(*it)->prisms.begin(),(*it)->prisms.end());
+	fillit_ ( edgeToElement,(*it)->pyramids.begin(),(*it)->pyramids.end());
+	fillit_ ( edgeToElement,(*it)->polyhedra.begin(),(*it)->polyhedra.end());
+      }    
+    }
+    {
+      std::multimap<MEdge,MElement*,Less_Edge>::iterator it = edgeToElement.begin();
+      Equal_Edge oper;
+      while ( it != edgeToElement.end() ){
+	MEdge e = it->first;
+	std::vector<MElement*> voe;
+	do {
+	  voe.push_back(it->second);
+	  ++it;
+	  if (it ==  edgeToElement.end() ) break;
+	}while (oper (e,it->first));
+	assignPartitionBoundary (model,e,pedges,voe,pfaces);
+      }
+    }
+  }
+
+  // make partition vertices
+  if (meshDim > 1){
+    std::multimap<MVertex*,MElement*> vertexToElement;
+    if (meshDim == 2){
+      for(GModel::fiter it = model->firstFace(); it != model->lastFace(); ++it){
+	fillit_ ( vertexToElement,(*it)->triangles.begin(),(*it)->triangles.end());
+	fillit_ ( vertexToElement,(*it)->quadrangles.begin(),(*it)->quadrangles.end());
+	fillit_ ( vertexToElement,(*it)->polygons.begin(),(*it)->polygons.end());
+      }
+    }
+    else if (meshDim == 3){
+      for(GModel::riter it = model->firstRegion(); it != model->lastRegion(); ++it){
+	fillit_ ( vertexToElement,(*it)->tetrahedra.begin(),(*it)->tetrahedra.end());
+	fillit_ ( vertexToElement,(*it)->hexahedra.begin(),(*it)->hexahedra.end());
+	fillit_ ( vertexToElement,(*it)->prisms.begin(),(*it)->prisms.end());
+	fillit_ ( vertexToElement,(*it)->pyramids.begin(),(*it)->pyramids.end());
+	fillit_ ( vertexToElement,(*it)->polyhedra.begin(),(*it)->polyhedra.end());
+      }
+    }    
+    {
+      std::multimap<MVertex*,MElement*>::iterator it = vertexToElement.begin();
+      while ( it != vertexToElement.end() ){
+	MVertex *v = it->first;
+	std::vector<MElement*> voe;
+	do {
+	  voe.push_back(it->second);
+	  ++it;
+	  if (it ==  vertexToElement.end() ) break;
+	}while (v == it->first);
+	assignPartitionBoundary (model,v,pvertices,voe,pedges,pfaces);
+      }
+    }
+  }
+
+  return 1;
+}
+  
 /*******************************************************************************
  *
  * Explicit instantiations of routine MakeGraphDIM
