@@ -7,7 +7,7 @@
 #define _NUMERIC_H_
 
 #include <math.h>
-#include "GmshMatrix.h"
+#include "fullMatrix.h"
 
 #define myhypot(a,b) (sqrt((a)*(a)+(b)*(b)))
 #define sign(x)      (((x)>=0)?1:-1)
@@ -58,19 +58,20 @@ double inv2x2(double mat[2][2], double inv[2][2]);
 double angle_02pi(double A3);
 double angle_plan(double V[3], double P1[3], double P2[3], double n[3]);
 double triangle_area(double p0[3], double p1[3], double p2[3]);
+void circumCenterXY(double *p1, double *p2, double *p3, double *res);
+void circumCenterXYZ(double *p1, double *p2, double *p3, double *res, double *uv=0);
 char float2char(float f);
 float char2float(char c);
 void eigenvalue(double mat[3][3], double re[3]);
 void FindCubicRoots(const double coeff[4], double re[3], double im[3]);
 void eigsort(double d[3]);
-double InterpolateIso(double *X, double *Y, double *Z, 
-                      double *Val, double V, int I1, int I2, 
-                      double *XI, double *YI ,double *ZI);
 void gradSimplex(double *x, double *y, double *z, double *v, double *grad);
 double ComputeVonMises(double *val);
 double ComputeScalarRep(int numComp, double *val);
 void invert_singular_matrix3x3(double MM[3][3], double II[3][3]);
-bool newton_fd(void (*func)(gmshVector<double> &, gmshVector<double> &, void *),
-               gmshVector<double> &x, void *data, double relax=1., double tolx=1.e-6);
+bool newton_fd(void (*func)(fullVector<double> &, fullVector<double> &, void *),
+               fullVector<double> &x, void *data, double relax=1., double tolx=1.e-6);
+double minimize_grad_fd(double (*func)(fullVector<double> &, void *),
+                        fullVector<double> &x, void *data);
 
 #endif
