@@ -79,84 +79,90 @@
      tBoundingBox = 295,
      tDraw = 296,
      tToday = 297,
-     tPoint = 298,
-     tCircle = 299,
-     tEllipse = 300,
-     tLine = 301,
-     tSphere = 302,
-     tPolarSphere = 303,
-     tSurface = 304,
-     tSpline = 305,
-     tVolume = 306,
-     tCharacteristic = 307,
-     tLength = 308,
-     tParametric = 309,
-     tElliptic = 310,
-     tPlane = 311,
-     tRuled = 312,
-     tTransfinite = 313,
-     tComplex = 314,
-     tPhysical = 315,
-     tCompound = 316,
-     tUsing = 317,
-     tPlugin = 318,
-     tRotate = 319,
-     tTranslate = 320,
-     tSymmetry = 321,
-     tDilate = 322,
-     tExtrude = 323,
-     tLoop = 324,
-     tRecombine = 325,
-     tSmoother = 326,
-     tSplit = 327,
-     tDelete = 328,
-     tCoherence = 329,
-     tIntersect = 330,
-     tLayers = 331,
-     tHole = 332,
-     tAlias = 333,
-     tAliasWithOptions = 334,
-     tText2D = 335,
-     tText3D = 336,
-     tInterpolationScheme = 337,
-     tTime = 338,
-     tCombine = 339,
-     tBSpline = 340,
-     tBezier = 341,
-     tNurbs = 342,
-     tOrder = 343,
-     tKnots = 344,
-     tColor = 345,
-     tColorTable = 346,
-     tFor = 347,
-     tIn = 348,
-     tEndFor = 349,
-     tIf = 350,
-     tEndIf = 351,
-     tExit = 352,
-     tField = 353,
-     tReturn = 354,
-     tCall = 355,
-     tFunction = 356,
-     tShow = 357,
-     tHide = 358,
-     tGetValue = 359,
-     tGMSH_MAJOR_VERSION = 360,
-     tGMSH_MINOR_VERSION = 361,
-     tGMSH_PATCH_VERSION = 362,
-     tAFFECTDIVIDE = 363,
-     tAFFECTTIMES = 364,
-     tAFFECTMINUS = 365,
-     tAFFECTPLUS = 366,
-     tOR = 367,
-     tAND = 368,
-     tNOTEQUAL = 369,
-     tEQUAL = 370,
-     tGREATEROREQUAL = 371,
-     tLESSOREQUAL = 372,
-     UNARYPREC = 373,
-     tMINUSMINUS = 374,
-     tPLUSPLUS = 375
+     tSyncModel = 298,
+     tCreateTopology = 299,
+     tDistanceFunction = 300,
+     tPoint = 301,
+     tCircle = 302,
+     tEllipse = 303,
+     tLine = 304,
+     tSphere = 305,
+     tPolarSphere = 306,
+     tSurface = 307,
+     tSpline = 308,
+     tVolume = 309,
+     tCharacteristic = 310,
+     tLength = 311,
+     tParametric = 312,
+     tElliptic = 313,
+     tPlane = 314,
+     tRuled = 315,
+     tTransfinite = 316,
+     tComplex = 317,
+     tPhysical = 318,
+     tCompound = 319,
+     tUsing = 320,
+     tPlugin = 321,
+     tDegenerated = 322,
+     tOCCShape = 323,
+     tRotate = 324,
+     tTranslate = 325,
+     tSymmetry = 326,
+     tDilate = 327,
+     tExtrude = 328,
+     tLevelset = 329,
+     tLoop = 330,
+     tRecombine = 331,
+     tSmoother = 332,
+     tSplit = 333,
+     tDelete = 334,
+     tCoherence = 335,
+     tIntersect = 336,
+     tLayers = 337,
+     tHole = 338,
+     tAlias = 339,
+     tAliasWithOptions = 340,
+     tText2D = 341,
+     tText3D = 342,
+     tInterpolationScheme = 343,
+     tTime = 344,
+     tCombine = 345,
+     tBSpline = 346,
+     tBezier = 347,
+     tNurbs = 348,
+     tNurbsOrder = 349,
+     tNurbsKnots = 350,
+     tColor = 351,
+     tColorTable = 352,
+     tFor = 353,
+     tIn = 354,
+     tEndFor = 355,
+     tIf = 356,
+     tEndIf = 357,
+     tExit = 358,
+     tField = 359,
+     tReturn = 360,
+     tCall = 361,
+     tFunction = 362,
+     tShow = 363,
+     tHide = 364,
+     tGetValue = 365,
+     tGMSH_MAJOR_VERSION = 366,
+     tGMSH_MINOR_VERSION = 367,
+     tGMSH_PATCH_VERSION = 368,
+     tAFFECTDIVIDE = 369,
+     tAFFECTTIMES = 370,
+     tAFFECTMINUS = 371,
+     tAFFECTPLUS = 372,
+     tOR = 373,
+     tAND = 374,
+     tNOTEQUAL = 375,
+     tEQUAL = 376,
+     tGREATEROREQUAL = 377,
+     tLESSOREQUAL = 378,
+     UNARYPREC = 379,
+     tMINUSMINUS = 380,
+     tPLUSPLUS = 381
    };
 #endif
 /* Tokens.  */
@@ -200,91 +206,97 @@
 #define tBoundingBox 295
 #define tDraw 296
 #define tToday 297
-#define tPoint 298
-#define tCircle 299
-#define tEllipse 300
-#define tLine 301
-#define tSphere 302
-#define tPolarSphere 303
-#define tSurface 304
-#define tSpline 305
-#define tVolume 306
-#define tCharacteristic 307
-#define tLength 308
-#define tParametric 309
-#define tElliptic 310
-#define tPlane 311
-#define tRuled 312
-#define tTransfinite 313
-#define tComplex 314
-#define tPhysical 315
-#define tCompound 316
-#define tUsing 317
-#define tPlugin 318
-#define tRotate 319
-#define tTranslate 320
-#define tSymmetry 321
-#define tDilate 322
-#define tExtrude 323
-#define tLoop 324
-#define tRecombine 325
-#define tSmoother 326
-#define tSplit 327
-#define tDelete 328
-#define tCoherence 329
-#define tIntersect 330
-#define tLayers 331
-#define tHole 332
-#define tAlias 333
-#define tAliasWithOptions 334
-#define tText2D 335
-#define tText3D 336
-#define tInterpolationScheme 337
-#define tTime 338
-#define tCombine 339
-#define tBSpline 340
-#define tBezier 341
-#define tNurbs 342
-#define tOrder 343
-#define tKnots 344
-#define tColor 345
-#define tColorTable 346
-#define tFor 347
-#define tIn 348
-#define tEndFor 349
-#define tIf 350
-#define tEndIf 351
-#define tExit 352
-#define tField 353
-#define tReturn 354
-#define tCall 355
-#define tFunction 356
-#define tShow 357
-#define tHide 358
-#define tGetValue 359
-#define tGMSH_MAJOR_VERSION 360
-#define tGMSH_MINOR_VERSION 361
-#define tGMSH_PATCH_VERSION 362
-#define tAFFECTDIVIDE 363
-#define tAFFECTTIMES 364
-#define tAFFECTMINUS 365
-#define tAFFECTPLUS 366
-#define tOR 367
-#define tAND 368
-#define tNOTEQUAL 369
-#define tEQUAL 370
-#define tGREATEROREQUAL 371
-#define tLESSOREQUAL 372
-#define UNARYPREC 373
-#define tMINUSMINUS 374
-#define tPLUSPLUS 375
+#define tSyncModel 298
+#define tCreateTopology 299
+#define tDistanceFunction 300
+#define tPoint 301
+#define tCircle 302
+#define tEllipse 303
+#define tLine 304
+#define tSphere 305
+#define tPolarSphere 306
+#define tSurface 307
+#define tSpline 308
+#define tVolume 309
+#define tCharacteristic 310
+#define tLength 311
+#define tParametric 312
+#define tElliptic 313
+#define tPlane 314
+#define tRuled 315
+#define tTransfinite 316
+#define tComplex 317
+#define tPhysical 318
+#define tCompound 319
+#define tUsing 320
+#define tPlugin 321
+#define tDegenerated 322
+#define tOCCShape 323
+#define tRotate 324
+#define tTranslate 325
+#define tSymmetry 326
+#define tDilate 327
+#define tExtrude 328
+#define tLevelset 329
+#define tLoop 330
+#define tRecombine 331
+#define tSmoother 332
+#define tSplit 333
+#define tDelete 334
+#define tCoherence 335
+#define tIntersect 336
+#define tLayers 337
+#define tHole 338
+#define tAlias 339
+#define tAliasWithOptions 340
+#define tText2D 341
+#define tText3D 342
+#define tInterpolationScheme 343
+#define tTime 344
+#define tCombine 345
+#define tBSpline 346
+#define tBezier 347
+#define tNurbs 348
+#define tNurbsOrder 349
+#define tNurbsKnots 350
+#define tColor 351
+#define tColorTable 352
+#define tFor 353
+#define tIn 354
+#define tEndFor 355
+#define tIf 356
+#define tEndIf 357
+#define tExit 358
+#define tField 359
+#define tReturn 360
+#define tCall 361
+#define tFunction 362
+#define tShow 363
+#define tHide 364
+#define tGetValue 365
+#define tGMSH_MAJOR_VERSION 366
+#define tGMSH_MINOR_VERSION 367
+#define tGMSH_PATCH_VERSION 368
+#define tAFFECTDIVIDE 369
+#define tAFFECTTIMES 370
+#define tAFFECTMINUS 371
+#define tAFFECTPLUS 372
+#define tOR 373
+#define tAND 374
+#define tNOTEQUAL 375
+#define tEQUAL 376
+#define tGREATEROREQUAL 377
+#define tLESSOREQUAL 378
+#define UNARYPREC 379
+#define tMINUSMINUS 380
+#define tPLUSPLUS 381
 
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 72 "Gmsh.y"
+#line 78 "Gmsh.y"
 {
   char *c;
   int i;
@@ -295,7 +307,7 @@ typedef union YYSTYPE
   List_T *l;
 }
 /* Line 1529 of yacc.c.  */
-#line 299 "Gmsh.tab.hpp"
+#line 311 "Gmsh.tab.hpp"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
