@@ -16,16 +16,15 @@ extern "C"
 
 class GMSH_AnnotatePlugin : public GMSH_PostPlugin
 {
-private:
+ private:
   static double callback(int num, int action, double value, double *opt,
                          double step, double min, double max);
   static std::string callbackStr(int num, int action, std::string value,
                                  std::string &opt);
-public:
+ public:
   GMSH_AnnotatePlugin(){}
-  void getName(char *name) const;
-  void getInfos(char *author, char *copyright, char *helpText) const;
-  void catchErrorMessage(char *errorMessage) const;
+  std::string getName() const { return "Annotate"; }
+  std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
   int getNbOptionsStr() const;
