@@ -19,15 +19,14 @@ class GMSH_CutGridPlugin : public GMSH_PostPlugin
                          double step, double min, double max);
   void addInView(int numsteps, int connect, int nbcomp, 
                  double ***pnts, double ***vals, 
-                 List_T *P, int *nP, 
-                 List_T *L, int *nL, 
-                 List_T *Q, int *nQ);
+                 std::vector<double> &P, int *nP, 
+                 std::vector<double> &L, int *nL, 
+                 std::vector<double> &Q, int *nQ);
   PView *GenerateView (PView *v, int connectPoints);
-public:
+ public:
   GMSH_CutGridPlugin(){}
-  void getName(char *name) const;
-  void getInfos(char *author, char *copyright, char *help_text) const;
-  void catchErrorMessage(char *errorMessage) const;
+  std::string getName() const { return "CutGrid"; }
+  std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
   PView *execute(PView *);
