@@ -7,14 +7,8 @@
 #include "CutPlane.h"
 #include "Context.h"
 
-#if defined(HAVE_FLTK)
+#if defined(HAVE_OPENGL)
 #include "drawContext.h"
-#include "Draw.h"
-#endif
-
-#if defined(WIN32)
-#undef min
-#undef max
 #endif
 
 int GMSH_CutPlanePlugin::iview = 0;
@@ -40,13 +34,13 @@ extern "C"
 
 void GMSH_CutPlanePlugin::draw(void *context)
 {
-#if defined(HAVE_FLTK)
+#if defined(HAVE_OPENGL)
   int num = (int)CutPlaneOptions_Number[7].def;
   drawContext *ctx = (drawContext*)context;
   if(num < 0) num = iview;
   if(num >= 0 && num < (int)PView::list.size()){
     glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
-    glLineWidth(CTX::instance()->lineWidth);
+    glLineWidth((float)CTX::instance()->lineWidth);
     SBoundingBox3d bb = PView::list[num]->getData()->getBoundingBox();
     ctx->drawPlaneInBoundingBox(bb.min().x(), bb.min().y(), bb.min().z(), 
                                 bb.max().x(), bb.max().y(), bb.max().z(), 
@@ -69,9 +63,7 @@ double GMSH_CutPlanePlugin::callback(int num, int action, double value, double *
   default: break;
   }
   *opt = value;
-#if defined(HAVE_FLTK)
-  DrawPlugin(draw);
-#endif
+  GMSH_Plugin::setDrawFunction(draw);
   return 0.;
 }
 
@@ -96,7 +88,8 @@ double GMSH_CutPlanePlugin::callbackC(int num, int action, double value)
 double GMSH_CutPlanePlugin::callbackD(int num, int action, double value)
 {
   return callback(num, action, value, &CutPlaneOptions_Number[3].def,
-                  CTX::instance()->lc/200., -CTX::instance()->lc, CTX::instance()->lc);
+                  CTX::instance()->lc / 200., -CTX::instance()->lc, 
+                  CTX::instance()->lc);
 }
 
 double GMSH_CutPlanePlugin::callbackVol(int num, int action, double value)
@@ -117,25 +110,16 @@ double GMSH_CutPlanePlugin::callbackTarget(int num, int action, double value)
                   0.01, 0., 1.);
 }
 
-void GMSH_CutPlanePlugin::getName(char *name) const
+std::string GMSH_CutPlanePlugin::getHelp() const
 {
-  strcpy(name, "Cut Plane");
-}
-
-void GMSH_CutPlanePlugin::getInfos(char *author, char *copyright,
-                                   char *help_text) const
-{
-  strcpy(author, "C. Geuzaine, J.-F. Remacle");
-  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
-  strcpy(help_text,
-         "Plugin(CutPlane) cuts the view `iView' with\n"
+  return "Plugin(CutPlane) cuts the view `iView' with\n"
          "the plane `A'*X + `B'*Y + `C'*Z + `D' = 0. If\n"
          "`ExtractVolume' is nonzero, the plugin extracts\n"
          "the elements on one side of the plane (depending\n"
          "on the sign of `ExtractVolume'). If `iView' < 0,\n"
          "the plugin is run on the current view.\n"
          "\n"
-         "Plugin(CutPlane) creates one new view.\n");
+         "Plugin(CutPlane) creates one new view.\n";
 }
 
 int GMSH_CutPlanePlugin::getNbOptions() const
@@ -148,19 +132,13 @@ StringXNumber *GMSH_CutPlanePlugin::getOption(int iopt)
   return &CutPlaneOptions_Number[iopt];
 }
 
-void GMSH_CutPlanePlugin::catchErrorMessage(char *errorMessage) const
-{
-  strcpy(errorMessage, "CutPlane failed...");
-}
-
 double GMSH_CutPlanePlugin::levelset(double x, double y, double z, double val) const
 {
-  return CutPlaneOptions_Number[0].def * x +
-    CutPlaneOptions_Number[1].def * y +
+  return CutPlaneOptions_Number[0].def * x + CutPlaneOptions_Number[1].def * y +
     CutPlaneOptions_Number[2].def * z + CutPlaneOptions_Number[3].def;
 }
 
-bool GMSH_CutPlanePlugin::geometricalFilter(gmshMatrix<double> *node_positions) const
+bool GMSH_CutPlanePlugin::geometricalFilter(fullMatrix<double> *node_positions) const
 {
   const double l0 = levelset((*node_positions)(0, 0),
                              (*node_positions)(0, 1),
