@@ -19,15 +19,14 @@ class GMSH_CutPlanePlugin : public GMSH_LevelsetPlugin
   static double callback(int num, int action, double value, double *opt,
                          double step, double min, double max);
   static int iview;
-public:
+ public:
   GMSH_CutPlanePlugin(){}
-  void getName(char *name) const;
-  void getInfos(char *author, char *copyright, char *helpText) const;
-  void catchErrorMessage(char *errorMessage) const;
+  std::string getName() const { return "CutPlane"; }
+  std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
   PView *execute(PView *);
-  virtual bool geometricalFilter(gmshMatrix<double> *) const;
+  virtual bool geometricalFilter(fullMatrix<double> *) const;
 
   static double callbackA(int, int, double);
   static double callbackB(int, int, double);
