@@ -17,9 +17,8 @@ class GMSH_GradientPlugin : public GMSH_PostPlugin
 {
  public:
   GMSH_GradientPlugin(){}
-  void getName(char *name) const;
-  void getInfos(char *author, char *copyright, char *help_text) const;
-  void catchErrorMessage(char *errorMessage) const;
+  std::string getName() const { return "Gradient"; }
+  std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
   PView *execute(PView *);
