@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -13,13 +13,12 @@ extern "C"
   GMSH_Plugin *GMSH_RegisterMakeSimplexPlugin();
 }
 
-class GMSH_MakeSimplexPlugin : public GMSH_Post_Plugin
+class GMSH_MakeSimplexPlugin : public GMSH_PostPlugin
 {
 public:
-  GMSH_MakeSimplexPlugin();
-  void getName(char *name) const;
-  void getInfos(char *author, char *copyright, char *help_text) const;
-  void catchErrorMessage(char *errorMessage) const;
+  GMSH_MakeSimplexPlugin(){}
+  std::string getName() const { return "MakeSimplex"; }
+  std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber* getOption(int iopt);  
   PView *execute(PView *);
