@@ -18,11 +18,10 @@ class GMSH_ProbePlugin : public GMSH_PostPlugin
   double levelset(double x, double y, double z, double val) const;
   static double callback(int num, int action, double value, double *opt);
   static int iview;
-public:
+ public:
   GMSH_ProbePlugin(){}
-  void getName(char *name) const;
-  void getInfos(char *author, char *copyright, char *helpText) const;
-  void catchErrorMessage(char *errorMessage) const;
+  std::string getName() const { return "Probe"; }
+  std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
   PView *execute(PView *);
