@@ -30,22 +30,13 @@ extern "C"
   }
 }
 
-void GMSH_RemovePlugin::getName(char *name) const
+std::string GMSH_RemovePlugin::getHelp() const
 {
-  strcpy(name, "Remove");
-}
-
-void GMSH_RemovePlugin::getInfos(char *author, char *copyright,
-                                   char *help_text) const
-{
-  strcpy(author, "C. Geuzaine, J.-F. Remacle");
-  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
-  strcpy(help_text,
-         "Plugin(Remove) removes the marked items\n"
+  return "Plugin(Remove) removes the marked items\n"
          "from the view `iView'. If `iView' < 0, the plugin\n"
          "is run on the current view.\n"
          "\n"
-         "Plugin(Remove) is executed in-place.\n");
+         "Plugin(Remove) is executed in-place.\n";
 }
 
 int GMSH_RemovePlugin::getNbOptions() const
@@ -58,11 +49,6 @@ StringXNumber *GMSH_RemovePlugin::getOption(int iopt)
   return &RemoveOptions_Number[iopt];
 }
 
-void GMSH_RemovePlugin::catchErrorMessage(char *errorMessage) const
-{
-  strcpy(errorMessage, "Remove failed...");
-}
-
 PView *GMSH_RemovePlugin::execute(PView *v)
 {
   int iView = (int)RemoveOptions_Number[13].def;
@@ -78,50 +64,50 @@ PView *GMSH_RemovePlugin::execute(PView *v)
   int tensor = (int)RemoveOptions_Number[12].def;
 
   if(RemoveOptions_Number[0].def){ 
-    data1->NbT2 = 0; List_Reset(data1->T2D); List_Reset(data1->T2C); 
+    data1->NbT2 = 0; data1->T2D.clear(); data1->T2C.clear(); 
   }
   if(RemoveOptions_Number[1].def){ 
-    data1->NbT3 = 0; List_Reset(data1->T3D); List_Reset(data1->T3C); 
+    data1->NbT3 = 0; data1->T3D.clear(); data1->T3C.clear(); 
   }
   if(RemoveOptions_Number[2].def){ 
-    if(scalar){ data1->NbSP = 0; List_Reset(data1->SP); }
-    if(vector){ data1->NbVP = 0; List_Reset(data1->VP); }
-    if(tensor){ data1->NbTP = 0; List_Reset(data1->TP); }
+    if(scalar){ data1->NbSP = 0; data1->SP.clear(); }
+    if(vector){ data1->NbVP = 0; data1->VP.clear(); }
+    if(tensor){ data1->NbTP = 0; data1->TP.clear(); }
   }
   if(RemoveOptions_Number[3].def){
-    if(scalar){ data1->NbSL = 0; List_Reset(data1->SL); }
-    if(vector){ data1->NbVL = 0; List_Reset(data1->VL); }
-    if(tensor){ data1->NbTL = 0; List_Reset(data1->TL); }
+    if(scalar){ data1->NbSL = 0; data1->SL.clear(); }
+    if(vector){ data1->NbVL = 0; data1->VL.clear(); }
+    if(tensor){ data1->NbTL = 0; data1->TL.clear(); }
   }
   if(RemoveOptions_Number[4].def){ 
-    if(scalar){ data1->NbST = 0; List_Reset(data1->ST); }
-    if(vector){ data1->NbVT = 0; List_Reset(data1->VT); }
-    if(tensor){ data1->NbTT = 0; List_Reset(data1->TT); }
+    if(scalar){ data1->NbST = 0; data1->ST.clear(); }
+    if(vector){ data1->NbVT = 0; data1->VT.clear(); }
+    if(tensor){ data1->NbTT = 0; data1->TT.clear(); }
   }
   if(RemoveOptions_Number[5].def){ 
-    if(scalar){ data1->NbSQ = 0; List_Reset(data1->SQ); }
-    if(vector){ data1->NbVQ = 0; List_Reset(data1->VQ); }
-    if(tensor){ data1->NbTQ = 0; List_Reset(data1->TQ); }
+    if(scalar){ data1->NbSQ = 0; data1->SQ.clear(); }
+    if(vector){ data1->NbVQ = 0; data1->VQ.clear(); }
+    if(tensor){ data1->NbTQ = 0; data1->TQ.clear(); }
   }
   if(RemoveOptions_Number[6].def){
-    if(scalar){ data1->NbSS = 0; List_Reset(data1->SS); }
-    if(vector){ data1->NbVS = 0; List_Reset(data1->VS); }
-    if(tensor){ data1->NbTS = 0; List_Reset(data1->TS); }
+    if(scalar){ data1->NbSS = 0; data1->SS.clear(); }
+    if(vector){ data1->NbVS = 0; data1->VS.clear(); }
+    if(tensor){ data1->NbTS = 0; data1->TS.clear(); }
   }
   if(RemoveOptions_Number[7].def){
-    if(scalar){ data1->NbSH = 0; List_Reset(data1->SH); }
-    if(vector){ data1->NbVH = 0; List_Reset(data1->VH); }
-    if(tensor){ data1->NbTH = 0; List_Reset(data1->TH); }
+    if(scalar){ data1->NbSH = 0; data1->SH.clear(); }
+    if(vector){ data1->NbVH = 0; data1->VH.clear(); }
+    if(tensor){ data1->NbTH = 0; data1->TH.clear(); }
   }
   if(RemoveOptions_Number[8].def){
-    if(scalar){ data1->NbSI = 0; List_Reset(data1->SI); }
-    if(vector){ data1->NbVI = 0; List_Reset(data1->VI); }
-    if(tensor){ data1->NbTI = 0; List_Reset(data1->TI); }
+    if(scalar){ data1->NbSI = 0; data1->SI.clear(); }
+    if(vector){ data1->NbVI = 0; data1->VI.clear(); }
+    if(tensor){ data1->NbTI = 0; data1->TI.clear(); }
   }
   if(RemoveOptions_Number[9].def){
-    if(scalar){ data1->NbSY = 0; List_Reset(data1->SY); }
-    if(vector){ data1->NbVY = 0; List_Reset(data1->VY); }
-    if(tensor){ data1->NbTY = 0; List_Reset(data1->TY); }
+    if(scalar){ data1->NbSY = 0; data1->SY.clear(); }
+    if(vector){ data1->NbVY = 0; data1->VY.clear(); }
+    if(tensor){ data1->NbTY = 0; data1->TY.clear(); }
   }
 
   data1->finalize();
