@@ -15,11 +15,10 @@ extern "C"
 
 class GMSH_RemovePlugin : public GMSH_PostPlugin
 {
-public:
+ public:
   GMSH_RemovePlugin(){}
-  void getName(char *name) const;
-  void getInfos(char *author, char *copyright, char *helpText) const;
-  void catchErrorMessage(char *errorMessage) const;
+  std::string getName() const { return "Remove"; }
+  std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
   PView *execute(PView *);
