@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -13,13 +13,12 @@ extern "C"
   GMSH_Plugin *GMSH_RegisterSphericalRaisePlugin();
 }
 
-class GMSH_SphericalRaisePlugin : public GMSH_Post_Plugin
+class GMSH_SphericalRaisePlugin : public GMSH_PostPlugin
 {
-public:
+ public:
   GMSH_SphericalRaisePlugin(){}
-  void getName(char *name) const;
-  void getInfos(char *author, char *copyright, char *helpText) const;
-  void catchErrorMessage(char *errorMessage) const;
+  std::string getName() const { return "SphericalRaise"; }
+  std::string getHelp() const;
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
   PView *execute(PView *);
