@@ -10,10 +10,8 @@
 #include "Context.h"
 #include "PViewOptions.h"
 
-#if defined(HAVE_FLTK)
-#include <FL/gl.h>
+#if defined(HAVE_OPENGL)
 #include "drawContext.h"
-#include "Draw.h"
 #endif
 
 StringXNumber StreamLinesOptions_Number[] = {
@@ -45,7 +43,7 @@ extern "C"
 
 void GMSH_StreamLinesPlugin::draw(void *context)
 {
-#if defined(HAVE_FLTK)
+#if defined(HAVE_OPENGL)
   glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
   drawContext *ctx = (drawContext*)context;
   double p[3];
@@ -68,9 +66,7 @@ double GMSH_StreamLinesPlugin::callback(int num, int action, double value, doubl
   default: break;
   }
   *opt = value;
-#if defined(HAVE_FLTK)
-  DrawPlugin(draw);
-#endif
+  GMSH_Plugin::setDrawFunction(draw);
   return 0.;
 }
 
@@ -140,18 +136,9 @@ double GMSH_StreamLinesPlugin::callbackV(int num, int action, double value)
                   1, 1, 100);
 }
 
-void GMSH_StreamLinesPlugin::getName(char *name) const
+std::string GMSH_StreamLinesPlugin::getHelp() const
 {
-  strcpy(name, "Stream Lines");
-}
-
-void GMSH_StreamLinesPlugin::getInfos(char *author, char *copyright,
-                                      char *help_text) const
-{
-  strcpy(author, "C. Geuzaine, J.-F. Remacle");
-  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
-  strcpy(help_text,
-         "Plugin(StreamLines) computes stream lines\n"
+  return "Plugin(StreamLines) computes stream lines\n"
          "from the `TimeStep'-th time step of a vector\n"
          "view `iView' and optionally interpolates the\n"
          "scalar view `dView' on the resulting stream\n"
@@ -172,7 +159,7 @@ void GMSH_StreamLinesPlugin::getInfos(char *author, char *copyright,
          "\n"
          "Plugin(StreamLines) creates one new view. This\n"
          "view contains multi-step vector points if `dView'\n"
-         "< 0, or single-step scalar lines if `dView' >= 0.\n");
+         "< 0, or single-step scalar lines if `dView' >= 0.\n";
 }
 
 int GMSH_StreamLinesPlugin::getNbOptions() const
@@ -185,11 +172,6 @@ StringXNumber *GMSH_StreamLinesPlugin::getOption(int iopt)
   return &StreamLinesOptions_Number[iopt];
 }
 
-void GMSH_StreamLinesPlugin::catchErrorMessage(char *errorMessage) const
-{
-  strcpy(errorMessage, "StreamLines failed...");
-}
-
 int GMSH_StreamLinesPlugin::getNbU()
 {
   return (int)StreamLinesOptions_Number[9].def;
@@ -244,7 +226,7 @@ PView *GMSH_StreamLinesPlugin::execute(PView *v)
     o2 = new OctreePost(v2);
   }
 
-  PView *v3 = new PView(true);
+  PView *v3 = new PView();
   PViewDataList *data3 = getDataList(v3);
 
   const double b1 = 1. / 3., b2 = 2. / 3., b3 = 1. / 3., b4 = 1. / 6.;
@@ -261,9 +243,9 @@ PView *GMSH_StreamLinesPlugin::execute(PView *v)
       }
       else{
         data3->NbVP++;
-        List_Add(data3->VP, &X[0]);
-        List_Add(data3->VP, &X[1]);
-        List_Add(data3->VP, &X[2]);            
+        data3->VP.push_back(X[0]);
+        data3->VP.push_back(X[1]);
+        data3->VP.push_back(X[2]);            
       }
 
       int currentTimeStep = 0;
@@ -275,11 +257,11 @@ PView *GMSH_StreamLinesPlugin::execute(PView *v)
         if(timeStep < 0){
           double T0 = data1->getTime(0);
           double currentT = T0 + DT * iter;
-          List_Add(data3->Time, &currentT);
+          data3->Time.push_back(currentT);
           for(; currentTimeStep < data1->getNumTimeSteps() - 1 && 
                 currentT > 0.5 * (data1->getTime(currentTimeStep) + 
                                   data1->getTime(currentTimeStep + 1));
-              currentTimeStep++);
+          currentTimeStep++);
         }
         else{
           currentTimeStep = timeStep;
@@ -291,7 +273,7 @@ PView *GMSH_StreamLinesPlugin::execute(PView *v)
         // X3 = X + a3 * DT * V(X2)
         // X4 = X + a4 * DT * V(X3)
         // X = X + b1 X1 + b2 X2 + b3 X3 + b4 x4
-	double val[3];
+        double val[3];
         o1.searchVector(X[0], X[1], X[2], val, currentTimeStep);
         for(int k = 0; k < 3; k++) X1[k] = X[k] + DT * val[k] * a1;
         o1.searchVector(X1[0], X1[1], X1[2], val, currentTimeStep);
@@ -308,22 +290,19 @@ PView *GMSH_StreamLinesPlugin::execute(PView *v)
 
         if(data2){
           data3->NbSL++;
-          List_Add(data3->SL, &XPREV[0]);
-          List_Add(data3->SL, &X[0]);
-          List_Add(data3->SL, &XPREV[1]);
-          List_Add(data3->SL, &X[1]);
-          List_Add(data3->SL, &XPREV[2]);
-          List_Add(data3->SL, &X[2]);
+          data3->SL.push_back(XPREV[0]); data3->SL.push_back(X[0]);
+          data3->SL.push_back(XPREV[1]); data3->SL.push_back(X[1]);
+          data3->SL.push_back(XPREV[2]); data3->SL.push_back(X[2]);
           for(int k = 0; k < data2->getNumTimeSteps(); k++)
-            List_Add(data3->SL, &val2[k]);
+            data3->SL.push_back(val2[k]);
           o2->searchScalar(X[0], X[1], X[2], val2, -1);
           for(int k = 0; k < data2->getNumTimeSteps(); k++)
-            List_Add(data3->SL, &val2[k]);
+            data3->SL.push_back(val2[k]);
         }
         else{
-          List_Add(data3->VP, &DX[0]);
-          List_Add(data3->VP, &DX[1]);
-          List_Add(data3->VP, &DX[2]);         
+          data3->VP.push_back(DX[0]);
+          data3->VP.push_back(DX[1]);
+          data3->VP.push_back(DX[2]);         
         }
       }
     }
