@@ -23,18 +23,9 @@ extern "C"
   }
 }
 
-void GMSH_WarpPlugin::getName(char *name) const
+std::string GMSH_WarpPlugin::getHelp() const
 {
-  strcpy(name, "Warp");
-}
-
-void GMSH_WarpPlugin::getInfos(char *author, char *copyright,
-			       char *help_text) const
-{
-  strcpy(author, "C. Geuzaine, J.-F. Remacle");
-  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
-  strcpy(help_text,
-         "Plugin(Warp) transforms the elements in the\n"
+  return "Plugin(Warp) transforms the elements in the\n"
          "view `iView' by adding to their node coordinates\n"
          "the vector field stored in the `TimeStep'-th time\n"
          "step of the view `dView', scaled by `Factor'. If\n"
@@ -45,7 +36,7 @@ void GMSH_WarpPlugin::getInfos(char *author, char *copyright,
          "parameter.) If `iView' < 0, the plugin is run on\n"
          "the current view.\n"
          "\n"
-         "Plugin(Warp) is executed in-place.\n");
+         "Plugin(Warp) is executed in-place.\n";
 }
 
 int GMSH_WarpPlugin::getNbOptions() const
@@ -58,11 +49,6 @@ StringXNumber *GMSH_WarpPlugin::getOption(int iopt)
   return &WarpOptions_Number[iopt];
 }
 
-void GMSH_WarpPlugin::catchErrorMessage(char *errorMessage) const
-{
-  strcpy(errorMessage, "Warp failed...");
-}
-
 PView *GMSH_WarpPlugin::execute(PView *v)
 {
   double factor = WarpOptions_Number[0].def;
@@ -97,16 +83,16 @@ PView *GMSH_WarpPlugin::execute(PView *v)
     normals = new smooth_normals(AngleTol);
     for(int ent = 0; ent < data1->getNumEntities(0); ent++){
       for(int ele = 0; ele < data1->getNumElements(0, ent); ele++){
-	if(data1->skipElement(0, ent, ele)) continue;
-	int numEdges = data1->getNumEdges(0, ent, ele);
-	if(numEdges == 3 || numEdges == 4){
-	  double x[4], y[4], z[4], n[4];
-	  for(int nod = 0; nod < numEdges; nod++)
-	    data1->getNode(0, ent, ele, nod, x[nod], y[nod], z[nod]);
-	  normal3points(x[0], y[0], z[0], x[1], y[1], z[1], x[2], y[2], z[2], n);
-	  for(int nod = 0; nod < numEdges; nod++)
-	    normals->add(x[nod], y[nod], z[nod], n[0], n[1], n[2]);
-	}
+        if(data1->skipElement(0, ent, ele)) continue;
+        int numEdges = data1->getNumEdges(0, ent, ele);
+        if(numEdges == 3 || numEdges == 4){
+          double x[4], y[4], z[4], n[4];
+          for(int nod = 0; nod < numEdges; nod++)
+            data1->getNode(0, ent, ele, nod, x[nod], y[nod], z[nod]);
+          normal3points(x[0], y[0], z[0], x[1], y[1], z[1], x[2], y[2], z[2], n);
+          for(int nod = 0; nod < numEdges; nod++)
+            normals->add(x[nod], y[nod], z[nod], n[0], n[1], n[2]);
+        }
       }
     }
   }
@@ -115,9 +101,9 @@ PView *GMSH_WarpPlugin::execute(PView *v)
   for(int step = 0; step < data1->getNumTimeSteps(); step++){
     for(int ent = 0; ent < data1->getNumEntities(step); ent++){
       for(int ele = 0; ele < data1->getNumElements(step, ent); ele++){
-	if(data1->skipElement(step, ent, ele)) continue;
-	for(int nod = 0; nod < data1->getNumNodes(step, ent, ele); nod++)
-	  data1->tagNode(step, ent, ele, nod, 0);
+        if(data1->skipElement(step, ent, ele)) continue;
+        for(int nod = 0; nod < data1->getNumNodes(step, ent, ele); nod++)
+          data1->tagNode(step, ent, ele, nod, 0);
       }
     }
   }
@@ -126,35 +112,35 @@ PView *GMSH_WarpPlugin::execute(PView *v)
   for(int step = 0; step < data1->getNumTimeSteps(); step++){
     for(int ent = 0; ent < data1->getNumEntities(step); ent++){
       for(int ele = 0; ele < data1->getNumElements(step, ent); ele++){
-	if(data1->skipElement(step, ent, ele)) continue;
-	int numNodes = data1->getNumNodes(step, ent, ele);
-	double x[8], y[8], z[8], n[3] = {0., 0., 0.};
-	int tag[8];
-	for(int nod = 0; nod < numNodes; nod++)
-	  tag[nod] = data1->getNode(step, ent, ele, nod, x[nod], y[nod], z[nod]);
-	int dim = data1->getDimension(step, ent, ele);
-	if(normals && dim == 2)
-	  normal3points(x[0], y[0], z[0], x[1], y[1], z[1], x[2], y[2], z[2], n);
-	for(int nod = 0; nod < numNodes; nod++){
-	  if(tag[nod]) continue; // already transformed
-	  double mult = 1., val[3] = {n[0], n[1], n[2]};
-	  if(normals){
-	    if(dim == 2){
-	      normals->get(x[nod], y[nod], z[nod], val[0], val[1], val[2]);
-	      data1->getScalarValue(step, ent, ele, nod, mult);
-	    }
-	  }
-	  else if(data2->getNumComponents(TimeStep, ent, ele) == 3 &&
-		  data2->getNumNodes(TimeStep, ent, ele) == numNodes){
-	    for(int comp = 0; comp < 3; comp++)
-	      data2->getValue(TimeStep, ent, ele, nod, comp, val[comp]);
-	  }
-	  x[nod] += factor * mult * val[0];
-	  y[nod] += factor * mult * val[1];
-	  z[nod] += factor * mult * val[2];
-	  data1->setNode(step, ent, ele, nod, x[nod], y[nod], z[nod]);
-	  data1->tagNode(step, ent, ele, nod, 1);
-	}
+        if(data1->skipElement(step, ent, ele)) continue;
+        int numNodes = data1->getNumNodes(step, ent, ele);
+        double x[8], y[8], z[8], n[3] = {0., 0., 0.};
+        int tag[8];
+        for(int nod = 0; nod < numNodes; nod++)
+          tag[nod] = data1->getNode(step, ent, ele, nod, x[nod], y[nod], z[nod]);
+        int dim = data1->getDimension(step, ent, ele);
+        if(normals && dim == 2)
+          normal3points(x[0], y[0], z[0], x[1], y[1], z[1], x[2], y[2], z[2], n);
+        for(int nod = 0; nod < numNodes; nod++){
+          if(tag[nod]) continue; // already transformed
+          double mult = 1., val[3] = {n[0], n[1], n[2]};
+          if(normals){
+            if(dim == 2){
+              normals->get(x[nod], y[nod], z[nod], val[0], val[1], val[2]);
+              data1->getScalarValue(step, ent, ele, nod, mult);
+            }
+          }
+          else if(data2->getNumComponents(TimeStep, ent, ele) == 3 &&
+                  data2->getNumNodes(TimeStep, ent, ele) == numNodes){
+            for(int comp = 0; comp < 3; comp++)
+              data2->getValue(TimeStep, ent, ele, nod, comp, val[comp]);
+          }
+          x[nod] += factor * mult * val[0];
+          y[nod] += factor * mult * val[1];
+          z[nod] += factor * mult * val[2];
+          data1->setNode(step, ent, ele, nod, x[nod], y[nod], z[nod]);
+          data1->tagNode(step, ent, ele, nod, 1);
+        }
       }
     }
   }
