@@ -5,7 +5,6 @@
 
 #include "Octree.h"
 #include "OctreePost.h"
-#include "ListUtils.h"
 #include "PView.h"
 #include "PViewDataList.h"
 #include "PViewDataGModel.h"
@@ -192,14 +191,10 @@ void pyrCentroid(void *a, double *x)
   centroid(5, X, Y, Z, x);
 }
 
-static void addListOfStuff(Octree *o, List_T *l, int nbelm)
+static void addListOfStuff(Octree *o, std::vector<double> &l, int nbelm)
 {
-  if(!l) return;
- 
-  for(int i = 0; i < List_Nbr(l); i += nbelm){
-    double * X = (double *)List_Pointer_Fast(l, i);
-    Octree_Insert(X, o);
-  }
+  for(unsigned int i = 0; i < l.size(); i += nbelm)
+    Octree_Insert(&l[i], o);
 }
 
 // OctreePost implementation
@@ -230,8 +225,8 @@ OctreePost::OctreePost(PView *v)
     SBoundingBox3d bb = _theViewDataList->getBoundingBox();
     double min[3] = {bb.min().x(), bb.min().y(), bb.min().z()};
     double size[3] = {bb.max().x() - bb.min().x(),
-		      bb.max().y() - bb.min().y(),
-		      bb.max().z() - bb.min().z()};                   
+                      bb.max().y() - bb.min().y(),
+                      bb.max().z() - bb.min().z()};                   
     const int maxElePerBucket = 100; // memory vs. speed trade-off
     
     PViewDataList *l = _theViewDataList;
@@ -339,7 +334,7 @@ bool OctreePost::_getValue(void *in, int dim, int nbNod, int nbComp,
 } 
 
 bool OctreePost::_getValue(void *in, int nbComp, double P[3], int timestep,
-			   double *values, double *elementSize)
+                           double *values, double *elementSize)
 {
   if(!in) return false;
 
@@ -364,7 +359,7 @@ bool OctreePost::_getValue(void *in, int nbComp, double P[3], int timestep,
       for(int nod = 0; nod < e->getNumVertices(); nod++){
         for(int comp = 0; comp < nbComp; comp++){
           if(!_theViewDataGModel->getValueByIndex(step, dataIndex[nod], nod, comp, 
-						  nodeval[nod * nbComp + comp]))
+                                                  nodeval[nod * nbComp + comp]))
             return false;
         }
       }
