@@ -1,8 +1,9 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
+#include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "PViewDataGModel.h"
 #include "MVertex.h"
@@ -14,8 +15,12 @@ bool PViewDataGModel::addData(GModel *model, std::map<int, std::vector<double> >
                               int step, double time, int partition)
 {
   if(data.empty()) return false;
-  
-  int numComp = data.begin()->second.size();
+
+  int numComp = 9;
+  for(std::map<int, std::vector<double> >::iterator it = data.begin(); 
+      it != data.end(); it++)
+    numComp = std::min(numComp, (int)it->second.size());
+
   while(step >= (int)_steps.size())
     _steps.push_back(new stepData<double>(model, numComp));
   
@@ -107,15 +112,8 @@ bool PViewDataGModel::writeMSH(std::string fileName, bool binary)
     return false;
   }
 
-  if(_type != NodeData){
-    Msg::Error("Can only export node-based datasets for now");
-    return false;
-  }
-
   GModel *model = _steps[0]->getModel();
 
-  binary = true;
-
   if(!model->writeMSH(fileName, 2.0, binary, true)) return false;
 
   // append data
@@ -126,36 +124,79 @@ bool PViewDataGModel::writeMSH(std::string fileName, bool binary)
   }
 
   for(unsigned int step = 0; step < _steps.size(); step++){
-    int numNodes = 0, numComp = _steps[step]->getNumComponents();
+    int numEnt = 0, numComp = _steps[step]->getNumComponents();
     for(int i = 0; i < _steps[step]->getNumData(); i++)
-      if(_steps[step]->getData(i)) numNodes++;
-    if(numNodes){
-      fprintf(fp, "$NodeData\n");
-      fprintf(fp, "1\n\"%s\"\n", getName().c_str());
-      fprintf(fp, "1\n%.16g\n", _steps[step]->getTime());
-      fprintf(fp, "3\n%d\n%d\n%d\n", step, numComp, numNodes);
-      for(int i = 0; i < _steps[step]->getNumData(); i++){
-        if(_steps[step]->getData(i)){
-          MVertex *v = _steps[step]->getModel()->getMeshVertexByTag(i);
-          if(!v){
-            Msg::Error("Unknown vertex %d in data", i);
-            return false;
-          }
-          int num = v->getIndex();
-          if(binary){
-            fwrite(&num, sizeof(int), 1, fp);
-            fwrite(_steps[step]->getData(i), sizeof(double), numComp, fp);
-          }
-          else{
-            fprintf(fp, "%d", num);
-            for(int k = 0; k < numComp; k++)
-              fprintf(fp, " %.16g", _steps[step]->getData(i)[k]);
-            fprintf(fp, "\n");
+      if(_steps[step]->getData(i)) numEnt++;
+    if(numEnt){
+      if(_type == NodeData){
+        fprintf(fp, "$NodeData\n");
+        fprintf(fp, "1\n\"%s\"\n", getName().c_str());
+        fprintf(fp, "1\n%.16g\n", _steps[step]->getTime());
+        fprintf(fp, "3\n%d\n%d\n%d\n", step, numComp, numEnt);
+        for(int i = 0; i < _steps[step]->getNumData(); i++){
+          if(_steps[step]->getData(i)){
+            MVertex *v = _steps[step]->getModel()->getMeshVertexByTag(i);
+            if(!v){
+              Msg::Error("Unknown vertex %d in data", i);
+              return false;
+            }
+            int num = v->getIndex();
+            if(binary){
+              fwrite(&num, sizeof(int), 1, fp);
+              fwrite(_steps[step]->getData(i), sizeof(double), numComp, fp);
+            }
+            else{
+              fprintf(fp, "%d", num);
+              for(int k = 0; k < numComp; k++)
+                fprintf(fp, " %.16g", _steps[step]->getData(i)[k]);
+              fprintf(fp, "\n");
+            }
           }
         }
+        if(binary) fprintf(fp, "\n");
+        fprintf(fp, "$EndNodeData\n");
+      }
+      else{
+        if(_type == ElementNodeData)
+          fprintf(fp, "$ElementNodeData\n");
+        else
+          fprintf(fp, "$ElementData\n");
+        fprintf(fp, "1\n\"%s\"\n", getName().c_str());
+        fprintf(fp, "1\n%.16g\n", _steps[step]->getTime());
+        fprintf(fp, "3\n%d\n%d\n%d\n", step, numComp, numEnt);
+        for(int i = 0; i < _steps[step]->getNumData(); i++){
+          if(_steps[step]->getData(i)){
+            MElement *e = model->getMeshElementByTag(i);
+            if(!e){
+              Msg::Error("Unknown element %d in data", i);
+              return false;
+            }
+            int mult = 1;
+            if(_type == ElementNodeData)
+              mult = e->getNumVertices();
+            int num = e->getNum();
+            if(binary){
+              fwrite(&num, sizeof(int), 1, fp);
+              if(_type == ElementNodeData)
+                fwrite(&mult, sizeof(int), 1, fp);
+              fwrite(_steps[step]->getData(i), sizeof(double), numComp * mult, fp);
+            }
+            else{
+              fprintf(fp, "%d", num);
+              if(_type == ElementNodeData)
+                fprintf(fp, " %d", mult);
+              for(int k = 0; k < numComp * mult; k++)
+                fprintf(fp, " %.16g", _steps[step]->getData(i)[k]);
+              fprintf(fp, "\n");
+            }
+          }
+        }
+        if(binary) fprintf(fp, "\n");
+        if(_type == ElementNodeData)
+          fprintf(fp, "$EndElementNodeData\n");
+        else
+          fprintf(fp, "$EndElementData\n");
       }
-      if(binary) fprintf(fp, "\n");
-      fprintf(fp, "$EndNodeData\n");
     }
   }
     
@@ -475,7 +516,7 @@ bool PViewDataGModel::writeMED(std::string fileName)
       for(int k = 0; k < numComp; k++)
         val[i * numComp + k] = _steps[step]->getData(indices[i])[k];
     if(MEDchampEcr(fid, meshName, fieldName, (unsigned char*)&val[0], 
-                   MED_FULL_INTERLACE, numNodes, MED_NOGAUSS, MED_ALL,
+                   MED_FULL_INTERLACE, numNodes, (char*)MED_NOGAUSS, MED_ALL,
                    profileName, MED_COMPACT, MED_NOEUD, MED_NONE, (med_int)step,
                    (char*)"unknown", time, MED_NONOR) < 0) {
       Msg::Error("Could not write MED field");
