@@ -7,15 +7,101 @@
 #include <set>
 #include "PViewDataList.h"
 #include "Numeric.h"
+#include "StringUtils.h"
 #include "GmshMessage.h"
+#include "GmshDefines.h"
 #include "Context.h"
 #include "adaptiveData.h"
 
-bool PViewDataList::readPOS(FILE *fp, double version, int format, int size)
+static void dVecRead(std::vector<double> &v, int n, FILE *fp, 
+                     bool binary, int swap)
+{
+  if(!n) return;
+  v.resize(n);
+  if(binary){
+    if(!fread(&v[0], sizeof(double), n, fp))
+      Msg::Error("Read error");
+    if(swap) SwapBytes((char*)&v[0], sizeof(double), n);
+  }
+  else{
+    for(int i = 0; i < n; i++){
+      if(!fscanf(fp, "%lf", &v[i])){
+        Msg::Error("Read error");
+        break;
+      }
+    }
+  }
+}
+
+static void cVecRead(std::vector<char> &v, int n, FILE *fp, 
+                     bool binary, int swap, bool oldStyle)
+{
+  if(!n) return;
+  v.resize(n);
+  if(binary){
+    if(!fread(&v[0], sizeof(char), n, fp))
+      Msg::Error("Read error");
+    if(swap) SwapBytes((char*)&v[0], sizeof(char), n);
+  }
+  else{
+    if(oldStyle){
+      for(int i = 0; i < n; i++){
+        if(!fscanf(fp, "%c", &v[i])){
+          Msg::Error("Read error");
+          break;
+        }
+        if(v[i] == '^') v[i] = '\0';
+      }
+    }
+    else{
+      for(int i = 0; i < n; i++){
+        char c = (char)fgetc(fp);
+        if(c == EOF){
+          Msg::Error("Read error");
+          break;
+        }
+        else{
+          v[i] = c;
+        }
+      }
+    }
+  }
+}
+
+static void dVecWrite(std::vector<double> &v, FILE *fp, bool binary)
+{
+  if(v.empty()) return;
+  if(binary)
+    fwrite(&v[0], sizeof(double), v.size(), fp);
+  else
+    for(unsigned i = 0; i < v.size(); i++)
+      fprintf(fp, " %.16g", v[i]);
+}
+
+static void cVecWrite(std::vector<char> &v, FILE *fp, bool binary)
+{
+  if(v.empty()) return;
+  if(binary)
+    fwrite(&v[0], sizeof(char), v.size(), fp);
+  else
+    for(unsigned i = 0; i < v.size(); i++)
+      fputc(v[i], fp);
+}
+
+bool PViewDataList::readPOS(FILE *fp, double version, bool binary)
 {
   char name[256];
   int t2l, t3l;
 
+  int NbSL2 = 0, NbVL2 = 0, NbTL2 = 0, NbST2 = 0, NbVT2 = 0, NbTT2 = 0;
+  int NbSQ2 = 0, NbVQ2 = 0, NbTQ2 = 0, NbSS2 = 0, NbVS2 = 0, NbTS2 = 0;
+  int NbSH2 = 0, NbVH2 = 0, NbTH2 = 0, NbSI2 = 0, NbVI2 = 0, NbTI2 = 0;
+  int NbSY2 = 0, NbVY2 = 0, NbTY2 = 0;
+  std::vector<double> SL2, VL2, TL2, ST2, VT2, TT2;
+  std::vector<double> SQ2, VQ2, TQ2, SS2, VS2, TS2;
+  std::vector<double> SH2, VH2, TH2, SI2, VI2, TI2;
+  std::vector<double> SY2, VY2, TY2;
+
   if(version <= 1.0) {
     Msg::Debug("Detected post-processing view format <= 1.0");
     if(!fscanf(fp, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
@@ -67,13 +153,13 @@ bool PViewDataList::readPOS(FILE *fp, double version, int format, int size)
     Msg::Error("Unknown post-processing file format (version %g)", version);
     return false;
   }
-  
+
   for(int i = 0; i < (int)strlen(name); i++)
     if(name[i] == '^')
       name[i] = ' ';
   
   int swap = 0;
-  if(format == LIST_FORMAT_BINARY) {
+  if(binary) {
     int testone;
     if(!fread(&testone, sizeof(int), 1, fp)){
       Msg::Error("Read error");
@@ -85,172 +171,129 @@ bool PViewDataList::readPOS(FILE *fp, double version, int format, int size)
     }
   }
   
-  DataSize = size;
+  dVecRead(Time, NbTimeStep, fp, binary, swap);
+  dVecRead(SP, NbSP * (NbTimeStep * 1 + 3), fp, binary, swap);
+  dVecRead(VP, NbVP * (NbTimeStep * 3 + 3), fp, binary, swap);
+  dVecRead(TP, NbTP * (NbTimeStep * 9 + 3), fp, binary, swap);
+  dVecRead(SL, NbSL * (NbTimeStep * 2 * 1 + 6), fp, binary, swap);
+  dVecRead(VL, NbVL * (NbTimeStep * 2 * 3 + 6), fp, binary, swap);
+  dVecRead(TL, NbTL * (NbTimeStep * 2 * 9 + 6), fp, binary, swap);
+  dVecRead(ST, NbST * (NbTimeStep * 3 * 1 + 9), fp, binary, swap);
+  dVecRead(VT, NbVT * (NbTimeStep * 3 * 3 + 9), fp, binary, swap);
+  dVecRead(TT, NbTT * (NbTimeStep * 3 * 9 + 9), fp, binary, swap);
+  dVecRead(SQ, NbSQ * (NbTimeStep * 4 * 1 + 12), fp, binary, swap);
+  dVecRead(VQ, NbVQ * (NbTimeStep * 4 * 3 + 12), fp, binary, swap);
+  dVecRead(TQ, NbTQ * (NbTimeStep * 4 * 9 + 12), fp, binary, swap);
+  dVecRead(SS, NbSS * (NbTimeStep * 4 * 1 + 12), fp, binary, swap);
+  dVecRead(VS, NbVS * (NbTimeStep * 4 * 3 + 12), fp, binary, swap);
+  dVecRead(TS, NbTS * (NbTimeStep * 4 * 9 + 12), fp, binary, swap);
+  dVecRead(SH, NbSH * (NbTimeStep * 8 * 1 + 24), fp, binary, swap);
+  dVecRead(VH, NbVH * (NbTimeStep * 8 * 3 + 24), fp, binary, swap);
+  dVecRead(TH, NbTH * (NbTimeStep * 8 * 9 + 24), fp, binary, swap);
+  dVecRead(SI, NbSI * (NbTimeStep * 6 * 1 + 18), fp, binary, swap);
+  dVecRead(VI, NbVI * (NbTimeStep * 6 * 3 + 18), fp, binary, swap);
+  dVecRead(TI, NbTI * (NbTimeStep * 6 * 9 + 18), fp, binary, swap);
+  dVecRead(SY, NbSY * (NbTimeStep * 5 * 1 + 15), fp, binary, swap);
+  dVecRead(VY, NbVY * (NbTimeStep * 5 * 3 + 15), fp, binary, swap);
+  dVecRead(TY, NbTY * (NbTimeStep * 5 * 9 + 15), fp, binary, swap);
+
+  // overwrite first order data with second order data (if any)
+  dVecRead(SL, NbSL2 * (NbTimeStep * 3 * 1 + 9), fp, binary, swap);
+  dVecRead(VL, NbVL2 * (NbTimeStep * 3 * 3 + 9), fp, binary, swap);
+  dVecRead(TL, NbTL2 * (NbTimeStep * 3 * 9 + 9), fp, binary, swap);
+  dVecRead(ST, NbST2 * (NbTimeStep * 6 * 1 + 18), fp, binary, swap);
+  dVecRead(VT, NbVT2 * (NbTimeStep * 6 * 3 + 18), fp, binary, swap);
+  dVecRead(TT, NbTT2 * (NbTimeStep * 6 * 9 + 18), fp, binary, swap);
+  dVecRead(SQ, NbSQ2 * (NbTimeStep * 9 * 1 + 27), fp, binary, swap);
+  dVecRead(VQ, NbVQ2 * (NbTimeStep * 9 * 3 + 27), fp, binary, swap);
+  dVecRead(TQ, NbTQ2 * (NbTimeStep * 9 * 9 + 27), fp, binary, swap);
+  dVecRead(SS, NbSS2 * (NbTimeStep * 10 * 1 + 30), fp, binary, swap);
+  dVecRead(VS, NbVS2 * (NbTimeStep * 10 * 3 + 30), fp, binary, swap);
+  dVecRead(TS, NbTS2 * (NbTimeStep * 10 * 9 + 30), fp, binary, swap);
+  dVecRead(SH, NbSH2 * (NbTimeStep * 27 * 1 + 81), fp, binary, swap);
+  dVecRead(VH, NbVH2 * (NbTimeStep * 27 * 3 + 81), fp, binary, swap);
+  dVecRead(TH, NbTH2 * (NbTimeStep * 27 * 9 + 81), fp, binary, swap);
+  dVecRead(SI, NbSI2 * (NbTimeStep * 18 * 1 + 54), fp, binary, swap);
+  dVecRead(VI, NbVI2 * (NbTimeStep * 18 * 3 + 54), fp, binary, swap);
+  dVecRead(TI, NbTI2 * (NbTimeStep * 18 * 9 + 54), fp, binary, swap);
+  dVecRead(SY, NbSY2 * (NbTimeStep * 14 * 1 + 42), fp, binary, swap);
+  dVecRead(VY, NbVY2 * (NbTimeStep * 14 * 3 + 42), fp, binary, swap);
+  dVecRead(TY, NbTY2 * (NbTimeStep * 14 * 9 + 42), fp, binary, swap);
+  if(NbSL2){ NbSL = NbSL2; setOrder2(TYPE_LIN); }
+  if(NbVL2){ NbVL = NbVL2; setOrder2(TYPE_LIN); }
+  if(NbTL2){ NbTL = NbTL2; setOrder2(TYPE_LIN); }
+  if(NbST2){ NbST = NbST2; setOrder2(TYPE_TRI); }
+  if(NbVT2){ NbVT = NbVT2; setOrder2(TYPE_TRI); }
+  if(NbTT2){ NbTT = NbTT2; setOrder2(TYPE_TRI); }
+  if(NbSQ2){ NbSQ = NbSQ2; setOrder2(TYPE_QUA); }
+  if(NbVQ2){ NbVQ = NbVQ2; setOrder2(TYPE_QUA); }
+  if(NbTQ2){ NbTQ = NbTQ2; setOrder2(TYPE_QUA); }
+  if(NbSS2){ NbSS = NbSS2; setOrder2(TYPE_TET); }
+  if(NbVS2){ NbVS = NbVS2; setOrder2(TYPE_TET); }
+  if(NbTS2){ NbTS = NbTS2; setOrder2(TYPE_TET); }
+  if(NbSH2){ NbSH = NbSH2; setOrder2(TYPE_HEX); }
+  if(NbVH2){ NbVH = NbVH2; setOrder2(TYPE_HEX); }
+  if(NbTH2){ NbTH = NbTH2; setOrder2(TYPE_HEX); }
+  if(NbSI2){ NbSI = NbSI2; setOrder2(TYPE_PRI); }
+  if(NbVI2){ NbVI = NbVI2; setOrder2(TYPE_PRI); }
+  if(NbTI2){ NbTI = NbTI2; setOrder2(TYPE_PRI); }
+  if(NbSY2){ NbSY = NbSY2; setOrder2(TYPE_PYR); }
+  if(NbVY2){ NbVY = NbVY2; setOrder2(TYPE_PYR); }
+  if(NbTY2){ NbTY = NbTY2; setOrder2(TYPE_PYR); }
+ 
+  dVecRead(T2D, NbT2 * 4, fp, binary, swap);
+  cVecRead(T2C, t2l, fp, binary, swap, (version <= 1.2));
+  dVecRead(T3D, NbT3 * 5, fp, binary, swap);
+  cVecRead(T3C, t3l, fp, binary, swap, (version <= 1.2));
   
-  // Time values
-  Time = List_CreateFromFile(NbTimeStep, 100, size, fp, format, swap);
-  
-  // Note: if nb==0, we still allocates the lists (so that they
-  // are ready to be filled later, e.g. in plugins)
-  
-#define LCD List_CreateFromFile(nb, 1000, size, fp, format, swap)
-  int nb;
-  // Points
-  nb = NbSP ? NbSP * (NbTimeStep * 1 + 3) : 0; SP = LCD;
-  nb = NbVP ? NbVP * (NbTimeStep * 3 + 3) : 0; VP = LCD;
-  nb = NbTP ? NbTP * (NbTimeStep * 9 + 3) : 0; TP = LCD;
-  
-  // Lines
-  nb = NbSL ? NbSL * (NbTimeStep * 2 * 1 + 6) : 0; SL = LCD;
-  nb = NbVL ? NbVL * (NbTimeStep * 2 * 3 + 6) : 0; VL = LCD;
-  nb = NbTL ? NbTL * (NbTimeStep * 2 * 9 + 6) : 0; TL = LCD;
-  
-  // Triangles
-  nb = NbST ? NbST * (NbTimeStep * 3 * 1 + 9) : 0; ST = LCD;
-  nb = NbVT ? NbVT * (NbTimeStep * 3 * 3 + 9) : 0; VT = LCD;
-  nb = NbTT ? NbTT * (NbTimeStep * 3 * 9 + 9) : 0; TT = LCD;
-  
-  // Quadrangles
-  nb = NbSQ ? NbSQ * (NbTimeStep * 4 * 1 + 12) : 0; SQ = LCD;
-  nb = NbVQ ? NbVQ * (NbTimeStep * 4 * 3 + 12) : 0; VQ = LCD;
-  nb = NbTQ ? NbTQ * (NbTimeStep * 4 * 9 + 12) : 0; TQ = LCD;
-  
-  // Tetrahedra
-  nb = NbSS ? NbSS * (NbTimeStep * 4 * 1 + 12) : 0; SS = LCD;
-  nb = NbVS ? NbVS * (NbTimeStep * 4 * 3 + 12) : 0; VS = LCD;
-  nb = NbTS ? NbTS * (NbTimeStep * 4 * 9 + 12) : 0; TS = LCD;
-  
-  // Hexahedra
-  nb = NbSH ? NbSH * (NbTimeStep * 8 * 1 + 24) : 0; SH = LCD;
-  nb = NbVH ? NbVH * (NbTimeStep * 8 * 3 + 24) : 0; VH = LCD;
-  nb = NbTH ? NbTH * (NbTimeStep * 8 * 9 + 24) : 0; TH = LCD;
-  
-  // Prisms
-  nb = NbSI ? NbSI * (NbTimeStep * 6 * 1 + 18) : 0; SI = LCD;
-  nb = NbVI ? NbVI * (NbTimeStep * 6 * 3 + 18) : 0; VI = LCD;
-  nb = NbTI ? NbTI * (NbTimeStep * 6 * 9 + 18) : 0; TI = LCD;
-  
-  // Pyramids
-  nb = NbSY ? NbSY * (NbTimeStep * 5 * 1 + 15) : 0; SY = LCD;
-  nb = NbVY ? NbVY * (NbTimeStep * 5 * 3 + 15) : 0; VY = LCD;
-  nb = NbTY ? NbTY * (NbTimeStep * 5 * 9 + 15) : 0; TY = LCD;
-  
-  // 2nd order Lines
-  nb = NbSL2 ? NbSL2 * (NbTimeStep * 3 * 1 + 9) : 0; SL2 = LCD;
-  nb = NbVL2 ? NbVL2 * (NbTimeStep * 3 * 3 + 9) : 0; VL2 = LCD;
-  nb = NbTL2 ? NbTL2 * (NbTimeStep * 3 * 9 + 9) : 0; TL2 = LCD;
-  
-  // 2nd order Triangles
-  nb = NbST2 ? NbST2 * (NbTimeStep * 6 * 1 + 18) : 0; ST2 = LCD;
-  nb = NbVT2 ? NbVT2 * (NbTimeStep * 6 * 3 + 18) : 0; VT2 = LCD;
-  nb = NbTT2 ? NbTT2 * (NbTimeStep * 6 * 9 + 18) : 0; TT2 = LCD;
-  
-  // 2nd order Quadrangles
-  nb = NbSQ2 ? NbSQ2 * (NbTimeStep * 9 * 1 + 27) : 0; SQ2 = LCD;
-  nb = NbVQ2 ? NbVQ2 * (NbTimeStep * 9 * 3 + 27) : 0; VQ2 = LCD;
-  nb = NbTQ2 ? NbTQ2 * (NbTimeStep * 9 * 9 + 27) : 0; TQ2 = LCD;
-  
-  // 2nd order Tetrahedra
-  nb = NbSS2 ? NbSS2 * (NbTimeStep * 10 * 1 + 30) : 0; SS2 = LCD;
-  nb = NbVS2 ? NbVS2 * (NbTimeStep * 10 * 3 + 30) : 0; VS2 = LCD;
-  nb = NbTS2 ? NbTS2 * (NbTimeStep * 10 * 9 + 30) : 0; TS2 = LCD;
-  
-  // 2nd order Hexahedra
-  nb = NbSH2 ? NbSH2 * (NbTimeStep * 27 * 1 + 81) : 0; SH2 = LCD;
-  nb = NbVH2 ? NbVH2 * (NbTimeStep * 27 * 3 + 81) : 0; VH2 = LCD;
-  nb = NbTH2 ? NbTH2 * (NbTimeStep * 27 * 9 + 81) : 0; TH2 = LCD;
-  
-  // 2nd order Prisms
-  nb = NbSI2 ? NbSI2 * (NbTimeStep * 18 * 1 + 54) : 0; SI2 = LCD;
-  nb = NbVI2 ? NbVI2 * (NbTimeStep * 18 * 3 + 54) : 0; VI2 = LCD;
-  nb = NbTI2 ? NbTI2 * (NbTimeStep * 18 * 9 + 54) : 0; TI2 = LCD;
-  
-  // 2nd order Pyramids
-  nb = NbSY2 ? NbSY2 * (NbTimeStep * 14 * 1 + 42) : 0; SY2 = LCD;
-  nb = NbVY2 ? NbVY2 * (NbTimeStep * 14 * 3 + 42) : 0; VY2 = LCD;
-  nb = NbTY2 ? NbTY2 * (NbTimeStep * 14 * 9 + 42) : 0; TY2 = LCD;
-#undef LCD
-  
-  // 2D strings
-  nb = NbT2 ? NbT2 * 4 : 0;
-  T2D = List_CreateFromFile(nb, 100, size, fp, format, swap);
-  if(version <= 1.2)
-    T2C = List_CreateFromFileOld(t2l, 100, sizeof(char), fp, format, swap);
-  else
-    T2C = List_CreateFromFile(t2l, 100, sizeof(char), fp, format, swap);
-  
-  // 3D strings
-  nb = NbT3 ? NbT3 * 5 : 0;
-  T3D = List_CreateFromFile(nb, 100, size, fp, format, swap);
-  if(version <= 1.2)
-    T3C = List_CreateFromFileOld(t3l, 100, sizeof(char), fp, format, swap);
-  else
-    T3C = List_CreateFromFile(t3l, 100, sizeof(char), fp, format, swap);
-  
-  Msg::Debug(
-      "Read View '%s' (%d TimeSteps): "
-      "SP(%d/%d) VP(%d/%d) TP(%d/%d) "
-      "SL(%d/%d) VL(%d/%d) TL(%d/%d) "
-      "ST(%d/%d) VT(%d/%d) TT(%d/%d) "
-      "SQ(%d/%d) VQ(%d/%d) TQ(%d/%d) "
-      "SS(%d/%d) VS(%d/%d) TS(%d/%d) "
-      "SH(%d/%d) VH(%d/%d) TH(%d/%d) "
-      "SI(%d/%d) VI(%d/%d) TI(%d/%d) "
-      "SY(%d/%d) VY(%d/%d) TY(%d/%d) " 
-      "SL2(%d/%d) VL2(%d/%d) TL2(%d/%d) "
-      "ST2(%d/%d) VT2(%d/%d) TT2(%d/%d) "
-      "SQ2(%d/%d) VQ2(%d/%d) TQ2(%d/%d) "
-      "SS2(%d/%d) VS2(%d/%d) TS2(%d/%d) " 
-      "SH2(%d/%d) VH2(%d/%d) TH2(%d/%d) "
-      "SI2(%d/%d) VI2(%d/%d) TI2(%d/%d) "
-      "SY2(%d/%d) VY2(%d/%d) TY2(%d/%d) "
-      "T2(%d/%d/%d) T3(%d/%d/%d) ", 
-      name, NbTimeStep,
-      NbSP, List_Nbr(SP), NbVP, List_Nbr(VP), NbTP, List_Nbr(TP),
-      NbSL, List_Nbr(SL), NbVL, List_Nbr(VL), NbTL, List_Nbr(TL),
-      NbST, List_Nbr(ST), NbVT, List_Nbr(VT), NbTT, List_Nbr(TT),
-      NbSQ, List_Nbr(SQ), NbVQ, List_Nbr(VQ), NbTQ, List_Nbr(TQ),
-      NbSS, List_Nbr(SS), NbVS, List_Nbr(VS), NbTS, List_Nbr(TS),
-      NbSH, List_Nbr(SH), NbVH, List_Nbr(VH), NbTH, List_Nbr(TH),
-      NbSI, List_Nbr(SI), NbVI, List_Nbr(VI), NbTI, List_Nbr(TI),
-      NbSY, List_Nbr(SY), NbVY, List_Nbr(VY), NbTY, List_Nbr(TY),
-      NbSL2, List_Nbr(SL2), NbVL2, List_Nbr(VL2), NbTL2, List_Nbr(TL2),
-      NbST2, List_Nbr(ST2), NbVT2, List_Nbr(VT2), NbTT2, List_Nbr(TT2),
-      NbSQ2, List_Nbr(SQ2), NbVQ2, List_Nbr(VQ2), NbTQ2, List_Nbr(TQ2),
-      NbSS2, List_Nbr(SS2), NbVS2, List_Nbr(VS2), NbTS2, List_Nbr(TS2),
-      NbSH2, List_Nbr(SH2), NbVH2, List_Nbr(VH2), NbTH2, List_Nbr(TH2),
-      NbSI2, List_Nbr(SI2), NbVI2, List_Nbr(VI2), NbTI2, List_Nbr(TI2),
-      NbSY2, List_Nbr(SY2), NbVY2, List_Nbr(VY2), NbTY2, List_Nbr(TY2),
-      NbT2, List_Nbr(T2D), List_Nbr(T2C), 
-      NbT3, List_Nbr(T3D), List_Nbr(T3C));
+  Msg::Debug("Read View '%s' (%d TimeSteps): "
+             "SP(%d/%d) VP(%d/%d) TP(%d/%d) "
+             "SL(%d/%d) VL(%d/%d) TL(%d/%d) "
+             "ST(%d/%d) VT(%d/%d) TT(%d/%d) "
+             "SQ(%d/%d) VQ(%d/%d) TQ(%d/%d) "
+             "SS(%d/%d) VS(%d/%d) TS(%d/%d) "
+             "SH(%d/%d) VH(%d/%d) TH(%d/%d) "
+             "SI(%d/%d) VI(%d/%d) TI(%d/%d) "
+             "SY(%d/%d) VY(%d/%d) TY(%d/%d) " 
+             "T2(%d/%d/%d) T3(%d/%d/%d) ", 
+             name, NbTimeStep,
+             NbSP, SP.size(), NbVP, VP.size(), NbTP, TP.size(),
+             NbSL, SL.size(), NbVL, VL.size(), NbTL, TL.size(),
+             NbST, ST.size(), NbVT, VT.size(), NbTT, TT.size(),
+             NbSQ, SQ.size(), NbVQ, VQ.size(), NbTQ, TQ.size(),
+             NbSS, SS.size(), NbVS, VS.size(), NbTS, TS.size(),
+             NbSH, SH.size(), NbVH, VH.size(), NbTH, TH.size(),
+             NbSI, SI.size(), NbVI, VI.size(), NbTI, TI.size(),
+             NbSY, SY.size(), NbVY, VY.size(), NbTY, TY.size(),
+             NbT2, T2D.size(), T2C.size(), 
+             NbT3, T3D.size(), T3C.size());
   
   setName(name);
   finalize();
-
   return true;
 }
 
-static void writeTimePOS(FILE *fp, List_T *list)
+static void writeTimePOS(FILE *fp, std::vector<double> &list)
 {
-  if(List_Nbr(list) > 1){
+  if(list.size() > 1){
     fprintf(fp, "TIME{");
-    for(int i = 0; i < List_Nbr(list); i ++){
+    for(unsigned int i = 0; i < list.size(); i++){
       if(i) fprintf(fp, ",");
-      fprintf(fp, "%.16g", *(double *)List_Pointer(list, i));
+      fprintf(fp, "%.16g", list[i]);
     }
     fprintf(fp, "};\n");
   }
 }
 
 static void writeElementPOS(FILE *fp, const char *str, int nbnod, int nb,
-                            List_T *list)
+                            std::vector<double> &list)
 {
   if(nb){
-    int n = List_Nbr(list) / nb;
-    for(int i = 0; i < List_Nbr(list); i += n) {
-      double *x = (double *)List_Pointer(list, i);
-      double *y = (double *)List_Pointer(list, i + nbnod);
-      double *z = (double *)List_Pointer(list, i + 2 * nbnod);
+    int n = list.size() / nb;
+    for(unsigned int i = 0; i < list.size(); i += n) {
+      double *x = &list[i];
+      double *y = &list[i + nbnod];
+      double *z = &list[i + 2 * nbnod];
       fprintf(fp, "%s(", str);
       for(int j = 0; j < nbnod; j++) {
         if(j) fprintf(fp, ",");
@@ -259,38 +302,38 @@ static void writeElementPOS(FILE *fp, const char *str, int nbnod, int nb,
       fprintf(fp, "){");
       for(int j = 3 * nbnod; j < n; j++) {
         if(j - 3 * nbnod) fprintf(fp, ",");
-        fprintf(fp, "%.16g", *(double *)List_Pointer(list, i + j));
+        fprintf(fp, "%.16g", list[i + j]);
       }
       fprintf(fp, "};\n");
     }
   }
 }
 
-static void writeTextPOS(FILE *fp, int nbc, int nb, List_T *TD, List_T *TC)
+static void writeTextPOS(FILE *fp, int nbc, int nb, std::vector<double> &TD, 
+                         std::vector<char> &TC)
 {
   if(!nb || (nbc != 4 && nbc != 5)) return;
-  for(int j = 0; j < List_Nbr(TD); j += nbc){
-    double x, y, z, style, start, end;
-    List_Read(TD, j, &x);
-    List_Read(TD, j+1, &y);
-    if(nbc == 5)
-      List_Read(TD, j+2, &z);
-    List_Read(TD, j+nbc-2, &style);
+  for(unsigned int j = 0; j < TD.size(); j += nbc){
+    double x = TD[j];
+    double y = TD[j + 1];
+    double z = (nbc == 5) ? TD[j + 2] : 0.;
+    double style = TD[j + nbc - 2];
     if(nbc == 4)
       fprintf(fp, "T2(%g,%g,%g){", x, y, style);
     else
       fprintf(fp, "T3(%g,%g,%g,%g){", x, y, z, style);
-    List_Read(TD, j+nbc-1, &start);
-    if(j+nbc*2-1 < List_Nbr(TD))
-      List_Read(TD, j+nbc*2-1, &end);
+    double start = TD[j + nbc - 1];
+    double end;
+    if(j + nbc * 2 - 1 < TD.size())
+      end = TD[j + nbc * 2 - 1];
     else
-      end = List_Nbr(TC);
+      end = TC.size();
     int l = 0;
-    while(l < end-start){
-      char *str = (char*)List_Pointer(TC, (int)start + l);
+    while(l < end - start){
+      char *str = &TC[(int)start + l];
       if(l) fprintf(fp, ",");
       fprintf(fp, "\"%s\"", str);
-      l += strlen(str)+1;
+      l += strlen(str) + 1;
     }
     fprintf(fp, "};\n");
   }
@@ -298,12 +341,10 @@ static void writeTextPOS(FILE *fp, int nbc, int nb, List_T *TD, List_T *TC)
 
 bool PViewDataList::writePOS(std::string fileName, bool binary, bool parsed, bool append)
 {
+  if(_adaptive)
+    return _adaptive->getData()->writePOS(fileName, binary, parsed, append);
 
-  if ( _adaptive ){
-    return _adaptive->getData()->writePOS (fileName,binary,parsed,append);
-  }
-
-  FILE *fp = fopen(fileName.c_str(), 
+  FILE *fp = fopen(fileName.c_str(),
                    append ? (binary ? "ab" : "a") : (binary ? "wb" : "w"));
   if(!fp){
     Msg::Error("Unable to open file '%s'", fileName.c_str());
@@ -328,13 +369,12 @@ bool PViewDataList::writePOS(std::string fileName, bool binary, bool parsed, boo
       fprintf(fp, "%s ", str.c_str());
     fprintf(fp, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
             "%d %d %d %d %d %d %d %d %d %d %d %d\n",
-            List_Nbr(Time),
+            (int)Time.size(),
             NbSP, NbVP, NbTP, NbSL, NbVL, NbTL,
             NbST, NbVT, NbTT, NbSQ, NbVQ, NbTQ,
             NbSS, NbVS, NbTS, NbSH, NbVH, NbTH,
             NbSI, NbVI, NbTI, NbSY, NbVY, NbTY,
-            NbT2, List_Nbr(T2C), NbT3, List_Nbr(T3C));
-    int f = binary ? LIST_FORMAT_BINARY : LIST_FORMAT_ASCII;
+            NbT2, (int)T2C.size(), NbT3, (int)T3C.size());
     if(binary) {
       int one = 1;
       if(!fwrite(&one, sizeof(int), 1, fp)){
@@ -342,21 +382,21 @@ bool PViewDataList::writePOS(std::string fileName, bool binary, bool parsed, boo
         return false;
       }
     }
-    List_WriteToFile(Time, fp, f);
-    List_WriteToFile(SP, fp, f); List_WriteToFile(VP, fp, f); 
-    List_WriteToFile(TP, fp, f); List_WriteToFile(SL, fp, f);
-    List_WriteToFile(VL, fp, f); List_WriteToFile(TL, fp, f);
-    List_WriteToFile(ST, fp, f); List_WriteToFile(VT, fp, f);
-    List_WriteToFile(TT, fp, f); List_WriteToFile(SQ, fp, f);
-    List_WriteToFile(VQ, fp, f); List_WriteToFile(TQ, fp, f);
-    List_WriteToFile(SS, fp, f); List_WriteToFile(VS, fp, f);
-    List_WriteToFile(TS, fp, f); List_WriteToFile(SH, fp, f);
-    List_WriteToFile(VH, fp, f); List_WriteToFile(TH, fp, f);
-    List_WriteToFile(SI, fp, f); List_WriteToFile(VI, fp, f);
-    List_WriteToFile(TI, fp, f); List_WriteToFile(SY, fp, f);
-    List_WriteToFile(VY, fp, f); List_WriteToFile(TY, fp, f);
-    List_WriteToFile(T2D, fp, f); List_WriteToFile(T2C, fp, f);
-    List_WriteToFile(T3D, fp, f); List_WriteToFile(T3C, fp, f);
+    dVecWrite(Time, fp, binary);
+    dVecWrite(SP, fp, binary); dVecWrite(VP, fp, binary); 
+    dVecWrite(TP, fp, binary); dVecWrite(SL, fp, binary);
+    dVecWrite(VL, fp, binary); dVecWrite(TL, fp, binary);
+    dVecWrite(ST, fp, binary); dVecWrite(VT, fp, binary);
+    dVecWrite(TT, fp, binary); dVecWrite(SQ, fp, binary);
+    dVecWrite(VQ, fp, binary); dVecWrite(TQ, fp, binary);
+    dVecWrite(SS, fp, binary); dVecWrite(VS, fp, binary);
+    dVecWrite(TS, fp, binary); dVecWrite(SH, fp, binary);
+    dVecWrite(VH, fp, binary); dVecWrite(TH, fp, binary);
+    dVecWrite(SI, fp, binary); dVecWrite(VI, fp, binary);
+    dVecWrite(TI, fp, binary); dVecWrite(SY, fp, binary);
+    dVecWrite(VY, fp, binary); dVecWrite(TY, fp, binary);
+    dVecWrite(T2D, fp, binary); cVecWrite(T2C, fp, binary);
+    dVecWrite(T3D, fp, binary); cVecWrite(T3C, fp, binary);
     fprintf(fp, "\n");
     fprintf(fp, "$EndView\n");
   }
@@ -406,16 +446,17 @@ class pVertexLessThan{
   }
 };
 
-static void getNodeMSH(int nbelm, List_T *list, int nbnod, int nbcomp, int nbstep,
+static void getNodeMSH(int nbelm, std::vector<double> &list, 
+                       int nbnod, int nbcomp, int nbstep,
                        std::set<pVertex, pVertexLessThan> *nodes, int *numelm)
 {
   if(!nbelm) return;
-  int nb = List_Nbr(list) / nbelm;
-  for(int i = 0; i < List_Nbr(list); i += nb){
-    double *x = (double *)List_Pointer_Fast(list, i);
-    double *y = (double *)List_Pointer_Fast(list, i + nbnod);
-    double *z = (double *)List_Pointer_Fast(list, i + 2 * nbnod);
-    double *v = (double *)List_Pointer_Fast(list, i + 3 * nbnod);
+  int nb = list.size() / nbelm;
+  for(unsigned int i = 0; i < list.size(); i += nb){
+    double *x = &list[i];
+    double *y = &list[i + nbnod];
+    double *z = &list[i + 2 * nbnod];
+    double *v = &list[i + 3 * nbnod];
     for(int j = 0; j < nbnod; j++) {
       pVertex n(x[j], y[j], z[j]);
       std::set<pVertex, pVertexLessThan>::iterator it = nodes->find(n);
@@ -467,19 +508,19 @@ static void writeElementMSH(FILE *fp, int num, int nbnod, pVertex nod[8],
   }
 }
 
-static void writeElementsMSH(FILE *fp, int nbelm, List_T *list,
+static void writeElementsMSH(FILE *fp, int nbelm, std::vector<double> &list,
                              int nbnod, int nbcomp, int dim, 
                              std::set<pVertex, pVertexLessThan> *nodes,
                              int *numelm)
 {
   if(!nbelm) return;
   pVertex nod[8];
-  int nb = List_Nbr(list) / nbelm;
-  for(int i = 0; i < List_Nbr(list); i += nb){
-    double *x = (double *)List_Pointer_Fast(list, i);
-    double *y = (double *)List_Pointer_Fast(list, i + nbnod);
-    double *z = (double *)List_Pointer_Fast(list, i + 2 * nbnod);
-    double *v = (double *)List_Pointer_Fast(list, i + 3 * nbnod);
+  int nb = list.size() / nbelm;
+  for(unsigned int i = 0; i < list.size(); i += nb){
+    double *x = &list[i];
+    double *y = &list[i + nbnod];
+    double *z = &list[i + 2 * nbnod];
+    double *v = &list[i + 3 * nbnod];
     for(int j = 0; j < nbnod; j++) {
       pVertex n(x[j], y[j], z[j]);
       std::set<pVertex, pVertexLessThan>::iterator it = nodes->find(n);
