@@ -8,8 +8,8 @@
 #include <set>
 #include "adaptiveData.h"
 #include "Plugin.h"
-#include "ListUtils.h"
 #include "OS.h"
+#include "GmshDefines.h"
 
 //#define TIMER
 
@@ -50,9 +50,9 @@ static void cleanElement()
   T::allPoints.clear();
 }
 
-static void computeShapeFunctions(gmshMatrix<double> *coeffs, gmshMatrix<double> *eexps,
-				  double u, double v, double w, gmshVector<double> *sf,
-                                  gmshVector<double> *tmp)
+static void computeShapeFunctions(fullMatrix<double> *coeffs, fullMatrix<double> *eexps,
+                                  double u, double v, double w, fullVector<double> *sf,
+                                  fullVector<double> *tmp)
 {
   for(int i = 0; i < eexps->size1(); i++) {
     (*tmp)(i) = pow(u, (*eexps)(i, 0));
@@ -830,7 +830,7 @@ void adaptivePrism::recurError(adaptivePrism *p, double AVG, double tol)
 }
 
 template <class T>
-adaptiveElements<T>::adaptiveElements(std::vector<gmshMatrix<double>*> &p)
+adaptiveElements<T>::adaptiveElements(std::vector<fullMatrix<double>*> &p)
   : _coeffsVal(0), _eexpsVal(0), _interpolVal(0),
     _coeffsGeom(0), _eexpsGeom(0), _interpolGeom(0)
 {
@@ -864,15 +864,15 @@ void adaptiveElements<T>::init(int level)
   int numNodes = _coeffsGeom ? _coeffsGeom->size1() : T::numNodes;
   
   if(_interpolVal) delete _interpolVal;
-  _interpolVal = new gmshMatrix<double>(T::allPoints.size(), numVals);
+  _interpolVal = new fullMatrix<double>(T::allPoints.size(), numVals);
   
   if(_interpolGeom) delete _interpolGeom;
-  _interpolGeom = new gmshMatrix<double>(T::allPoints.size(), numNodes);
+  _interpolGeom = new fullMatrix<double>(T::allPoints.size(), numNodes);
   
-  gmshVector<double> sfv(numVals), *tmpv = 0;
-  gmshVector<double> sfg(numNodes), *tmpg = 0;
-  if(_eexpsVal) tmpv = new gmshVector<double>(_eexpsVal->size1());
-  if(_eexpsGeom) tmpg = new gmshVector<double>(_eexpsGeom->size1());
+  fullVector<double> sfv(numVals), *tmpv = 0;
+  fullVector<double> sfg(numNodes), *tmpg = 0;
+  if(_eexpsVal) tmpv = new fullVector<double>(_eexpsVal->size1());
+  if(_eexpsGeom) tmpg = new fullVector<double>(_eexpsGeom->size1());
 
   int i = 0;
   for(std::set<adaptivePoint>::iterator it = T::allPoints.begin(); 
@@ -927,7 +927,7 @@ void adaptiveElements<T>::adapt(double tol, int numComp,
   }
   
   int numVals = _coeffsVal ? _coeffsVal->size1() : T::numNodes;
-  if(numVals != values.size()){
+  if(numVals != (int)values.size()){
     Msg::Error("Wrong number of values in adaptation %d != %i", 
                numVals, values.size());
     return;
@@ -937,7 +937,7 @@ void adaptiveElements<T>::adapt(double tol, int numComp,
   double t1 = GetTimeInSeconds();
 #endif
   
-  gmshVector<double> val(numVals), res(numPoints);
+  fullVector<double> val(numVals), res(numPoints);
   if(numComp == 1){
     for(int i = 0; i < numVals; i++)
       val(i) = values[i].v[0];
@@ -957,10 +957,10 @@ void adaptiveElements<T>::adapt(double tol, int numComp,
   }
   if(onlyComputeMinMax) return;
   
-  gmshMatrix<double> *resxyz = 0;
+  fullMatrix<double> *resxyz = 0;
   if(numComp == 3){
-    gmshMatrix<double> valxyz(numVals, 3);
-    resxyz = new gmshMatrix<double>(numPoints, 3);
+    fullMatrix<double> valxyz(numVals, 3);
+    resxyz = new fullMatrix<double>(numPoints, 3);
     for(int i = 0; i < numVals; i++){
       valxyz(i, 0) = values[i].v[0];
       valxyz(i, 1) = values[i].v[1];
@@ -970,13 +970,13 @@ void adaptiveElements<T>::adapt(double tol, int numComp,
   }
   
   int numNodes = _coeffsGeom ? _coeffsGeom->size1() : T::numNodes;
-  if(numNodes != coords.size()){
+  if(numNodes != (int)coords.size()){
     Msg::Error("Wrong number of nodes in adaptation %d != %i", 
                numNodes, coords.size());
     return;
   }
   
-  gmshMatrix<double> xyz(numNodes, 3), XYZ(numPoints, 3);
+  fullMatrix<double> xyz(numNodes, 3), XYZ(numPoints, 3);
   for(int i = 0; i < numNodes; i++){
     xyz(i, 0) = coords[i].c[0];
     xyz(i, 1) = coords[i].c[1];
@@ -1044,42 +1044,42 @@ void adaptiveElements<T>::addInView(double tol, int step,
   if(numComp != 1 && numComp != 3) return;
   
   int numEle = 0, *outNb = 0;
-  List_T *outList = 0;
+  std::vector<double> *outList = 0;
   switch(T::numEdges){
   case 1: 
     numEle = in->getNumLines(); 
     outNb = (numComp == 1) ? &out->NbSL : &out->NbVL;
-    outList = (numComp == 1) ? out->SL : out->VL;
+    outList = (numComp == 1) ? &out->SL : &out->VL;
     break;
   case 3:
     numEle = in->getNumTriangles();
     outNb = (numComp == 1) ? &out->NbST : &out->NbVT;
-    outList = (numComp == 1) ? out->ST : out->VT;
+    outList = (numComp == 1) ? &out->ST : &out->VT;
     break;
   case 4:
     numEle = in->getNumQuadrangles();
     outNb = (numComp == 1) ? &out->NbSQ : &out->NbVQ;
-    outList = (numComp == 1) ? out->SQ : out->VQ;
+    outList = (numComp == 1) ? &out->SQ : &out->VQ;
     break;
   case 6:
     numEle = in->getNumTetrahedra();
     outNb = (numComp == 1) ? &out->NbSS : &out->NbVS;
-    outList = (numComp == 1) ? out->SS : out->VS;
+    outList = (numComp == 1) ? &out->SS : &out->VS;
     break;
   case 9: 
     numEle = in->getNumPrisms();
     outNb = (numComp == 1) ? &out->NbSI : &out->NbVI;
-    outList = (numComp == 1) ? out->SI : out->VI;
+    outList = (numComp == 1) ? &out->SI : &out->VI;
     break;
   case 12: 
     numEle = in->getNumHexahedra();
     outNb = (numComp == 1) ? &out->NbSH : &out->NbVH;
-    outList = (numComp == 1) ? out->SH : out->VH;
+    outList = (numComp == 1) ? &out->SH : &out->VH;
     break;
   }
   if(!numEle) return;
   
-  List_Reset(outList);
+  outList->clear();
   *outNb = 0;
   
   for(int ent = 0; ent < in->getNumEntities(step); ent++){
@@ -1115,14 +1115,14 @@ void adaptiveElements<T>::addInView(double tol, int step,
       *outNb += coords.size() / T::numNodes;
       for(unsigned int i = 0; i < coords.size() / T::numNodes; i++){
         for(int k = 0; k < T::numNodes; ++k) 
-          List_Add(outList, &coords[T::numNodes * i + k].c[0]);
+          outList->push_back(coords[T::numNodes * i + k].c[0]);
         for(int k = 0; k < T::numNodes; ++k) 
-          List_Add(outList, &coords[T::numNodes * i + k].c[1]);
+          outList->push_back(coords[T::numNodes * i + k].c[1]);
         for(int k = 0; k < T::numNodes; ++k) 
-          List_Add(outList, &coords[T::numNodes * i + k].c[2]);
+          outList->push_back(coords[T::numNodes * i + k].c[2]);
         for(int k = 0; k < T::numNodes; ++k)
           for(int l = 0; l < numComp; ++l)
-            List_Add(outList, &values[T::numNodes * i + k].v[l]);
+            outList->push_back(values[T::numNodes * i + k].v[l]);
       }
     }
   }
@@ -1133,30 +1133,30 @@ adaptiveData::adaptiveData(PViewData *data)
     _lines(0), _triangles(0), _quadrangles(0), 
     _tetrahedra(0), _hexahedra(0), _prisms(0)
 {
-  _outData = new PViewDataList(true);
-  std::vector<gmshMatrix<double>*> p;
+  _outData = new PViewDataList();
+  std::vector<fullMatrix<double>*> p;
   if(_inData->getNumLines()){
-    _inData->getInterpolationMatrices(1, p);
+    _inData->getInterpolationMatrices(TYPE_LIN, p);
     _lines = new adaptiveElements<adaptiveLine>(p);
   }
   if(_inData->getNumTriangles()){
-    _inData->getInterpolationMatrices(3, p);
+    _inData->getInterpolationMatrices(TYPE_TRI, p);
     _triangles = new adaptiveElements<adaptiveTriangle>(p);
   }
   if(_inData->getNumQuadrangles()){
-    _inData->getInterpolationMatrices(4, p);
+    _inData->getInterpolationMatrices(TYPE_QUA, p);
     _quadrangles = new adaptiveElements<adaptiveQuadrangle>(p);
   }
   if(_inData->getNumTetrahedra()){
-    _inData->getInterpolationMatrices(6, p);
+    _inData->getInterpolationMatrices(TYPE_TET, p);
     _tetrahedra = new adaptiveElements<adaptiveTetrahedron>(p);
   }
   if(_inData->getNumPrisms()){
-    _inData->getInterpolationMatrices(9, p);
+    _inData->getInterpolationMatrices(TYPE_PRI, p);
     _prisms = new adaptiveElements<adaptivePrism>(p);
   }
   if(_inData->getNumHexahedra()){
-    _inData->getInterpolationMatrices(12, p);
+    _inData->getInterpolationMatrices(TYPE_HEX, p);
     _hexahedra = new adaptiveElements<adaptiveHexahedron>(p);
   }
 }
