@@ -84,6 +84,7 @@ public:
       }
       return sqrt(SQU(jac[0][0])+SQU(jac[0][1])+SQU(jac[0][2]));
     default:
+      jac[0][0] = jac[1][1] = jac[2][2] = 1.;
       return 1.;
     }
   }
@@ -832,21 +833,21 @@ public:
   void getGaussPoint(int num, double &u, double &v, double &w, double &weight)
   {
     double u8[8] = {0.2631840555694285,-0.2631840555694285,
-		    0.2631840555694285,-0.2631840555694285,
-		    0.5066163033492386,-0.5066163033492386,
-		    0.5066163033492386,-0.5066163033492386};
+                    0.2631840555694285,-0.2631840555694285,
+                    0.5066163033492386,-0.5066163033492386,
+                    0.5066163033492386,-0.5066163033492386};
     double v8[8] = {0.2631840555694285,0.2631840555694285,
-		    -0.2631840555694285,-0.2631840555694285,
-		    0.5066163033492386,0.5066163033492386,
-		    -0.5066163033492386,-0.5066163033492386};
+                    -0.2631840555694285,-0.2631840555694285,
+                    0.5066163033492386,0.5066163033492386,
+                    -0.5066163033492386,-0.5066163033492386};
     double w8[8] = {0.544151844011225,0.544151844011225,
-		    0.544151844011225,0.544151844011225,
-		    0.122514822655441,0.122514822655441,
-		    0.122514822655441,0.122514822655441};
+                    0.544151844011225,0.544151844011225,
+                    0.122514822655441,0.122514822655441,
+                    0.122514822655441,0.122514822655441};
     double p8[8] = {0.100785882079825,0.100785882079825,
-		    0.100785882079825,0.100785882079825,
-		    0.232547451253508,0.232547451253508,
-		    0.232547451253508,0.232547451253508};
+                    0.100785882079825,0.100785882079825,
+                    0.232547451253508,0.232547451253508,
+                    0.232547451253508,0.232547451253508};
     if(num < 0 || num > 7) return;
     u = u8[num];
     v = v8[num];
@@ -873,20 +874,20 @@ public:
     if(w == 1. && num != 4) {
       switch(num) {
       case 0 : s[0] = -0.25 ; 
-	       s[1] = -0.25 ;
-	       s[2] = -0.25 ; break ;
+               s[1] = -0.25 ;
+               s[2] = -0.25 ; break ;
       case 1 : s[0] =  0.25 ; 
-	       s[1] = -0.25 ;
-	       s[2] = -0.25 ; break ;
+               s[1] = -0.25 ;
+               s[2] = -0.25 ; break ;
       case 2 : s[0] =  0.25 ; 
-	       s[1] =  0.25 ;
+               s[1] =  0.25 ;
                s[2] = -0.25 ; break ;
       case 3 : s[0] = -0.25 ; 
-	       s[1] =  0.25 ;
+               s[1] =  0.25 ;
                s[2] = -0.25 ; break ;
       case 4 : s[0] =  0. ; 
-	       s[1] =  0. ;
-	       s[2] =  1. ; break ;
+               s[1] =  0. ;
+               s[2] =  1. ; break ;
       default : s[0] = s[1] = s[2] = 0.; break;
       }
     }
