@@ -15,5 +15,28 @@ extern void Sort (const ARRAY<double> & values,
 extern void QickSort (const ARRAY<double> & values,
 		      ARRAY<int> & order);
 
-		  
+
+
+
+template <class T>
+inline void BubbleSort (int size, T * data)
+{
+  T hv;
+  for (int i = 0; i < size; i++)
+    for (int j = i+1; j < size; j++)
+      if (data[i] > data[j])
+	{
+	  hv = data[i];
+	  data[i] = data[j];
+	  data[j] = hv;
+	}
+}
+
+template <class T>
+inline void BubbleSort (ARRAY<T> & data)
+{
+  if(data.Size() > 0)
+	  BubbleSort (data.Size(), &data[data.Begin()]);
+}
+
 #endif
